#ident "Kalopa: $Id: bankdump.c,v 1.1 2008/10/15 15:14:37 dtynan Exp $"

/*
 * $Id: bankdump.c,v 1.1 2008/10/15 15:14:37 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: bankdump.c,v $
 * Revision 1.1  2008/10/15 15:14:37  dtynan
 * New applications for checking and verifying bank records.
 *
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <regex.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

char	*smonth[12] = {
	"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
	"JUL", "AUG", "SEP", "OCT", "NOV", "DEC"
};

char	*lmonth[12] = {
	"January", "February", "March", "April", "May", "June",
	"July", "August", "September", "October", "November", "December"
};

int	verbose;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, acno;
	char *cname;
	dbow_conn *dbp;
	struct db_company *comp;
	struct db_bankacct *bap;
	struct db_bank_stmnt *bsp;
	struct db_bankdata *bp;
	struct tm *tmp;
	time_t when;

	/*
	 *
	 */
	verbose = 0;
	acno = 1;
	cname = NULL;
	while ((i = getopt(argc, argv, "a:c:v")) != EOF) {
		switch (i) {
		case 'a':
			if ((acno = atoi(optarg)) < 1)
				usage();
			break;

		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1)
		usage();
	if (parsedate(argv[optind], &when) < 0)
		usage();
	if (verbose)
		printf("Bank records for %s", ctime(&when));
	loadconfig(cname);
	dbp = getdbase();
	/*
	 *
	 */
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("bankdump", "cannot find company");
	if (verbose)
		printf("Company:\t%s\n", comp->cname);
	/*
	 * Find the appropriate bank account...
	 */
	if ((bap = db_findbankacctbyid(dbp, acno)) == NULL)
		berror("bankdump", "cannot find bank account");
	if (verbose)
		printf("Bank Account:\t%s\n", bap->acname);
	/*
	 * Find the bank statement entry...
	 */
	if ((bsp = findstatement(dbp, bap->id, when)) == NULL)
		berror("bankdump", "cannot find appropriate statement");
	/*
	 * Now find all the bank records for that statement...
	 */
	tmp = localtime((time_t *)&bsp->edate);
	printf(".ft HR\n.ce 99\n.sp 4c\n.ps 20\n.vs 22\nULSTER BANK\n");
	printf(".ps 16\n.vs 18\nBusiness Card Statement\n.sp 2c\n.ps 10\n");
	printf(".vs 12\n.ce 0\n.in +9c\n.TS\n;\nlb r.\n");
	printf("Card Number\t5473 6880 0010 2102\n_\n");
	printf("Statement Date\t%02d %s %04d\n_\n", tmp->tm_mday,
				lmonth[tmp->tm_mon], tmp->tm_year + 1900);
	printf("Credit Limit\t\\(Eu2,000\n.TE\n.in 0c\n.sp 2\n.TS\n");
	printf("expand box;\ncbss\ncb cb cb\nr l r.\nCARDHOLDER ACTIVITY\n");
	printf("TRAN DATE\tTRANSACTION DESCRIPTION\tAMOUNT\n_\n");
	for (bp = db_findbankdatabybank_stmnt_id(dbp, bsp->id);
					bp != NULL;
					bp = db_findbankdatanext(dbp, bp)) {
		tmp = localtime((time_t *)&bp->rdate);
		printf("%02d %s\t%s\t%.2f\n", tmp->tm_mday,
				smonth[tmp->tm_mon],
				bp->descr, -bp->amount);
	}
	printf(".TE\n");
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: bankdump [-c #][-v] <date>\n");
	exit(2);
}
