#ident "Kalopa: $Id: brcheck.c,v 1.1 2008/10/15 15:14:37 dtynan Exp $"

/*
 * $Id: brcheck.c,v 1.1 2008/10/15 15:14:37 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: brcheck.c,v $
 * Revision 1.1  2008/10/15 15:14:37  dtynan
 * New applications for checking and verifying bank records.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

int	noeffect;
int	verbose;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, acctno, year, sdate, edate;
	char *cname;
	struct db_company *comp;
	struct db_bankacct *bap;
	struct db_bankdata *bp;
	struct db_journal *jp;
	dbow_conn *dbp;
	struct tm *tmp;
	time_t now;

	/*
	 * Process command-line options.
	 */
	cname = NULL;
	acctno = 1;
	time(&now);
	if ((tmp = localtime(&now)) != NULL)
		year = tmp->tm_year;
	else
		year = 104;
	opterr = noeffect = verbose = 0;
	while ((i = getopt(argc, argv, "a:c:nvy:")) != EOF) {
		switch (i) {
		case 'a':
			if ((acctno = atoi(optarg)) < 1)
				usage();
			break;

		case 'c':
			cname = optarg;
			break;

		case 'y':
			if ((i = atoi(optarg)) < 1990 || i > 2099) {
				fprintf(stderr, "?Error - invalid year.\n");
				exit(2);
			}
			year = i - 1900;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("brcheck", "cannot find specified company");
	if ((bap = db_findbankacctbyid(dbp, acctno)) == NULL)
		berror("brcheck", "cannot find bank account");
	if (verbose)
		printf("Reconciling bank account \"%s\"...\n", bap->acname);
	/*
	 * OK, start trawling through the journal looking for bank account
	 * information.
	 */
	tmp->tm_isdst = -1;
	tmp->tm_mday = 1;
	tmp->tm_mon = 0;
	tmp->tm_year = year;
	tmp->tm_hour = tmp->tm_min = tmp->tm_sec = 0;
	sdate = mktime(tmp);
	printf("On or after:%s", ctime((time_t *)&sdate));
	tmp->tm_year++;
	edate = mktime(tmp);
	printf("Before:%s", ctime((time_t *)&edate));
	printf("NAID: %d\n", bap->nomacct_id);
	jp = findjournalbynomacct(dbp, bap->nomacct_id, sdate, edate);
	for (; jp != NULL; jp = db_findjournalnext(dbp, jp)) {
		if (jp->bankdata_id != 0)
			continue;
		if ((bp = db_findbankdatabyjournal_id(dbp, jp->id)) == NULL) {
			printf(">> %4d %4d %-30.30s\n", jp->id, jp->bankdata_id,
							jp->descr);
			continue;
		}
		jp->bankdata_id = bp->id;
		printf("Fixing %4d [%s] [%d]\n", jp->id, jp->descr, jp->bankdata_id);
		if (!noeffect && db_updatejournalbyid(dbp, jp, jp->id) < 0)
			berror("brcheck", "cannot update journal");
		db_bankdatafree(bp);
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: brcheck [-a a/c #][-nv][-c #][-y year]\n");
	exit(2);
}
