#ident "Kalopa: $Id: brfix.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: brfix.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: brfix.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:23:07  dtynan
 * New utilities to automatically reconcile a bank account.
 *
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <regex.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

int	verbose;
int	noeffect;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, jid, bid;
	char *cname;
	dbow_conn *dbp;
	struct db_company *comp;
	struct db_journal *jp;
	struct db_bankdata *bp;

	/*
	 *
	 */
	verbose = noeffect = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 *
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("brfix", "cannot find company");
	if (verbose)
		printf("Company:\t%s\n", comp->cname);
	if ((argc - optind) != 2)
		usage();
	jid = atoi(argv[optind++]);
	bid = atoi(argv[optind]);
	/*
	 * Find the journal entry and bank data entry...
	 */
	if ((jp = db_findjournalbyid(dbp, jid)) == NULL)
		berror("brfix", "cannot find journal entry");
	if ((bp = db_findbankdatabyid(dbp, bid)) == NULL)
		berror("brfix", "cannot find bank data entry");
	if (verbose) {
		printf("Journal:\t%s\n", jp->descr);
		printf("Bank data:\t%s\n", bp->descr);
		printf("Amount:\t\t%.2f\n", bp->amount);
	}
	if (jp->bankdata_id != 0) {
		printf("?Error - journal is already mapped to bank data.\n");
		exit(1);
	}
	if (bp->journal_id != 0) {
		printf("?Error - bank data is already mapped to journal.\n");
		exit(1);
	}
	jp->bankdata_id = bp->id;
	bp->journal_id = jp->id;
	if (!noeffect && db_updatejournalbyid(dbp, jp, jp->id) < 0)
		berror("brfix", "cannot update the journal");
	if (!noeffect && db_updatebankdatabyid(dbp, bp, bp->id) < 0)
		berror("brfix", "cannot update the bank data");
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: brfix [-c #][-nv] j.id bd.id\n");
	exit(2);
}
