#ident "Kalopa: $Id: main.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: main.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: main.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:23:07  dtynan
 * New utilities to automatically reconcile a bank account.
 *
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <regex.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

#define RECON_WINDOW		604800		/* Seven days */

/*
 *
 */
struct	relist	{
	struct	relist		*next;
	regex_t			preg;
	struct	db_autorecon	*arp;
};

int	verbose;
int	noeffect;
struct	relist	*rehead, *retail;

void	usage();

struct	db_vat	*db_findvatcodes(dbow_conn *);

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, acno, jid, drcrf, goodf, vcode, sdate, edate;
	char *cname;
	dbow_conn *dbp;
	struct db_company *comp;
	struct db_bankdata *bp;
	struct db_bankacct *bap;
	struct db_autorecon *arp;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct relist *rp;
	double netamount;

	/*
	 *
	 */
	acno = 1;
	verbose = noeffect = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "a:c:nv")) != EOF) {
		switch (i) {
		case 'a':
			if ((acno = atoi(optarg)) < 1)
				usage();
			break;

		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 *
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("bankrecon", "cannot find company");
	if (verbose)
		printf("Company:\t%s\n", comp->cname);
	/*
	 * Find the appropriate bank account...
	 */
	if ((bap = db_findbankacctbyid(dbp, acno)) == NULL)
		berror("bankrecon", "cannot find bank account");
	if (verbose)
		printf("Bank Account:\t%s\n", bap->acname);
	/*
	 * First off, find all the RE expressions which refer to this
	 * bank account and generate an 'relist' entry for them.  This
	 * also means compiling the regular expression.
	 */
	rehead = retail = NULL;
	for (arp = db_findautoreconbybankacct_id(dbp, acno); arp != NULL;
				arp = db_findautoreconnext(dbp, NULL)) {
		if ((rp = (struct relist *)malloc(sizeof(*rp))) == NULL)
			berror("bankrecon", "malloc failed");
		rp->next = NULL;
		rp->arp = arp;
		if (regcomp(&rp->preg, rp->arp->expr,
					REG_EXTENDED|REG_ICASE|REG_NOSUB) != 0)
			berror("bankrecon", "regcomp failure");
		if (rehead == NULL)
			rehead = rp;
		else
			retail->next = rp;
		retail = rp;
	}
	/*
	 * Now find each unposted bank transaction and try to create a
	 * journal entry for it.
	 */
	for (bp = findunposted(dbp, bap->id); bp != NULL;
					bp = db_findbankdatanext(dbp, bp)) {
		if (bp->amount < 0) {
			netamount = -bp->amount;
			drcrf = 1;
		} else {
			netamount = bp->amount;
			drcrf = 0;
		}
		printf("%4d  %s  ", bp->id, prabbrev(&bp->rdate));
		if (bp->number > 0)
			printf("%6d  ", bp->number);
		else
			printf("        ");
		printf("%-30.30s  %8.2f(%d)\n", bp->descr, netamount, drcrf);
		for (rp = rehead; rp != NULL; rp = rp->next)
			if (regexec(&rp->preg, bp->descr, 0, NULL, 0) == 0)
				break;
		if (rp == NULL)
			continue;
		arp = rp->arp;
		/*
		 * OK, we've found a live one.  Now see if we can find an
		 * existing journal entry.  If not, we'll need to create
		 * one.  The 'find' function looks for journal entries with
		 * a posting date a week before or after the bank date and
		 * with the same nominal A/C number.  Then we go through
		 * and see if the description is the same.
		 */
		printf("JDescr: [%s]\n", arp->jdescr);
		sdate = bp->rdate - RECON_WINDOW;
		edate = bp->rdate + RECON_WINDOW;
		jp = findjournalbynomacct(dbp, arp->nomacct_id, sdate, edate);
		for (; jp != NULL; jp = db_findjournalnext(dbp, jp)) {
			if (jp->bankdata_id != 0)
				continue;
			if (strcmp(jp->descr, arp->jdescr) == 0)
				break;
		}
		jid = goodf = 0;
		if (jp != NULL) {
			/*
			 * OK, we have a match.  Now try and line up the
			 * journal_item records, looking for the bank
			 * details and looking for a similar amount.
			 */
			printf(">Journal[%d]:\t\"%s\" - %s", jp->id, jp->descr, ctime((time_t *)&jp->jdate));
			printf("Bankdata ID: %d\n", jp->bankdata_id);
			for (jip = findjournalitems(dbp, jp->id);
					jip != NULL; jip =
					db_findjournal_itemnext(dbp, jip)) {
				printf(">>\t\t%1d %04d %1d %8.2f - ",
					jip->item_id,
					jip->nomacct_id,
					jip->drcrf,
					jip->amount);
				if (jip->amount == -bp->amount)
					printf("Amount Good: ");
				if (jip->nomacct_id == bap->nomacct_id && jip->drcrf == 1) {
					if (jip->amount != -bp->amount)
						goodf = 0;
					else
						goodf |= 02;
					printf("FROM BANK A/C!\n");
				} else if (jip->nomacct_id == arp->nomacct_id && jip->drcrf == 0) {
					printf("TO CORRECT A/C!\n");
					goodf |= 01;
				} else
					printf("Dunno?!\n");
			}
			if (goodf == 03) {
				printf(">Good record - a keeper.\n");
				jid = jp->id;
			}
		}
		if (jid == 0) {
			/*
			 * No existing record found...
			 */
			 vcode = -1;
			if (arp->vat_id != 0) {
				vcode = arp->vat_id;
				netamount = vat_findnet(netamount, vcode);
			}
			printf("VCode: %d\n", vcode);
			if (!noeffect) {
				printf(">Creating a new journal entry...\n");
				if (drcrf) {
					jid = journal_entry((time_t*)&bp->rdate,
						NULL, arp->jdescr, vcode,
						arp->nomacct_id,
						bap->nomacct_id,
						netamount);
				} else {
					jid = journal_entry((time_t*)&bp->rdate,
						NULL, arp->jdescr, vcode,
						bap->nomacct_id,
						arp->nomacct_id,
						netamount);
				}
			}
			jp = db_findjournalbyid(dbp, jid);
		}
		/*
		 * At this point, if we have a journal ID, we can
		 * update the bank data record.
		 */
		if (jid > 0) {
			printf(">Journal ID is %d\n", jid);
			bp->journal_id = jid;
			if (!noeffect)
				db_updatebankdatabyid(dbp, bp, bp->id);
		}
		if (jp != NULL && jp->bankdata_id == 0) {
			jp->bankdata_id = bp->id;
			if (!noeffect)
				db_updatejournalbyid(dbp, jp, jp->id);
		}
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: bankrecon [-c #][-v][-a bankacct]\n");
	exit(2);
}
