#
# $Id: bankdata.d,v 1.7 2008/07/08 16:36:01 dtynan Exp $
#
# Copyright (c) 2006, Kalopa Research Limited.  All rights reserved.
# This is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your
# option) any later version.
#
# It is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this product; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
# USA.
#
# THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
# RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# ABSTRACT
#
# $Log: bankdata.d,v $
# Revision 1.7  2008/07/08 16:36:01  dtynan
# Extensive changes.
#
# Revision 1.6  2008/01/23 17:52:16  dtynan
# Added a 'edate' field as well as the sdate field and a function to see
# if a statement exists for a given date.
#
# Revision 1.5  2008/01/23 16:43:02  dtynan
# Added new tables for managing bank statement reconciliation
#
# Revision 1.4  2008/01/15 18:10:39  dtynan
# Changed to use new company name and copyright, also fixed a few old
# files with the wrong license.
#
# Revision 1.3  2007/08/06 10:20:43  dtynan
# Added a bankdata_id to track the bank data record associated with this
# journal entry.  Also added some more utility routines.
#
# Revision 1.2  2006/07/31 09:07:54  dtynan
# Renamed all primary keys from xxx_id to id (where xxx is the table
# name).  This is both convenient and supports the Ruby/Rails
# methodology.
#
# Revision 1.1  2006/07/31 08:48:53  dtynan
# Added a new table for bank data.
#
#
# COMPANYDB
#
%table	bank_stmnt	{
	id mediumint(7) not null auto_increment primary key,
	bankacct_id mediumint(7) not null,
	sdate date not null,
	edate date not null
%}

%search bank_stmnt id
%search bank_stmnt bankacct_id
%search bank_stmnt sdate

%table	bankdata	{
	id mediumint(7) not null auto_increment primary key,
	bank_stmnt_id mediumint(7) not null,
	journal_id mediumint(7),
	rdate date not null,
	number mediumint(7),
	descr varchar(254),
	amount double
%}

%search bankdata id
%search bankdata bank_stmnt_id
%search bankdata journal_id
%search bankdata rdate
%update bankdata id

#
# The autorecon table matches a regular expression (regex(3)) with a
# bankdata record and if found, updates the journal to reflect the
# record.  This surprisingly useful little tool can zip through a
# bank statement (bankdata) in seconds and reconcile it with the
# journal, adding records where appropriate.
#
%table	autorecon	{
	id mediumint(7) not null auto_increment primary key,
	bankacct_id mediumint(7) not null,
	expr varchar(254) not null,
	jdescr varchar(254) not null,
	nomacct_id mediumint(7) not null,
	vat_id smallint(5)
%}

%search autorecon bankacct_id

%type C emit {
/*
 *
 */
struct db_bank_stmnt *
findstatement(dbow_conn *c, int bid, int when)
{
	if (dbow_query(c, "SELECT * FROM bank_stmnt WHERE bankacct_id = $c AND sdate <= $i AND edate >= $i", bid, when, when) < 0)
		return(NULL);
	return(db_findbank_stmntnext(c, NULL));
}

/*
 *
 */
struct db_bankdata *
findunposted(dbow_conn *c, int bid)
{
	if (dbow_query(c, "SELECT d.* FROM bankdata d, bank_stmnt s WHERE s.bankacct_id = $c AND d.bank_stmnt_id = s.id AND d.journal_id = 0 ORDER BY d.rdate",
bid) < 0)
		return(NULL);
	return(db_findbankdatanext(c, NULL));
}
%}
