#
# $Id: journal.d,v 1.15 2008/10/15 15:11:13 dtynan Exp $
#
# Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
# Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
# This is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your
# option) any later version.
#
# It is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this product; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
# USA.
#
# THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
# RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# ABSTRACT
#
# $Log: journal.d,v $
# Revision 1.15  2008/10/15 15:11:13  dtynan
# Mammoth change to convert from old-style database configuration to new
# schema where an RC file defines the database access.
#
# Revision 1.14  2008/01/15 18:10:40  dtynan
# Changed to use new company name and copyright, also fixed a few old
# files with the wrong license.
#
# Revision 1.13  2007/08/07 08:16:22  dtynan
# Added a function to find if any journal items for a specific journal
# entry reference a specific nominal account number.
#
# Revision 1.12  2007/08/06 10:20:43  dtynan
# Added a bankdata_id to track the bank data record associated with this
# journal entry.  Also added some more utility routines.
#
# Revision 1.11  2006/10/01 10:14:50  dtynan
# Created new mechanism for running overnight tasks.
#
# Revision 1.10  2006/07/31 09:07:54  dtynan
# Renamed all primary keys from xxx_id to id (where xxx is the table
# name).  This is both convenient and supports the Ruby/Rails
# methodology.
#
# Revision 1.9  2005/08/12 18:44:53  dtynan
# Added a new comment in each file to show whether the table belongs in
# the main or the per-company databases.
#
# Revision 1.8  2005/03/22 09:41:11  dtynan
# Extensive upgrade to make sure the copyright block was
# consistent and GPL'ed.
#
# Revision 1.7  2005/02/17 12:02:30  dtynan
# Added a user_id field to show who created the entry.
#
# Revision 1.6  2005/01/13 20:44:08  dtynan
# Added a new function to post end of year totals.
#
# Revision 1.5  2004/12/05 22:58:58  dtynan
# Added a 'folio' field to the journal and totals to the invoice.
#
# Revision 1.4  2004/08/26 17:25:28  dtynan
# Renamed amount field.
#
# Revision 1.3  2004/08/05 12:45:17  dtynan
# Removed reference to company from journal (each company is now in a
# separate database).
#
# Revision 1.2  2004/08/04 11:01:50  dtynan
# Various changes for new & improved system.  Made the invoice header
# company-specific, modified the primary key of the journal and removed
# extraneous income types.
#
# Revision 1.1  2004/07/26 12:53:17  dtynan
# Added new table for journal entries.
#
# COMPANYDB
#
%table	journal	{
	id mediumint(7) NOT NULL AUTO_INCREMENT primary key,
	jdate date NOT NULL,
	folio varchar(16),
	descr varchar(254),
	user_id mediumint(7),
	bankdata_id mediumint(7),
	state tinyint(3)
%}

%insert journal
%search journal id
%search journal folio
%search journal state
%search journal descr
%update journal id
%delete journal id

%table	journal_item {
	journal_id mediumint(7) NOT NULL primary key,
	item_id mediumint(7) NOT NULL primary key,
	nomacct_id mediumint(7),
	drcrf tinyint(4),
	amount double
%}

%type C emit {
/*
 *
 */
struct db_journal *
findjournalentries(dbow_conn *c)
{
	return(db_runjournalquery(c, "SELECT * FROM journal ORDER BY jdate,id"));
}


/*
 * Find journal entry based on Nominal A/C ID and a date window.
 * This is used by the bank a/c reconciliation program to determine
 * if the given record already has a journal entry.
 */
struct db_journal *
findjournalbynomacct(dbow_conn *c, int naid, int sdate, int edate)
{
	if (dbow_query(c, "SELECT j.* FROM journal j, journal_item ji WHERE j.id = ji.journal_id AND ji.nomacct_id=$c AND j.jdate >= $i AND j.jdate < $i ORDER BY jdate,id", naid, sdate, edate) < 0)
		return(NULL);
	return(db_findjournalnext(c, NULL));
}

/*
 *
 */
struct db_journal_item *
findjournalitems(dbow_conn *c, int jno)
{
	if (dbow_query(c, "SELECT * FROM journal_item WHERE journal_id=$c ORDER BY item_id", jno) < 0)
		return(NULL);
	return(db_findjournal_itemnext(c, NULL));
}

/*
 *
 */
struct db_journal_item *
findjournalitembynomacct(dbow_conn *c, int jno, int nomacct)
{
	if (dbow_query(c, "SELECT * FROM journal_item WHERE journal_id=$c AND nomacct_id = $c", jno, nomacct) < 0)
		return(NULL);
	return(db_findjournal_itemnext(c, NULL));
}

/*
 *
 */
int
updatejournal_item(dbow_conn *c, struct db_journal_item *p)
{
	if (dbow_query(c, "UPDATE journal_item SET nomacct_id = $c,drcrf = $a,amount = $g WHERE journal_id = $c AND item_id = $c",
						p->nomacct_id,
						p->drcrf,
						p->amount,
			p->journal_id, p->item_id) < 0)
		return(-1);
	return(0);
}

/*
 *
 */
void
deletejournalitems(dbow_conn *c, int jno)
{
	dbow_query(c, "DELETE FROM journal_item WHERE journal_id=$c", jno);
}
%}
