#
# $Id: runcom.d,v 1.3 2008/01/15 18:10:40 dtynan Exp $
#
# Copyright (c) 2004, Kalopa Media Limited.
# Copyright (c) 2006, Kalopa Research Limited.  All rights reserved.
# This is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your
# option) any later version.
#
# It is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this product; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
# USA.
#
# THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
# RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# ABSTRACT
# The RunCom system is used to run certain commands at various
# times.  It is akin to the cron system under UNIX, but the
# configuration is more in line with how Beanie works.  The 'rtype'
# specifies how often the RunCom should be executed.  See beanie.h
# for more details.  The 'ndate' is the next, predicted run-date.
# 'edate' is when the RC should terminate.
#
# $Log: runcom.d,v $
# Revision 1.3  2008/01/15 18:10:40  dtynan
# Changed to use new company name and copyright, also fixed a few old
# files with the wrong license.
#
# Revision 1.2  2007/10/29 17:13:37  dtynan
# Various database changes to support hourly billing.
#
# Revision 1.1  2006/10/01 10:14:50  dtynan
# Created new mechanism for running overnight tasks.
#
# From when the file was called rjentry.d...
# Revision 1.8  2006/07/31 09:07:54  dtynan
# Renamed all primary keys from xxx_id to id (where xxx is the table
# name).  This is both convenient and supports the Ruby/Rails
# methodology.
#
# Revision 1.7  2005/08/12 18:44:53  dtynan
# Added a new comment in each file to show whether the table belongs in
# the main or the per-company databases.
#
# Revision 1.6  2005/03/31 10:08:34  dtynan
# The recurring journals now use the same frequency table as
# everything else.
#
# Revision 1.5  2005/03/22 09:41:12  dtynan
# Extensive upgrade to make sure the copyright block was
# consistent and GPL'ed.
#
# Revision 1.4  2004/12/05 22:58:58  dtynan
# Added a 'folio' field to the journal and totals to the invoice.
#
# Revision 1.3  2004/09/30 09:55:50  dtynan
# Added a new 'delete' function to the rjentry code and added a comment
# block to the nomacct file.
#
# Revision 1.2  2004/08/26 17:26:55  dtynan
# Minor changes for recurring journals.
#
# Revision 1.1  2004/08/07 15:43:33  dtynan
# Added new database table for managing recurring journal entries.
#
# COMPANYDB
#
%table	runcom	{
	id mediumint(7) not null auto_increment primary key,
	rtype tinyint(3),
	ndate date not null,
	edate date,
	seqno tinyint(3) not null,
	command varchar(254) not null,
	state tinyint(3)
%}

%type C emit {
/*
 *
 */
struct db_runcom *
findruncom(dbow_conn *c, int when)
{
	if (dbow_query(c, "SELECT id,rtype,ndate,edate,seqno,command,state FROM runcom WHERE ndate < $i AND state = 0 ORDER BY ndate,id", when) < 0)
		return(NULL);
	return(db_findruncomnext(c, NULL));
}
%}
