#
# $Id: salesorder.d,v 1.9 2008/01/15 18:10:40 dtynan Exp $
#
# Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
# This is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your
# option) any later version.
#
# It is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this product; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
# USA.
#
# THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
# RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# ABSTRACT
#
# Sales order table.  There is one of these for each order in the system.
# If it is in a READY state, then it is possible to produce an invoice
# against this sales order.
#	id	Unique sales order ID
#	customer_id	Customer ID
#	odate		Order date
#	yref		Your reference (probably a PO number)
#	descr		Description of order (parts for new vehicle)
#
# Sales order item table.
#	salesorder_id	Reference to the main sales order.
#	item_id		Specific item on the sales order (1...N)
#	product_id	Reference to the product table.
#	rdate		Run date - the last time this item was actioned
#	cdate		Completion date - move to DONE after this date
#	state		Current state of the order (see SALESORDER_*)
#	freq		How often to run this item (see FREQUENCY_*)
#	descr		Description of this line-item as shown on invoice
#	vcode		VAT Code {1=A,etc}
#	quant		Quantity (as per invoice)
#	unitc		Unit cost (as per invoice)
#	disc		Discount (as per invoice)
#
# $Log: salesorder.d,v $
# Revision 1.9  2008/01/15 18:10:40  dtynan
# Changed to use new company name and copyright, also fixed a few old
# files with the wrong license.
#
# Revision 1.8  2007/11/11 14:14:08  dtynan
# Fixed a bug in updateorderitems and added a new function which just
# updates the rundate.
#
# Revision 1.7  2007/10/29 17:13:37  dtynan
# Various database changes to support hourly billing.
#
# Revision 1.6  2006/07/31 09:07:54  dtynan
# Renamed all primary keys from xxx_id to id (where xxx is the table
# name).  This is both convenient and supports the Ruby/Rails
# methodology.
#
# Revision 1.5  2005/09/21 18:02:29  dtynan
# Major changes to deal with data import/export.
#
# Revision 1.4  2005/08/12 18:43:44  dtynan
# Added additional components.
#
# Revision 1.3  2005/03/31 10:09:08  dtynan
# Added definitions for sales order state.
#
# Revision 1.2  2005/03/22 09:41:12  dtynan
# Extensive upgrade to make sure the copyright block was
# consistent and GPL'ed.
#
# Revision 1.1  2005/03/11 13:15:44  dtynan
# Added new sales order table entry (will require drastic changes later
# on).  For now, just track the sales order with the invoice.
#
# COMPANYDB
#
%table	salesorder	{
	id mediumint(7) not null auto_increment primary key,
	customer_id mediumint(7) not null,
	odate date not null,
	yref varchar(254),
	descr varchar(254)
%}

%insert salesorder
%search salesorder id
%search salesorder customer_id
%dump salesorder

%table	sorder_item {
	salesorder_id mediumint(7) not null primary key,
	item_id mediumint(7) not null primary key,
	product_id mediumint(7),
	rdate date not null,
	cdate date,
	state tinyint(3),
	freq tinyint(3) not null,
	descr varchar(254) not null,
	vcode tinyint,
	quant real,
	unitc real,
	disc real
%}

%search sorder_item salesorder_id
%dump sorder_item

%type C proto {
/*
 * NEW means the sales order has just been created and isn't ready for
 * prime time.  Ignore it.  READY means that it is billable if the
 * run-date allows.  DONE means the sales order is closed out.  This
 * happens to a "ONCE" sales order after it has been invoiced.  If the
 * sales order is cancelled, it is set to the DONE state.  If it is
 * marked as BLOCKED then the invoice cannot be billed until some
 * external stimulus moves it out of that state.  Note that a "MANUAL"
 * invoice goes to BLOCKED each time an invoice is generated.  This
 * should be linked in with the work-request system.
 */
#define SALESORDER_NEW		0
#define SALESORDER_READY	1
#define SALESORDER_DONE		2
#define SALESORDER_BLOCKED	3
%}

%type C emit {
/*
 *
 */
struct db_sorder_item *
findsalesorderitems(dbow_conn *c, int soid)
{
	if (dbow_query(c, "SELECT * FROM sorder_item WHERE salesorder_id = $c ORDER BY item_id", soid) < 0)
		return(NULL);
	return(db_findsorder_itemnext(c, NULL));
}

/*
 *
 */
struct db_sorder_item *
findsalesorderitem(dbow_conn *c, int soid, int itemno)
{
	if (dbow_query(c, "SELECT * FROM sorder_item WHERE salesorder_id = $c AND item_id = $c", soid, itemno) < 0)
		return(NULL);
	return(db_findsorder_itemnext(c, NULL));
}

/*
 * Update the specified sales order item.
 */
int
updateorderitem(dbow_conn *c, struct db_sorder_item *p)
{
	if (dbow_query(c, "UPDATE sorder_item SET product_id = $c,rdate = $i,cdate = $i,state = $a,freq = $a,descr = $o,vcode = $a,quant = $g,unitc = $g,disc = $g WHERE salesorder_id = $c AND item_id = $c",
						p->product_id,
						p->rdate,
						p->cdate,
						p->state,
						p->freq,
						p->descr,
						p->vcode,
						p->quant,
						p->unitc,
						p->disc,
			 p->salesorder_id, p->item_id) < 0)
		return(-1);
	return(0);
}

/*
 * Only update the run-date.
 */
int
newrundate(dbow_conn *c, struct db_sorder_item *p)
{
	if (dbow_query(c, "UPDATE sorder_item SET rdate = $i WHERE salesorder_id = $c AND item_id = $c",
				p->rdate, p->salesorder_id, p->item_id) < 0)
		return(-1);
	return(0);
}
%}
