#
# $Id: vatret.d,v 1.9 2008/10/13 20:59:37 dtynan Exp $
#
# Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
# Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
# This is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your
# option) any later version.
#
# It is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this product; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
# USA.
#
# THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
# RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# ABSTRACT
# Table for maintaining VAT returns.
#
# $Log: vatret.d,v $
# Revision 1.9  2008/10/13 20:59:37  dtynan
# Extensive changes (part 1) for change to database configuration.
#
# Revision 1.8  2008/07/08 16:36:01  dtynan
# Extensive changes.
#
# Revision 1.7  2008/01/15 18:10:40  dtynan
# Changed to use new company name and copyright, also fixed a few old
# files with the wrong license.
#
# Revision 1.6  2006/10/01 10:14:50  dtynan
# Created new mechanism for running overnight tasks.
#
# Revision 1.5  2006/08/25 11:30:56  dtynan
# Added new code for managing VAT returns.
#
# Revision 1.4  2006/07/31 09:07:54  dtynan
# Renamed all primary keys from xxx_id to id (where xxx is the table
# name).  This is both convenient and supports the Ruby/Rails
# methodology.
#
# Revision 1.3  2005/08/12 18:44:53  dtynan
# Added a new comment in each file to show whether the table belongs in
# the main or the per-company databases.
#
# Revision 1.2  2005/03/22 09:41:13  dtynan
# Extensive upgrade to make sure the copyright block was
# consistent and GPL'ed.
#
# Revision 1.1  2004/08/26 17:26:08  dtynan
# Added new SQL tables for cheque register and VAT returns.
#
# COMPANYDB
#
%table vatret {
	id mediumint(7) NOT NULL AUTO_INCREMENT primary key,
	sdate date not null,
	edate date not null,
	vtype tinyint(1),
	sales int(9) not null,
	purchs int(9) not null,
	goodsto int(9) not null,
	goodsfrom int(9) not null
%}

%search vatret id

%table vatdata	{
	id mediumint(7) not null auto_increment primary key,
	vatret_id mediumint(7),
	journal_id mediumint(7) not null,
	vdtype tinyint(1),
	vat_id mediumint(7) not null,
	amount double not null,
	total double not null
%}

%search vatdata id
%search vatdata vatret_id
%search vatdata journal_id
%search vatdata vat_id

%type C proto {
/*
 * Definitions for VAT3 types...
 */
#define VAT3_CURRENCY_EURO	"E"
#define VAT3_CURRENCY_POUND	"P"
#define VAT3_FREQ_BIMONTHLY	0
#define VAT3_FREQ_OTHER		1
#define VAT3_TYPE_ORIGINAL	0
#define VAT3_TYPE_SUPPLEMENTARY	1
#define VAT3_TYPE_AMENDED	1

/*
 * Definitions for VAT data types...
 */
#define VATDATA_NOPOST		0	/* Do not post VAT data records */
#define VATDATA_NONE		1	/* Does not correlate to a VAT return */
#define VATDATA_SALES		2	/* VAT Sales data */
#define VATDATA_PURCHS		3	/* VAT Purchases data */
#define VATDATA_GOODSTO		4	/* VAT Goods to EU data */
#define VATDATA_GOODSFROM	5	/* VAT Goods from EU data */
%}

%type C emit {
/*
 *
 */
struct db_vatret *
findvatreturn(dbow_conn *c, time_t when)
{
	if (dbow_query(c, "SELECT id,sdate,edate,vtype,sales,purchs,goodsto,goodsfrom FROM vatret WHERE sdate<= $i AND edate>= $i", when, when) < 0)
		return(NULL);
	return(db_findvatretnext(c, NULL));
}

/*
 *
 */
struct db_vatdata *
findvatdata(dbow_conn *c, time_t when, int retno)
{
	if (dbow_query(c, "SELECT v.id,v.vatret_id,v.journal_id,v.vdtype,v.vat_id,v.amount,v.total FROM vatdata v, journal j WHERE (v.vatret_id = 0 OR v.vatret_id = $c) AND v.journal_id = j.id AND j.jdate <= $i", retno, when) < 0)
		return(NULL);
	return(db_findvatdatanext(c, NULL));
}
%}
