#ident "$Id: customer.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: customer.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA
 * 02139, USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: customer.c,v $
 * Revision 1.5  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.4  2008/01/15 18:10:40  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2006/07/31 10:17:47  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.2  2005/08/12 19:37:15  dtynan
 * Minor tweaks prior to first tagged release.
 *
 * Revision 1.1  2005/08/12 18:49:25  dtynan
 * First pass at a utility to pull data from the database and produce an
 * XML representation for exporting to other applications.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#include "export.h"

/*
 *
 */
void
docustomer(FILE *fp, int id, int in)
{
	char tagbuf[64], *vcp;
	struct db_customer *cp;
	dbow_conn *dbp = getdbase();

	if ((cp = db_findcustomerbyid(dbp, id)) == NULL)
		berror("export", "cannot find customer");
	sprintf(tagbuf, "number=\"%d\"", cp->id);
	block(fp, "customer", BLOCK_OPEN|BLOCK_MULTILINE, tagbuf, NULL, in++);
	docontact(fp, cp->contact_id, in);
	if (cp->cperson_id != 0)
		docontactperson(fp, cp->contact_id, cp->cperson_id, in);
	if (cp->currency_id != HOME_CURRENCY)
		blockint(fp, "currency", NULL, cp->currency_id, in);
	switch (cp->billtype) {
	case CUST_BTYPE_POST:
		blockstr(fp, "postbill", NULL, NULL, in);
		break;

	case CUST_BTYPE_FAX:
		blockstr(fp, "faxbill", NULL, NULL, in);
		break;

	case CUST_BTYPE_EMAIL:
		blockstr(fp, "emailbill", NULL, NULL, in);
		break;
	}
	switch (cp->vattype) {
	default:
		vcp = NULL;
		break;

	case CUST_VTYPE_HOME:
		vcp = "vtype=\"home\"";
		break;

	case CUST_VTYPE_EUNR:
		vcp = "vtype=\"EUnonr\"";
		break;

	case CUST_VTYPE_EUREG:
		vcp = "vtype=\"EUreg\"";
		break;

	case CUST_VTYPE_EXT:
		vcp = "vtype=\"external\"";
		break;
	}
	if (vcp != NULL)
		blockstr(fp, "vatno", vcp, cp->vat, in);
	if (cp->terms != NULL)
		blockstr(fp, "terms", NULL, cp->terms, in);
	blockfloat(fp, "balance", NULL, cp->balance, in);
	blockint(fp, "flag", NULL, cp->cusflag, in);
	block(fp, "customer", BLOCK_CLOSE, NULL, NULL, --in);
	db_customerfree(cp);
}
