#ident "$Id: invoice.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: invoice.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA
 * 02139, USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: invoice.c,v $
 * Revision 1.5  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.4  2008/01/15 18:10:40  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2006/07/31 10:17:47  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.2  2005/09/21 14:53:29  dtynan
 * The order state is now on a per-item basis.
 *
 * Revision 1.1  2005/08/12 18:49:25  dtynan
 * First pass at a utility to pull data from the database and produce an
 * XML representation for exporting to other applications.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#include "export.h"

/*
 *
 */
void
doinvoice(FILE *fp, int id, int in)
{
	char tagbuf[64];
	struct db_invoice *ip;
	struct db_invoice_item *iip;
	dbow_conn *dbp = getdbase();

	if ((ip = db_findinvoicebyid(dbp, id)) == NULL)
		berror("export", "cannot find invoice");
	sprintf(tagbuf, "number=\"%d\"", ip->id);
	block(fp, "invoice", BLOCK_OPEN|BLOCK_MULTILINE, tagbuf, NULL, in++);
	blockdate(fp, "date", NULL, ip->idate, in);
	blockdate(fp, "duedate", NULL, ip->ddate, in);
	blockdate(fp, "orderdate", NULL, ip->odate, in);
	docustomer(fp, ip->customer_id, in);
	if (ip->salesorder_id != 0) {
		sprintf(tagbuf, "number=\"%d\"", ip->salesorder_id);
		blockstr(fp, "salesorder", tagbuf, NULL, in);
	}
	blockfloat(fp, "total", NULL, ip->total, in);
	blockfloat(fp, "vat_total", NULL, ip->vatttl, in);
	if (ip->shipping != 0.0)
		blockfloat(fp, "shipping", NULL, ip->shipping, in);
	if (ip->yref != NULL)
		blockstr(fp, "yourref", NULL, ip->yref, in);
	if (ip->descr != NULL)
		blockstr(fp, "descr", NULL, ip->descr, in);
	blockint(fp, "state", NULL, ip->state, in);
	iip = db_findinvoice_itembyinvoice_id(dbp, ip->id);
	while (iip != NULL) {
		block(fp, "item", BLOCK_OPEN|BLOCK_MULTILINE, NULL, NULL, in++);
		blockint(fp, "nomacct", NULL, iip->nomacct_id, in);
		block(fp, "descr", BLOCK_OPEN|BLOCK_MULTILINE|BLOCK_CLOSE,
							NULL, iip->descr, in);
		blockint(fp, "vatcode", NULL, iip->vcode, in);
		blockint(fp, "quantity", NULL, iip->quant, in);
		blockfloat(fp, "unitcost", NULL, iip->unitc, in);
		if (iip->disc != 0.0)
			blockfloat(fp, "discount", NULL, iip->disc, in);
		block(fp, "item", BLOCK_CLOSE, NULL, NULL, --in);
		iip = db_findinvoice_itemnext(dbp, iip);
	}
	block(fp, "invoice", BLOCK_CLOSE, NULL, NULL, --in);
	db_invoicefree(ip);
}
