#ident "$Id: main.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: main.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA
 * 02139, USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: main.c,v $
 * Revision 1.7  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.6  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.5  2008/01/15 18:10:40  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2008/01/15 15:38:14  dtynan
 * Fixed issues with new table names.
 *
 * Revision 1.3  2006/07/31 10:17:47  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.2  2005/08/15 12:03:25  dtynan
 * Added functionality to export share certificates.
 *
 * Revision 1.1  2005/08/12 18:49:25  dtynan
 * First pass at a utility to pull data from the database and produce an
 * XML representation for exporting to other applications.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <expat.h>

#include "beanie.h"
#include "hacks.h"

#include "export.h"

#define DOCUSTOMER	1
#define DOINVOICE	2

/*
 * Command-line options...
 */
struct	cmdopts	{
	char	*name;
	char	*xslname;
	char	*dtdname;
	void	(*func)(FILE *, int, int);
} options[] = {
	{"customer", "customer.1.0.xml", "customer.1.0.dtd", docustomer},
	{"invoice", "invoice.1.0.xml", "invoice.1.0.dtd", doinvoice},
	{"product", "product.1.0.xml", "product.1.0.dtd", doproduct},
	{"salesorder", "salesorder.1.0.xml", "salesorder.1.0.dtd", dosorder},
	{"sharecert", "sharecert.1.0.xml", "sharecert.1.0.dtd", dosharecert},
	{NULL, NULL, NULL, NULL}
};

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, verbose;
	char *cname, *fname;
	struct db_company *comp;
	dbow_conn *dbp;
	struct cmdopts *optp = NULL;
	FILE *fp;

	cname = fname = NULL;
	verbose = 0;
	while ((i = getopt(argc, argv, "c:vo:-:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		case 'o':
			fname = optarg;
			break;

		case '-':
			for (optp = options; optp->name != NULL; optp++)
				if (strcmp(optp->name, optarg) == 0)
					break;
			if (optp->name == NULL)
				usage();
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1 || optp == NULL)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Find the company that we're processing...
	 */
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("export", "cannot find company");
	if ((i = atoi(argv[optind++])) < 1)
		berror("export", "invalid argument");
	if (fname == NULL) {
		if ((fname = (char *)malloc(strlen(optp->name) + 20)) == NULL) {
			perror("export: malloc");
			exit(1);
		}
		sprintf(fname, "%s%d.xml", optp->name, i);
	}
	if (strcmp(fname, "-") == 0)
		fp = stdout;
	else if ((fp = fopen(fname, "w")) == NULL) {
		fprintf(stderr, "export: file open: ");
		perror(fname);
		exit(1);
	}
	fprintf(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	fprintf(fp, "<!-- Data produced by BEANIE export utility -->\n");
	fprintf(fp, "<!-- $Revision: 1.7 $ -->\n<!-- -->\n");
	fprintf(fp, "<!-- Company: %s -->\n", comp->cname);
#if 0
	fprintf(fp, "<?xml-stylesheet type=\"text/xsl\" href=\"%s\"?>\n",
							optp->xslname);
#endif
	fprintf(fp, "<!DOCTYPE %s SYSTEM \"%s\">\n",	optp->name,
							optp->dtdname);
	optp->func(fp, i, 0);
	fclose(fp);
	exit(0);
}

/*
 *
 */
void
usage()
{
	struct cmdopts *optp;

	fprintf(stderr, "Usage: export [-c <comp>][-v][-o <file>] --type <no>\n");
	fprintf(stderr, "Where type is one of the following:\n");
	for (optp = options; optp->name != NULL; optp++)
		fprintf(stderr, "\t%s\n", optp->name);
	exit(2);
}
