#ident "$Id: salesorder.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: salesorder.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA
 * 02139, USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: salesorder.c,v $
 * Revision 1.6  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/01/15 18:10:40  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2006/07/31 10:17:47  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.3  2005/09/21 15:16:44  dtynan
 * The 'cdate' field is actually the completion date of the task, not the
 * creation date.
 *
 * Revision 1.2  2005/09/21 14:53:29  dtynan
 * The order state is now on a per-item basis.
 *
 * Revision 1.1  2005/08/12 18:49:25  dtynan
 * First pass at a utility to pull data from the database and produce an
 * XML representation for exporting to other applications.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#include "export.h"

/*
 *
 */
void
dosorder(FILE *fp, int id, int in)
{
	char tagbuf[64];
	struct db_salesorder *sp;
	struct db_sorder_item *sip;
	dbow_conn *dbp = getdbase();

	if ((sp = db_findsalesorderbyid(dbp, id)) == NULL)
		berror("export", "cannot find sales order");
	sprintf(tagbuf, "number=\"%d\"", sp->id);
	block(fp, "salesorder", BLOCK_OPEN|BLOCK_MULTILINE, tagbuf, NULL, in++);
	docustomer(fp, sp->customer_id, in);
	blockdate(fp, "orderdate", NULL, sp->odate, in);
	if (sp->yref != NULL)
		blockstr(fp, "yourref", NULL, sp->yref, in);
	if (sp->descr != NULL)
		blockstr(fp, "descr", NULL, sp->descr, in);
	sip = findsalesorderitems(dbp, sp->id);
	while (sip != NULL) {
		block(fp, "item", BLOCK_OPEN|BLOCK_MULTILINE, NULL, NULL, in++);
		if (sip->product_id != 0)
			doproduct(fp, sip->product_id, in);
		blockdate(fp, "rundate", NULL, sip->rdate, in);
		if (sip->cdate != 0)
			blockdate(fp, "enddate", NULL, sip->cdate, in);
		switch (sip->state) {
		case SALESORDER_NEW:
			blockstr(fp, "neworder", NULL, NULL, in);
			break;

		case SALESORDER_READY:
			blockstr(fp, "ready", NULL, NULL, in);
			break;

		case SALESORDER_DONE:
			blockstr(fp, "done", NULL, NULL, in);
			break;

		case SALESORDER_BLOCKED:
			blockstr(fp, "blocked", NULL, NULL, in);
			break;
		}
		dofrequency(fp, sip->freq, in);
		block(fp, "descr", BLOCK_OPEN|BLOCK_MULTILINE|BLOCK_CLOSE,
							NULL, sip->descr, in);
		blockint(fp, "vatcode", NULL, sip->vcode, in);
		blockint(fp, "quantity", NULL, sip->quant, in);
		blockfloat(fp, "unitcost", NULL, sip->unitc, in);
		if (sip->disc != 0.0)
			blockfloat(fp, "discount", NULL, sip->disc, in);
		block(fp, "item", BLOCK_CLOSE, NULL, NULL, --in);
		sip = db_findsorder_itemnext(dbp, sip);
	}
	block(fp, "salesorder", BLOCK_CLOSE, NULL, NULL, --in);
	db_salesorderfree(sp);
}

/*
 *
 */
void
dofrequency(FILE *fp, int id, int in)
{
	switch (id) {
	case FREQ_ONCE:
		blockstr(fp, "frequency", "once", NULL, in);
		break;

	case FREQ_HOURLY:
		blockstr(fp, "frequency", "hour=\"1\"", NULL, in);
		break;

	case FREQ_DAILY:
		blockstr(fp, "frequency", "day=\"1\"", NULL, in);
		break;

	case FREQ_WEEKLY:
		blockstr(fp, "frequency", "day=\"7\"", NULL, in);
		break;

	case FREQ_WEEKLY2:
		blockstr(fp, "frequency", "day=\"14\"", NULL, in);
		break;

	case FREQ_WEEKLY4:
		blockstr(fp, "frequency", "day=\"28\"", NULL, in);
		break;

	case FREQ_MONTHLY:
		blockstr(fp, "frequency", "month=\"1\"", NULL, in);
		break;

	case FREQ_MONTHLY2:
		blockstr(fp, "frequency", "month=\"2\"", NULL, in);
		break;

	case FREQ_QUARTERLY:
		blockstr(fp, "frequency", "month=\"3\"", NULL, in);
		break;

	case FREQ_BIANNUAL:
		blockstr(fp, "frequency", "month=\"6\"", NULL, in);
		break;

	case FREQ_ANNUALLY:
		blockstr(fp, "frequency", "year=\"1\"", NULL, in);
		break;

	case FREQ_MANUALLY:
		blockstr(fp, "frequency", "manual", NULL, in);
		break;
	}
}
