#ident "$Id: xml.c,v 1.2 2008/01/15 18:10:40 dtynan Exp $"

/*
 * $Id: xml.c,v 1.2 2008/01/15 18:10:40 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA
 * 02139, USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: xml.c,v $
 * Revision 1.2  2008/01/15 18:10:40  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.1  2005/08/12 18:49:25  dtynan
 * First pass at a utility to pull data from the database and produce an
 * XML representation for exporting to other applications.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#include "export.h"

#define BLOCK_OPEN	01
#define BLOCK_CLOSE	02
#define BLOCK_MULTILINE	04

/*
 *
 */
void
blockstr(FILE *fp, char *label, char *opts, char *val, int indent)
{
	block(fp, label, BLOCK_OPEN|BLOCK_CLOSE, opts, val, indent);
}

/*
 *
 */
void
blockint(FILE *fp, char *label, char *opts, int val, int indent)
{
	char vtmp[64];

	sprintf(vtmp, "%d", val);
	blockstr(fp, label, opts, vtmp, indent);
}

/*
 *
 */
void
blockfloat(FILE *fp, char *label, char *opts, double val, int indent)
{
	char vtmp[64];

	sprintf(vtmp, "%.2f", val);
	blockstr(fp, label, opts, vtmp, indent);
}

/*
 *
 */
void
blockdate(FILE *fp, char *label, char *opts, int val, int indent)
{
	char vtmp[64];
	struct tm *tmp;

	if ((tmp = localtime((time_t *)&val)) == NULL)
		return;
	sprintf(vtmp, "%04d-%02d-%02d", tmp->tm_year + 1900,
					tmp->tm_mon + 1, tmp->tm_mday);
	blockstr(fp, label, opts, vtmp, indent);
}

/*
 *
 */
void
block(FILE *fp, char *label, int flag, char *opts, char *data, int indent)
{
	int i, needf;

	blockindent(fp, indent);
	if (flag & BLOCK_OPEN) {
		fprintf(fp, "<%s", label);
		if (opts != NULL)
			fprintf(fp, " %s", opts);
		if ((flag & (BLOCK_MULTILINE|BLOCK_CLOSE)) == BLOCK_CLOSE &&
							data == NULL) {
			fprintf(fp, "/>\n");
			return;
		}
		fputc('>', fp);
	}
	needf = 0;
	if (flag & BLOCK_MULTILINE) {
		fputc('\n', fp);
		needf = 1;
	}
	if (data != NULL) {
		while ((i = *data++) != '\0') {
			if (needf) {
				blockindent(fp, indent + 1);
				needf = 0;
			}
			switch (i) {
			case '<':
				fprintf(fp, "&lt;");
				break;

			case '>':
				fprintf(fp, "&gt;");
				break;

			case '&':
				fprintf(fp, "&amp;");
				break;

			case '"':
				fprintf(fp, "&quot;");
				break;

			case '\'':
				fprintf(fp, "&apos;");
				break;

			case '\n':
				fputc('\n', fp);
				if (i == '\n' && (flag & BLOCK_MULTILINE))
					needf = 1;
				break;

			default:
				fputc(i, fp);
				break;
			}
		}
		if (needf == 0 && (flag & BLOCK_MULTILINE)) {
			fputc('\n', fp);
			needf = 1;
		}
	}
	if (flag & BLOCK_CLOSE) {
		if (needf)
			blockindent(fp, indent);
		fprintf(fp, "</%s>\n", label);
		needf = 1;
	}
	if (!needf)
		fputc('\n', fp);
}

/*
 *
 */
void
blockindent(FILE *fp, int indent)
{
	int i;

	for (i = indent/2; i > 0; i--)
		fputc('\t', fp);
	if (indent & 01)
		fprintf(fp, "    ");
}
