#ident "Kalopa: $Id: acctview.c,v 1.3 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: acctview.c,v 1.3 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: acctview.c,v $
 * Revision 1.3  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.2  2006/07/31 10:17:47  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.1  2006/07/31 09:00:00  dtynan
 * Added missing files.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

/*
 *
 */
enum	{
	COLUMN_DATE,
	COLUMN_JID,
	COLUMN_FOLIO,
	COLUMN_DESCR,
	COLUMN_DEBIT,
	COLUMN_CREDIT,
	COLUMN_TOTAL,
	NCOLUMNS
};

struct	db_journal	*findjournalentries(dbow_conn *);
struct	db_journal_item	*findjournalitems(dbow_conn *, int);

/*
 *
 */
void
acctview(int acctno, char *title)
{
	int drcr, year;
	char mtitle[64];
	struct db_journal *jp;
	struct db_journal_item *jip;
	dbow_conn *db;
	double tot;
	struct tm *tmp;
	GtkWidget *dialog;
	GtkWidget *scroll;
	GtkWidget *tree;
	GtkTreeIter iter;
	GtkListStore *store;

	if ((db = get_company_dbase()) == NULL)
		return;

	year = get_current_year() - 1900;
	store = gtk_list_store_new(NCOLUMNS,
					G_TYPE_STRING,
					G_TYPE_UINT,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING);

	tot = 0.0;
	for (jp = findjournalentries(db); jp != NULL;
				jp = db_findjournalnext(db, jp)) {
		if ((tmp = localtime((time_t *)&jp->jdate)) == NULL)
			continue;
		if (tmp->tm_year != year)
			continue;
		for (jip = findjournalitems(db, jp->id);
				jip != NULL;
				jip = db_findjournal_itemnext(db, jip)) {
			if (jip->nomacct_id != acctno)
				continue;
			gtk_list_store_append(store, &iter);
			if (jip->drcrf) {
				drcr = COLUMN_DEBIT;
				tot -= jip->amount;
			} else {
				drcr = COLUMN_CREDIT;
				tot += jip->amount;
			}
			gtk_list_store_set(store, &iter,
				COLUMN_DATE, strdup(prabbrev(&jp->jdate)),
				COLUMN_JID, jp->id,
				COLUMN_FOLIO, strdup(jp->folio == NULL ? "" : jp->folio),
				COLUMN_DESCR, strdup(jp->descr),
				drcr, strdup(bal(jip->amount, NULL)),
				COLUMN_TOTAL, strdup(bal(tot, NULL)),
				-1);
		}
	}
	if (title == NULL) {
		sprintf(mtitle, "List of activities on A/C #%d", acctno);
		title = mtitle;
	}
	dialog = gtk_dialog_new_with_buttons(title, NULL, 0,
				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
				NULL);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 600, 600);
	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
	g_signal_connect_swapped(G_OBJECT(dialog), "response",
				G_CALLBACK(gtk_widget_destroy), dialog);
	gtk_widget_realize(dialog);

	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
					GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
					GTK_POLICY_NEVER,
					GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scroll,
						TRUE, TRUE, 0);

	tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
	gtk_tree_view_set_search_column(GTK_TREE_VIEW(tree), COLUMN_DESCR);
	gtk_container_add(GTK_CONTAINER(scroll), tree);
	g_object_unref(store);

	columnheader(tree, "Date", COLUMN_DATE, 100, 100, CR_RALIGN);
	columnheader(tree, "J.ID", COLUMN_JID, 50, 50, CR_RALIGN);
	columnheader(tree, "Folio", COLUMN_FOLIO, 80, 80, CR_RALIGN);
	columnheader(tree, "Description", COLUMN_DESCR, 200, 0, 0);
	columnheader(tree, "Debit", COLUMN_DEBIT, 100, 100, CR_RALIGN);
	columnheader(tree, "Credit", COLUMN_CREDIT, 100, 100, CR_RALIGN);
	columnheader(tree, "Balance", COLUMN_TOTAL, 110, 110, CR_RALIGN);

	gtk_widget_show(tree);
	gtk_widget_show(scroll);
	gtk_widget_show(dialog);
}
