#ident "Kalopa: $Id: balsheet.c,v 1.7 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: balsheet.c,v 1.7 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: balsheet.c,v $
 * Revision 1.7  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.6  2005/09/21 18:10:27  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.5  2005/04/11 17:52:30  dtynan
 * Various fixes and improvements.
 *
 * Revision 1.4  2005/02/17 16:55:04  dtynan
 * Make sure we load the nominal account list from the main database.
 *
 * Revision 1.3  2004/12/09 20:17:53  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.2  2004/12/09 19:17:48  dtynan
 * Extensive functionality improvements.
 *
 * Revision 1.1  2004/12/08 14:28:14  dtynan
 * First pass at a useable GUI.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

static	struct	nabal	*nlist;

/*
 *
 */
void
balsheet(GtkWidget *widget)
{
	int year;
	char mtitle[256], ftitle[256];
	struct db_company *cp = get_current_company();
	dbow_conn *dbp = get_main_dbase();
	dbow_conn *dbcp = get_company_dbase();
	struct nabal *np;
	GtkTextBuffer *bp;
	struct tm *tmp;
	double tot1, tot2;

	year = get_current_year() - 1900;
	sprintf(mtitle, "Balance Sheet for %s", cp->name);
	strcpy(ftitle, mtitle);
	strcat(ftitle, "\non ");
	strcat(ftitle, prdate(tmp->tm_mday, tmp->tm_mon+1, tmp->tm_year+1900));
	strcat(ftitle, "\n\n");

	if ((bp = genreport(mtitle, ftitle)) == NULL)
		return;

	if ((nlist = loadnabal(dbp)) == NULL)
		return;
	loadbalances(dbcp, nlist, year);
	/*
	 * Start with fixed assets...
	 */
	tot1 = tot2 = 0.0;
	reportline(bp, "Fixed Assets", 0.0, BS_BOLD);
	for (np = nlist; np != NULL; np = np->next) {
		if (np->bal == 0.0 || np->atype != ASSET_FIXED)
			continue;
		reportline(bp, np->name, -np->bal, BS_COL1);
		tot2 -= np->bal;
	}
	reportline(bp, NULL, tot2, BS_COL2|BS_LINE|BS_BLANK);
	/*
	 * Now current assets.
	 */
	tot1 += tot2;
	tot2 = 0.0;
	reportline(bp, "Current Assets", 0.0, BS_BOLD);
	for (np = nlist; np != NULL; np = np->next) {
		if (np->bal == 0.0 || np->atype != ASSET_CURRENT)
			continue;
		reportline(bp, np->name, -np->bal, BS_COL1);
		tot2 -= np->bal;
	}
	reportline(bp, NULL, tot2, BS_COL2|BS_LINE|BS_BLANK);
	/*
	 * Total assets.
	 */
	tot1 += tot2;
	tot2 = 0.0;
	reportline(bp, "Assets", tot1, BS_COL2|BS_LINE|BS_BOLD|BS_BLANK);
	/*
	 * Liabilities.
	 */
	reportline(bp, "Liabilities", 0.0, BS_BOLD);
	for (np = nlist; np != NULL; np = np->next) {
		if (np->bal == 0.0 || np->atype != LIABILITY)
			continue;
		reportline(bp, np->name, np->bal, BS_COL1);
		tot2 += np->bal;
	}
	reportline(bp, NULL, tot2, BS_COL2|BS_LINE|BS_BLANK);
	/*
	 * Net assets.
	 */
	tot1 -= tot2;
	reportline(bp, "Net assets less liabilities", tot1, BS_COL2|BS_DLINE|BS_BOLD);
	reportline(bp, NULL, 0.0, 0);
	/*
	 * Shareholders equity.
	 */
	tot2 = 0.0;
	reportline(bp, "Capital and Reserves", 0.0, BS_BOLD);
	for (np = nlist; np != NULL; np = np->next) {
		if (np->bal == 0.0 || np->atype != OEQUITY)
			continue;
		reportline(bp, np->name, np->bal, BS_COL1);
		tot2 += np->bal;
	}
	reportline(bp, "Profit and loss account", tot1 - tot2, BS_COL1);
	reportline(bp, "Shareholders funds", tot1, BS_COL2|BS_LINE|BS_BOLD);
	reportline(bp, NULL, 0.0, BS_DLINE);
}

/*
 *
 */
void
pandloss(GtkWidget *widget)
{
	int year;
	char mtitle[256], ftitle[256];
	struct db_company *cp = get_current_company();
	dbow_conn *dbp = get_main_dbase();
	dbow_conn *dbcp = get_company_dbase();
	struct nabal *np;
	GtkTextBuffer *bp;
	struct tm *tmp;
	double tot1, tot2;

	/*
	 *
	 */
	year = get_current_year() - 1900;
	sprintf(mtitle, "Profit and Loss for %s", cp->name);
	strcpy(ftitle, mtitle);
	strcat(ftitle, "\non ");
	strcat(ftitle, prdate(tmp->tm_mday, tmp->tm_mon+1, tmp->tm_year+1900));
	strcat(ftitle, "\n\n");

	if ((bp = genreport(mtitle, ftitle)) == NULL)
		return;

	if ((nlist = loadnabal(dbp)) == NULL)
		return;
	loadbalances(dbcp, nlist, year);
	/*
	 * Start with turnover...
	 */
	tot1 = tot2 = 0.0;
	reportline(bp, "Turnover", 0.0, BS_BOLD);
	for (np = nlist; np != NULL; np = np->next) {
		if (np->bal == 0.0 || np->atype != INCOME)
			continue;
		reportline(bp, np->name, np->bal, BS_COL1);
		tot2 += np->bal;
	}
	reportline(bp, NULL, tot2, BS_COL2|BS_LINE|BS_BLANK);
	/*
	 * Cost of goods sold...
	 */
	tot1 += tot2;
	tot2 = 0.0;
	reportline(bp, "Cost of Sales", 0.0, BS_BOLD);
	for (np = nlist; np != NULL; np = np->next) {
		if (np->bal == 0.0 || np->atype != COSTOFSALES)
			continue;
		reportline(bp, np->name, -np->bal, BS_COL1);
		tot2 -= np->bal;
	}
	reportline(bp, NULL, tot2, BS_COL2|BS_LINE|BS_BLANK);
	tot1 -= tot2;
	reportline(bp, "Gross Profit", tot1, BS_COL2|BS_DLINE|BS_BLANK|BS_BOLD);
	/*
	 * Expenses...
	 */
	tot2 = 0.0;
	reportline(bp, "Expenses", 0.0, BS_BOLD);
	for (np = nlist; np != NULL; np = np->next) {
		if (np->bal == 0.0 || np->atype != EXPENSE)
			continue;
		reportline(bp, np->name, -np->bal, BS_COL1);
		tot2 -= np->bal;
	}
	reportline(bp, NULL, tot2, BS_COL2|BS_LINE);
	reportline(bp, NULL, 0.0, 0);
	if (tot1 > tot2)
		reportline(bp, "Profit on ordinary activities", tot1 - tot2, BS_COL2|BS_DLINE|BS_BOLD);
	else
		reportline(bp, "Loss on ordinary activities", tot1 - tot2, BS_COL2|BS_DLINE|BS_BOLD);
}
