#ident "Kalopa: $Id: company.c,v 1.9 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: company.c,v 1.9 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: company.c,v $
 * Revision 1.9  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.8  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.7  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.6  2006/07/31 10:17:47  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.5  2005/09/21 18:10:27  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.4  2005/04/11 17:52:30  dtynan
 * Various fixes and improvements.
 *
 * Revision 1.3  2004/12/09 20:17:54  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.2  2004/12/09 19:17:49  dtynan
 * Extensive functionality improvements.
 *
 * Revision 1.1  2004/12/08 14:28:15  dtynan
 * First pass at a useable GUI.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

/*
 *
 */
struct	comps	{
	struct	comps		*next;
	struct	db_company	*data;
};

static	struct	comps	*comp = NULL;

static	int		curr = -1;
static	dbow_conn	*maindb = NULL;
static	dbow_conn	*compdb = NULL;
static	GtkWidget	*combo;

static	struct	db_company	*currcomp = NULL;

static	void	setcomp(struct comps *);

/*
 *
 */
void
select_company(GtkMenuItem *menuitem, gpointer udata)
{
	int i;
	GtkWidget *dialog;
	GtkWidget *frame;
	struct comps *xcp;

	setcomp(NULL);

	dialog = gtk_dialog_new_with_buttons("Select Company", NULL,
				0,
				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
				GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
				NULL);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 250, -1);
	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
	gtk_widget_realize(dialog);

	frame = gtk_frame_new("Select a company from the list");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), frame,
						TRUE, TRUE, 0);

	combo = gtk_combo_box_new_text();
	for (xcp = comp; xcp != NULL; xcp = xcp->next)
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), xcp->data->name);
	if (curr >= 0)
		gtk_combo_box_set_active(GTK_COMBO_BOX(combo), curr);

	gtk_container_add(GTK_CONTAINER(frame), combo);
	gtk_widget_show(combo);

	gtk_widget_show(frame);
	gtk_widget_show(dialog);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		curr = gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
		for (i = 0, xcp = comp; i < curr; i++)
			if (xcp && xcp->next != NULL)
				xcp = xcp->next;
		setcomp(xcp);
	}
	gtk_widget_destroy(dialog);
}

/*
 *
 */
struct db_company *
get_current_company()
{
	if (compdb == NULL || currcomp == NULL)
		select_company(NULL, NULL);
	return(currcomp);
}

/*
 *
 */
static void
setcomp(struct comps *xcp)
{
	/*
	 * If we haven't initialised the linked-list of companies,
	 * we'd better do that now.
	 */
	if (comp == NULL) {
		struct db_company *cp;
		struct comps *nxcp, *tail = NULL;

		for (cp = db_findcompanyfirst(get_main_dbase());
					cp != NULL;
					cp = db_findcompanynext(maindb, NULL)) {
			nxcp = (struct comps *)malloc(sizeof(struct comps));
			if (nxcp == NULL)
				return;
			if (comp == NULL)
				comp = nxcp;
			else
				tail->next = nxcp;
			tail = nxcp;
			nxcp->next = NULL;
			nxcp->data = cp;
		}
		currcomp = NULL;
	}
	/*
	 * Now, set the company if that's what they want...
	 */
	if (xcp != NULL) {
		if (compdb != NULL)
			dbow_close(compdb);
		compdb = beanie_init(xcp->data->tag, 0);
		currcomp = xcp->data;
		update_details();
	}
}
