#ident "Kalopa: $Id: debtors.c,v 1.12 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: debtors.c,v 1.12 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: debtors.c,v $
 * Revision 1.12  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.11  2006/07/31 10:17:47  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.10  2006/03/06 12:31:17  dtynan
 * This is still a work-in-process.  If it compiles and works for you,
 * great!  If not, oh well...
 *
 * Revision 1.9  2005/09/21 18:10:27  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.8  2005/03/09 11:20:12  dtynan
 * Fixed bug in aged debtors report.
 *
 * Revision 1.7  2005/03/03 12:05:24  dtynan
 * Added IBAN details to invoices.
 *
 * Revision 1.6  2005/02/17 16:23:22  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.5  2005/01/13 17:43:31  dtynan
 * Added an aged debtors report.
 *
 * Revision 1.4  2004/12/09 20:32:29  dtynan
 * Cleaned up the organisation display.
 *
 * Revision 1.3  2004/12/09 20:17:54  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.2  2004/12/09 19:17:49  dtynan
 * Extensive functionality improvements.
 *
 * Revision 1.1  2004/12/08 14:28:15  dtynan
 * First pass at a useable GUI.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

/*
 *
 */
enum	{
	COLUMN_CUSTOMER,
	COLUMN_TOTAL,
	COLUMN_CURRENT,
	COLUMN_30,
	COLUMN_60,
	COLUMN_OVER,
	COLUMN_AGE,
	NCOLUMNS
};

/*
 *
 */
static char *
showmoney(double val, int boldf)
{
	char mval[128];

	if (val == 0.0)
		sprintf(mval, "        ");
	else if (boldf)
		sprintf(mval, "<b>%.2f</b>", val);
	else
		sprintf(mval, "%.2f", val);
	return(strdup(mval));
}

/*
 *
 */
void
aged_debtors(GtkWidget *widget)
{
	int i, oldest, told;
	time_t now;
	dbow_conn *dbp, *dbcp;
	struct db_invoice *invp;
	struct db_customer *cusp;
	struct db_contact *cp;
	GtkWidget *dialog;
	GtkWidget *scroll;
	GtkWidget *tree;
	GtkTreeIter iter;
	GtkListStore *store;
	double total, current, pd30, pd60, over;
	double ttotal, tcurrent, tpd30, tpd60, tover;

	dbp = get_main_dbase();
	if ((dbcp = get_company_dbase()) == NULL)
		return;

	time(&now);
	store = gtk_list_store_new(NCOLUMNS,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_UINT);
	ttotal = tcurrent = tpd30 = tpd60 = tover = 0.0;
	told = 0;
	for (cusp = db_findcustomerfirst(dbcp); cusp != NULL;
				cusp = db_findcustomernext(dbcp, cusp)) {
		if ((cp = contact_findbyid(dbp, cusp->contact_id)) == NULL)
			continue;
		total = current = pd30 = pd60 = over = 0.0;
		oldest = -1;
		for (invp=db_findinvoicebycustomer_id(dbcp, cusp->id);
					invp != NULL;
					invp = db_findinvoicenext(dbcp, invp)) {
			if (invp->state < INVOICE_STATE_SENT)
				continue;
			total += invp->htotal;
			if ((i = (now - invp->idate) / (24*60*60)) > oldest)
				oldest = i;
			switch (invp->state) {
			case INVOICE_STATE_SENT:
				current += invp->htotal;
				break;

			case INVOICE_STATE_LATE1:
				pd30 += invp->htotal;
				break;

			case INVOICE_STATE_LATE2:
				pd60 += invp->htotal;
				break;

			case INVOICE_STATE_LATE3:
			case INVOICE_STATE_LATE4:
			case INVOICE_STATE_LATE5:
				over += invp->htotal;
				break;
			}
		}
		if (oldest < 0)
			continue;
		ttotal += total;
		tcurrent += current;
		tpd30 += pd30;
		tpd60 += pd60;
		tover += over;
		if (told < oldest)
			told = oldest;
		gtk_list_store_append(store, &iter);
		gtk_list_store_set(store, &iter,
				COLUMN_CUSTOMER, orgname(cp),
				COLUMN_TOTAL, showmoney(total, 0),
				COLUMN_CURRENT, showmoney(current, 0),
				COLUMN_30, showmoney(pd30, 0),
				COLUMN_60, showmoney(pd60, 0),
				COLUMN_OVER, showmoney(over, 0),
				COLUMN_AGE, oldest,
				-1);
	}

	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter,
			COLUMN_CUSTOMER, strdup("<b>Total</b>"),
			COLUMN_TOTAL, showmoney(ttotal, 1),
			COLUMN_CURRENT, showmoney(tcurrent, 1),
			COLUMN_30, showmoney(tpd30, 1),
			COLUMN_60, showmoney(tpd60, 1),
			COLUMN_OVER, showmoney(tover, 1),
			COLUMN_AGE, told,
			-1);

	dialog = gtk_dialog_new_with_buttons("Aged Debtors Report", NULL,
				0,
				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
				NULL);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 500, 200);
	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
	g_signal_connect_swapped(G_OBJECT(dialog), "response", 
				G_CALLBACK (gtk_widget_destroy), dialog);
	gtk_widget_realize(dialog);

	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
					GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
					GTK_POLICY_NEVER,
					GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scroll,
						TRUE, TRUE, 0);

	tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
	gtk_tree_view_set_search_column(GTK_TREE_VIEW(tree),
				COLUMN_CUSTOMER);
	gtk_container_add(GTK_CONTAINER(scroll), tree);

	columnheader(tree, "Customer Name",COLUMN_CUSTOMER,0,0,CR_MARKUP);
	columnheader(tree, "Total",COLUMN_TOTAL,100,0,CR_MARKUP|CR_RALIGN);
	columnheader(tree, "Current",COLUMN_CURRENT,80,0,CR_MARKUP|CR_RALIGN);
	columnheader(tree, "Over 30 Days",COLUMN_30,80,0,CR_MARKUP|CR_RALIGN);
	columnheader(tree, "Over 45 Days",COLUMN_60,80,0,CR_MARKUP|CR_RALIGN);
	columnheader(tree, "Over 60 Days",COLUMN_OVER,80,0,CR_MARKUP|CR_RALIGN);
	columnheader(tree, "Age",COLUMN_AGE,50,0,CR_MARKUP|CR_RALIGN);

	gtk_widget_show(tree);
	gtk_widget_show(scroll);
	gtk_widget_show(dialog);
}
