#ident "Kalopa: $Id: invoice.c,v 1.6 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: invoice.c,v 1.6 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: invoice.c,v $
 * Revision 1.6  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.5  2006/07/31 10:17:47  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.4  2006/03/06 12:31:17  dtynan
 * This is still a work-in-process.  If it compiles and works for you,
 * great!  If not, oh well...
 *
 * Revision 1.3  2005/09/21 18:10:27  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.2  2004/12/09 20:17:54  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.1  2004/12/09 19:17:49  dtynan
 * Extensive functionality improvements.
 *
 * Revision 1.1  2004/12/08 14:28:15  dtynan
 * First pass at a useable GUI.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

/*
 *
 */
enum	{
	COLUMN_ITEM,
	COLUMN_NOMACCT,
	COLUMN_DESCR,
	COLUMN_VCODE,
	COLUMN_QUANT,
	COLUMN_UNITC,
	COLUMN_DISC,
	NCOLUMNS
};

/*
 *
 */
void
show_invoice(GtkWidget *widget)
{
	int invid;
	dbow_conn *db;
	struct db_invoice *invp;
	struct db_invoice_item *iip;
	char vcode[4], temp[64];
	GtkWidget *dialog;
	GtkWidget *scroll;
	GtkWidget *tree;
	GtkTreeIter iter;
	GtkListStore *store;

	if ((db = get_company_dbase()) == NULL)
		return;

	invid = 453; /* XXX */
	store = gtk_list_store_new(NCOLUMNS,
					G_TYPE_UINT,
					G_TYPE_UINT,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_UINT,
					G_TYPE_STRING,
					G_TYPE_STRING);
	if ((invp = db_findinvoicebyid(db, invid)) == NULL) {
		warning("Could not find specified invoice.");
		return;
	}
	for (iip = db_findinvoice_itembyinvoice_id(db, invp->id);
				iip != NULL;
				iip = db_findinvoice_itemnext(db, iip)) {
		gtk_list_store_append(store, &iter);
		if (iip->vcode == 0)
			sprintf(vcode, "Z");
		else
			sprintf(vcode, "%c", iip->vcode + 'A' - 1);
		sprintf(temp, "%.2f%%", iip->disc);
		gtk_list_store_set(store, &iter,
				COLUMN_ITEM, iip->item_id,
				COLUMN_NOMACCT, iip->nomacct_id,
				COLUMN_DESCR, strdup(iip->descr),
				COLUMN_VCODE, strdup(vcode),
				COLUMN_QUANT, iip->quant,
				COLUMN_UNITC, strdup(bal(iip->unitc, NULL)),
				COLUMN_DISC, strdup(temp),
				-1);
	}

	sprintf(temp, "Invoice #%d", invid);
	dialog = gtk_dialog_new_with_buttons(temp, NULL, 0,
				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
				NULL);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 550, 300);
	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
	g_signal_connect_swapped(G_OBJECT(dialog), "response", 
				G_CALLBACK (gtk_widget_destroy), dialog);
	gtk_widget_realize(dialog);

	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
					GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
					GTK_POLICY_NEVER,
					GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scroll,
						TRUE, TRUE, 0);

	tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
	gtk_tree_view_set_search_column(GTK_TREE_VIEW(tree), COLUMN_DESCR);
	gtk_container_add(GTK_CONTAINER(scroll), tree);

	columnheader(tree, "#",COLUMN_ITEM,20,20,CR_RALIGN);
	columnheader(tree, "NL",COLUMN_NOMACCT,50,50,CR_RALIGN);
	columnheader(tree, "Description",COLUMN_DESCR,200,0,CR_MARKUP);
	columnheader(tree, "VAT",COLUMN_VCODE,20,20,0);
	columnheader(tree, "Quant.",COLUMN_QUANT,50,50,CR_RALIGN);
	columnheader(tree, "Cost",COLUMN_UNITC,100,100,CR_RALIGN);
	columnheader(tree, "Discount",COLUMN_DISC,80,80,CR_RALIGN);

	gtk_widget_show(tree);
	gtk_widget_show(scroll);
	gtk_widget_show(dialog);
}
