#ident "Kalopa: $Id: journal.c,v 1.6 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: journal.c,v 1.6 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: journal.c,v $
 * Revision 1.6  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.5  2005/09/21 18:10:27  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.4  2005/04/11 17:52:30  dtynan
 * Various fixes and improvements.
 *
 * Revision 1.3  2005/02/17 16:55:04  dtynan
 * Make sure we load the nominal account list from the main database.
 *
 * Revision 1.2  2004/12/09 20:17:54  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.1  2004/12/09 19:17:49  dtynan
 * Extensive functionality improvements.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

static	struct	nabal	*nlist;

/*
 *
 */
void
journalbals(GtkWidget *widget)
{
	int year;
	char mtitle[256], ftitle[256];
	struct db_company *cp = get_current_company();
	dbow_conn *dbp = get_main_dbase();
	dbow_conn *dbcp = get_company_dbase();
	double val;
	struct nabal *np;
	GtkTextBuffer *bp;
	struct tm *tmp;

	year = get_current_year() - 1900;
	sprintf(mtitle, "Journal Balances for %s", cp->name);
	strcpy(ftitle, mtitle);
	strcat(ftitle, "\non ");
	strcat(ftitle, prdate(tmp->tm_mday, tmp->tm_mon+1, tmp->tm_year+1900));
	strcat(ftitle, "\n\n");

	if ((bp = genreport(mtitle, ftitle)) == NULL)
		return;

	if ((nlist = loadnabal(dbp)) == NULL)
		return;
	loadbalances(dbcp, nlist, year);
	/*
	 * Start with fixed assets...
	 */
	for (np = nlist; np != NULL; np = np->next) {
		if ((val = np->bal) == 0.0)
			continue;
		if (np->atype == ASSET_FIXED || np->atype == ASSET_CURRENT)
			val = -val;
		reportline(bp, np->name, val, BS_COL2);
	}
}

/*
 *
 */
void
journalstmnt(GtkWidget *widget)
{
	char mtitle[64];
	struct nabal *np;

	if ((np = select_nomacct(NULL, NULL)) == NULL)
		return;

	sprintf(mtitle, "Nominal Account Statement for %d (%s)",
						np->nomacct_id, np->name);
	acctview(np->nomacct_id, mtitle);
}

/*
 *
 */
struct nabal *
select_nomacct(GtkMenuItem *menuitem, gpointer udata)
{
	int i, curr;
	char tmplab[256];
	struct nabal *np, *nlist;
	GtkWidget *dialog;
	GtkWidget *frame;
	GtkWidget *combo;

	if ((nlist = loadnabal(get_main_dbase())) == NULL)
		return(NULL);
	dialog = gtk_dialog_new_with_buttons("Select Nominal Account", NULL,
				0,
				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
				GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
				NULL);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 250, -1);
	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
	gtk_widget_realize(dialog);

	frame = gtk_frame_new("Select a nominal account from the list");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), frame,
						TRUE, TRUE, 0);

	combo = gtk_combo_box_new_text();
	for (np = nlist; np != NULL; np = np->next) {
		sprintf(tmplab, "%d - %s", np->nomacct_id, np->name);
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), tmplab);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);

	gtk_container_add(GTK_CONTAINER(frame), combo);
	gtk_widget_show(combo);
	gtk_widget_show(frame);
	gtk_widget_show(dialog);

	np = NULL;
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		curr = gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
		for (i = 0, np = nlist; i < curr; i++)
			if (np != NULL && np->next != NULL)
				np = np->next;
	}
	gtk_widget_destroy(dialog);
	return(np);
}
