#ident "Kalopa: $Id: main.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: main.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: main.c,v $
 * Revision 1.6  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2005/09/21 18:10:27  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.3  2004/12/09 20:17:55  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.2  2004/12/09 19:17:50  dtynan
 * Extensive functionality improvements.
 *
 * Revision 1.1  2004/12/08 14:28:15  dtynan
 * First pass at a useable GUI.
 */
#include <stdio.h>
#include <gnome.h>

#include "beanie_gui.h"

void	usage();

/*
 *
 */
static gint
save_session(GnomeClient *client, gint phase, GnomeSaveStyle save_style,
		gint is_shutdown, GnomeInteractStyle interact_style,
		gint is_fast, gpointer client_data)
{
	printf("Save Session!\n");
	return(TRUE);
}

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	GtkWidget *app;
	GnomeClient *client;

#ifdef ENABLE_NLS
	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
	textdomain(GETTEXT_PACKAGE);
#endif
	gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE,
				argc, argv,
				GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
				NULL);
	/*
	 * Process our own command-line options...
	 */
	cname = NULL;
	while ((i = getopt(argc, argv, "c:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		default:
			usage();
			break;
		}
	}
	if (loadconfig(cname) < 0)
		error("cannot load configuration file");
	client = gnome_master_client();
	g_signal_connect(G_OBJECT(client), "save_yourself",
			G_CALLBACK(save_session), argv[0]);
	g_signal_connect(G_OBJECT(client), "die",
			G_CALLBACK(session_die), NULL);

	if (get_main_dbase() == NULL)
		error("Unable to connect to database.");

	app = create_beanie();
	gtk_widget_show(app);
	gtk_main();
	exit(0);
}

/*
 *
 */
gint
session_die(GnomeClient *client, gpointer client_data)
{
	printf("session_die()\n");
	gtk_main_quit();
	return(TRUE);
}

/*
 *
 */
void
warning(char *msg)
{
	GtkWidget *dialog;

	dialog = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_WARNING,
							"OK", NULL);
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
	gtk_object_set(GTK_OBJECT(dialog), "type", GTK_WINDOW_POPUP, NULL);
	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
}

/*
 *
 */
void
error(char *msg)
{
	GtkWidget *dialog;

	dialog = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_ERROR,
							"Exit", NULL);
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
	gtk_object_set(GTK_OBJECT(dialog), "type", GTK_WINDOW_POPUP, NULL);
	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
	exit(1);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: beanie [-c #]\n");
	exit(2);
}
