#ident "Kalopa: $Id: mainapp.c,v 1.9 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: mainapp.c,v 1.9 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: mainapp.c,v $
 * Revision 1.9  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.8  2005/09/21 18:10:27  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.7  2005/04/01 17:04:59  dtynan
 * Added new functions to handle contacts, customers and countries.
 *
 * Revision 1.6  2005/03/03 12:05:24  dtynan
 * Added IBAN details to invoices.
 *
 * Revision 1.5  2005/02/17 16:55:04  dtynan
 * Make sure we load the nominal account list from the main database.
 *
 * Revision 1.4  2005/01/13 17:43:32  dtynan
 * Added an aged debtors report.
 *
 * Revision 1.3  2004/12/09 20:17:55  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.2  2004/12/09 19:17:50  dtynan
 * Extensive functionality improvements.
 *
 * Revision 1.1  2004/12/08 14:28:15  dtynan
 * First pass at a useable GUI.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

static GnomeUIInfo file_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_NEW_ITEM(N_("Select _Company"), NULL, select_company, NULL),
	GNOMEUIINFO_MENU_NEW_ITEM(N_("Select _Financial Year"), NULL, select_year, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_OPEN_ITEM(open_activate, NULL),
	GNOMEUIINFO_MENU_SAVE_ITEM(save_activate, NULL),
	GNOMEUIINFO_MENU_SAVE_AS_ITEM(save_as_activate, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM(exit_activate, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_CUT_ITEM(cut_activate, NULL),
	GNOMEUIINFO_MENU_COPY_ITEM(copy_activate, NULL),
	GNOMEUIINFO_MENU_PASTE_ITEM(paste_activate, NULL),
	GNOMEUIINFO_MENU_CLEAR_ITEM(clear_activate, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PROPERTIES_ITEM(payment_activate, NULL),
	GNOMEUIINFO_MENU_PROPERTIES_ITEM(props_activate, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(prefs_activate, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo cash_menu_uiinfo[] = {
	GNOMEUIINFO_ITEM_NONE("Bank Statement", NULL, bankstmnt),
	GNOMEUIINFO_END
};

static GnomeUIInfo journal_menu_uiinfo[] = {
	GNOMEUIINFO_ITEM_NONE("Journal Balances", NULL, journalbals),
	GNOMEUIINFO_ITEM_NONE("Nominal Account Statement", NULL, journalstmnt),
	GNOMEUIINFO_END
};

static GnomeUIInfo purch_menu_uiinfo[] = {
	GNOMEUIINFO_ITEM_NONE("Enter Purchases Receipt", NULL, sales_receipt),
	GNOMEUIINFO_ITEM_NONE("Display Invoice", NULL, show_invoice),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_NONE("Edit Vendor Details", NULL, show_invoice),
	GNOMEUIINFO_END
};

static GnomeUIInfo sales_menu_uiinfo[] = {
	GNOMEUIINFO_ITEM_NONE("Enter Sales Receipt", NULL, sales_receipt),
	GNOMEUIINFO_ITEM_NONE("Display Invoice", NULL, show_invoice),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_NONE("Edit Customer Details", NULL, edit_customer),
	GNOMEUIINFO_ITEM_NONE("Customer Statement", NULL, customer_statement),
	GNOMEUIINFO_END
};

static GnomeUIInfo report_menu_uiinfo[] = {
	GNOMEUIINFO_ITEM_NONE("Balance Sheet", NULL, balsheet),
	GNOMEUIINFO_ITEM_NONE("Profit & Loss", NULL, pandloss),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_NONE("Aged Creditors Report", NULL, aged_debtors),
	GNOMEUIINFO_ITEM_NONE("Aged Debtors Report", NULL, aged_debtors),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] = {
	GNOMEUIINFO_MENU_ABOUT_ITEM(about_activate, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] = {
	GNOMEUIINFO_MENU_FILE_TREE(file_menu_uiinfo),
	GNOMEUIINFO_MENU_EDIT_TREE(edit_menu_uiinfo),
	GNOMEUIINFO_SUBTREE("_Cash Book", cash_menu_uiinfo),
	GNOMEUIINFO_SUBTREE("_Journal", journal_menu_uiinfo),
	GNOMEUIINFO_SUBTREE("_Purchases", purch_menu_uiinfo),
	GNOMEUIINFO_SUBTREE("_Sales", sales_menu_uiinfo),
	GNOMEUIINFO_SUBTREE("_Reports", report_menu_uiinfo),
	GNOMEUIINFO_MENU_HELP_TREE(help_menu_uiinfo),
	GNOMEUIINFO_END
};

void	beanie_die(GtkWidget *, gpointer *);

/*
 *
 */
GtkWidget *
create_beanie(void)
{
	GtkWidget *app;
	GtkWidget *bonobodock;
	GtkWidget *toolbar;
	GtkWidget *frame;

	(void )get_company_dbase();

	app = gnome_app_new("beanie", _("Beanie -- The Accounts Processor"));
	gtk_window_set_position(GTK_WINDOW(app), GTK_WIN_POS_CENTER);
	gtk_window_set_default_size(GTK_WINDOW(app), 650, 400);

	gtk_signal_connect(GTK_OBJECT(app), "delete_event",
			GTK_SIGNAL_FUNC(beanie_die), NULL);

	bonobodock = GNOME_APP(app)->dock;
	gtk_widget_show(bonobodock);

	gnome_app_create_menus(GNOME_APP(app), menubar_uiinfo);

	toolbar = gtk_toolbar_new();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),
			gtk_tool_button_new_from_stock(GTK_STOCK_OPEN), -1);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),
			gtk_tool_button_new_from_stock(GTK_STOCK_PRINT), -1);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),
			gtk_tool_button_new_from_stock(GTK_STOCK_OPEN), -1);
	gnome_app_add_toolbar(GNOME_APP(app), GTK_TOOLBAR(toolbar), "toolbar",
				BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,
				BONOBO_DOCK_TOP, 1, 0, 0);
	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);

	gtk_widget_show(toolbar);

	frame = gtk_frame_new("Company Details");
	make_details(frame);
	gtk_widget_show(frame);

	gnome_app_set_contents(GNOME_APP(app), frame);
	g_object_set_data(G_OBJECT(app), "beanie", app);
	return(app);
}

/*
 *
 */
void
beanie_die(GtkWidget *widget, gpointer *data)
{
	gtk_main_quit();
}
