#ident "Kalopa: $Id: payment.c,v 1.3 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: payment.c,v 1.3 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: payment.c,v $
 * Revision 1.3  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.2  2004/12/09 20:17:55  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.1  2004/12/08 14:28:15  dtynan
 * First pass at a useable GUI.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

void button_clicked(GtkWidget *widget, gpointer udata);
void payment_changed(GtkWidget *, gpointer);
void payment_delete_text(GtkWidget *, gpointer);
void payment_insert_text(GtkWidget *, gpointer);

/*
 *
 */
void
payment_activate(GtkMenuItem *menuitem, gpointer udata)
{
	GtkWidget *dialog;
	GtkWidget *dialog_vbox;
	GtkWidget *frame;
	GtkWidget *alignment;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *action;
	GtkWidget *button;
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new();
	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), _("Enter / Allocate Sales Ledger Receipts"));
	gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
	dialog_vbox = GTK_DIALOG(dialog)->vbox;
	gtk_widget_show(dialog_vbox);

	frame = gtk_frame_new(NULL);
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), frame, TRUE, TRUE, 0);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);

	alignment = gtk_alignment_new(0.5, 0.5, 1, 1);
	gtk_widget_show(alignment);
	gtk_container_add(GTK_CONTAINER(frame), alignment);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 12, 0);

	label = gtk_label_new(_("<b>Enter Payment Details</b>"));
	gtk_widget_show(label);
	gtk_frame_set_label_widget(GTK_FRAME(frame), label);
	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);

	table = gtk_table_new(3, 2, FALSE);
	gtk_widget_show(table);
	gtk_container_add(GTK_CONTAINER(alignment), table);
	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);

	label = gtk_label_new(_("Customer"));
	gtk_widget_show(label);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
			(GtkAttachOptions )GTK_FILL,
			(GtkAttachOptions )0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	entry = gtk_entry_new();
	gtk_widget_show(entry);
	gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 0, 1,
			(GtkAttachOptions )(GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions )0, 0, 0);
	gtk_entry_set_has_frame(GTK_ENTRY(entry), FALSE);
	gtk_signal_connect(GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(payment_changed), NULL);
	g_signal_connect((gpointer )entry, "delete_text",
			G_CALLBACK(payment_delete_text), NULL);
	g_signal_connect((gpointer )entry, "insert_text",
			G_CALLBACK(payment_insert_text), NULL);

	label = gtk_label_new(_("Description"));
	gtk_widget_show(label);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
			(GtkAttachOptions )GTK_FILL,
			(GtkAttachOptions )0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	entry = gtk_entry_new();
	gtk_widget_show(entry);
	gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 1, 2,
			(GtkAttachOptions )(GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions )0, 0, 0);
	gtk_entry_set_has_frame(GTK_ENTRY(entry), FALSE);

	label = gtk_label_new(_("Amount"));
	gtk_widget_show(label);
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
			(GtkAttachOptions )GTK_FILL,
			(GtkAttachOptions )0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	entry = gtk_entry_new();
	gtk_widget_show(entry);
	gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 2, 3,
			(GtkAttachOptions )(GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions )0, 0, 0);
	gtk_entry_set_has_frame(GTK_ENTRY(entry), FALSE);

	action = GTK_DIALOG(dialog)->action_area;
	gtk_widget_show(action);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(action), GTK_BUTTONBOX_END);

	button = gtk_button_new_from_stock("gtk-cancel");
	gtk_widget_show(button);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(button_clicked), "Cancel");

	button = gtk_button_new_from_stock("gtk-ok");
	gtk_widget_show(button);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(button_clicked), "OK");

	gtk_widget_show(dialog);
}

/*
 *
 */
void
button_clicked(GtkWidget *widget, gpointer udata)
{
	GtkWidget *dialog;

	printf("Button clicked! U=%s\n", (char *)udata);
	if (udata == 0) {
		dialog = gnome_ok_dialog("You pressed a button!\n");
		gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(widget));
	}
}

/*
 *
 */
void
payment_changed(GtkWidget *widget, gpointer udata)
{
	printf("Changed.\n");
}

/*
 *
 */
void
payment_delete_text(GtkWidget *widget, gpointer udata)
{
	printf("Delete Text.\n");
}

/*
 *
 */
void
payment_insert_text(GtkWidget *widget, gpointer udata)
{
	printf("Insert Text.\n");
}
