#ident "Kalopa: $Id: report.c,v 1.3 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: report.c,v 1.3 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: report.c,v $
 * Revision 1.3  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.2  2004/12/09 20:17:55  dtynan
 * Cleaned up include topology.
 *
 * Revision 1.1  2004/12/09 19:17:50  dtynan
 * Extensive functionality improvements.
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <bonobo.h>
#include <gnome.h>

#include "beanie_gui.h"

#define BLANK_LINE	"\t__________________________\n"
#define BLANK_DLINE	"\t\t____________\n"

/*
 *
 */
GtkTextBuffer *
genreport(char *mtitle, char *ftitle)
{
	GtkWidget *dialog;
	GtkWidget *scroll;
	GtkWidget *view;
	GtkTextBuffer *bp;
	GtkTextIter iter;
	PangoTabArray *tabs;

	dialog = gtk_dialog_new_with_buttons(mtitle, NULL, 0,
				GTK_STOCK_PRINT, GTK_RESPONSE_ACCEPT,
				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
				NULL);
	g_signal_connect_swapped(G_OBJECT(dialog), "response", 
				G_CALLBACK (gtk_widget_destroy), dialog);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 550, 600);
	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
	gtk_widget_realize(dialog);

	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
					GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
					GTK_POLICY_NEVER,
					GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scroll,
						TRUE, TRUE, 0);

	tabs = pango_tab_array_new_with_positions(3, TRUE,
					PANGO_TAB_LEFT, 0,
					PANGO_TAB_LEFT, 300,
					PANGO_TAB_LEFT, 400);

	view = gtk_text_view_new();
	gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
	gtk_text_view_set_tabs(GTK_TEXT_VIEW(view), tabs);
	gtk_container_border_width(GTK_CONTAINER(scroll), 5);
	gtk_container_add(GTK_CONTAINER(scroll), view);

	gtk_widget_show(view);
	gtk_widget_show(scroll);
	gtk_widget_show(dialog);

	bp = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
	gtk_text_buffer_create_tag(bp, "bold", "weight",
						PANGO_WEIGHT_BOLD, NULL);
	gtk_text_buffer_create_tag(bp, "center", "justification",
						GTK_JUSTIFY_CENTER, NULL);
	gtk_text_buffer_create_tag(bp, "monospace", "family",
						"monospace", NULL);
	gtk_text_buffer_create_tag(bp, "red", "foreground",
						"red", NULL);
	gtk_text_buffer_get_iter_at_offset(bp, &iter, 0);
	gtk_text_buffer_insert_with_tags_by_name(bp, &iter,
					ftitle, -1, "bold", "center", NULL);
	return(bp);
}

/*
 *
 */
void
reportline(GtkTextBuffer *bp, char *title, double val, int flag)
{
	int len, minus;
	GtkTextIter iter;
	char data[256], d2[64], *cp;

	gtk_text_buffer_get_iter_at_offset(bp, &iter, -1);
	if (flag & BS_LINE)
		gtk_text_buffer_insert(bp, &iter, BLANK_LINE, -1);
	else if (flag & BS_DLINE)
		gtk_text_buffer_insert(bp, &iter, BLANK_DLINE, -1);
	if (title != NULL) {
		if (flag & BS_BOLD) {
			gtk_text_buffer_insert_with_tags_by_name(bp, &iter,
						title, -1, "bold", NULL);
		} else
			gtk_text_buffer_insert(bp, &iter, title, -1);
	}
	cp = data;
	if (!(flag & BS_COL1))
		*cp++ = '\t';
	if (flag & (BS_COL2|BS_COL1)) {
		*cp++ = '\t';
		if (val < 0.0) {
			sprintf(d2, "(%.2f)", -val);
			len = strlen(d2) - 1;
			minus = 1;
		} else {
			sprintf(d2, "%.2f", val);
			len = strlen(d2);
			minus = 0;
		}
		while (len++ < 10)
			*cp++ = ' ';
		strcpy(cp, d2);
		if (minus) {
			gtk_text_buffer_insert_with_tags_by_name(bp, &iter,
						data, -1, "monospace",
						"red", NULL);
		} else {
			gtk_text_buffer_insert_with_tags_by_name(bp, &iter,
						data, -1, "monospace", NULL);
		}
	}
	gtk_text_buffer_insert(bp, &iter, "\n", -1);
	if (flag & BS_BLANK)
		gtk_text_buffer_insert(bp, &iter, "\n", -1);
}
