#ident "$Id: contact.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: contact.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: contact.c,v $
 * Revision 1.6  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.4  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2006/07/31 10:17:48  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.2  2005/09/26 15:44:36  dtynan
 * Fixed import bug where records didn't get inserted in some cases.
 *
 * Revision 1.1  2005/09/21 14:14:33  dtynan
 * Major addition of data-import functionality.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include <expat.h>

#include "beanie.h"
#include "hacks.h"

#include "import.h"

int	editcontact(struct db_contact *);

/*
 *
 */
int
docontact(struct ctxt *cp, int token)
{
	struct db_contact *conp = (struct db_contact *)cp->tail->udata;
	struct db_contact *conp2;
	dbow_conn *dbp = getdbase();

	switch (token) {
	case TOKEN_CCODE:
		conp->ccode = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_FIRSTNAME:
		conp->fname = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_LASTNAME:
		conp->lname = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_NAME:
		conp->fname = NULL;
		conp->lname = cp->data;
		cp->data = NULL;

	case TOKEN_ADDR:
		if (conp->a1 == NULL)
			conp->a1 = cp->data;
		else if (conp->a2 == NULL)
			conp->a2 = cp->data;
		else
			conp->a3 = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_CITY:
		conp->city = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_STATE:
		conp->state_county = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_ZIP:
		conp->postcode_zip = cp->data;
		cp->data = NULL;
		break;

#if 0
	XXX - FIXME
	case TOKEN_COUNTRY:
		conp->country_ = cp->data;
		cp->data = NULL;
		break;
#endif

	case TOKEN_WEBSITE:
		conp->website = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_PHONE:
		conp->phone = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_FAX:
		conp->fax = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_EMAIL:
		conp->email = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_CONTACT:
		break;

	default:
		berror("import", "invalid XML tag in contact field");
		break;
	}
	if (token != TOKEN_CONTACT) {
		cp->sawf = 1;
		return(0);
	}
	if (conp->id == 0 && conp->ccode != NULL &&
			(conp2 = contact_findcode(dbp, conp->ccode)) != NULL) {
		conp->id = conp2->id;
		db_contactfree(conp2);
	}
	if (cp->sawf) {
		if (conp->id == 0 || editcontact(conp) == -1)
			db_insertcontact(dbp, conp);
	}
	return(conp->id);
}

/*
 *
 */
int
editcontact(struct db_contact *conp)
{
	struct db_contact *conp2;
	dbow_conn *dbp = getdbase();

	if ((conp2 = contact_findbyid(dbp, conp->id)) == NULL)
		return(-1);
	db_updatecontactbyid(dbp, conp2, conp2->id);
	return(0);
}
