#ident "$Id: invoice.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: invoice.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: invoice.c,v $
 * Revision 1.6  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2006/07/31 10:17:48  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.3  2005/09/26 15:44:36  dtynan
 * Fixed import bug where records didn't get inserted in some cases.
 *
 * Revision 1.2  2005/09/21 14:14:34  dtynan
 * Major addition of data-import functionality.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include <expat.h>

#include "beanie.h"
#include "hacks.h"

#include "import.h"

int	itemno = 0;

int	editinvoice(struct db_invoice *);

/*
 *
 */
int
doinvoice(struct ctxt *cp, int token)
{
	struct db_invoice *invp = (struct db_invoice *)cp->tail->udata;
	dbow_conn *dbp = getdbase();

	switch (token) {
	case TOKEN_DATE:
		parsedate(cp->data, &invp->idate);
		break;

	case TOKEN_DUEDATE:
		parsedate(cp->data, &invp->ddate);
		break;

	case TOKEN_ORDERDATE:
		parsedate(cp->data, &invp->odate);
		break;

	case TOKEN_CUSTOMER:
		invp->customer_id = cp->tail->id;
		break;

	case TOKEN_SALESORDER:
		invp->salesorder_id = cp->tail->id;
		break;

	case TOKEN_TOTAL:
		invp->total = atof(cp->data);
		break;

	case TOKEN_VAT_TOTAL:
		invp->vatttl = atof(cp->data);
		break;

	case TOKEN_SHIPPING:
		invp->shipping = atof(cp->data);
		break;

	case TOKEN_YOURREF:
		invp->yref = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_DESCR:
		invp->descr = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_STATE:
		invp->state = atoi(cp->data);
		break;

	case TOKEN_INVOICE:
	case TOKEN_ITEM:
		break;

	default:
		berror("import", "invalid XML tag in invoice field");
		break;
	}
	if (token != TOKEN_INVOICE) {
		if (token != TOKEN_ITEM)
			itemno = 0;
		cp->sawf = 1;
		return(0);
	}
	if (cp->sawf) {
		if (invp->id == 0 || editinvoice(invp) == -1)
			db_insertinvoice(dbp, invp);
	}
	return(invp->id);
}

/*
 *
 */
int
doinvoiceitem(struct ctxt *cp, int token)
{
	struct db_invoice_item *iip = (struct db_invoice_item *)cp->tail->udata;
	struct nest *np;
	dbow_conn *dbp = getdbase();

	switch (token) {
	case TOKEN_NOMACCT:
		iip->nomacct_id = atoi(cp->data);
		break;

	case TOKEN_DESCR:
		iip->descr = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_VATCODE:
		iip->vcode = atoi(cp->data);
		break;

	case TOKEN_QUANTITY:
		iip->quant = atoi(cp->data);
		break;

	case TOKEN_UNITCOST:
		iip->unitc = atof(cp->data);
		break;

	case TOKEN_DISCOUNT:
		iip->disc = atof(cp->data);
		break;

	case TOKEN_ITEM:
		break;

	default:
		berror("import", "invalid XML tag in invoice item field");
		break;
	}
	if (token != TOKEN_ITEM) {
		cp->sawf = 1;
		return(0);
	}
	if (cp->tail->id != 0) {
		/*
		 * Dig out the parent nesting context.
		 */
		for (np = cp->head; np->next != cp->tail; np = np->next)
			;
		/*
		 * Kludge!  Don't have the invoice ID number
		 * so we call runentry to stuff the database
		 * and get the number for us.
		 */
		cp->tail = np;
		np = cp->tail->next;
		cp->tail->next = NULL;
		np->id = dosalesorder(cp, TOKEN_INVOICE);
		cp->tail->next = np;
		cp->tail = np;
	}
	iip->item_id = ++itemno;
	iip->invoice_id = cp->tail->id;
	db_insertinvoice_item(dbp, iip);
	return(iip->item_id);
}

/*
 *
 */
int
editinvoice(struct db_invoice *invp)
{
	struct db_invoice *invp2;
	dbow_conn *dbp = getdbase();

	if ((invp2=db_findinvoicebyid(dbp, invp->id)) == NULL)
		return(-1);
	db_updateinvoicebyid(dbp, invp2, invp2->id);
	return(0);
}
