#ident "$Id: main.c,v 1.9 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: main.c,v 1.9 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * The software contained herein is proprietary to and embodies the
 * confidential technology of Kalopa Internet Solutions Limited.
 * Possession, use, duplication or dissemination of the software and
 * media is authorized only pursuant to a valid written license from
 * Kalopa Internet Solutions Limited.
 *
 * ABSTRACT
 *
 * $Log: main.c,v $
 * Revision 1.9  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.8  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.7  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.6  2006/07/31 10:17:48  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.5  2005/10/17 13:31:23  dtynan
 * Make sure the customer currency is defined.
 *
 * Revision 1.4  2005/09/21 15:22:14  dtynan
 * Don't default the cdate to today.  It's OK to do this for the rdate.
 *
 * Revision 1.3  2005/09/21 15:16:43  dtynan
 * The 'cdate' field is actually the completion date of the task, not the
 * creation date.
 *
 * Revision 1.2  2005/09/21 14:14:34  dtynan
 * Major addition of data-import functionality.
 *
 * Revision 1.1  2005/08/12 18:50:10  dtynan
 * First pass at a utility to import data from an XML file into the
 * database.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include <expat.h>

#include "beanie.h"
#include "hacks.h"

#include "import.h"

int	verbose;

void	usage();

/*
 *
 */
struct	token	{
	char	*name;
	int	value;
} tokens[] = {
	{"addr",	TOKEN_ADDR},
	{"balance",	TOKEN_BALANCE},
	{"blocked",	TOKEN_BLOCKED},
	{"ccode",	TOKEN_CCODE},
	{"city",	TOKEN_CITY},
	{"contact",	TOKEN_CONTACT},
	{"country",	TOKEN_COUNTRY},
	{"currency",	TOKEN_CURRENCY},
	{"customer",	TOKEN_CUSTOMER},
	{"date",	TOKEN_DATE},
	{"descr",	TOKEN_DESCR},
	{"discount",	TOKEN_DISCOUNT},
	{"done",	TOKEN_DONE},
	{"duedate",	TOKEN_DUEDATE},
	{"email",	TOKEN_EMAIL},
	{"emailbill",	TOKEN_EMAILBILL},
	{"enddate",	TOKEN_ENDDATE},
	{"endno",	TOKEN_ENDNO},
	{"fax",		TOKEN_FAX},
	{"faxbill",	TOKEN_FAXBILL},
	{"firstname",	TOKEN_FIRSTNAME},
	{"flag",	TOKEN_FLAG},
	{"frequency",	TOKEN_FREQUENCY},
	{"idescr",	TOKEN_IDESCR},
	{"invoice",	TOKEN_INVOICE},
	{"item",	TOKEN_ITEM},
	{"lastname",	TOKEN_LASTNAME},
	{"manufactured",TOKEN_MANUFACTURED},
	{"name",	TOKEN_NAME},
	{"neworder",	TOKEN_NEWORDER},
	{"nomacct",	TOKEN_NOMACCT},
	{"noshares",	TOKEN_NOSHARES},
	{"orderdate",	TOKEN_ORDERDATE},
	{"pcode",	TOKEN_PCODE},
	{"phone",	TOKEN_PHONE},
	{"postbill",	TOKEN_POSTBILL},
	{"price",	TOKEN_PRICE},
	{"product",	TOKEN_PRODUCT},
	{"quantity",	TOKEN_QUANTITY},
	{"ready",	TOKEN_READY},
	{"resale",	TOKEN_RESALE},
	{"retail_price",TOKEN_RETAIL_PRICE},
	{"rundate",	TOKEN_RUNDATE},
	{"salesorder",	TOKEN_SALESORDER},
	{"service",	TOKEN_SERVICE},
	{"sharecert",	TOKEN_SHARECERT},
	{"sharedate",	TOKEN_SHAREDATE},
	{"shipping",	TOKEN_SHIPPING},
	{"startno",	TOKEN_STARTNO},
	{"state",	TOKEN_STATE},
	{"terms",	TOKEN_TERMS},
	{"total",	TOKEN_TOTAL},
	{"trade_price",	TOKEN_TRADE_PRICE},
	{"unitcost",	TOKEN_UNITCOST},
	{"vat_total",	TOKEN_VAT_TOTAL},
	{"vatcode",	TOKEN_VATCODE},
	{"vatno",	TOKEN_VATNO},
	{"website",	TOKEN_WEBSITE},
	{"wsale_price",	TOKEN_WSALE_PRICE},
	{"yourref",	TOKEN_YOURREF},
	{"zip",		TOKEN_ZIP},
	{NULL,		0}
};

struct	attr	{
	char	*name;
	int	value;
} attrs[] = {
	{"day",		ATTR_DAY},
	{"hour",	ATTR_HOUR},
	{"manual",	ATTR_MANUAL},
	{"month",	ATTR_MONTH},
	{"number",	ATTR_NUMBER},
	{"once",	ATTR_ONCE},
	{"vtype",	ATTR_VTYPE},
	{"year",	ATTR_YEAR},
	{NULL,		0}
};

/*
 *
 */
struct nest *
newnesting(struct ctxt *cp, int token)
{
	struct nest *np;

	if ((np = (struct nest *)malloc(sizeof(struct nest))) == NULL)
		berror("import", "nesting malloc failure");
	memset((char *)np, 0, sizeof(struct nest));
	np->token = token;
	if (cp->head == NULL)
		cp->head = np;
	else
		cp->tail->next = np;
	cp->tail = np;
	return(np);
}

/*
 *
 */
int
runentry(struct ctxt *cp, int token)
{
	int idret;

	assert(cp->tail != NULL);
	switch (cp->tail->token) {
	case TOKEN_SHARECERT:
		idret = dosharecert(cp, token);
		break;

	case TOKEN_SALESORDER:
		idret = dosalesorder(cp, token);
		break;

	case TOKEN_ORDER_ITEM:
		idret = doorderitem(cp, token);
		break;

	case TOKEN_INVOICE:
		idret = doinvoice(cp, token);
		break;

	case TOKEN_INVOICE_ITEM:
		idret = doinvoiceitem(cp, token);
		break;

	case TOKEN_CUSTOMER:
		idret = docustomer(cp, token);
		break;

	case TOKEN_CONTACT:
		idret = docontact(cp, token);
		break;

	case TOKEN_PRODUCT:
		idret = doproduct(cp, token);
		break;
	}
	return(idret);
}

/*
 *
 */
static void
startelement(void *udp, const char *name, const char **attr)
{
	int i, j, number = 0;
	struct ctxt *cp = (struct ctxt *)udp;
	struct nest *np;
	struct token *tp;
	struct attr *ap;
	struct db_sharecert *scertp;
	struct db_salesorder *sorderp;
	struct db_sorder_item *soitemp;
	struct db_invoice *invoicep;
	struct db_invoice_item *invitemp;
	struct db_customer *customerp;
	struct db_contact *contactp;
	struct db_product *productp;

	cp->level++;
	for (tp = tokens; tp->name != NULL; tp++)
		if (strcmp(tp->name, name) == 0)
			break;
	if (verbose)
		printf("Tag: %s\n", name);
	if (tp->name == NULL)
		berror("import", "invalid XML in source file");
	cp->elem = tp->value;
	for (i = 0; attr[i]; i += 2) {
		for (ap = attrs; ap->name != NULL; ap++)
			if (strcmp(ap->name, attr[i]) == 0)
				break;
		if (ap->name == NULL)
			berror("import", "invalid XML in source file");
		j = atoi(attr[i + 1]);
		switch (ap->value) {
		case ATTR_VTYPE:
			cp->vtype = strdup(attr[i + 1]);
			break;

		case ATTR_NUMBER:
			number = j;
			break;

		case ATTR_HOUR:
			if (j < 24) {
				cp->freq = FREQ_HOURLY;
				break;
			}
			j = (j + 23) / 24;
			/* Fall through... */

		case ATTR_DAY:
			if (j == 7)
				cp->freq = FREQ_WEEKLY;
			else if (j == 14)
				cp->freq = FREQ_WEEKLY2;
			else if (j == 28)
				cp->freq = FREQ_WEEKLY4;
			else
				cp->freq = FREQ_DAILY;
			break;

		case ATTR_MONTH:
			if (j == 2)
				cp->freq = FREQ_MONTHLY2;
			else if (j == 3)
				cp->freq = FREQ_QUARTERLY;
			else if (j == 6)
				cp->freq = FREQ_BIANNUAL;
			else
				cp->freq = FREQ_MONTHLY;
			break;

		case ATTR_YEAR:
			cp->freq = FREQ_ANNUALLY;
			break;

		case ATTR_ONCE:
			cp->freq = FREQ_ONCE;
			break;

		case ATTR_MANUAL:
			cp->freq = FREQ_MANUALLY;
			break;
		}
	}
	switch (tp->value) {
	case TOKEN_SHARECERT:
		np = newnesting(cp, tp->value);
		np->udata = scertp = db_sharecertalloc();
		scertp->id = number;
		cp->sawf = 0;
		break;

	case TOKEN_SALESORDER:
		np = newnesting(cp, tp->value);
		np->udata = sorderp = db_salesorderalloc();
		sorderp->id = number;
		cp->sawf = 0;
		break;

	case TOKEN_INVOICE:
		np = newnesting(cp, tp->value);
		np->udata = invoicep = db_invoicealloc();
		invoicep->id = number;
		cp->sawf = 0;
		break;

	case TOKEN_CUSTOMER:
		np = newnesting(cp, tp->value);
		np->udata = customerp = db_customeralloc();
		customerp->id = number;
		customerp->currency_id = HOME_CURRENCY;
		customerp->vattype = CUST_VTYPE_NONE;
		cp->sawf = 0;
		break;

	case TOKEN_CONTACT:
		np = newnesting(cp, tp->value);
		np->udata = contactp = db_contactalloc();
		contactp->id = number;
		contactp->country_id = HOME_COUNTRY;
		cp->sawf = 0;
		break;

	case TOKEN_PRODUCT:
		np = newnesting(cp, tp->value);
		np->udata = productp = db_productalloc();
		productp->id = number;
		cp->sawf = 0;
		break;

	case TOKEN_ITEM:
		switch (cp->tail->token) {
		case TOKEN_SALESORDER:
			np = newnesting(cp, TOKEN_ORDER_ITEM);
			np->udata = soitemp = db_sorder_itemalloc();
			soitemp->item_id = number;
			time(&soitemp->rdate);
			break;

		case TOKEN_INVOICE:
			np = newnesting(cp, TOKEN_INVOICE_ITEM);
			np->udata = invitemp = db_invoice_itemalloc();
			invitemp->item_id = number;
			break;

		default:
			berror("import", "invalid XML 'item'");
			break;
		}
		cp->sawf = 0;
	}
}

/*
 *
 */
static void
endelement(void *udp, const char *name)
{
	int idret = 0;
	struct ctxt *cp = (struct ctxt *)udp;
	struct nest *np;
	struct token *tp;

	cp->level--;
	for (tp = tokens; tp->name != NULL; tp++)
		if (strcmp(tp->name, name) == 0)
			break;
	if (tp->name == NULL)
		berror("import", "invalid XML in source file");
	if (cp->data != NULL) {
		char *xp;

		xp = strchr(cp->data, '\0');
		while (isspace(*--xp))
			;
		*++xp = '\0';
	}
	if (cp->tail == NULL)
		berror("import", "weird parsing failure");
	idret = runentry(cp, tp->value);
	if (cp->data != NULL) {
		free(cp->data);
		cp->data = NULL;
	}
	if (idret != 0) {
		np = cp->head;
		if (np->next != NULL) {
			for (; np->next != NULL; np = np->next)
				if (np->next == cp->tail)
					break;
			cp->tail = np;
			free((char *)np->next);
			np->next = NULL;
			np->id = idret;
			runentry(cp, tp->value);
		} else
			np->id = idret;
	}
}

/*
 *
 */
void
chardata(void *udp, const XML_Char *s, int len)
{
	struct ctxt *cp = (struct ctxt *)udp;

	if (cp->tail == NULL)
		berror("import", "invalid raw data in XML file");
	if (cp->elem == -1)
		return;
	if (cp->data == NULL) {
		cp->dlen = 0;
		while (isspace(*s))
			s++, len--;
	}
	if (len == 0)
		return;
	if ((cp->data = realloc(cp->data, cp->dlen + len + 1)) == NULL)
		berror("import", "out of character memory");
	memcpy(cp->data + cp->dlen, s, len);
	cp->dlen += len;
	cp->data[cp->dlen] = '\0';
}

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, done;
	char *cname, buf[BUFSIZ];
	struct ctxt ctxt;
	size_t len;
	XML_Parser parser = XML_ParserCreate(NULL);
	FILE *fp;

	cname = NULL;
	opterr = verbose = 0;
	while ((i = getopt(argc, argv, "c:v")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1)
		usage();
	loadconfig(cname);
	if ((fp = fopen(argv[optind], "r")) == NULL) {
		perror(argv[optind]);
		exit(1);
	}
	/*
	 * Find the company that we're processing...
	 */
	memset((char *)&ctxt, 0, sizeof(struct ctxt));
	ctxt.elem = -1;
	XML_SetUserData(parser, &ctxt);
	XML_SetElementHandler(parser, startelement, endelement);
	XML_SetCharacterDataHandler(parser, chardata);
	do {
		len = fread(buf, 1, sizeof(buf), fp);
		done = len < sizeof(buf);
		if (XML_Parse(parser, buf, len, done) == XML_STATUS_ERROR) {
			fprintf(stderr, "%s at line %d\n",
				XML_ErrorString(XML_GetErrorCode(parser)),
				XML_GetCurrentLineNumber(parser));
			exit(1);
		}
	} while(!done);
	XML_ParserFree(parser);
	fclose(fp);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: import [-c <comp>][-v] <file>.xml\n");
	exit(2);
}
