#ident "$Id: salesorder.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: salesorder.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: salesorder.c,v $
 * Revision 1.7  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.6  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.5  2006/07/31 10:17:49  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.4  2005/09/26 15:44:36  dtynan
 * Fixed import bug where records didn't get inserted in some cases.
 *
 * Revision 1.3  2005/09/21 15:16:43  dtynan
 * The 'cdate' field is actually the completion date of the task, not the
 * creation date.
 *
 * Revision 1.2  2005/09/21 14:53:05  dtynan
 * The order state is now on a per-line basis.
 *
 * Revision 1.1  2005/09/21 14:14:34  dtynan
 * Major addition of data-import functionality.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include <expat.h>

#include "beanie.h"
#include "hacks.h"

#include "import.h"

int	itemno;

int	editsalesorder(struct db_salesorder *);

/*
 *
 */
int
dosalesorder(struct ctxt *cp, int token)
{
	struct db_salesorder *sop = (struct db_salesorder *)cp->tail->udata;
	dbow_conn *dbp = getdbase();

	switch (token) {
	case TOKEN_CUSTOMER:
		sop->customer_id = cp->tail->id;
		cp->tail->id = 0;
		break;

	case TOKEN_ORDERDATE:
		parsedate(cp->data, &sop->odate);
		break;

	case TOKEN_YOURREF:
		sop->yref = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_DESCR:
		sop->descr = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_SALESORDER:
	case TOKEN_ITEM:
		break;

	default:
		berror("import", "invalid XML tag in sales order field");
		break;
	}
	if (token != TOKEN_SALESORDER) {
		if (token != TOKEN_ITEM)
			itemno = 0;
		cp->sawf = 1;
		return(0);
	}
	if (cp->sawf) {
		if (sop->id == 0 || editsalesorder(sop) == -1)
			db_insertsalesorder(dbp, sop);
	}
	return(sop->id);
}

/*
 *
 */
int
doorderitem(struct ctxt *cp, int token)
{
	struct db_sorder_item *sip = (struct db_sorder_item *)cp->tail->udata;
	struct nest *np;
	dbow_conn *dbp = getdbase();

	switch (token) {
	case TOKEN_PRODUCT:
		sip->product_id = cp->tail->id;
		break;

	case TOKEN_RUNDATE:
		parsedate(cp->data, &sip->rdate);
		break;

	case TOKEN_ENDDATE:
		parsedate(cp->data, &sip->cdate);
		break;

	case TOKEN_NEWORDER:
		sip->state = SALESORDER_NEW;
		break;

	case TOKEN_READY:
		sip->state = SALESORDER_READY;
		break;

	case TOKEN_DONE:
		sip->state = SALESORDER_DONE;
		break;

	case TOKEN_BLOCKED:
		sip->state = SALESORDER_BLOCKED;
		break;

	case TOKEN_FREQUENCY:
		sip->freq = cp->freq;
		break;

	case TOKEN_DESCR:
		sip->descr = cp->data;
		cp->data = NULL;
		break;

	case TOKEN_VATCODE:
		sip->vcode = atoi(cp->data);
		break;

	case TOKEN_QUANTITY:
		sip->quant = atoi(cp->data);
		break;

	case TOKEN_UNITCOST:
		sip->unitc = atof(cp->data);
		break;

	case TOKEN_DISCOUNT:
		sip->disc = atof(cp->data);
		break;

	case TOKEN_ITEM:
		break;

	default:
		berror("import", "invalid XML tag in order item field");
		break;
	}
	if (token != TOKEN_ITEM) {
		cp->sawf = 1;
		return(0);
	}
	if (cp->tail->id != 0) {
		/*
		 * Dig out the parent nesting context.
		 */
		for (np = cp->head; np->next != cp->tail; np = np->next)
			;
		/*
		 * Kludge!  Don't have the sales order ID number
		 * so we call runentry to stuff the database
		 * and get the number for us.
		 */
		cp->tail = np;
		np = cp->tail->next;
		cp->tail->next = NULL;
		np->id = dosalesorder(cp, TOKEN_SALESORDER);
		cp->tail->next = np;
		cp->tail = np;
	}
	sip->item_id = ++itemno;
	sip->salesorder_id = cp->tail->id;
	db_insertsorder_item(dbp, sip);
	return(sip->item_id);
}

/*
 *
 */
int
editsalesorder(struct db_salesorder *sop)
{
	struct db_salesorder *sop2;
	dbow_conn *dbp = getdbase();

	sop2 = db_findsalesorderbyid(dbp, sop->id);
	if (sop2 == NULL)
		return(-1);
	db_updatesalesorderbyid(dbp, sop2, sop2->id);
	return(0);
}
