#ident "Kalopa: $Id: charge.c,v 1.13 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: charge.c,v 1.13 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: charge.c,v $
 * Revision 1.13  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.12  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.11  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.10  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.9  2006/10/01 10:18:21  dtynan
 * Major changes for VAT.
 *
 * Revision 1.8  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.7  2006/07/31 08:51:20  dtynan
 * Added an option to calculate VAT and create a journal entry for same.
 *
 * Revision 1.6  2005/03/22 09:41:14  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.5  2005/02/17 16:23:23  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.4  2005/02/16 10:15:43  dtynan
 * Changed nominal accounts to use main database (not the per-company one).
 *
 * Revision 1.3  2005/01/13 20:44:09  dtynan
 * Added a new function to post end of year totals.
 *
 * Revision 1.2  2005/01/13 17:44:27  dtynan
 * Added new program to post a customer payment to the database.  Also did
 * cleanups of functions across the board.
 *
 * Revision 1.1  2005/01/13 14:54:07  dtynan
 * Added new program to add a simple journal item.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, dracct, cracct, vcode;
	dbow_conn *dbp;
	char *cname, *folio, *descr;
	struct db_nomacct *nomp;
	double amount;
	time_t when;

	folio = cname = NULL;
	time(&when);
	opterr = 0;
	vcode = -1;
	while ((i = getopt(argc, argv, "c:d:f:V")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				usage();
			break;

		case 'f':
			folio = optarg;
			break;

		case 'V':
			vcode = vatid(DEFAULT_VATCODE);
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Make sure we have the right number of arguments.  Then
	 * connect to the appropriate company database.
	 */
	if ((argc - optind) != 4)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	/*
	 *
	 */
	dracct = atoi(argv[optind++]);
	cracct = atoi(argv[optind++]);
	descr  = argv[optind++];
	amount = atof(argv[optind++]);
	printf("Date:\t\t%s\n", prtime(&when));
	printf("Description:\t\"%s\"\n", descr);
	if ((nomp = db_findnomacctbyid(dbp, dracct)) == NULL)
		berror("charge", "invalid debit account number");
	printf("Debit account:\t%s\n", nomp->aname);
	if ((nomp = db_findnomacctbyid(dbp, cracct)) == NULL)
		berror("charge", "invalid credit account number");
	printf("Credit account:\t%s\n", nomp->aname);
	if (folio != NULL)
		printf("Folio:\t\t%s\n", folio);
	printf("Net Amount:\t%.2f\n", amount);
	if (yesorno("Is this correct")) {
		/*
		 * Now do the posting.
		 */
		journal_entry(&when,folio,descr,vcode,dracct,cracct,amount);
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: charge [-c <comp>][-d date][-f folio] [-V] dracct cracct descr amount\n");
	exit(2);
}
