#ident "Kalopa: $Id: cheque.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: cheque.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: cheque.c,v $
 * Revision 1.7  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.6  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.5  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.3  2006/10/01 10:18:21  dtynan
 * Major changes for VAT.
 *
 * Revision 1.2  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.1  2005/10/17 13:29:25  dtynan
 * New application for entering cheque details.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, chqno, dracct, vcode;
	dbow_conn *dbp;
	char *cname, *payee, *descr, folio[16];
	struct db_nomacct *nomp;
	struct db_chqreg *cqp;
	double amount;
	time_t when;

	cname = NULL;
	time(&when);
	opterr = 0;
	vcode = -1;
	while ((i = getopt(argc, argv, "c:d:V")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				usage();
			break;

		case 'V':
			vcode = vatid(DEFAULT_VATCODE);
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Make sure we have the right number of arguments.  Then
	 * connect to the appropriate company database.
	 */
	if ((argc - optind) != 5)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	/*
	 *
	 */
	chqno  = atoi(argv[optind++]);
	dracct = atoi(argv[optind++]);
	payee  = argv[optind++];
	descr  = argv[optind++];
	amount = atof(argv[optind++]);
	if (chqno < 1 || chqno > 999999)
		usage();
	printf("Cheque #:\t%d\n", chqno);
	printf("Date:\t\t%s\n", prtime(&when));
	printf("Payee:\t\t\"%s\"\n", payee);
	printf("Description:\t\"%s\"\n", descr);
	if ((nomp = db_findnomacctbyid(dbp, dracct)) == NULL)
		berror("cheque", "invalid debit account number");
	printf("Debit account:\t%s\n", nomp->aname);
	printf("Net Amount:\t%.2f\n", amount);
	sprintf(folio, "%d", chqno);
	if (yesorno("Is this correct")) {
		/*
		 * Now do the posting.
		 */
		journal_entry(&when, folio, descr, vcode, dracct,
						NA_MAINBANK, amount);
		/*
		 * Now do the posting to the cheque register.
		 */
		if ((cqp = db_chqregalloc()) == NULL)
			berror("cheque", "cannot create cheque register item");
		cqp->chqno = chqno;
		cqp->cdate = when;
		cqp->payee = strdup(payee);
		cqp->amount = amount;
		if (db_insertchqreg(dbp, cqp) < 0)
			berror("cheque", "cheque register insert failed");
		db_chqregfree(cqp);
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: cheque [-c <comp>][-d date][-V] chqno dracct payee descr amount\n");
	exit(2);
}
