#ident "Kalopa: $Id: eoyupdate.c,v 1.13 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: eoyupdate.c,v 1.13 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: eoyupdate.c,v $
 * Revision 1.13  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.12  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.11  2008/07/08 16:40:08  dtynan
 * Excessive changes.
 *
 * Revision 1.10  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.9  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.8  2006/10/01 10:18:21  dtynan
 * Major changes for VAT.
 *
 * Revision 1.7  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.6  2005/03/22 09:41:14  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.5  2005/02/17 17:25:26  dtynan
 * Fixed some compiler warnings.
 *
 * Revision 1.4  2005/02/17 16:23:23  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.3  2005/02/17 11:03:03  dtynan
 * Update the year-end as well as the other EOY tasks.
 *
 * Revision 1.2  2005/02/16 10:15:43  dtynan
 * Changed nominal accounts to use main database (not the per-company one).
 *
 * Revision 1.1  2005/01/13 20:44:09  dtynan
 * Added a new function to post end of year totals.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

#define JOURNALDESCR	"Opening Balance"

struct	balance	{
	struct	balance	*next;
	int		posted;
	struct	nabal	*nomp;
	double		abal;
};

struct	balance	*head, *tail;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, ndiffs, year, noeffect, verbose, yesman;
	char *cname;
	dbow_conn *dbp;
	struct nabal *nlist, *np;
	struct db_company *comp;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct tm *tmp, mytm;
	struct balance *bp;
	double newretpl;
	time_t when, yend;

	cname = NULL;
	time(&when);
	head = tail = NULL;
	opterr = noeffect = verbose = yesman = 0;
	while ((i = getopt(argc, argv, "c:nvy")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		case 'y':
			yesman = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Make sure we have the right number of arguments.  Then
	 * connect to the appropriate company database.
	 */
	if ((argc - optind) != 1)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("eoyupdate", "cannot find company");
	if (!yesman)
		verbose = 1;
	if (verbose)
		printf("Company: %s\n", comp->cname);
	if ((year = atoi(argv[optind])) < 100) {
		if (year < 50)
			year += 2000;
		else
			year += 1900;
	}
	if (year < 1900 || year > 2100)
		usage();
	year -= 1900;
	/*
	 * Load the current year balances and calculate the end of year
	 * totals (for posting).
	 */
	newretpl = 0.0;
	if ((nlist = loadnabal(dbp)) == NULL)
		berror("eoyypdate", "cannot load nominal accounts");
	loadbalances(dbp, nlist, year);
	for (np = nlist; np != NULL; np = np->next) {
		if (np->nomacct_id != NA_RETAINEDPL && np->bal == 0.0)
			continue;
		printf("%5d: %2d %10.2f %s\n", np->nomacct_id, np->atype, np->bal, np->name);
		switch (np->atype) {
		case INCOME:
		case EXPENSE:
		case COSTOFSALES:
			newretpl += np->bal;
			break;

		case ASSET_FIXED:
		case ASSET_CURRENT:
		case LIABILITY_CURRENT:
		case LIABILITY_LONG:
		case OEQUITY:
			if ((bp = (struct balance *)malloc(sizeof(*bp))) == NULL)
				berror("eoyupdate", "malloc failed");
			bp->next = NULL;
			bp->posted = 0;
			bp->nomp = np;
			bp->abal = 0.0;
			if (head == NULL)
				head = bp;
			else
				tail->next = bp;
			tail = bp;
			break;
		}
	}
	/*
	 * Calculate new retained profit and loss.
	 */
	printf("New Retained P+L=%.2f\n", newretpl);
	for (bp = head; bp != NULL; bp = bp->next)
		if (bp->nomp->nomacct_id == NA_RETAINEDPL)
			bp->nomp->bal += newretpl;
	/*
	 * Find the next years opening balance statement.
	 */
	year++;
	mytm.tm_mday = 1;
	mytm.tm_mon = 0;
	mytm.tm_year = year;
	mytm.tm_hour = 0;
	mytm.tm_min = 0;
	mytm.tm_sec = 0;
	mytm.tm_isdst = -1;
	/*
	 * Is there an opening balance?
	 */
	for (jp = db_findjournalbydescr(dbp, JOURNALDESCR);
				jp != NULL;
				jp = db_findjournalnext(dbp, jp)) {
		if ((tmp = localtime((time_t *)&jp->jdate)) != NULL &&
							(tmp->tm_year == year))
			break;
	}
	if (jp == NULL) {
		if (verbose)
			printf("No opening balance statement found.\n");
		if (!yesman && !yesorno("Create an opening balance"))
			exit(0);
		if ((jp = db_journalalloc()) == NULL)
			berror("eoyupdate", "cannot create journal entry");
		jp->jdate = mktime(&mytm);
		jp->folio = NULL;
		jp->descr = strdup(JOURNALDESCR);
		jp->user_id = getuid();
		jp->state = 0;
		if (!noeffect && db_insertjournal(dbp, jp) < 0)
			berror("eoyupdate", "journal insert failed");
	}
	if (verbose)
		printf("Opening balance ID: %d\n", jp->id);
	ndiffs = 0;
	for (jip = db_findjournal_itembyjournal_id(dbp, jp->id);
				jip != NULL;
				jip = db_findjournal_itemnext(dbp, jip)) {
		printf("%d,%d,%d,%.2f\n", jip->item_id,jip->nomacct_id,jip->drcrf,jip->amount);
		for (bp = head; bp != NULL; bp = bp->next)
			if (bp->nomp->nomacct_id == jip->nomacct_id)
				break;
		if (bp == NULL) {
			if (verbose)
				printf("Extraneous balance item for N/A %d\n", jip->nomacct_id);
			ndiffs++;
		} else {
			bp->posted = 1;
			if (jip->drcrf)
				bp->abal = jip->amount;
			else
				bp->abal = -jip->amount;
		}
	}
	for (bp = head; bp != NULL; bp = bp->next) {
		printf("%d, NA ID: %d, %.2f (%.2f)\n", bp->posted, bp->nomp->nomacct_id, bp->nomp->bal, bp->abal);
		if (bp->posted == 0)
			ndiffs++;
		else if (fabs(bp->nomp->bal - bp->abal) >= 0.01)
			ndiffs++;
	}
	if (ndiffs > 0) {
		if (verbose)
			printf("There are %d differences.\n", ndiffs);
		if (!yesman && !yesorno("Post new updates"))
			exit(0);
		db_deletejournal_itembyjournal_id(dbp, jp->id);
		if ((jip = db_journal_itemalloc()) == NULL)
			berror("eoyupdate", "cannot create journal item");
		jip->journal_id = jp->id;
		jip->item_id = 0;
		for (bp = head; bp != NULL; bp = bp->next) {
			jip->item_id++;
			jip->nomacct_id = bp->nomp->nomacct_id;
			if (bp->nomp->bal < 0.0) {
				jip->drcrf = 0;
				jip->amount = -bp->nomp->bal;
			} else {
				jip->drcrf = 1;
				jip->amount = bp->nomp->bal;
			}
			if (!noeffect && db_insertjournal_item(dbp, jip) < 0)
				berror("eoyupdate", "journal item insert failed");
		}
	}
	/*
	 * Calculate the year-end for this company and the specified year.
	 */
	mytm.tm_mon  = getyend(comp) - 1;
	mytm.tm_mday = mdays(mytm.tm_mon + 1, mytm.tm_year + 1900);
	mytm.tm_hour = 12;
	yend = mktime(&mytm);
	if (comp->yend < yend) {
		printf("Current: %s", ctime((time_t *)&comp->yend));
		if (verbose)
			printf("Revised year end: %s", ctime(&yend));
		if (!yesman && !yesorno("Update year end"))
			exit(0);
		comp->yend = yend;
		if (!noeffect && db_updatecompanybyid(dbp, comp, comp->id) < 0)
			berror("eoyupdate", "cannot update company");
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: eoyupdate [-nvy][-c <comp>] year\n");
	exit(2);
}
