#ident "$Id: fixvat.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: fixvat.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: fixvat.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:40:10  dtynan
 * Excessive changes.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, jno, noeffect, verbose;
	char *cname;
	struct db_journal *jp;
	struct db_journal_item *jip;
	double amount, vatamt;
	dbow_conn *dbp;

	cname =  NULL;
	opterr = noeffect = verbose = 0;
	while ((i = getopt(argc, argv, "c:va:d:n:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Make sure we have the right number of arguments.  Then
	 * connect to the appropriate company database.
	 */
	if ((argc - optind) != 1)
		usage();
	jno = atoi(argv[optind]);
	loadconfig(cname);
	if (verbose)
		printf("Journal #:\t%d\n", jno);
	/*
	 * OK, we're good to go.  If we are changing the description,
	 * we'll need the journal entry.  Start there...
	 */
	for (jp = db_findjournalfirst(dbp); jp != NULL;
					jp = db_findjournalnext(dbp, jp)) {
		if (jp->id < jno)
			continue;
		if ((jip=findjournalitembynomacct(dbp, jp->id, 2600)) == NULL)
			continue;
		printf("J.ID:\t%d...\n", jp->id);
		if ((jip = findjournalitems(dbp, jp->id)) == NULL)
			berror("fixvat", "cannot find journal items");
		for (; jip != NULL; jip = db_findjournal_itemnext(dbp, jip)) {
			if (verbose)
				printf("Fix J%d.%d (%d)\n", jip->journal_id, jip->item_id, jip->nomacct_id);
			amount = vat_findnet(jip->amount, 2);
			printf("%.2f/%.2f\n", jip->amount, amount);
			switch (jip->item_id) {
			case 1:
				vatamt = jip->amount - amount;
				jip->amount = amount;
				break;

			case 2:
				jip->amount = vatamt;
				break;
			}
			printf("New amt:%.2f\n", jip->amount);
			if (!noeffect && updatejournal_item(dbp, jip) < 0)
				berror("fixvat", "cannot update journal item");
		}
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage:\tfixvat [-c <comp>][-nv] j.id\n");
	exit(2);
}
