#ident "$Id: jdelete.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: jdelete.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2007, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: jdelete.c,v $
 * Revision 1.5  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.4  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.3  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.2  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.1  2007/08/06 10:32:45  dtynan
 * Added some new programs for manipulating journal entries and modified
 * the VAT return code.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, jno, verbose;
	char *cname;
	struct db_company *comp;
	struct db_bankdata *bp;
	dbow_conn *dbp;

	opterr = verbose = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:v")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Make sure we have the right number of arguments.  Then
	 * connect to the appropriate company database.
	 */
	if ((argc - optind) != 1)
		usage();
	jno = atoi(argv[optind++]);
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("jdelete", "cannot find company");
	if (verbose)
		printf("Company:\t%s\n", comp->cname);
	/*
	 *
	 */
	if ((bp = db_findbankdatabyjournal_id(dbp, jno)) != NULL)
		berror("jdelete", "journal item is referenced by bank data");
	deletejournalitems(dbp, jno);
	db_deletejournalbyid(dbp, jno);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage:\tjdelete [-c company][-v] j.id\n");
	exit(2);
}
