#ident "$Id: jdump.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: jdump.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2007, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: jdump.c,v $
 * Revision 1.7  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.6  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.5  2008/07/08 16:40:10  dtynan
 * Excessive changes.
 *
 * Revision 1.4  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.2  2007/10/29 17:24:08  dtynan
 * Minor tweaks.
 *
 * Revision 1.1  2007/08/06 10:32:45  dtynan
 * Added some new programs for manipulating journal entries and modified
 * the VAT return code.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

char	*month[12] = {
	"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};

void	showjournal(struct db_journal *);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, jid, verbose, pflag;
	struct db_journal *jp;
	time_t now, sdate, edate;
	char *cname;
	dbow_conn *dbp;

	time(&now);
	cname = NULL;
	opterr = verbose = pflag = 0;
	while ((i = getopt(argc, argv, "c:vs:e:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		case 's':
			if (parsedate(optarg, &sdate) < 0)
				usage();
			if (!pflag)
				edate = sdate + DAYS(30);
			pflag = 1;
			break;

		case 'e':
			if (parsedate(optarg, &edate) < 0)
				usage();
			if (!pflag)
				sdate = edate - DAYS(30);
			pflag = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (!pflag && (argc - optind) < 1)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	/*
	 *
	 */
	if ((argc - optind) > 0) {
		while (optind < argc) {
			jid = atoi(argv[optind++]);
			if ((jp = db_findjournalbyid(dbp, jid)) == NULL) {
				fprintf(stderr, "?jdump: cannot find journal entry.\n");
				continue;
			}
			showjournal(jp);
			db_journalfree(jp);
		}
	} else {
		for (jp = findjournalentries(dbp); jp != NULL;
							jp = db_findjournalnext(dbp, jp)) {
			if (jp->jdate < sdate || jp->jdate >= edate)
				continue;
			showjournal(jp);
		}
	}
	exit(0);
}

/*
 *
 */
void
showjournal(struct db_journal *jp)
{
	struct db_nomacct *np;
	struct db_journal_item *jip;
	struct tm *tmp;
	char *cp;
	dbow_conn *dbp = getdbase();

	tmp = localtime((time_t *)&jp->jdate);
	printf("%2d-%s-%02d  %4d  ", tmp->tm_mday, month[tmp->tm_mon], tmp->tm_year % 100,
										jp->id);
	if (jp->folio != NULL)
		printf("%-4.4s", jp->folio);
	else
		printf("    ");
	printf("  %-50.50s\n", jp->descr);
	for (jip = findjournalitems(dbp, jp->id); jip != NULL;
				jip = db_findjournal_itemnext(dbp, jip)) {
		if ((np = db_findnomacctbyid(dbp, jip->nomacct_id)) == NULL)
			cp = "<Unknown>";
		else
			cp = np->aname;
		printf("\t%04d  %-40.40s", jip->nomacct_id, cp);
		if (jip->drcrf)
			printf("          %8.2f\n", jip->amount);
		else
			printf("%8.2f\n", jip->amount);
	}
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage:\tjdump [-c #][-v][-s startdate][-e enddate][j.id]\n");
	exit(2);
}
