#ident "$Id: jfix.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: jfix.c,v 1.7 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2006, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: jfix.c,v $
 * Revision 1.7  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.6  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.5  2008/07/08 16:40:10  dtynan
 * Excessive changes.
 *
 * Revision 1.4  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.2  2007/10/29 17:24:08  dtynan
 * Minor tweaks.
 *
 * Revision 1.1  2006/10/01 10:18:21  dtynan
 * Major changes for VAT.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, jno, itemno, noeffect, verbose;
	int aflag, dflag, nflag;
	char *cname, *cp, *descr;
	struct db_company *comp;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct db_nomacct *np;
	double amount;
	dbow_conn *dbp;

	opterr = noeffect = verbose = aflag = dflag = nflag = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:nva:d:N:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		case 'a':
			aflag = 1;
			amount = atof(optarg);
			break;

		case 'd':
			dflag = 1;
			descr = strdup(optarg);
			break;

		case 'N':
			nflag = 1;
			if ((np=db_findnomacctbyid(dbp, atoi(optarg))) == NULL)
				usage();
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Make sure we have the right number of arguments.  Then
	 * connect to the appropriate company database.
	 */
	if ((!aflag && !dflag && !nflag) || (argc - optind) != 1)
		usage();
	if ((cp = strchr(argv[optind], '.')) != NULL) {
		*cp++ = '\0';
		itemno = atoi(cp);
	} else
		itemno = 0;
	jno = atoi(argv[optind]);
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("jfix", "cannot find company");
	if (verbose) {
		printf("Company:\t%s\n", comp->cname);
		if (aflag)
			printf("Amount:\t%.2f\n", amount);
		if (dflag)
			printf("Description:\t%s\n", descr);
		if (nflag)
			printf("Account:\t%s\n", np->aname);
		printf("Journal #:\t%d\nItem #:\t\t", jno);
		if (itemno == 0)
			printf("ALL\n");
		else
			printf("%d\n", itemno);
	}
	/*
	 * OK, we're good to go.  If we are changing the description,
	 * we'll need the journal entry.  Start there...
	 */
	if (dflag) {
		if ((jp = db_findjournalbyid(dbp, jno)) == NULL)
			berror("jfix", "cannot find journal entry");
		if (verbose)
			printf("Old Descr:\t%s\n", jp->descr);
		jp->descr = descr;
		if (!noeffect && db_updatejournalbyid(dbp, jp, jp->id) < 0)
			berror("jfix", "cannot update journal entry");
	}
	/*
	 * Do we need to change the items?  If not, we're done.
	 */
	if (!aflag && !nflag)
		exit(0);
	if ((jip = findjournalitems(dbp, jno)) == NULL)
		berror("jfix", "cannot find journal items");
	for (; jip != NULL; jip = db_findjournal_itemnext(dbp, jip)) {
		if (itemno != 0 && itemno != jip->item_id)
			continue;
		if (verbose)
			printf("Fix J%d.%d:", jip->journal_id, jip->item_id);
		if (aflag) {
			if (verbose)
				printf(" Old Amount: %.2f", jip->amount);
			jip->amount = amount;
		}
		if (nflag) {
			if (verbose)
				printf(" Old N/A: %d", jip->nomacct_id);
			jip->nomacct_id = np->id;
		}
		putchar('\n');
		if (!noeffect && updatejournal_item(dbp, jip) < 0)
			berror("jfix", "cannot update journal item");
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage:\tjfix [-c comp][-nv][-a amount][-d descr][-N nomacct] j.id\n");
	exit(2);
}
