#ident "Kalopa: $Id: payment.c,v 1.18 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: payment.c,v 1.18 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: payment.c,v $
 * Revision 1.18  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.17  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.16  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.15  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.14  2006/10/01 10:18:21  dtynan
 * Major changes for VAT.
 *
 * Revision 1.13  2006/08/25 11:27:13  dtynan
 * Lots of changes as per the end-of-year.
 *
 * Revision 1.12  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.11  2005/12/18 16:54:24  dtynan
 * Fixed some issues with the batch-payment.
 *
 * Revision 1.10  2005/12/18 16:00:55  dtynan
 * Added batch-payment code.
 *
 * Revision 1.9  2005/10/17 13:29:58  dtynan
 * Changed to use 'bal()' function for consistent output.
 *
 * Revision 1.8  2005/08/12 18:51:58  dtynan
 * Attempted fix at unusual bug where payment doesn't appear in the
 * sales ledger with todays date.
 *
 * Revision 1.7  2005/08/01 10:52:12  dtynan
 * Update the sales ledger with the payment information.
 *
 * Revision 1.6  2005/04/04 14:06:14  dtynan
 * Added a printf to show what amount hasn't been allocated (if any).
 *
 * Revision 1.5  2005/03/22 09:41:14  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.4  2005/03/09 11:22:11  dtynan
 * Removed custcode from the customer table.  Now replaced with ccode
 * from the contact table.
 *
 * Revision 1.3  2005/02/17 16:23:23  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.2  2005/01/13 20:44:09  dtynan
 * Added a new function to post end of year totals.
 *
 * Revision 1.1  2005/01/13 17:44:27  dtynan
 * Added new program to post a customer payment to the database.  Also did
 * cleanups of functions across the board.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

void	payment(char *, char *, char *, time_t);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, forex;
	char *cname, *cp, *ccode, *folio, input[256], *batchf;
	double localamt;
	dbow_conn *dbp;
	time_t when;
	FILE *fp;


	cname = NULL;
	time(&when);
	opterr = forex = 0;
	batchf = NULL;
	while ((i = getopt(argc, argv, "c:d:f:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				usage();
			break;

		case 'f':
			batchf = optarg;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Make sure we have the right number of arguments.  Then
	 * connect to the appropriate company database.
	 */
	if (batchf == NULL) {
		if ((argc - optind) == 4) {
			localamt = atof(argv[optind + 3]);
			forex = 1;
		} else if ((argc - optind) != 3)
			usage();
	} else {
		if ((argc - optind) != 1)
			usage();
	}
	loadconfig(cname);
	dbp = getdbase();
	/*
	 *
	 */
	printf("Date:\t\t%s\n", prtime(&when));
	if (forex)
		printf("Local currency:\t%.2f\n", localamt);
	if (batchf == NULL) {
		ccode = argv[optind++];
		folio = argv[optind++];
		payment(ccode, folio, argv[optind], when);
	} else {
		folio = argv[optind];
		if ((fp = fopen(batchf, "r")) == NULL) {
			fprintf(stderr, "payment: ");
			perror(batchf);
			exit(1);
		}
		while (fgets(input, sizeof(input), fp) != NULL) {
			cp = input;
			while (isspace(*cp))
				cp++;
			if (*cp == '#')
				continue;
			ccode = cp++;
			while (*cp != '\0' && !isspace(*cp))
				cp++;
			*cp++ = '\0';
			while (isspace(*cp))
				cp++;
			folio = cp++;
			while (*cp != '\0' && !isspace(*cp))
				cp++;
			*cp++ = '\0';
			while (isspace(*cp))
				cp++;
			payment(ccode, folio, cp, when);
		}
		fclose(fp);
	}
	exit(0);
}
\
/*
 *
 */
void
payment(char *ccode, char *folio, char *aval, time_t when)
{
	char invstring[512];
	struct db_customer *cusp;
	struct db_contact *cp;
	struct db_invoice *invp;
	struct db_sledger *slp;
	double amount;
	dbow_conn *dbp = getdbase();

	if ((amount = atof(aval)) < 0.01 || amount > 1000000.0) {
		fprintf(stderr,
			"?Error: invalid payment amount %.2f for customer %s\n",
			amount, ccode);
		exit(2);
	}
	if ((cp = contact_findcode(dbp, ccode)) == NULL)
		berror("payment", "invalid contact code");
	if ((cusp = db_findcustomerbycontact_id(dbp, cp->id)) == NULL)
		berror("payment", "can't find customer details");
	if (isatty(fileno(stdin))) {
		printf("Customer:\t%s\n", orgname(cp));
		printf("Balance:\t%s\n", bal(cusp->balance, NULL));
		printf("Folio:\t\t%s\n", folio);
		printf("Amount:\t\t%.2f\n", amount);
	}
	if (cusp->balance >= 0.0)
		printf("\n***** This customer does not have an outstanding balance *****\n");
	if (!yesorno("Is this correct"))
		exit(0);
	/*
	 * Now do the journal posting.
	 */
	sprintf(invstring, "Payment from %s", cp->ccode);
	journal_entry(&when, folio, invstring, -1, NA_MAINBANK, NA_DEBTORSCA,
								amount);
	/*
	 * Adjust the outstanding amount from the customer.
	 */
	cusp->balance += amount;
	if (db_updatecustomerbyid(dbp, cusp, cusp->id))
		berror("payment", "cannot update customer record");
	/*
	 * Update the sales ledger.
	 */
	if ((slp = db_sledgeralloc()) != NULL) {
		slp->customer_id = cusp->id;
		slp->sldate = when;
		slp->folio = strdup(folio);
		slp->descr = strdup("Payment on Account");
		slp->amount = amount;
		db_insertsledger(dbp, slp);
		db_sledgerfree(slp);
	}
	/*
	 * Now allocate the payment against each invoice.
	 */
	invp = db_findinvoicebycustomer_id(dbp, cusp->id);
	for (; invp != NULL; invp = db_findinvoicenext(dbp, invp)) {
		if (invp->state < INVOICE_STATE_SENT)
			continue;
		printf("Invoice #%d, outstanding balance: %.2f\n", invp->id, invp->total);
		if (amount < invp->total)
			continue;
		if (!yesorno("Apply amount to this invoice"))
			continue;
		amount -= invp->total;
		invp->state = INVOICE_STATE_PAID;
		if (db_updateinvoicebyid(dbp, invp, invp->id))
			berror("payment", "cannot update invoice record");
	}
	printf("Remaining (unallocated) balance: %.2f\n", amount);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage:\tpayment [-c <comp>][-d date] ccode folio amount [local_amount]\n");
	fprintf(stderr, "\tpayment [-c <comp>][-d date] -f batchfile folio\n");
	exit(2);
}
