#ident "Kalopa: $Id: runcom.c,v 1.6 2008/10/20 17:53:25 dtynan Exp $"

/*
 * $Id: runcom.c,v 1.6 2008/10/20 17:53:25 dtynan Exp $
 *
 * Copyright (c) 2006, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: runcom.c,v $
 * Revision 1.6  2008/10/20 17:53:25  dtynan
 * Added a document for installation (quite basic at this point), removed
 * some debug output from tpadjust(), and fixed a lingering issue with the
 * way invoice due-dates were calculated.
 *
 * Revision 1.5  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.4  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.3  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.2  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.1  2006/10/01 10:18:21  dtynan
 * Major changes for VAT.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

#define NULLDEV		"/dev/null"

int	verbose;
int	lineno;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, delete;
	char *cname, *cp, *xp;
	struct db_runcom *rp;
	struct tm *tp;
	time_t now;
	dbow_conn *dbp;

	/*
	 * Process command-line options.
	 */
	now = opterr = verbose = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:vd:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		case 'd':
			if (parsedate(optarg, &now) < 0) {
				fprintf(stderr, "?Error - invalid date.\n");
				exit(2);
			}
			break;

		default:
			usage();
			break;
		}
	}
	if (now == 0)
		usage();
	printf("RUNCOM When: %s", ctime(&now));
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Run through the list of runcom tasks for those whose
	 * trigger date is in the "past".  We don't compare them
	 * to todays date, but instead compare them against the
	 * specified date on the command line.
	 */
	for (rp = findruncom(dbp, now); rp != NULL;
					rp = db_findruncomnext(dbp, rp)) {
		if (verbose) {
			printf("RCEntry: %d\n", rp->id);
			printf("RType:   %d\n", rp->rtype);
			printf("Ndate:   %s", ctime((time_t *)&rp->ndate));
			printf("Edate:   %s", ctime((time_t *)&rp->edate));
			printf("Command: [%s]\n", rp->command);
			printf("State:   %d\n", rp->state);
		}
		/*
		 * Run the specified command.
		 */
		cp = makepath("bin", rp->command, NULL);
		if ((xp = malloc(strlen(cp) + sizeof(NULLDEV) + 4)) == NULL) {
			bwarn("runcom", "cannot malloc memory");
			continue;
		}
		sprintf(xp, "%s < %s", cp, NULLDEV);
		if (verbose) {
			printf("Running [%s]...\n", xp);
			printf("----- START OF COMMAND OUTPUT -----\n");
		}
		fflush(stdout);
		if (system(xp) != 0)
			bwarn("runcom", "command returned an error");
		free(cp);
		free(xp);
		if (verbose)
			printf("----- END OF COMMAND OUTPUT -----\n");
		/*
		 * Now, advance this task into the future for the next run.
		 */
		if ((tp = localtime((time_t *)&rp->ndate)) == NULL)
			continue;
		delete = 0;
		switch (rp->rtype) {
		default:
			/*
			 * Only run the entry once...
			 */
			delete = 1;
			break;

		case FREQ_DAILY:
			/*
			 * Run the entry every day...
			 */
			while (rp->ndate <= now) {
				tp->tm_mday++;
				tpadjust(tp);
				rp->ndate = mktime(tp);
			}
			break;

		case FREQ_WEEKLY:
			/*
			 * Run the entry every week...
			 */
			tp->tm_mday += 7;
			break;

		case FREQ_WEEKLY2:
			/*
			 * Run the entry every two weeks...
			 */
			tp->tm_mday += 14;
			break;

		case FREQ_WEEKLY4:
			/*
			 * Run the entry every four weeks...
			 */
			tp->tm_mday += 28;
			break;

		case FREQ_MONTHLY:
			/*
			 * Run the entry every month...
			 */
			tp->tm_mon++;
			break;

		case FREQ_MONTHLY2:
			/*
			 * Run the entry every two months...
			 */
			tp->tm_mon += 2;
			break;

		case FREQ_QUARTERLY:
			/*
			 * Run the entry every quarter...
			 */
			tp->tm_mon += 3;
			break;

		case FREQ_BIANNUAL:
			/*
			 * Run the entry twice a year...
			 */
			tp->tm_mon += 6;
			break;

		case FREQ_ANNUALLY:
			/*
			 * Run the entry every year...
			 */
			tp->tm_year++;
			break;
		}
		if (delete || (rp->edate != 0 && rp->edate < rp->ndate)) {
			if (verbose)
				printf("Deleting entry #%d.\n", rp->id);
			db_deleteruncombyid(dbp, rp->id);
		} else {
			rp->ndate = mktime(tp);
			if (verbose)
				printf("Next run-date: %s", ctime((time_t *)&rp->ndate));
			db_updateruncombyid(dbp, rp, rp->id);
		}
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: runcom [-v][-c #] -d <date>\n");
	exit(2);
}
