#ident "$Id: vatreturn.c,v 1.8 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: vatreturn.c,v 1.8 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2006, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: vatreturn.c,v $
 * Revision 1.8  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.7  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.6  2008/07/08 16:40:10  dtynan
 * Excessive changes.
 *
 * Revision 1.5  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2008/01/15 15:34:02  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.3  2008/01/15 14:08:53  dtynan
 * Added support for VAT periods other than bimonthly.
 *
 * Revision 1.2  2007/08/06 10:32:45  dtynan
 * Added some new programs for manipulating journal entries and modified
 * the VAT return code.
 *
 * Revision 1.1  2006/10/01 10:18:21  dtynan
 * Major changes for VAT.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

#define ROUNDOFF_DESCR	"VAT Roundoff Error"
#define VATPAYMENT	"VAT Payment"
#define VATREFUND	"VAT Refund"

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, period, noeffect, verbose;
	int isales, ipurchs, igoodsto, igoodsfrom;
	double sales, purchs, goodsto, goodsfrom;
	char *cname;
	struct db_vatret *vp;
	struct db_vatdata *vdp;
	struct tm *tmp;
	time_t when;
	dbow_conn *dbp;


	cname = NULL;
	time(&when);
	opterr = noeffect = verbose = period = 0;
	while ((i = getopt(argc, argv, "c:d:qmynv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				usage();
			break;

		case 'q':
			/*
			 * Returns every quarter.
			 */
			period = 2;
			break;

		case 'm':
			/*
			 * Returns every six months.
			 */
			period = 3;
			break;

		case 'y':
			/*
			 * Annual returns.
			 */
			period = 4;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (period == 0)
		period = 1;
	/*
	 * Connect to the appropriate company database.
	 */
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Find the appropriate VAT return or create a new one.
	 */
	if (verbose)
		printf("Date:\t\t%s\n", prtime(&when));
	if ((vp = findvatreturn(dbp, when)) == NULL) {
		if (verbose)
			printf("No eligible VAT return - creating...\n");
		/*
		 * Create an empty VAT return for the specified period.
		 */
		if ((vp = db_vatretalloc()) == NULL)
			berror("vatreturn", "cannot allocate return");
		tmp = localtime(&when);
		tmp->tm_hour = 12;
		tmp->tm_min = tmp->tm_sec = 0;
		switch (period) {
		case 1:
			tmp->tm_mon &= ~01;
			tmp->tm_mday = 1;
			vp->sdate = mktime(tmp);
			tmp->tm_mon++;
			break;

		case 2:
			tmp->tm_mon  = (tmp->tm_mon / 3) * 3;
			tmp->tm_mday = 1;
			vp->sdate = mktime(tmp);
			tmp->tm_mon += 2;
			break;

		case 3:
			tmp->tm_mon  = (tmp->tm_mon > 5) ? 6 : 0;
			tmp->tm_mday = 1;
			vp->sdate = mktime(tmp);
			tmp->tm_mon += 5;
			break;

		case 4:
			tmp->tm_mon  = 0;
			tmp->tm_mday = 1;
			vp->sdate = mktime(tmp);
			tmp->tm_mon = 11;
			tmp->tm_year++;
			break;
		}
		tmp->tm_mday = mdays(tmp->tm_mon + 1, tmp->tm_year + 1900);
		vp->edate = mktime(tmp);
		vp->vtype = VAT3_TYPE_ORIGINAL;
		vp->sales = vp->purchs = vp->goodsto = vp->goodsfrom = 0;
		if (!noeffect && db_insertvatret(dbp, vp) < 0)
			berror("vatreturn", "cannot insert vat return");
	}
	if (verbose) {
		printf("VAT Return ID:\t%d\n", vp->id);
		printf("VAT Start Date:\t%s", ctime(&vp->sdate));
		printf("VAT End Date:\t%s", ctime(&vp->edate));
	}
	/*
	 * Find all the VAT records relating to this VAT period.
	 */
	sales = purchs = goodsto = goodsfrom = 0.0;
	for (vdp = findvatdata(dbp, vp->edate, vp->id); vdp != NULL;
					vdp = db_findvatdatanext(dbp, vdp)) {
		if (vdp->vdtype < VATDATA_SALES)
			continue;
		if (verbose) {
			printf(">> ID#%d, J.ID:%d, VRet #%d, Type%d, Amt: %10.2f\n",
							vdp->id,
							vdp->journal_id,
							vdp->vatret_id,
							vdp->vdtype,
							vdp->amount);
		}
		switch (vdp->vdtype) {
		case VATDATA_SALES:
			sales += vdp->amount;
			break;

		case VATDATA_PURCHS:
			purchs += vdp->amount;
			break;

		case VATDATA_GOODSTO:
			break;

		case VATDATA_GOODSFROM:
			break;
		}
		vdp->vatret_id = vp->id;
		if (!noeffect && db_updatevatdatabyid(dbp, vdp, vdp->id) < 0)
			berror("vatreturn", "cannot update vat data");
	}
	isales     = (int )roundoff(sales, ROUNDOFF_INTEGER);
	ipurchs    = (int )roundoff(purchs, ROUNDOFF_INTEGER);
	igoodsto   = (int )roundoff(goodsto, ROUNDOFF_INTEGER);
	igoodsfrom = (int )roundoff(goodsfrom, ROUNDOFF_INTEGER);
	if (verbose) {
		printf("Sales:\t\t%6d (%d)\n", isales, vp->sales);
		printf("Purchases:\t%6d (%d)\n", ipurchs, vp->purchs);
		printf("Goods From EU:\t%6d (%d)\n", igoodsto, vp->goodsto);
		printf("Goods To EU:\t%6d (%d)\n", igoodsfrom, vp->goodsfrom);
		if (sales < purchs)
			printf("Refund:\t\t%6d\n", ipurchs - isales);
		else
			printf("Amount Due:\t%6d\n", isales - ipurchs);
	}
	if (verbose)
		printf("Posting entries...\n");
	vp->sales     = isales;
	vp->purchs    = ipurchs;
	vp->goodsto   = igoodsto;
	vp->goodsfrom = igoodsfrom;
	if (!noeffect && db_updatevatretbyid(dbp, vp, vp->id) < 0)
		berror("vatreturn", "cannot update VAT return");
	isales -= ipurchs;
	sales = (double )isales + purchs - sales;
	if (verbose)
		printf("Roundoff:\t%10.2f\n", sales);
	if (!noeffect) {
		journal_entry((time_t *)&vp->edate, NULL, ROUNDOFF_DESCR, -1,
					NA_ROUNDOFF, NA_VATACCOUNT, sales);
		if (isales < 0.0) {
			journal_entry((time_t *)&vp->edate, NULL, VATREFUND, -1,
					NA_ROSRDI, NA_VATACCOUNT, -isales);
		} else {
			journal_entry((time_t *)&vp->edate, NULL, VATPAYMENT,-1,
					NA_VATACCOUNT, NA_ROSRDI, isales);
		}
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage:\tvatreturn [-qmynv][-c <comp>][-d date]\n");
	exit(2);
}
