#ident "$Id: vatrtd.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: vatrtd.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: vatrtd.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:40:19  dtynan
 * Excessive changes.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

struct	vrs	{
	struct	vrs	*next;
	int		vat_id;
	int		vdtype;
	double		total;
};

char	*vat_types[] = {
	"No Posting", "None", "Sales", "Purchases", "Goods To", "Goods From"
};

int		noeffect;
int		verbose;
struct vatamt	*vap;
static struct vrs *head = NULL;

void	dovatdata(struct db_vatdata *);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, year;
	char *cname;
	struct db_vatret *vp;
	struct db_vatdata *vdp;
	struct tm *tmp;
	struct vrs *vrsp;
	time_t when;
	dbow_conn *dbp;

	cname = NULL;
	time(&when);
	tmp = localtime(&when);
	year = tmp->tm_year;
	opterr = noeffect = verbose = 0;
	while ((i = getopt(argc, argv, "c:y:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'y':
			if ((year = atoi(optarg)) < 1900 || year > 2199)
				usage();
			year -= 1900;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Connect to the appropriate company database.
	 */
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Scan through all the VAT returns for the given year...
	 */
	vap = vat_init(dbp);
	for (vp = db_findvatretfirst(dbp); vp != NULL;
					vp = db_findvatretnext(dbp, vp)) {
		tmp = localtime((time_t *)&vp->sdate);
		if (tmp->tm_year != year)
			continue;
		printf("\n>> SDate:\t%s", ctime((time_t *)&vp->sdate));
		printf("EDate:\t\t%s", ctime((time_t *)&vp->edate));
		printf("VType:\t\t%d\n", vp->vtype);
		printf("Sales:\t\t%d\n", vp->sales);
		printf("Purchases:\t%d\n", vp->purchs);
		printf("Goods To:\t%d\n", vp->goodsto);
		printf("Goods From:\t%d\n", vp->goodsfrom);
		for (vdp = db_findvatdatabyvatret_id(dbp, vp->id);
					vdp != NULL;
					vdp = db_findvatdatanext(dbp, vdp))
			dovatdata(vdp);
	}
	for (vrsp = head; vrsp != NULL; vrsp = vrsp->next) {
		printf("VATID:\t%d\n", vrsp->vat_id);
		printf("Type:\t%s\n", vat_types[vrsp->vdtype]);
		printf("Total:\t%.2f\n", vrsp->total);
	}
	exit(0);
}

/*
 *
 */
void
dovatdata(struct db_vatdata *vdp)
{
	struct vrs *vrsp;

	printf(">>>> VD.ID:\t%d\n", vdp->id);
	printf("Return ID:\t%d\n", vdp->vatret_id);
	printf("Journal ID:\t%d\n", vdp->journal_id);
	printf("VD Type:\t%s\n", vat_types[vdp->vdtype]);
	printf("VAT ID:\t\t%d\n", vdp->vat_id);
	printf("Amount:\t\t%.2f\n", vdp->amount);
	for (vrsp = head; vrsp != NULL; vrsp = vrsp->next)
		if (vrsp->vat_id == vdp->vat_id && vrsp->vdtype == vdp->vdtype)
			break;
	if (vrsp == NULL) {
		vrsp = (struct vrs *)malloc(sizeof(*vrsp));
		if (vrsp == NULL)
			berror("vatrtd", "malloc failure");
		vrsp->vat_id = vdp->vat_id;
		vrsp->vdtype = vdp->vdtype;
		vrsp->total = 0.0;
		vrsp->next = head;
		head = vrsp;
	}
	vrsp->total += vdp->total;
	printf("VT:\t\t%.2f\n", vrsp->total);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage:\tvatrtd [-nv][-c <comp>][-y year]\n");
	exit(2);
}
