#ident "Kalopa: $Id: beanie.c,v 1.13 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: beanie.c,v 1.13 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: beanie.c,v $
 * Revision 1.13  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.12  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.11  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.10  2005/08/12 20:11:11  dtynan
 * Fixed a password bug.
 *
 * Revision 1.9  2005/03/22 09:41:16  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.8  2005/02/16 11:04:54  dtynan
 * Changed default database to always point to main system.
 *
 * Revision 1.7  2004/09/03 13:46:07  dtynan
 * Added syslog statements to dbinit() and changed the way the DB
 * password is handled.
 *
 * Revision 1.6  2004/08/26 17:27:47  dtynan
 * Added error output to dbinit().
 *
 * Revision 1.5  2004/08/05 11:39:41  dtynan
 * Added missing prototypes and includes
 *
 * Revision 1.4  2004/08/04 13:15:53  dtynan
 * Changed handling of database opening (now need to specify main or
 * sub database).
 *
 * Revision 1.3  2004/05/12 15:21:12  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.2  2003/11/08 15:35:07  dtynan
 * Moved logging to LOCAL1 from LOCAL0
 *
 * Revision 1.1  2003/11/08 14:53:47  dtynan
 * Added functions to getyend.c to determine quarter-ends and where we are
 * in the quarter.  Also added new functions for date manipulation, roff
 * processing and initialization.
 */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>
#include <string.h>

#include "beanie.h"

/*
 * New method for determining database.  The database name and parameters
 * are gleaned from the configuration system.  Note that we set the
 * 'norecurse' variable to prevent the 'getconfig()' function actually
 * calling us recursively which would not be good at this point.
 */
dbow_conn *
getdbase()
{
	static int norecurse = 0;
	static dbow_conn *dbp = NULL;
	char *dbuser, *dbpass, *host, *dbase;

	/*
	 * If we've already created the database connection, just use that.
	 */
	if (dbp != NULL)
		return(dbp);
	if (norecurse)
		return(NULL);
	norecurse++;
	/*
	 * Open a connection to syslog.
	 */
	openlog("beanie", LOG_PID, LOG_LOCAL1);
	/*
	 * Set the parameters, based on the configuration file...
	 * If $BEANIEHOST is specified, connect to that database.
	 * Otherwise, connect to the official one.
	 */
	dbuser = getconfig("dbuser", "beanie");
	dbpass = getconfig("dbpass", NULL);
	dbase = getconfig("dbase", NULL);
	if ((host = getenv("BEANIEHOST")) != NULL)
		host = strdup(host);
	else
		host = getconfig("dbhost", "localhost");
	/*
	 * Now, do the actual database connection.
	 */
	dbp = dbow_init(host, dbuser, dbpass, dbase);
	if (dbuser != NULL)
		free(dbuser);
	if (dbpass != NULL)
		free(dbpass);
	if (dbase != NULL)
		free(dbase);
	if (host != NULL)
		free(host);
	norecurse--;
	/*
	 * Did it work?
	 */
	if (dbp == NULL) {
		syslog(LOG_ERR, "init: cannot connect to database");
		fprintf(stderr, "init: cannot connect to db %s as %s@%s\n",
				dbase, dbuser, host);
		exit(1);
	}
	return(dbp);
}
