/*
 * $Id: beanie.h,v 1.57 2008/11/12 10:16:33 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: beanie.h,v $
 * Revision 1.57  2008/11/12 10:16:33  dtynan
 * Deprecated the config.h file and some minor tweaks prior to release 0.8.2
 *
 * Revision 1.56  2008/10/31 15:17:13  dtynan
 * Extensive changes for version 0.8.2.
 *
 * Revision 1.55  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.54  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.53  2008/07/18 11:07:37  dtynan
 * Fixed issue with isholiday/skipholiday where it wouldn't actually detect
 * bank holidays (only weekends).  It now interrogates the database.
 *
 * Revision 1.52  2008/07/08 17:08:53  dtynan
 * Extensive changes.
 *
 * Revision 1.51  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.50  2008/01/15 14:05:04  dtynan
 * Added payroll hooks.
 *
 * Revision 1.49  2007/10/29 17:14:31  dtynan
 * Odds and ends.
 *
 * Revision 1.48  2007/08/07 08:17:27  dtynan
 * Minor tweaks.
 *
 * Revision 1.47  2007/08/06 10:54:05  dtynan
 * Added new prototypes.
 *
 * Revision 1.46  2006/10/01 10:22:35  dtynan
 * Major changes for VAT.
 *
 * Revision 1.45  2006/07/31 08:49:49  dtynan
 * Fixed a bug in the round-up function.
 *
 * Revision 1.44  2005/12/19 22:18:39  dtynan
 * Extensive changes to provide a solid VAT framework.
 *
 * Revision 1.43  2005/12/18 16:10:52  dtynan
 * Minor fixes.
 *
 * Revision 1.42  2005/10/17 15:38:35  dtynan
 * Added new round-off types and fixed some minor gotchas.
 *
 * Revision 1.41  2005/10/17 15:06:42  dtynan
 * Created a new roundoff routine specifically for beanie.  It should
 * handle all the usual roundoff issues.
 *
 * Revision 1.40  2005/10/17 13:33:55  dtynan
 * New function to convert from a foreign currency into the home
 * currency.
 *
 * Revision 1.39  2005/09/21 18:10:28  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.38  2005/08/12 18:52:56  dtynan
 * Changed invoicemail prototype and added some stuff to the hacks directory.
 *
 * Revision 1.37  2005/08/01 10:37:49  dtynan
 * Added new functions for addresses and for printing numbers in a useful
 * way.  Also removed kalmap DBOW files from include file.
 *
 * Revision 1.36  2005/04/11 17:47:39  dtynan
 * Fixed a minor bug in the row printing code.
 *
 * Revision 1.35  2005/04/11 16:06:49  dtynan
 * New function for printing address blocks (for troff).
 *
 * Revision 1.34  2005/04/11 13:52:41  dtynan
 * Changed the nominal account code to only load the list of nominal
 * accounts once.
 *
 * Revision 1.33  2005/03/31 10:07:17  dtynan
 * Added a new 'manual' designation to periodic frequencies.  Not quite
 * the same as a "run-once" but close.  Also added some missing includes.
 *
 * Revision 1.32  2005/03/22 09:41:16  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.31  2005/03/11 14:20:13  dtynan
 * Added new definitions for event frequency (need to change the RJ
 * system too).
 *
 * Revision 1.30  2005/03/08 16:37:18  dtynan
 * Added a new function to return the VAT character code.
 *
 * Revision 1.29  2005/03/05 19:26:11  dtynan
 * Minor tweaks.
 *
 * Revision 1.28  2005/03/03 12:05:25  dtynan
 * Added IBAN details to invoices.
 *
 * Revision 1.27  2005/02/20 20:04:24  dtynan
 * Changed email string (dropped 'Media').
 *
 * Revision 1.26  2005/02/17 16:23:24  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.25  2005/02/17 11:04:24  dtynan
 * Slight reorganisation.
 *
 * Revision 1.24  2005/02/16 11:04:54  dtynan
 * Changed default database to always point to main system.
 *
 * Revision 1.23  2005/01/13 17:44:28  dtynan
 * Added new program to post a customer payment to the database.  Also
 * did cleanups of functions across the board.
 *
 * Revision 1.22  2005/01/13 17:12:51  dtynan
 * Added new function to display a balance (uses parens for negative).
 *
 * Revision 1.21  2004/12/09 20:20:30  dtynan
 * Added new include files.
 *
 * Revision 1.20  2004/12/09 20:11:08  dtynan
 * Created new function to return the organisation name in a predictable way.
 *
 * Revision 1.19  2004/12/09 19:19:13  dtynan
 * Added extra functions to produce date strings.
 *
 * Revision 1.18  2004/12/08 17:42:29  dtynan
 * Added functions to manage and manipulate journal entry balances.
 *
 * Revision 1.17  2004/08/26 17:29:14  dtynan
 * Two new prototypes.
 *
 * Revision 1.16  2004/08/05 12:45:46  dtynan
 * Removed company ID from journal.
 *
 * Revision 1.15  2004/08/05 11:39:41  dtynan
 * Added missing prototypes and includes
 *
 * Revision 1.14  2004/08/04 13:15:53  dtynan
 * Changed handling of database opening (now need to specify main or
 * sub database).
 *
 * Revision 1.13  2004/08/04 11:01:51  dtynan
 * Various changes for new & improved system.  Made the invoice header
 * company-specific, modified the primary key of the journal and removed
 * extraneous income types.
 *
 * Revision 1.12  2004/07/16 19:24:35  dtynan
 * Changed path system to use BEANIEROOT environment variable instead of
 * hard-coded paths and added a new function to build the appropriate path.
 *
 * Revision 1.11  2004/07/14 14:47:34  dtynan
 * Fixed various bugs and minor issues with building.
 *
 * Revision 1.10  2004/07/12 19:07:14  dtynan
 * Extensive changes for new directory structure and other things.
 *
 * Revision 1.9  2004/05/12 15:21:12  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.8  2004/01/05 18:38:05  dtynan
 * Added functions for including .sig files and regular files in an email.
 *
 * Revision 1.7  2004/01/05 18:07:27  dtynan
 * Various tweaks to the email back-end, as well as new functions for
 * dealing with errors in a common, predicable way and for determining
 * whether or not the specified date is a business day or not.
 *
 * Revision 1.6  2004/01/05 11:53:45  dtynan
 * Various changes around the filesystem layout and other minor details.
 *
 * Revision 1.5  2003/12/09 12:30:15  dtynan
 * Moved location of invoice template and added nominal accounts data.
 *
 * Revision 1.4  2003/11/08 16:18:21  dtynan
 * Added definition for old accounting system.
 *
 * Revision 1.3  2003/11/08 15:24:28  dtynan
 * Added a new hack include file to overcome limitations in DBOW and added
 * extra stuff to beanie.h
 *
 * Revision 1.2  2003/11/07 16:04:41  dtynan
 * Added prototypes for email functions.
 *
 * Revision 1.1  2003/10/31 17:18:06  dtynan
 * Various changes and fixes for new/improved accounting system.
 */

#include <dbow.h>

#include "company.h"
#include "config.h"
#include "sharecert.h"
#include "nomacct.h"
#include "currency.h"
#include "asset.h"
#include "journal.h"
#include "runcom.h"
#include "bankacct.h"
#include "bankdata.h"
#include "chqreg.h"
#include "crcard.h"
#include "country.h"
#include "employee.h"
#include "payroll.h"
#include "hours.h"
#include "customer.h"
#include "contact.h"
#include "product.h"
#include "salesorder.h"
#include "invoice.h"
#include "sledger.h"
#include "vat.h"
#include "vatret.h"
#include "holiday.h"
#include "terms.h"

#include "iepaye.h"

#define MAXLINELEN	512

/*
 * Some handy time macros
 */
#define SECONDS(n)	(n)
#define MINUTES(n)	(SECONDS(n)*60)
#define HOURS(n)	(MINUTES(n)*60)
#define DAYS(n)		(HOURS(n)*24)

#define FREQ_ONCE		0		/* Once only */
#define FREQ_HOURLY		1		/* Once an hour */
#define FREQ_DAILY		2		/* Once a day */
#define FREQ_WEEKLY		3		/* Once a week */
#define FREQ_WEEKLY2		4		/* Every two weeks */
#define FREQ_WEEKLY4		5		/* Every four weeks */
#define FREQ_MONTHLY		6		/* Once a month */
#define FREQ_MONTHLY2		7		/* Once every two months */
#define FREQ_QUARTERLY		8		/* Once a quarter */
#define FREQ_BIANNUAL		9		/* Twice a year */
#define FREQ_ANNUALLY		10		/* Once a year */
#define FREQ_MANUALLY		11		/* Run manually */

#define PAPERSIZE_LETTER	0		/* US Letter */
#define PAPERSIZE_A4		1		/* A4 Paper */

/*
 * Debit/Credit definitions
 */
#define DEBIT			0
#define CREDIT			1

/*
 * Different roundoff types.
 */
#define ROUNDOFF_NORMAL		0
#define ROUNDOFF_ROUNDDOWN	1
#define ROUNDOFF_ROUNDUP	2
#define ROUNDOFF_UKVAT		3
#define ROUNDOFF_INTEGER	4

/*
 * Nominal Ledger types
 */
#define INCOME			0
#define EXPENSE			4
#define ASSET			8
#define ASSET_FIXED		8
#define ASSET_CURRENT		10
#define LIABILITY		12
#define LIABILITY_CURRENT	12
#define LIABILITY_LONG		14
#define OEQUITY			16
#define COSTOFSALES		20

struct	nabal	{
	struct	nabal	*next;
	int		nomacct_id;
	int		atype;
	char		*name;
	double		bal;
};

/*
 * Structure for calculating VAT totals (only used in environments
 * where VAT is an issue).
 */
struct	vatamt	{
	struct	vatamt	*next;
	short		vat_id;
	int		vcode;
	double		total;
	double		rate;
	double		amount;
	int		journal_id;
	int		vdtype;
};

/*
 * Prototypes...
 */
dbow_conn	*getdbase();
int		loadconfig(char *);
char		*getconfig(char *, char *);
char		*makepath(char *, ...);
char		*makepdf(char *);

struct	nabal	*loadnabal(dbow_conn *);
struct	nabal	*findnabal(struct nabal *, int);
void		loadbalances(dbow_conn *, struct nabal *, int);

void		invoicemail(struct db_company *, struct db_customer *, struct db_invoice *, char *);

int	mdays(int, int);
void	tpadjust(struct tm *);
char	*prdate(int, int, int);
char	*prtmptr(struct tm *);
char	*prabbrev(time_t *);
char	*prtime(time_t *);
int	parsedate(char *, time_t *);
char	*getanswer(char *, char *);
int	yesorno(char *);
int	getyend();
void	getqends();
int	whichquarter(int, int[]);
int	isholiday(time_t);
time_t	skipholiday(time_t);
void	gentroffaddr(FILE *, dbow_conn *, int, int);
void	gentroffaddrline(FILE *, dbow_conn *, int);
int	email_open(char *, char *, char *);
int	email_close();
void	email_setarg(int, char *);
int	email_text(char *);
int	email_include(char *);
int	email_signature(char *);
int	email_file(char *, char *);
int	email_fptr(FILE *, char *, char *);
void	base64encode(FILE *, FILE *);
char	*roffize(char *);
char	*bal(double, char *);
double	roundoff(double, int);
int	journal_entry(time_t *, char *, char *, int, int, int, double);
int	invoice_total(struct db_invoice *, struct db_invoice_item *, double *, double *);
double	currconv(double, int);
void	calcdate(int *, int *, int *, char *);
struct	vatamt *vat_init();
void	vat_clear(struct vatamt *);
void	vat_lineitem(struct vatamt *, double, int, int, int);
double	vat_total(struct vatamt *);
int	vatcode(int);
int	vatid(int);
double	vat_findnet(double, int);
void	berror(char *, char *);
void	bwarn(char *, char *);
char	*orgname(struct db_contact *);
char	*localphone(char *);
int	makenumberdigit(char *, int, int);
int	makenumbertext(char *, int, int);
char	*invoice_number(int);
char	*crnote_number(int);
char	*makenotice(struct db_company *);
char	*makefaxemail(char *);
int	papersize();
void	dunning(struct db_invoice *);
