#ident "Kalopa: $Id: cdate.c,v 1.6 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: cdate.c,v 1.6 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 1996, Claddagh Films Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: cdate.c,v $
 * Revision 1.6  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.5  2005/03/22 09:41:16  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.4  2004/05/12 15:21:13  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.3  2003/11/08 16:10:13  dtynan
 * Added old accounting system functions.
 *
 * Revision 1.2  1997/01/05 03:31:18  dtynan
 * Baseline check-in - most functionality is now present.
 *
 * Revision 1.1  1997/01/03  21:37:25  dtynan
 * Initial check-in.
 */
#include <stdio.h>
#include <time.h>

#include "beanie.h"

/*
 *
 */
void
calcdate(int *dp, int *mp, int *yp, char *off)
{
	int i, val;

	/*
	 * Process the offset.
	 */
	for (val = 0; (i = *off) != '\0'; off++) {
		switch (i) {
		case '0': case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8': case '9':
			/*
			 * Process a numeric value.
			 */
			val = val * 10 + i - '0';
			break;

		case 'W': case 'w':
			/*
			 * Weeks are in multiples of 7 days.
			 */
			val *= 7;

		case 'D': case 'd':
			/*
			 * Add the day count and handle month wraparounds.
			 */
			*dp += val;
			while (*dp > mdays(*mp, *yp)) {
				*dp -= mdays(*mp, *yp);
				if (*mp++ > 12) {
					*mp = 1;
					(*yp)++;
				}
			}
			val = 0;
			break;

		case 'Y': case 'y':
			/*
			 * Years are in multiples of 12 months.
			 */
			val *= 12;

		case 'M': case 'm':
			/*
			 * Add the number of months and handle year
			 * wraparounds.
			 */
			*mp += val;
			while (*mp > 12) {
				*mp -= 12;
				(*yp)++;
			}
			/*
			 * Make sure we still have a valid day.
			 */
			if (*dp > mdays(*mp, *yp))
				*dp = mdays(*mp, *yp);
			val = 0;
			break;
		}
	}
	if (val != 0) {
		/*
		 * No specifier for the period - assume days.
		 */
		while (*dp > mdays(*mp, *yp)) {
			*dp -= mdays(*mp, *yp);
			if (*mp++ > 12) {
				*mp = 1;
				(*yp)++;
			}
		}
	}
}
