#ident "Kalopa: $Id: config.c,v 1.4 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: config.c,v 1.4 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: config.c,v $
 * Revision 1.4  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.3  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.2  2008/07/08 17:08:53  dtynan
 * Extensive changes.
 *
 * Revision 1.1  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <pwd.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

#define RCFILENAME	".beanierc"

struct	config	{
	struct	config	*next;
	char		*name;
	char		*value;
} *chead = NULL;

/*
 * Strip leading and trailing blanks from the argument given.
 */
static char *
_trim(char *strp)
{
	char *cp;

	if (strp == NULL)
		return(NULL);
	while (isspace(*strp))
		strp++;
	cp = strp + strlen(strp) - 1;
	while (cp > strp && isspace(*cp))
		cp--;
	*++cp = '\0';
	return(strp);
}

/*
 * Open the default RC file and parse it.  If we're given a configuration
 * name, then we'll search for that configuration.  Otherwise, we'll take
 * the first one we see.  Returns -1 if no configuration found.
 * TODO:
 * Might be nice to be able to specify aliases, like printcap does.  Along
 * the lines of "kis|2:" so that either "kis" or "2" would work?
 */
int
loadconfig(char *cfgname)
{
	int state = 0;
	char *cp, *xp, input[1024];
	struct config *cfgp, *ctail;
	struct passwd *pp;
	FILE *fp;

	/*
	 * Clear out any existing configuration entries
	 */
	for (cfgp = chead; cfgp != NULL; cfgp = chead) {
		chead = cfgp->next;
		if (cfgp->name != NULL)
			free(cfgp->name);
		if (cfgp->value != NULL)
			free(cfgp->value);
		free((char *)cfgp);
	}
	/*
	 * Default the configuration settings to be /etc in case we can't
	 * find the user or their home directory.
	 */
	strcpy(input, "/etc");
	if ((pp = getpwuid(getuid())) != NULL && pp->pw_dir != NULL)
		strncpy(input, pp->pw_dir, sizeof(input)-sizeof(RCFILENAME)-4);
	/*
	 * Build up the filename and open it.
	 */
	strcat(input, "/");
	strcat(input, RCFILENAME);
	if ((fp = fopen(input, "r")) == NULL)
		return(-1);
	/*
	 * Parse the file.
	 */
	while (fgets(input, sizeof(input), fp) != NULL) {
		/*
		 * Trim each line and remove comments.
		 */
		if ((cp = strpbrk(input, "\r\n")) != NULL)
			*cp = '\0';
		cp = _trim(input);
		if (*cp == '#' || *cp == '\0')
			continue;
		/*
		 * We need either a ':' or an '=' on a valid line.
		 */
		if ((xp = strpbrk(cp, "=:")) == NULL)
			continue;
		if (*xp == ':') {
			/*
			 * Configuration identifier.
			 */
			*xp = '\0';
			cp = _trim(cp);
			/*
			 * If we haven't seen a configuration yet (state=0)
			 * and either no specific configuration has been
			 * specified, or this actual one matches, then we're
			 * in a LOAD state (=1).  Otherwise, if we were
			 * loading, then we're done now.  Skip everything
			 * else.
			 */
			if (state == 0 && (cfgname == NULL ||
						strcmp(cfgname, cp) == 0))
				state++;
			else if (state == 1)
				state = 2;
		} else if (state == 1) {
			/*
			 * Actual configuration line, and we're loading...
			 */
			*xp++ = '\0';
			cp = _trim(cp);
			xp = _trim(xp);
			cfgp = (struct config *)malloc(sizeof(*cfgp));
			if (cfgp == NULL) {
				fclose(fp);
				return(0);
			}
			cfgp->next = NULL;
			cfgp->name = strdup(cp);
			cfgp->value = strdup(xp);
			if (chead == NULL)
				chead = cfgp;
			else
				ctail->next = cfgp;
			ctail = cfgp;
		}
	}
	/*
	 * We're done.  Close the file and return whether or not we saw
	 * a valid configuration.
	 */
	fclose(fp);
	return(state > 0 ? 0 : -1);
}

/*
 * Start by searching through the hard-coded configuration lines which we
 * loaded in the first place, and then try the database...
 */
char *
getconfig(char *name, char *def)
{
	char *value;
	struct config *cfgp;
	struct db_config *cp;
	dbow_conn *dbp;

	/*
	 * Check the in-memory configuration first...
	 */
	for (cfgp = chead; cfgp != NULL; cfgp = cfgp->next)
		if (strcmp(cfgp->name, name) == 0)
			return(strdup(cfgp->value));
	/*
	 * Now check for a database entry...
	 */
	if ((dbp = getdbase()) != NULL &&
			(cp = db_findconfigbyvname(dbp, name)) != NULL) {
		value = cp->value;
		cp->value = NULL;
		db_configfree(cp);
		return(value);
	}
	return(def == NULL ? NULL : strdup(def));
}
