#ident "Kalopa: $Id: cvat.c,v 1.12 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: cvat.c,v 1.12 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 1996, Claddagh Films Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: cvat.c,v $
 * Revision 1.12  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.11  2008/08/12 16:37:54  dtynan
 * Remember to post the VAT total to the vatdata table.
 *
 * Revision 1.10  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.9  2007/08/06 10:52:29  dtynan
 * Added a routine to calculate the net value of a transaction.
 *
 * Revision 1.8  2006/10/01 10:22:35  dtynan
 * Major changes for VAT.
 *
 * Revision 1.7  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.6  2005/12/19 22:18:39  dtynan
 * Extensive changes to provide a solid VAT framework.
 *
 * Revision 1.5  2005/03/22 09:41:16  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.4  2005/03/08 16:37:18  dtynan
 * Added a new function to return the VAT character code.
 *
 * Revision 1.3  2004/05/12 15:21:14  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.2  1997/01/03 22:20:49  dtynan
 * Fixed problem with invalid atof(3) prototype.
 *
 * Revision 1.1  1997/01/03  21:37:26  dtynan
 * Initial check-in.
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "beanie.h"

struct	db_vat	*db_findvatcodes(dbow_conn *);

/*
 *
 */
struct vatamt *
vat_init()
{
#ifdef VAT_CALCULATE
	struct vatamt *vp, *head, *tail;
	struct db_vat *vatp;
	dbow_conn *dbp = getdbase();

	for (head = NULL, vatp = db_findvatcodes(dbp); vatp != NULL;
					vatp = db_findvatnext(dbp, vatp)) {
		if ((vp = (struct vatamt *)malloc(sizeof(*vp))) == NULL)
			berror("vatinit", "out of memory");
		if (head == NULL)
			head = vp;
		else
			tail->next = vp;
		tail = vp;
		vp->next = NULL;
		vp->vat_id = vatp->id;
		vp->vcode = vatp->vcode[0];
		vp->rate = vatp->rate;
		vp->total = vp->amount = 0.0;
		vp->journal_id = 0;
		vp->vdtype = VATDATA_NOPOST;
	}
	return(head);
#else
	return(NULL);
#endif
}

/*
 *
 */
void
vat_lineitem(struct vatamt *head, double total, int vatid, int jid, int vdtype)
{
	struct vatamt *vp;

#ifdef VAT_CALCULATE
	for (vp = head; vp != NULL && vp->vat_id != vatid; vp = vp->next)
		;
	if (vp == NULL) {
		for (vp = head; vp != NULL; vp = vp->next)
			if (vp->vcode == 'Z')
				break;
		if (vp == NULL)
			return;
	}
#ifdef VAT_LINEITEM
	vp->amount += roundoff(vp->rate * total / 100.0, VAT_ROUNDING);
#endif
	vp->total += total;
	vp->journal_id = jid;
	vp->vdtype = vdtype;
#endif
}

/*
 *
 */
double
vat_total(struct vatamt *head)
{
	struct vatamt *vp;
	struct db_vatdata *vdp = NULL;
	double vatttl = 0.0;
	dbow_conn *dbp;

#ifdef VAT_CALCULATE
#ifndef VAT_LINEITEM
	for (vp = head; vp != NULL; vp = vp->next) {
		vp->amount = roundoff(vp->rate * vp->total / 100.0,
								VAT_ROUNDING);
	}
#endif
	/*
	 * Now post the VAT entries to the VAT return data table.
	 */
	for (vp = head; vp != NULL; vp = vp->next) {
		vatttl += vp->amount;
		if (vp->vdtype == VATDATA_NOPOST)
			continue;
		if (vdp == NULL && (vdp = db_vatdataalloc()) == NULL) {
			bwarn("vat_total", "cannot post VAT data");
			continue;
		}
		vdp->id         = 0;
		vdp->vatret_id  = 0;
		vdp->journal_id = vp->journal_id;
		vdp->vdtype     = vp->vdtype;
		vdp->vat_id     = vp->vat_id;
		vdp->amount     = vp->amount;
		vdp->total      = vp->total;
		db_insertvatdata(dbp, vdp);
	}
#endif
	return(vatttl);
}

/*
 *
 */
void
vat_clear(struct vatamt *head)
{
#ifdef VAT_CALCULATE
	while (head != NULL) {
		head->amount = head->total = 0.0;
		head = head->next;
	}
#endif
}

/*
 *
 */
int
vatcode(int vatid)
{
	int vc;
	struct db_vat *vatp;
	dbow_conn *dbp = getdbase();

	if ((vatp = findvat(dbp, vatid)) == NULL)
		return('Z');
	vc = vatp->vcode[0];
	db_vatfree(vatp);
	return(vc);
}

/*
 *
 */
int
vatid(int vatcode)
{
	int vid;
	char vatstr[2];
	struct db_vat *vatp;
	dbow_conn *dbp = getdbase();

	vatstr[0] = vatcode;
	vatstr[1] = '\0';
	if ((vatp = findvatcode(dbp, vatstr)) == NULL)
		return(0);
	vid = vatp->id;
	db_vatfree(vatp);
	return(vid);
}

/*
 *
 */
double
vat_findnet(double amount, int vatid)
{
	int minus, inetval;
	struct db_vat *vatp;
	dbow_conn *dbp = getdbase();

#if VAT_CALCULATE
	if (amount < 0.0) {
		amount = -amount;
		minus = -1;
	} else
		minus = 1;
	if ((vatp = findvat(dbp, vatid)) == NULL)
		return(amount * minus);
	inetval = (int )(amount * 100000.0) / (vatp->rate + 100.0);
	if ((inetval % 10) >= 5)
		inetval += 10;
	inetval /= 10;
	amount = (double )inetval / 100.0;
	return(amount * minus);
#else
	return(amount);
#endif
}
