#ident "Kalopa$Id: digits.c,v 1.2 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: digits.c,v 1.2 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: digits.c,v $
 * Revision 1.2  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.1  2005/08/01 10:37:49  dtynan
 * Added new functions for addresses and for printing numbers in a useful
 * way.  Also removed kalmap DBOW files from include file.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "beanie.h"

/*
 *
 */
int
makenumberdigit(char *buf, int len, int val)
{
	int n = 0;

	if (val >= 1000) {
		n = makenumberdigit(buf, len, val / 1000);
		val %= 1000;
		buf += n;
		len -= n;
		if (len < 5)
			return(-1);
		sprintf(buf, ",%03d", val);
		n += 4;
	} else {
		if (len < 4)
			return(-1);
		sprintf(buf, "%d", val);
		n = strlen(buf);
	}
	return(n);
}

/*
 *
 */
int
makenumbertext(char *buf, int len, int val)
{
	int i, n = 0, delim = ' ', and = 0;
	char *cp;

	if (val == 0) {
		if (len < 6)
			return(-1);
		strcpy(buf, " ZERO");
		return(5);
	}
	if (val > 999999999) {
		n = makenumbertext(buf, len, val / 1000000000);
		val %= 1000000000;
		buf += n;
		len -= n;
		if (len < 9)
			return(-1);
		strcpy(buf, " BILLION");
		buf += 8;
		len -= 8;
		n += 8;
		and = 1;
	} else if (val > 999999) {
		n = makenumbertext(buf, len, val / 1000000);
		val %= 1000000;
		buf += n;
		len -= n;
		if (len < 9)
			return(-1);
		strcpy(buf, " MILLION");
		buf += 8;
		len -= 8;
		n += 8;
		and = 1;
	}
	if (val > 999) {
		n = makenumbertext(buf, len, val / 1000);
		val %= 1000;
		buf += n;
		len -= n;
		if (len < 10)
			return(-1);
		strcpy(buf, " THOUSAND");
		buf += 9;
		len -= 9;
		n += 9;
		and = 1;
	}
	if (val > 99) {
		n = makenumbertext(buf, len, val / 100);
		val %= 100;
		buf += n;
		len -= n;
		if (len < 9)
			return(-1);
		strcpy(buf, " HUNDRED");
		buf += 8;
		len -= 8;
		n += 8;
		and = 1;
	}
	if (and && val != 0) {
		if (len < 5)
			return(-1);
		strcpy(buf, " AND");
		buf += 4;
		len -= 4;
		n += 4;
	}
	if (val >= 20) {
		*buf++ = ' ';
		len--;
		n++;
		switch (val / 10) {
		case 2:
			cp = "TWENTY";
			i = 6;
			break;

		case 3:
			cp = "THIRTY";
			i = 6;
			break;

		case 4:
			cp = "FORTY";
			i = 5;
			break;

		case 5:
			cp = "FIFTY";
			i = 5;
			break;

		case 6:
			cp = "SIXTY";
			i = 5;
			break;

		case 7:
			cp = "SEVENTY";
			i = 7;
			break;

		case 8:
			cp = "EIGHTY";
			i = 6;
			break;

		case 9:
			cp = "NINETY";
			i = 6;
			break;
		}
		if (len <= i)
			return(-1);
		strcpy(buf, cp);
		buf += i;
		len -= i;
		n += i;
		val %= 10;
		delim = '-';
	}
	if (val == 0)
		return(n);
	*buf++ = delim;
	len--;
	n++;
	switch (val) {
	case 1:
		cp = "ONE";
		i = 3;
		break;

	case 2:
		cp = "TWO";
		i = 3;
		break;

	case 3:
		cp = "THREE";
		i = 5;
		break;

	case 4:
		cp = "FOUR";
		i = 4;
		break;

	case 5:
		cp = "FIVE";
		i = 4;
		break;

	case 6:
		cp = "SIX";
		i = 3;
		break;

	case 7:
		cp = "SEVEN";
		i = 5;
		break;

	case 8:
		cp = "EIGHT";
		i = 5;
		break;

	case 9:
		cp = "NINE";
		i = 4;
		break;

	case 10:
		cp = "TEN";
		i = 3;
		break;

	case 11:
		cp = "ELEVEN";
		i = 6;
		break;

	case 12:
		cp = "TWELVE";
		i = 6;
		break;

	case 13:
		cp = "THIRTEEN";
		i = 8;
		break;

	case 14:
		cp = "FOURTEEN";
		i = 8;
		break;

	case 15:
		cp = "FIFTEEN";
		i = 7;
		break;

	case 16:
		cp = "SIXTEEN";
		i = 7;
		break;

	case 17:
		cp = "SEVENTEEN";
		i = 9;
		break;

	case 18:
		cp = "EIGHTEEN";
		i = 8;
		break;

	case 19:
		cp = "NINETEEN";
		i = 8;
		break;
	}
	if (len <= i)
		return(-1);
	strcpy(buf, cp);
	buf += i;
	len -= i;
	return(n + i);
}
