#ident "Kalopa: $Id: email.c,v 1.12 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: email.c,v 1.12 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: email.c,v $
 * Revision 1.12  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.11  2006/10/01 10:22:35  dtynan
 * Major changes for VAT.
 *
 * Revision 1.10  2006/03/06 12:30:26  dtynan
 * Sometimes we need to use the basic form of an email address.  In this
 * case, we use argstrip() to remove everything outside the <>s.
 *
 * Revision 1.9  2005/08/15 12:04:10  dtynan
 * Minor tweaks to fix some MIME issues.
 *
 * Revision 1.8  2005/03/22 09:41:17  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.7  2004/07/16 19:24:35  dtynan
 * Changed path system to use BEANIEROOT environment variable instead of
 * hard-coded paths and added a new function to build the appropriate path.
 *
 * Revision 1.6  2004/07/12 19:07:15  dtynan
 * Extensive changes for new directory structure and other things.
 *
 * Revision 1.5  2004/01/05 18:38:05  dtynan
 * Added functions for including .sig files and regular files in an email.
 *
 * Revision 1.4  2004/01/05 18:24:46  dtynan
 * Made the close function conditional on being opened in the first place.
 *
 * Revision 1.3  2004/01/05 18:07:28  dtynan
 * Various tweaks to the email back-end, as well as new functions for
 * dealing with errors in a common, predicable way and for determining
 * whether or not the specified date is a business day or not.
 *
 * Revision 1.2  2003/11/07 16:00:10  dtynan
 * Added return-codes for email processor.
 *
 * Revision 1.1  2003/10/31 14:35:15  dtynan
 * First pass at automated billing software.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <paths.h>
#include <time.h>
#include <string.h>

#include "beanie.h"

#define MAXARGS		10
#define TEXT_TYPE	"text/plain; charset=us-ascii; format=flowed"

#ifndef ARGSTRIP
#define ARGSTRIP(ea)	(ea)
#endif

static	int	noch;
static	int	estate;
static	char	buffer[512];
static	char	*boundary;
static	char	*args[MAXARGS];
static	FILE	*pfp;

static	char	b64data[65] =
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

/*
 * Take an address of the form "Accounts System <beanie@domain.com>" and
 * return the string "beanie@domain.com" for those mailers that need this.
 */
static char *
argstrip(char *eaddr)
{
	char *cp, *xp;

	if ((cp = strchr(eaddr, '<')) == NULL)
		return(eaddr);
	xp = strdup(cp + 1);
	if ((cp = strchr(xp, '>')) != NULL)
		*cp = '\0';
	return(xp);
}

/*
 *
 */
static void
b64out(int vreg, int nch, FILE *ofp)
{
	int i, ch;

	for (i = 0; i < nch; i++) {
		ch = (vreg >> 18) & 0x3f;
		fputc(b64data[ch], ofp);
		vreg &= 0x03ffff;
		vreg <<= 6;
		noch++;
	}
}

/*
 *
 */
static int
emailproc(char *cp, FILE *ofp)
{
	int ch;

	while ((ch = *cp++) != '\0') {
		if (ch != '$') {
			fputc(ch, ofp);
			continue;
		}
		if ((ch = *cp) != '\0') {
			cp++;
			switch (ch) {
			case '$':
				fputc('$', ofp);
				break;

			case '0': case '1': case '2': case '3': case '4':
			case '5': case '6': case '7': case '8': case '9':
				ch -= '0';
				if (ch < MAXARGS && args[ch] != NULL)
					fprintf(ofp, "%s", args[ch]);
				break;
			}
		}
	}
	fputc('\r', ofp);
	fputc('\n', ofp);
	return(0);
}

/*
 *
 */
void
base64encode(FILE *ifp, FILE *ofp)
{
	int i, ch, vreg;

	i = vreg = noch = 0;
	while ((ch = fgetc(ifp)) != EOF) {
		vreg <<= 8;
		vreg |= ch;
		if (++i >= 3) {
			b64out(vreg, 4, ofp);
			if (noch >= 72) {
				fprintf(ofp, "\r\n");
				noch = 0;
			}
			i = vreg = 0;
		}
	}
	switch (i) {
	case 1:
		vreg <<= 16;
		b64out(vreg, 2, ofp);
		fprintf(ofp, "==");
		break;
	case 2:
		vreg <<= 8;
		b64out(vreg, 3, ofp);
		fputc('=', ofp);
		break;
	}
	fprintf(ofp, "\r\n\r\n");
}

/*
 *
 */
int
email_open(char *from, char *to, char *subj)
{
	char *cp;
	time_t now;
	struct tm *tmp;
#ifdef NOEMAIL
	int fd;
	char tmpf[128];
#endif

	time(&now);
	srand(now);
	tmp = localtime(&now);
	sprintf(buffer, "%s -i -f '%s' '%s' '%s'\n", _PATH_SENDMAIL,
							ARGSTRIP(from),
							ARGSTRIP(to),
							ARGSTRIP(from));
#ifdef NOEMAIL
	strcpy(tmpf, "/var/tmp/beanie.msg.XXXXXX");
	if ((fd = mkstemp(tmpf)) < 0 || (pfp = fdopen(fd, "w")) == NULL)
		return(-1);
	fprintf(pfp, ">> [%s]\n", buffer);
#else
	if ((pfp = popen(buffer, "w")) == NULL)
		return(-1);
#endif
	if (gethostname(buffer, sizeof(buffer)) < 0)
		return(-1);
	if ((cp = strchr(buffer, '.')) == NULL)
		cp = buffer;
	else
		cp++;
	fprintf(pfp, "Message-ID: <%08X.%08d@%s>\r\n", rand(), getpid(), cp);
	strftime(buffer, sizeof(buffer), "%a, %e %b %Y %T %z", tmp);
	fprintf(pfp, "Date: %s\r\n", buffer);
	fprintf(pfp, "From: %s\r\n", from);
	if ((cp = getenv("ORGANIZATION")) != NULL)
		fprintf(pfp, "Organization: %s\r\n", cp);

	fprintf(pfp, "MIME-Version: 1.0\r\n");
	fprintf(pfp, "To: %s\r\n", to);
	fprintf(pfp, "Cc: %s\r\n", from);
	fprintf(pfp, "Subject: ");
	emailproc(subj, pfp);

	sprintf(buffer, "------------%012d%012d", rand(), rand());
	if ((boundary = strdup(buffer)) == NULL)
		return(-1);
	fprintf(pfp, "Content-Type: multipart/mixed;\r\n");
	fprintf(pfp, "\tboundary=\"%s\"\r\n\r\n", boundary);
	fprintf(pfp, "This is a multi-part message in MIME format.\r\n\r\n");
	return(estate = 0);
}

/*
 *
 */
int
email_close()
{
	if (pfp != NULL) {
		fprintf(pfp, "--%s--\r\n\r\n", boundary);
#ifdef NOEMAIL
		fclose(pfp);
#else
		pclose(pfp);
#endif
		free(boundary);
	}
	argstrip("");
	return(0);
}

/*
 *
 */
int
email_text(char *data)
{
	if (estate == 0) {
		fprintf(pfp, "--%s\r\n", boundary);
		fprintf(pfp, "Content-Type: %s\r\n", TEXT_TYPE);
		fprintf(pfp, "Content-Transfer-Encoding: 7bit\r\n\r\n");
		estate = 1;
	}
	return(emailproc(data, pfp));
}

/*
 *
 */
int
email_signature(char *sname)
{
	return(email_include(makepath("email", "signature", sname, NULL)));
}

/*
 *
 */
int
email_include(char *fname)
{
	FILE *fp;
	char data[256], *cp;

	if ((fp = fopen(fname, "r")) == NULL)
		return(-1);
	while (fgets(data, sizeof(data), fp) != NULL) {
		if ((cp = strpbrk(data, "\r\n")) != NULL)
			*cp = '\0';
		email_text(data);
	}
	fclose(fp);
	return(0);
}

/*
 *
 */
int
email_file(char *fname, char *ctype)
{
	FILE *fp;

	if ((fp = fopen(fname, "r")) == NULL)
		return(-1);
	email_fptr(fp, fname, ctype);
	fclose(fp);
	return(estate = 0);
}

/*
 *
 */
int
email_fptr(FILE *fp, char *fname, char *ctype)
{
	fprintf(pfp, "--%s\r\n", boundary);
	fprintf(pfp, "Content-Type: %s;\r\n", ctype);
	fprintf(pfp, "\tname=\"%s\"\r\n", fname);
	fprintf(pfp, "Content-Transfer-Encoding: base64\r\n");
	fprintf(pfp, "Content-Disposition: inline;\r\n");
	fprintf(pfp, "\tfilename=\"%s\"\r\n\r\n", fname);
	base64encode(fp, pfp);
	return(estate = 0);
}

/*
 *
 */
void
email_setarg(int argno, char *val)
{
	if (argno < 0 || argno >= MAXARGS)
		return;
	if (args[argno] != NULL)
		free(args[argno]);
	args[argno] = (val == NULL) ? NULL : strdup(val);
}
