#ident "Kalopa: $Id: getdate.c,v 1.4 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: getdate.c,v 1.4 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 1996, Claddagh Films Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: getdate.c,v $
 * Revision 1.4  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2005/03/22 09:41:17  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.2  2003/11/08 16:10:14  dtynan
 * Added old accounting system functions.
 *
 * Revision 1.1  1997/01/05 03:31:24  dtynan
 * Baseline check-in - most functionality is now present.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "beanie.h"

/*
 * Parse a string of the form dd/mm/yy into its components.  While
 * this may seem trivial, it's not.  We need to ensure that the
 * specified dates are valid, not only in terms of themselves but
 * each other (for example, 29/2/97).  Also, if the caller won't
 * accept a date in the future, then we need to make sure the
 * date is historic.  Returns TRUE for a good date.
 */
int
getdate(char *cp, int *dp, int *mp, int *yp, int flag)
{
	int d, m, y;
	time_t now;
	char *xp;
	struct tm *tp;

	/*
	 * Find the delimiter of the 'dd' portion.
	 */
	if ((xp = strchr(cp, '/')) == NULL)
		return(0);
	/*
	 * We have at least a day/month.  First off,
	 * get the day and do a quick check on it.
	 */
	*xp++ = '\0';
	if ((d = atoi(cp)) < 1 || d > 31)
		return(0);
	/*
	 * Now find the delimiter of the 'mm' portion.
	 */
	if ((cp = strchr(xp, '/')) == NULL)
		return(0);
	/*
	 * Collect the month and check its validity.
	 */
	*cp++ = '\0';
	if ((m = atoi(xp)) < 1 || m > 12)
		return(0);
	/*
	 * Collect the year and allow the user to truncate
	 * the date (not a good idea these days).
	 */
	if ((y = atoi(cp)) < 100)
		y += 1900;
	/*
	 * If the year is bogus, or the day no longer matches
	 * (because we've changed the month and year), then
	 * fail now.  The dates are arbitrary.
	 */
	if (y < 1990 || y > 2199 || d > mdays(m, y))
		return(0);
	/*
	 * Save our interim values, and return if the future is OK.
	 */
	*dp = d;
	*mp = m;
	*yp = y;
	if (flag)
		return(1);
	/*
	 * OK, find todays date.  If the specified date is in the
	 * future the we have a problem, Heuston...
	 */
	time(&now);
	tp = localtime(&now);
	tp->tm_mon += 1;
	tp->tm_year += 1900;
	if (y != tp->tm_year)
		return(y < tp->tm_year);
	/*
	 * Same year - try the month.
	 */
	if (m != tp->tm_mon)
		return(m < tp->tm_mon);
	/*
	 * Same month.  Return with the status of the day.  Note
	 * that todays date is considered to be historic.
	 */
	return(d <= tp->tm_mday);
}
