#ident "Kalopa: $Id: getyend.c,v 1.6 2008/01/15 18:10:43 dtynan Exp $"

/*
 * $Id: getyend.c,v 1.6 2008/01/15 18:10:43 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: getyend.c,v $
 * Revision 1.6  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.5  2005/03/22 09:41:17  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.4  2004/12/09 20:11:08  dtynan
 * Created new function to return the organisation name in a predictable way.
 *
 * Revision 1.3  2003/11/08 16:01:06  dtynan
 * Improved handling of cardinal numbers and fixed data types in year-end.
 *
 * Revision 1.2  2003/11/08 14:53:48  dtynan
 * Added functions to getyend.c to determine quarter-ends and where we are
 * in the quarter.  Also added new functions for date manipulation, roff
 * processing and initialization.
 *
 * Revision 1.1  2003/10/31 17:16:15  dtynan
 * Various changes around the new/improved system.
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "beanie.h"

/*
 *
 */
int
getyend(struct db_company *cp)
{
	struct tm *tp;

	if (cp == NULL || cp->yend == 0 || (tp = localtime((time_t *)&cp->yend)) == NULL)
		return(12);
	return(tp->tm_mon + 1);
}

/*
 *
 */
void
getqends(struct db_company *cp, int qend[])
{
	int i, yend = getyend(cp);

	for (i = 3; i >= 0; i--) {
		qend[i] = yend;
		if ((yend -= 3) < 1)
			yend += 12;
	}
}

/*
 *
 */
int
whichquarter(int mon, int qend[])
{
	int i, qs, qe;

	qs = qend[3] + 1;
	qe = qend[0];
	for (i = 0; i < 4; i++) {
		if (qs > 12)
			qs -= 12;
		if (qe > 12)
			qe -= 12;
		if (qs <= mon && mon <= qe)
			break;
		qs += 3;
		qe += 3;
	}
	return(i + 1);
}
