#ident "Kalopa: $Id: holiday.c,v 1.5 2008/10/31 15:17:13 dtynan Exp $"

/*
 * $Id: holiday.c,v 1.5 2008/10/31 15:17:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: holiday.c,v $
 * Revision 1.5  2008/10/31 15:17:13  dtynan
 * Extensive changes for version 0.8.2.
 *
 * Revision 1.4  2008/07/18 11:07:37  dtynan
 * Fixed issue with isholiday/skipholiday where it wouldn't actually detect
 * bank holidays (only weekends).  It now interrogates the database.
 *
 * Revision 1.3  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.2  2005/03/22 09:41:18  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.1  2004/01/05 18:07:28  dtynan
 * Various tweaks to the email back-end, as well as new functions for
 * dealing with errors in a common, predicable way and for determining
 * whether or not the specified date is a business day or not.
 */

#include <stdio.h>
#include <time.h>

#include "beanie.h"

/*
 *
 */
int
isholiday(time_t when)
{
	struct tm *tmp;
	struct db_holiday *hp;

	tmp = localtime(&when);
	if (tmp->tm_wday == 0 || tmp->tm_wday == 6)
		return(1);
	if ((hp = db_findholidaybyhdate(getdbase(), (int )when)) != NULL) {
		db_holidayfree(hp);
		return(1);
	}
	return(0);
}

/*
 *
 */
time_t
skipholiday(time_t when)
{
	while (isholiday(when) > 0)
		when += DAYS(1);
	return(when);
}
