#ident "Kalopa: $Id: journal.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: journal.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2006, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 * Helper functions for general journal.
 *
 * $Log: journal.c,v $
 * Revision 1.6  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/07/08 17:08:53  dtynan
 * Extensive changes.
 *
 * Revision 1.4  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2007/08/07 08:17:16  dtynan
 * Quick update.
 *
 * Revision 1.2  2007/08/06 10:53:34  dtynan
 * Modified the journal_entry function to return the new journal ID.
 *
 * Revision 1.1  2006/10/01 10:22:35  dtynan
 * Major changes for VAT.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

/*
 *
 */
int
journal_entry(time_t *when, char *folio, char *descr, int vcode, int dracct, int cracct, double netamt)
{
	int jid, drcrf = 0;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct vatamt *vat;
	double vatamt, total;
	dbow_conn *dbp = getdbase();

	/*
	 * Make sure amounts are positive.
	 */
	if (netamt < 0.0) {
		drcrf = 1;
		netamt = -netamt;
	}
	/*
	 * Now do the journal posting.
	 */
	if ((jp = db_journalalloc()) == NULL)
		berror("journal_entry", "cannot create journal entry");
	if (when == NULL)
		time((time_t *)&jp->jdate);
	else
		jp->jdate = *when;
	jp->folio = (folio == NULL) ? NULL : strdup(folio);
	jp->descr = strdup(descr);
	jp->user_id = getuid();
	jp->state = 0;
	if (db_insertjournal(dbp, jp) < 0)
		berror("journal_entry", "journal insert failed");
	if ((jip = db_journal_itemalloc()) == NULL)
		berror("journal_entry", "cannot create journal item");
	jid = jip->journal_id = jp->id;
	/*
	 * Do the debit first.
	 */
	jip->item_id    = 1;
	jip->nomacct_id = dracct;
	jip->drcrf      = drcrf;
	jip->amount     = netamt;
	if (db_insertjournal_item(dbp, jip) < 0)
		berror("journal_entry", "journal debit item insert failed");
	/*
	 * Do the VAT if required.
	 */
	if (vcode >= 0) {
		/*
		 * Do the VAT posting.
		 */
		vat = vat_init();
		vat_lineitem(vat, netamt, vcode, jp->id, VATDATA_PURCHS);
		vatamt = vat_total(vat);
		total = netamt + vatamt;
		jip->item_id++;
		jip->nomacct_id = NA_VATACCOUNT;
		jip->drcrf      = drcrf;
		jip->amount     = vatamt;
		if (db_insertjournal_item(dbp, jip) < 0)
			berror("journal_entry", "journal vat insert failed");
	} else
		total = netamt;
	/*
	 * Now do the credit.
	 */
	jip->item_id++;
	jip->nomacct_id = cracct;
	jip->drcrf      = drcrf ? 0 : 1;
	jip->amount     = total;
	if (db_insertjournal_item(dbp, jip) < 0)
		berror("journal_entry", "journal credit item insert failed");
	db_journal_itemfree(jip);
	db_journalfree(jp);
	return(jid);
}
