#ident "Kalopa: $Id: mdays.c,v 1.8 2008/10/20 17:53:25 dtynan Exp $"

/*
 * $Id: mdays.c,v 1.8 2008/10/20 17:53:25 dtynan Exp $
 *
 * Copyright (c) 1996, Claddagh Films Limited.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: mdays.c,v $
 * Revision 1.8  2008/10/20 17:53:25  dtynan
 * Added a document for installation (quite basic at this point), removed
 * some debug output from tpadjust(), and fixed a lingering issue with the
 * way invoice due-dates were calculated.
 *
 * Revision 1.7  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.6  2005/03/22 09:41:18  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.5  2004/08/26 17:28:39  dtynan
 * Added new function to adjust a struct tm pointer to be a valid day.
 *
 * Revision 1.4  2004/05/12 15:21:16  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.3  2003/10/31 17:16:15  dtynan
 * Various changes around the new/improved system.
 *
 * Revision 1.2  1997/01/05 03:31:30  dtynan
 * Baseline check-in - most functionality is now present.
 *
 * Revision 1.1  1997/01/03  21:37:30  dtynan
 * Initial check-in.
 */

#include <stdio.h>
#include <time.h>

#include "beanie.h"

static	int	mdaylist[12] = {31,0,31,30,31,30,31,31,30,31,30,31};

/*
 * Return the number of days in the specified month.
 */
int
mdays(int m, int y)
{
	if (m != 2)
		return(mdaylist[m-1]);
	return((((y)%4) == 0 && (((y)%100) != 0 || ((y)%400) == 0)) ? 29 : 28);
}

/*
 *
 */
void
tpadjust(struct tm *tp)
{
	int md = mdays(tp->tm_mon + 1, tp->tm_year + 1900);

	while (tp->tm_mday > md) {
		tp->tm_mon++;
		tp->tm_mday -= md;
		while (tp->tm_mon > 11) {
			tp->tm_year++;
			tp->tm_mon -= 12;
		}
		md = mdays(tp->tm_mon + 1, tp->tm_year + 1900);
	}
}
