#ident "Kalopa: $Id: nabal.c,v 1.9 2008/01/15 18:10:43 dtynan Exp $"

/*
 * $Id: nabal.c,v 1.9 2008/01/15 18:10:43 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: nabal.c,v $
 * Revision 1.9  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.8  2008/01/15 15:25:36  dtynan
 * Changed table name.
 *
 * Revision 1.7  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.6  2005/04/11 13:52:41  dtynan
 * Changed the nominal account code to only load the list of nominal
 * accounts once.
 *
 * Revision 1.5  2005/03/22 09:41:18  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.4  2005/02/17 17:25:26  dtynan
 * Fixed some compiler warnings.
 *
 * Revision 1.3  2004/12/09 20:21:06  dtynan
 * Fixed wrong include files.
 *
 * Revision 1.2  2004/12/09 19:19:30  dtynan
 * Added a function to free nominal account balances.
 *
 * Revision 1.1  2004/12/08 17:42:29  dtynan
 * Added functions to manage and manipulate journal entry balances.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include "beanie.h"

struct	db_nomacct	*findnomaccts(dbow_conn *);
struct	db_journal	*findjournalentries(dbow_conn *);
struct	db_journal_item	*findjournalitems(dbow_conn *, int);

/*
 *
 */
struct nabal *
loadnabal(dbow_conn *conn)
{
	struct nabal *np;
	static struct nabal *head = NULL;

	/*
	 * Only load the nominal account data once, but clear the
	 * balances every time.
	 */
	if (head == NULL) {
		struct db_nomacct *ap;
		struct nabal *tail = NULL;

		for (ap = findnomaccts(conn); ap != NULL;
					ap = db_findnomacctnext(conn, ap)) {
			if ((np = (struct nabal *)malloc(sizeof(*np))) == NULL)
				return(NULL);
			np->next = NULL;
			np->nomacct_id = ap->id;
			np->atype = ap->atype;
			np->name = ap->aname;
			ap->aname = NULL;
			np->bal = 0.0;
			if (tail == NULL)
				head = np;
			else
				tail->next = np;
			tail = np;
		}
	} else
		for (np = head; np != NULL; np = np->next)
			np->bal = 0.0;
	return(head);
}

/*
 *
 */
struct nabal *
findnabal(struct nabal *nlist, int acctid)
{
	while (nlist != NULL && nlist->nomacct_id != acctid)
		nlist = nlist->next;
	return(nlist);
}

/*
 *
 */
void
loadbalances(dbow_conn *conn, struct nabal *nlist, int year)
{
	struct nabal *np;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct tm *tmp;

	for (jp = findjournalentries(conn); jp != NULL; jp =
						db_findjournalnext(conn, jp)) {
		if ((tmp = localtime((time_t *)&jp->jdate)) == NULL ||
						(tmp->tm_year != year))
			continue;
		for (jip = findjournalitems(conn, jp->id); jip != NULL;
				jip = db_findjournal_itemnext(conn, jip)) {
			if ((np = findnabal(nlist, jip->nomacct_id)) == NULL) {
				printf("loadbalances: can't find acct %d\n",
							jip->nomacct_id);
				continue;
			}
			if (jip->drcrf)
				np->bal += jip->amount;
			else
				np->bal -= jip->amount;
		}
	}
}
