#ident "Kalopa: $Id: path.c,v 1.6 2008/10/31 15:17:13 dtynan Exp $"

/*
 * $Id: path.c,v 1.6 2008/10/31 15:17:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: path.c,v $
 * Revision 1.6  2008/10/31 15:17:13  dtynan
 * Extensive changes for version 0.8.2.
 *
 * Revision 1.5  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2005/08/13 13:35:35  dtynan
 * Created a new define for the default home directory.
 *
 * Revision 1.3  2005/03/22 09:41:18  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.2  2005/02/17 11:05:03  dtynan
 * Added code to try to get the actual Beanie path from the password file
 * (home directory of user 'beanie') before resorting to hard-coded path.
 *
 * Revision 1.1  2004/07/16 19:24:35  dtynan
 * Changed path system to use BEANIEROOT environment variable instead of
 * hard-coded paths and added a new function to build the appropriate path.
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/types.h>
#include <string.h>
#include <time.h>
#include <pwd.h>

#include "beanie.h"

/*
 *
 */
char *
makepath(char *dir, ...)
{
	int len;
	char *cp, *memp;
	struct passwd *pp;
	va_list ap;

	if ((cp = getconfig("homedir", NULL)) == NULL) {
		if ((pp = getpwnam("beanie")) != NULL && pp->pw_dir != NULL)
			cp = pp->pw_dir;
		else
			cp = "beanie";
	}
	if (dir == NULL)
		return(strdup(cp));
	len = strlen(cp) + strlen(dir) + 2;
	if ((memp = (char *)malloc(len)) == NULL)
		return(NULL);
	sprintf(memp, "%s/%s", cp, dir);
	va_start(ap, dir);
	while ((cp = va_arg(ap, char *)) != NULL) {
		if (*cp == '\0')
			continue;
		len += strlen(cp) + 1;
		if ((memp = realloc(memp, len)) == NULL)
			return(NULL);
		strcat(memp, "/");
		strcat(memp, cp);
	}
	return(memp);
}
