#ident "Kalopa: $Id: roff.c,v 1.4 2008/01/15 18:10:43 dtynan Exp $"

/*
 * $Id: roff.c,v 1.4 2008/01/15 18:10:43 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: roff.c,v $
 * Revision 1.4  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2005/03/22 09:41:19  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.2  2004/05/12 15:21:16  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.1  2003/11/08 14:53:48  dtynan
 * Added functions to getyend.c to determine quarter-ends and where we are
 * in the quarter.  Also added new functions for date manipulation, roff
 * processing and initialization.
 */
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "beanie.h"

/*
 *
 */
char *
roffize(char *str)
{
	int ch, state;
	char *cp;
	static char output[512];

	cp = output;
	state = 0;
	while ((ch = *str++) != '\0') {
		switch (state) {
		case 0:
			switch (ch) {
			case '<':
				state = 1;
				break;
			case '&':
				state = 3;
				break;
			default:
				*cp++ = ch;
				break;
			}
			break;
		case 1:
			switch (ch) {
			case '/':
				*cp++ = '\\';
				*cp++ = 'f';
				*cp++ = 'P';
				state = 2;
				break;
			case 'i':
			case 'I':
				*cp++ = '\\';
				*cp++ = 'f';
				*cp++ = 'I';
				state = 2;
				break;
			case 'b':
			case 'B':
				*cp++ = '\\';
				*cp++ = 'f';
				*cp++ = 'B';
				state = 2;
				break;
			default:
				state = 2;
				break;
			}
			break;
		case 2:
			if (ch == '>')
				state = 0;
			break;
		case 3:
			switch (ch) {
			case 'g':
			case 'G':
				state = 4;
				break;
			case 'l':
			case 'L':
				state = 5;
				break;
			case 'a':
			case 'A':
				state = 6;
				break;
			default:
				state = 9;
			}
			break;
		case 4:
			if (ch == 't' || ch == 'T')
				*cp++ = '>';
			state = 99;
			break;
		case 5:
			if (ch == 't' || ch == 'T')
				*cp++ = '<';
			state = 9;
			break;
		case 6:
			if (ch == 'm' || ch == 'M')
				state = 7;
			else
				state = 9;
			break;
		case 7:
			if (ch == 'p' || ch == 'P')
				*cp++ = '&';
			state = 9;
			break;
		case 9:
			if (ch == ';')
				state = 0;
			break;
		}
	}
	*cp = '\0';
	return(output);
}
