#ident "Kalopa: $Id: round.c,v 1.5 2008/01/15 18:10:43 dtynan Exp $"

/*
 * $Id: round.c,v 1.5 2008/01/15 18:10:43 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: round.c,v $
 * Revision 1.5  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2006/10/01 10:22:35  dtynan
 * Major changes for VAT.
 *
 * Revision 1.3  2006/07/31 08:49:49  dtynan
 * Fixed a bug in the round-up function.
 *
 * Revision 1.2  2005/10/17 15:38:35  dtynan
 * Added new round-off types and fixed some minor gotchas.
 *
 * Revision 1.1  2005/10/17 15:06:42  dtynan
 * Created a new roundoff routine specifically for beanie.  It should
 * handle all the usual roundoff issues.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "beanie.h"

#define ROUNDSIZE	100.0

/*
 * Do a currency round-off to the nearest cent/penny.  Generally,
 * we round off by saying that anything less than half a cent is
 * ignored, and anything greater than or equal to half a cent is a
 * full cent.  This is type ROUNDOFF_NORMAL.  ROUNDOFF_ROUNDDOWN
 * always drops the error, ROUNDOFF_ROUNDUP adds a cent when there
 * is any error, and ROUNDOFF_UKVAT does the UK VAT trick when the
 * amount is exactly half a cent.  In this obscure case, round to
 * the nearest even whole integer.  For example, 17.5 pence rounds
 * to 18p whereas 16.5 pence rounds down to 16p.  ROUNDOFF_INTEGER
 * will round the amount to the nearest whole Euro (or Pound), using
 * ROUNDOFF_NORMAL.
 */
double
roundoff(double amount, int type)
{
	int m, n;
	double err, rndsize = ROUNDSIZE;

	/* 
	 * To make life easy, only deal with positive numbers and
	 * convert back any negative numbers at the end.
	 */
	if (amount < 0.0) {
		amount = -amount;
		m = -1;
	} else
		m = 1;
	/*
	 * Calculate the integer amount and the roundoff error.
	 */
	if (type == ROUNDOFF_INTEGER) {
		rndsize = 1.0;
		type = ROUNDOFF_NORMAL;
	}
	n = (int )(amount * rndsize);
	err = (amount * rndsize) - (double )n;
	switch (type) {
	case ROUNDOFF_ROUNDUP:
		if (err > 0.0)
			n++;
		break;

	case ROUNDOFF_UKVAT:
		if (err > 0.5 || (err == 0.5 && n & 01))
			n++;
		break;

	default:
		if (err >= 0.5)
			n++;
		break;
	}
	return((double )(n * m) / rndsize);
}
