#ident "Kalopa: $Id: troffaddr.c,v 1.8 2008/01/15 18:10:43 dtynan Exp $"

/*
 * $Id: troffaddr.c,v 1.8 2008/01/15 18:10:43 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: troffaddr.c,v $
 * Revision 1.8  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.7  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.6  2005/08/01 10:37:49  dtynan
 * Added new functions for addresses and for printing numbers in a useful
 * way.  Also removed kalmap DBOW files from include file.
 *
 * Revision 1.5  2005/04/11 17:50:28  dtynan
 * Make sure the variables are always initialized.
 *
 * Revision 1.4  2005/04/11 17:48:29  dtynan
 * Oops!  Minor gotcha.
 *
 * Revision 1.3  2005/04/11 17:47:39  dtynan
 * Fixed a minor bug in the row printing code.
 *
 * Revision 1.2  2005/04/11 16:17:10  dtynan
 * Don't need to return -1.
 *
 * Revision 1.1  2005/04/11 16:06:50  dtynan
 * New function for printing address blocks (for troff).
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "beanie.h"

/*
 *
 */
void
gentroffaddr(FILE *fp, dbow_conn *dbp, int contact_id, int cperson_id)
{
	int row;
	struct db_contact *cp = NULL;
	struct db_contact_person *cpp = NULL;
	struct db_country *cntp = NULL;

	/*
	 * Find the contact details first...
	 */
	if ((cp = contact_findbyid(dbp, contact_id)) == NULL)
		berror("gentroffaddr", "cannot find contact data for address");
	/*
	 * Now see if there's a specific person we need to find?
	 */
	row = 0;
	if (cperson_id > 0) {
		/*
		 * Yes - search for their details.
		 */
		row++;
		if ((cpp = person_findbyid(dbp, cperson_id)) == NULL)
			berror("gentroffaddr", "cannot find person record");
		if (cpp->fname == NULL)
			fprintf(fp, ".ds Name %s\n", cpp->lname);
		else
			fprintf(fp, ".ds Name %s %s\n", cpp->fname, cpp->lname);
		if (cpp->title != NULL) {
			fprintf(fp, ".ds Addr1 %s\n", cpp->title);
			row++;
		}

	}
	/*
	 * Now print the organisation name
	 */
	if (row == 0) {
		fprintf(fp, ".ds Name %s\n", orgname(cp));
		row = 1;
	} else
		fprintf(fp, ".ds Addr%d %s\n", row++, orgname(cp));
	/*
	 * Print the address block.
	 */
	if (cp->a1 != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, cp->a1);
	if (cp->a2 != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, cp->a2);
	if (cp->a3 != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, cp->a3);
	if (cp->city != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, cp->city);
	if (cp->state_county != NULL) {
		fprintf(fp, ".ds Addr%d %s", row++, cp->state_county);
		if (cp->postcode_zip != NULL)
			fprintf(fp, " %s", cp->postcode_zip);
		fputc('\n', fp);
	}
	/*
	 * Print the country (if it's not the same as ours.
	 */
	cntp = db_findcountrybyid(dbp, cp->country_id);
	if (cntp != NULL && cntp->id != HOME_COUNTRY)
		fprintf(fp, ".ds Addr%d %s\n", row++, cntp->cname);
	/*
	 * Print a contact code (for reference) and free the data.
	 */
	if (cp != NULL && cp->ccode != NULL)
		fprintf(fp, ".ds CCode %s\n", cp->ccode);
	if (cntp != NULL)
		db_countryfree(cntp);
	if (cpp != NULL)
		db_contact_personfree(cpp);
	if (cp != NULL)
		db_contactfree(cp);
}

/*
 *
 */
void
gentroffaddrline(FILE *fp, dbow_conn *dbp, int contact_id)
{
	struct db_contact *cp = NULL;
	struct db_country *cntp = NULL;

	/*
	 * Find the contact details first...
	 */
	if ((cp = contact_findbyid(dbp, contact_id)) == NULL)
		berror("gentroffaddr", "cannot find contact data for address");
	/*
	 * Now print the organisation name
	 */
	fprintf(fp, ".ds Name %s\n", orgname(cp));
	fprintf(fp, ".ds Addr %s", cp->a1);
	/*
	 * Print the address block.
	 */
	if (cp->a2 != NULL)
		fprintf(fp, ", %s", cp->a2);
	if (cp->a3 != NULL)
		fprintf(fp, ", %s", cp->a3);
	if (cp->city != NULL)
		fprintf(fp, ", %s", cp->city);
	if (cp->state_county != NULL) {
		fprintf(fp, ", %s", cp->state_county);
		if (cp->postcode_zip != NULL)
			fprintf(fp, " %s", cp->postcode_zip);
	}
	/*
	 * Print the country (if it's not the same as ours.
	 */
	cntp = db_findcountrybyid(dbp, cp->country_id);
	if (cntp != NULL && cntp->id != HOME_COUNTRY)
		fprintf(fp, ", %s", cntp->cname);
	fputc('\n', fp);
	if (cntp != NULL)
		db_countryfree(cntp);
	if (cp != NULL)
		db_contactfree(cp);
}
