#ident "Kalopa: $Id: ofxread.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: ofxread.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: ofxread.c,v $
 * Revision 1.6  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.4  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.2  2005/03/22 09:41:20  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.1  2005/03/07 16:51:13  dtynan
 * New utility to parse an OFX file and populate the database.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

#define PROGNAME	"ofxread"

#define OFXHDR_HEADER		0
#define OFXHDR_DTYPE		1
#define OFXHDR_VERSION		2
#define OFXHDR_SECURITY		3
#define OFXHDR_ENCODE		4
#define OFXHDR_CHARSET		5
#define OFXHDR_COMPR		6
#define OFXHDR_OLDUID		7
#define OFXHDR_NEWUID		8

#define OFXBODY_OFX		9
#define OFXBODY_SIGNON		10
#define OFXBODY_SONRS		11
#define OFXBODY_STATUS		12
#define OFXBODY_CCMSG		13
#define OFXBODY_CCTNRS		14
#define OFXBODY_CCSTMT		15
#define OFXBODY_CCFROM		16
#define OFXBODY_BANKTL		17
#define OFXBODY_STMNT		18
#define OFXBODY_LEDGER		19

#define OFXTAG_ACCTID		20
#define OFXTAG_BALAMT		21
#define OFXTAG_CODE		22
#define OFXTAG_CURDEF		23
#define OFXTAG_DTACCTUP		24
#define OFXTAG_DTASOF		25
#define OFXTAG_DTEND		26
#define OFXTAG_DTPOSTED		27
#define OFXTAG_DTSERVER		28
#define OFXTAG_DTSTART		29
#define OFXTAG_DTUSER		30
#define OFXTAG_FITID		31
#define OFXTAG_LANGUAGE		32
#define OFXTAG_NAME		33
#define OFXTAG_SEVERITY		34
#define OFXTAG_TRNAMT		35
#define OFXTAG_TRNTYPE		36
#define OFXTAG_TRNUID		37

/*
 * Header types...
 */
struct	header	{
	char	*name;
	int	type;
} headers[] = {
	{"ACCTID",		OFXTAG_ACCTID},
	{"BALAMT",		OFXTAG_BALAMT},
	{"BANKTRANLIST",	OFXBODY_BANKTL},
	{"CCACCTFROM",		OFXBODY_CCFROM},
	{"CCSTMTRS",		OFXBODY_CCSTMT},
	{"CCSTMTTRNRS",		OFXBODY_CCTNRS},
	{"CHARSET",		OFXHDR_CHARSET},
	{"CODE",		OFXTAG_CODE},
	{"COMPRESSION",		OFXHDR_COMPR},
	{"CREDITCARDMSGSRSV1",	OFXBODY_CCMSG},
	{"CURDEF",		OFXTAG_CURDEF},
	{"DATA",		OFXHDR_DTYPE},
	{"DTACCTUP",		OFXTAG_DTACCTUP},
	{"DTASOF",		OFXTAG_DTASOF},
	{"DTEND",		OFXTAG_DTEND},
	{"DTPOSTED",		OFXTAG_DTPOSTED},
	{"DTSERVER",		OFXTAG_DTSERVER},
	{"DTSTART",		OFXTAG_DTSTART},
	{"DTUSER",		OFXTAG_DTUSER},
	{"ENCODING",		OFXHDR_ENCODE},
	{"FITID",		OFXTAG_FITID},
	{"LANGUAGE",		OFXTAG_LANGUAGE},
	{"LEDGERBAL",		OFXBODY_LEDGER},
	{"NAME",		OFXTAG_NAME},
	{"NEWFILEUID",		OFXHDR_NEWUID},
	{"OFX",			OFXBODY_OFX},
	{"OFXHEADER",		OFXHDR_HEADER},
	{"OLDFILEUID",		OFXHDR_OLDUID},
	{"SECURITY",		OFXHDR_SECURITY},
	{"SEVERITY",		OFXTAG_SEVERITY},
	{"SIGNONMSGSRSV1",	OFXBODY_SIGNON},
	{"SONRS",		OFXBODY_SONRS},
	{"STATUS",		OFXBODY_STATUS},
	{"STMTTRN",		OFXBODY_STMNT},
	{"TRNAMT",		OFXTAG_TRNAMT},
	{"TRNTYPE",		OFXTAG_TRNTYPE},
	{"TRNUID",		OFXTAG_TRNUID},
	{"VERSION",		OFXHDR_VERSION},
	{NULL,			0}
};

/*
 *
 */
struct	ofxhdr	{
	int	headerid;
	int	dtype;
	int	version;
	int	security;
	int	encoding;
	int	charset;
	int	compression;
	int	oldfileuid;
	int	newfileuid;
};

int		lineno;
int		nerrors;
int		verbose;
int		nomacct;
char		*fname;

struct	db_creditcard	*ccp = NULL;

/*
 *
 */
int	handlestate(FILE *, int, int);
void	findaccount(char *);
void	error(char *);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	struct db_company *comp;
	FILE *fp;

	cname = NULL;
	opterr = verbose = 0;
	nomacct = 0;
	while ((i = getopt(argc, argv, "c:qv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
		}
	}
	/*
	 * Find the company that we're processing...
	 */
	if (loadconfig(cname) < 0)
		berror(PROGNAME, "cannot load configuration file");
	if ((argc - optind) < 1)
		usage();
	while (optind < argc) {
		lineno = nerrors = 0;
		fname = argv[optind++];
		if ((fp = fopen(fname, "r")) == NULL) {
			fprintf(stderr, "?ofxload: ");
			perror(fname);
		} else {
			handlestate(fp, 0, 0);
			fclose(fp);
		}
	}
	exit(0);
}

/*
 *
 */
int
handlestate(FILE *fp, int indent, int type)
{
	int i, isend;
	char *cp, *xp, input[512];
	struct header *hp;
	dbow_conn *dbp;

	while (fgets(input, sizeof(input), fp) != NULL) {
		lineno++;
		if ((cp = strpbrk(input, "\n\r")) != NULL)
			*cp = '\0';
		if (indent == 0) {
			if (strcmp(input, "<OFX>") == 0) {
				handlestate(fp, 1, OFXBODY_OFX);
				return(0);
			}
			if ((cp = strchr(input, ':')) == NULL) {
				error("invalid syntax");
				continue;
			}
			*cp++ = '\0';
			for (hp = headers; hp->name != NULL; hp++)
				if (strcmp(input, hp->name) == 0)
					break;
			if (hp->name == NULL) {
				error("invalid syntax");
				continue;
			}
		} else {
			cp = input;
			if (*cp++ != '<') {
				error("invalid syntax");
				continue;
			}
			if (*cp == '/') {
				isend = 1;
				cp++;
			} else
				isend = 0;
			if ((xp = strchr(cp, '>')) != NULL)
				*xp++ = '\0';
			for (hp = headers; hp->name != NULL; hp++)
				if (strcmp(cp, hp->name) == 0)
					break;
			if (hp->name == NULL) {
				error("invalid OFX symbol");
				return(0);
			}
			if (isend)
				return(hp->type);
			switch (hp->type) {
			case OFXBODY_SIGNON:
			case OFXBODY_SONRS:
			case OFXBODY_STATUS:
			case OFXBODY_CCMSG:
			case OFXBODY_CCTNRS:
			case OFXBODY_CCSTMT:
			case OFXBODY_CCFROM:
			case OFXBODY_BANKTL:
			case OFXBODY_LEDGER:
				i = handlestate(fp, indent + 1, hp->type);
				break;

			case OFXTAG_ACCTID:
				findaccount(xp);
				break;

			case OFXBODY_STMNT:
				if (nomacct == 0) {
					error("cannot find card account");
					break;
				}
				if ((ccp = db_creditcardalloc()) == NULL)
					break;
				if (verbose)
					printf(">>> Transaction:-\n");
				ccp->nomacct_id = nomacct;
				i = handlestate(fp, indent + 1, hp->type);
				if (ccp != NULL) {
					ccp->journal_id = 0;
					ccp->state = 0;
					db_insertcreditcard(dbp, ccp);
					db_creditcardfree(ccp);
					ccp = NULL;
				}
				break;

			case OFXTAG_TRNTYPE:
				if (ccp == NULL)
					break;
				if (verbose)
					printf("TX type:\t%s\n", xp);
				if (strcmp(xp, "DEBIT") == 0)
					ccp->crtype = DEBIT;
				else if (strcmp(xp, "CREDIT") == 0)
					ccp->crtype = CREDIT;
				break;

			case OFXTAG_DTPOSTED:
				if (ccp == NULL)
					break;
				parsedate(xp, &ccp->pdate);
				if (verbose)
					printf("Date posted:\t%s", ctime(&ccp->pdate));
				break;

			case OFXTAG_DTUSER:
				if (ccp == NULL)
					break;
				parsedate(xp, &ccp->udate);
				if (verbose)
					printf("Date of TX:\t%s", ctime(&ccp->udate));
				break;

			case OFXTAG_TRNAMT:
				if (ccp == NULL)
					break;
				ccp->amount = atof(xp);
				if (verbose)
					printf("Amount:\t\t%.2f\n", ccp->amount);
				break;

			case OFXTAG_FITID:
				if (ccp == NULL)
					break;
				if (db_findcreditcardbyfitid(dbp,xp) != NULL) {
					error("duplicate transaction");
					db_creditcardfree(ccp);
					ccp = NULL;
					break;
				}
				ccp->fitid = strdup(xp);
				if (verbose)
					printf("FITID:\t\t%s\n", ccp->fitid);
				break;

			case OFXTAG_NAME:
				if (ccp == NULL)
					break;
				ccp->descr = strdup(xp);
				if (verbose)
					printf("Description:\t%s\n", ccp->descr);
				break;
			}
		}
	}
	return(0);
}

/*
 *
 */
void
findaccount(char *acctno)
{
	char *cp, *xp;
	struct db_bankacct *bap;
	dbow_conn *dbp = getdbase();

	for (bap = db_findbankacctfirst(dbp); bap != NULL;
					bap = db_findbankacctnext(dbp, bap)) {
		for (cp = acctno, xp =bap->number; *cp != '\0'; cp++, xp++) {
			if (*xp == ' ')
				xp++;
			if (*cp != 'X' && *cp != 'x' && *cp != *xp)
				break;
		}
		if (*cp == '\0') {
			nomacct = bap->nomacct_id;
			if (verbose)
				printf("Account: %s\n", bap->name);
			break;
		}
	}
	if (bap != NULL)
		db_bankacctfree(bap);
}

/*
 *
 */
void
error(char *msg)
{
	fprintf(stderr, "\"%s\", line %d: %s.\n", fname, lineno, msg);
	nerrors++;
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: ofxload [-c <comp>] [-v] file.ofx ...\n");
	exit(2);
}
