#ident "Kalopa: $Id: overnight.c,v 1.31 2008/10/31 15:17:13 dtynan Exp $"

/*
 * $Id: overnight.c,v 1.31 2008/10/31 15:17:13 dtynan Exp $
 *
 * Copyright (c) 1996, Claddagh Films Limited.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: overnight.c,v $
 * Revision 1.31  2008/10/31 15:17:13  dtynan
 * Extensive changes for version 0.8.2.
 *
 * Revision 1.30  2008/10/20 15:16:04  dtynan
 * Fixed a bug in the overnight processing.
 *
 * Revision 1.29  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.28  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.27  2008/07/18 11:41:25  dtynan
 * Fixed gotcha; day and month reversed.
 *
 * Revision 1.26  2008/07/18 11:07:37  dtynan
 * Fixed issue with isholiday/skipholiday where it wouldn't actually detect
 * bank holidays (only weekends).  It now interrogates the database.
 *
 * Revision 1.25  2008/07/18 10:55:21  dtynan
 * Output from the overnight account run now goes to a separate log file
 * rather than to an email client which should reduce the email burden.
 * Also, weekly accounts are run on Sunday evenings now.
 *
 * Revision 1.24  2008/02/11 10:34:22  dtynan
 * Added -n option to prevent any kind of update or change.
 *
 * Revision 1.23  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.22  2007/10/29 17:30:03  dtynan
 * Small changes to handle hourly billing.
 *
 * Revision 1.21  2006/10/01 10:32:32  dtynan
 * Major changes for VAT.
 *
 * Revision 1.20  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.19  2005/08/12 18:54:23  dtynan
 * Fixed some bugs, added some new environment variables for DB access
 * and added some missing comments.
 *
 * Revision 1.18  2005/03/22 09:41:22  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.17  2005/03/14 12:06:04  dtynan
 * Don't do overnight processing if 'runorder' is zero.
 *
 * Revision 1.16  2005/02/17 11:08:05  dtynan
 * Make sure standard-error goes to email too.
 *
 * Revision 1.15  2005/02/17 11:00:59  dtynan
 * Uses new format where there are prolog and epilog scripts, and all
 * scripts are executed by being piped to the one email message.
 *
 * Revision 1.14  2004/12/09 20:23:38  dtynan
 * Cleaned up include mechanisms.
 *
 * Revision 1.13  2004/08/04 13:15:00  dtynan
 * Make sure that the company number is specified in all commands.
 *
 * Revision 1.12  2004/07/14 14:45:38  dtynan
 * Fixed bug where overnight updates weren't being posted and added a
 * 'verbose' option to display what the program is doing.
 *
 * Revision 1.11  2004/07/12 19:07:16  dtynan
 * Extensive changes for new directory structure and other things.
 *
 * Revision 1.10  2004/05/12 15:21:19  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.9  2004/01/14 10:50:39  dtynan
 * Fixed accounting scripts to only run on company #1 (for now).
 * Also changed 'overnight' script to use 'isholiday()' function.
 *
 * Revision 1.8  2003/11/08 15:27:57  dtynan
 * Optimized main function to use DBOW functions.
 *
 * Revision 1.7  2003/11/07 16:03:17  dtynan
 * Added cast for localtime() call.
 *
 * Revision 1.6  2003/11/07 16:01:22  dtynan
 * Minor porting changes.
 *
 * Revision 1.5  2003/10/31 17:14:41  dtynan
 * Various changes to use the database and the new/improved system.
 *
 * Revision 1.4  1997/01/04 15:14:16  dtynan
 * Modified so that the acct.monthly file is executed to handle the
 * quarter-end and year-end activities.
 *
 * Revision 1.3  1997/01/03  23:52:30  dtynan
 * Changed arguments to account scripts to have leading zeros.
 *
 * Revision 1.2  1997/01/03  21:47:43  dtynan
 * Upgraded with new functionality.
 *
 * Revision 1.1  1996/10/05  13:09:58  dtynan
 * Basic automated accounting system.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <paths.h>
#include <syslog.h>
#include <stdarg.h>
#include <time.h>
#include <errno.h>

#include "beanie.h"
#include "hacks.h"

int	d, m, y;
int	noeffect;
int	verbose;

struct	db_company	*findcompanyrun(dbow_conn *);

extern	int	optind;
extern	int	opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
void
incday()
{
	if (++d <= mdays(m, y))
		return;
	d = 1;
	if (++m <= 12)
		return;
	y++;
	m = 1;
}

/*
 *
 */
void
export(FILE *pfp, char *ename, char *value)
{
	fprintf(pfp, "%s=%s\nexport %s\n", ename, value, ename);
}

/*
 *
 */
void
execute(struct db_company *comp, char *sname, int d, int m, int y, int q)
{
	char *cp, cmdline[512];
	static FILE *pfp = NULL;

	/*
	 * If 'sname' is NULL, do the final (epilog) processing.
	 * Note that we don't do any epilog processing unless
	 * we already did the prolog processing.
	 */
	if (sname == NULL) {
		if (pfp != NULL) {
			/*
			 * Recursively call 'execute' to run the epilog
			 * script.  We do this so that the actual
			 * command to run the script is only written
			 * in one place.
			 */
			execute(comp, "epilog", 0, 0, 0, 0);
			if (!noeffect)
				pclose(pfp);
			pfp = NULL;
		}
		return;
	}
	/*
	 * Is this the first time we've been called?
	 */
	if (pfp == NULL) {
		/*
		 * Yes - open the pipe for feeding each of the commands
		 * through, and run the prolog script.  Note that the
		 * pipe is actually a bourne shell through which we
		 * feed the actual commands.  The output of this goes
		 * to a mail command for logging.
		 */
		syslog(LOG_INFO, "processing %s accounts on %d/%02d/%04d",
							comp->tag, d, m, y);
		sprintf(cmdline, "%s 2>&1 > %s/overnight-%s-%04d%02d%02d.log",
						_PATH_BSHELL,
						makepath("log", NULL),
						comp->tag, y, m, d);
		if (verbose)
			printf("Executing cmd [%s]\n", cmdline);
		if (noeffect)
			pfp = stdout;
		else {
			/*
			 * Open the pipe...
			 */
			if ((pfp = popen(cmdline, "w")) == NULL) {
				perror("overnight: popen");
				return;
			}
			/*
			 * Set up the environment...
			 */
			cp = makepath("bin", NULL);
			fprintf(pfp, "PATH=%s:/bin:/usr/bin:/usr/local/bin",cp);
			fprintf(pfp, "\nexport PATH\n");
			export(pfp, "BEANIEROOT", makepath(NULL));
			export(pfp, "EMAILADDR", getconfig("from_email", "beanie"));
			export(pfp, "DBUSER", getconfig("dbuser", "beanie"));
			export(pfp, "DBPASSWD", getconfig("dbpass", ""));
			export(pfp, "DBASE", getconfig("dbase", ""));
			fprintf(pfp, "env\n");
		}
		/*
		 * Execute the prolog command-set.
		 */
		execute(comp, "prolog", d, m, y, 0);
	}
	/*
	 * Execute the specific command by sending it to the shell
	 * on the other end of the pipe.
	 */
	if (noeffect) {
		printf(">> %s/acct.%s %s %02d %02d %04d %1d\n",
						makepath("bin", NULL),
						sname, comp->tag, d, m, y, q);
	} else {
		fprintf(pfp, "echo '****'\necho \"**** `date`: ");
		fprintf(pfp, "Running %s script for %s", sname, comp->cname);
		if (d > 0 && m > 0 && y > 0)
			fprintf(pfp, " on %d/%02d/%04d", d, m, y);
		fprintf(pfp, "...\"\necho '****'\n");
		fprintf(pfp, "%s/acct.%s %s %02d %02d %04d %1d\n",
						makepath("bin", NULL),
						sname, comp->tag, d, m, y, q);
	}
}

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, w, q, ed, em, ey, qend[4];
	char *cname;
	struct db_company *cp;
	dbow_conn *dbp;
	struct tm *tp;
	time_t now;

	opterr = verbose = noeffect = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (cname == NULL)
		usage();
	/*
	 * Find the particular company...
	 */
	if (loadconfig(cname) < 0)
		berror("overnight", "cannot load configuration");
	if ((dbp = getdbase()) == NULL)
		berror("overnight", "cannot connect to database");
	if ((cp = db_findcompanyfirst(dbp)) == NULL)
		berror("overnight", "cannot find company");
	if (verbose)
		printf("Running accounts for company %s...\n", cp->cname);
	time(&now);
	if (cp->rdate == 0)
		cp->rdate = now;
	tp = localtime((time_t *)&cp->rdate);
	d = tp->tm_mday; 
	m = tp->tm_mon + 1;
	y = tp->tm_year + 1900;
	w = tp->tm_wday;
	now += (24 * 60 * 60);
	tp = localtime(&now);
	ed = tp->tm_mday; 
	em = tp->tm_mon + 1;
	ey = tp->tm_year + 1900;
	qend[3] = getyend(cp);
	if ((qend[2] = qend[3] - 3) < 1)
		qend[2] += 12;
	if ((qend[1] = qend[2] - 3) < 1)
		qend[1] += 12;
	if ((qend[0] = qend[1] - 3) < 1)
		qend[0] += 12;
	while (d < ed || m < em || y < ey) {
		if (verbose)
			printf("%2d/%02d/%04d...\n", d, m, y);
		/*
		 * If it's not a holiday then run the daily accounts.
		 */
		if (!isholiday(cp->rdate))
			execute(cp, "daily", d, m, y, 0);
		/*
		 * Weekly accounts are executed every Sunday night.
		 */
		if (w == 0)
			execute(cp, "weekly", d, m, y, 0);
		if (d == mdays(m, y)) {
			/*
			 * Run the end of the month accounts.  Also
			 * make sure we handle quarter-ends properly.
			 */
			if (m == qend[0])
				q = 1;
			else if (m == qend[1])
				q = 2;
			else if (m == qend[2])
				q = 3;
			else if (m == qend[3])
				q = 4;
			else
				q = 0;
			execute(cp, "monthly", d, m, y, q);
		}
		cp->rdate += (24 * 60 * 60);
		if (!noeffect && db_updatecompanybyid(dbp, cp, cp->id) < 0)
			syslog(LOG_ERR, "overnight: unable to update database");
		incday();
		w = (w + 1) % 7;
	}
	/*
	 * Do epilog accounts and close pipe.
	 */
	execute(cp, NULL, 0, 0, 0, 0);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: overnight [-nv] -c <comp>\n");
	exit(2);
}
