#ident "$Id: p2cload.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: p2cload.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2007, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: p2cload.c,v $
 * Revision 1.5  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.4  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.3  2008/01/24 10:34:57  dtynan
 * Added the 'noeffect' option and cleaned up the P2C loader so that it updates tables rather than inserting new ones.  There is an issue here
 * though, because it makes it impossible to rerun an old payroll run with
 * the old data.  It might be better to tweak the 'sdate' if it's behind
 * us and set it to the current date and always insert.
 *
 * Revision 1.2  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.1  2008/01/15 15:46:50  dtynan
 * First pass at utilities for integrating the Irish Revenue Online
 * Service (ROS) files with the payroll system.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

#define DEFAULT_PRSI_CODE	"A1"

int			noeffect;
int			verbose;
int			lineno;
char			*fname;
struct db_company	*comp;

extern	int	optind, opterr;
extern	char	*optarg;

void	process(char *);
void	doheader(char *);
void	dotrailer(char *);
void	fetchstr(char *, char *, int);
void	fetchint(char *, int *, int);
void	fetchdate(char *, int *);
void	error(char *);
void	usage();

/*
 *
 */
void
dumpp2c(struct db_iepaye *iep)
{
	printf("ID:\t\t%d\n", iep->id);
	printf("Employee_id:\t%d\n", iep->employee_id);
	printf("Cdate:\t\t%s", ctime(&iep->cdate));
	printf("Sdate:\t\t%s", ctime(&iep->sdate));
	if (iep->edate != 0)
		printf("Edate:\t\t%s", ctime(&iep->edate));
	printf("SRCOP:\t\t%lf\n", iep->asrcop);
	printf("SRCOP (mon):\t%.2f\n", iep->msrcop);
	printf("SRCOP (wk):\t%.2f\n", iep->wsrcop);
	printf("Tax Credit:\t%.2f\n", iep->ataxcr);
	printf("Tax Cr. (mon):\t%.2f\n", iep->mtaxcr);
	printf("Tax Cr. (wk):\t%.2f\n", iep->wtaxcr);
	printf("Week1/Month1:\t%d\n", iep->w1m1f);
	printf("Total Pay:\t%.2f\n", iep->totalpay);
	printf("Total Tax:\t%.2f\n", iep->totaltax);
	printf("Exempt?:\t%d\n", iep->exempt);
	printf("Standard Rate:\t%.4f\n", iep->srate);
	printf("High Rate:\t%.4f\n", iep->hrate);
}

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, state;
	char *cname, input[1024], *cp;
	FILE *fp;
	time_t when;
	dbow_conn *dbp;

	noeffect = verbose = 0;
	cname = NULL;
	time(&when);
	while ((i = getopt(argc, argv, "c:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Find the company that we're processing...
	 */
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("p2cload", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	fname = argv[optind];
	if (verbose)
		printf("Loading P2C data from %s...\n", fname);
	if ((fp = fopen(fname, "r")) == NULL) {
		fprintf(stderr, "p2cload: ");
		perror(fname);
		exit(1);
	}
	/*
	 * Load the P2C file...
	 */
	state = lineno = 0;
	while (fgets(input, sizeof(input), fp) != NULL) {
		if ((cp = strpbrk(input, "\r\n")) != NULL)
			*cp = '\0';
		if (state == 0) {
			if (strncmp(input, "HC", 2) != 0)
				break;
			doheader(input);
			state = 1;
		} else if (state == 1) {
			if (strncmp(input, "TC", 2) == 0) {
				dotrailer(input);
				state = 2;
				break;
			}
			if (strncmp(input, "PC", 2) != 0)
				error("unknown record type");
			else
				process(input);
		}
	}
	fclose(fp);
	exit(0);
}

/*
 *
 */
void
process(char *cp)
{
	int val;
	char ppsno[12], worksno[13], lname[21], fname[21], junk[31];
	char flag[2];
	struct db_employee *ep;
	struct db_iepaye *iep, *oiep;
	time_t now;
	dbow_conn *dbp = getdbase();

	/*
	 * Stuff the structure...
	 */
	time(&now);
	iep = db_iepayealloc();
	fetchstr(cp + 2, ppsno, 9);
	fetchint(cp + 11, &val, 6);
	iep->asrcop = (double )val;
	fetchint(cp + 17, &val, 7);
	iep->msrcop = (double )val / 100.0;
	fetchint(cp + 24, &val, 6);
	iep->wsrcop = (double )val / 100.0;
	fetchstr(cp + 30, worksno, 12);
	fetchint(cp + 42, &val, 6);
	iep->ataxcr = (double )val;
	fetchint(cp + 48, &val, 7);
	iep->mtaxcr = (double )val / 100.0;
	fetchint(cp + 55, &val, 6);
	iep->wtaxcr = (double )val / 100.0;
	fetchdate(cp + 61, &iep->sdate);
	fetchdate(cp + 67, &iep->edate);
	fetchint(cp + 73, (int *)&iep->w1m1f, 1);
	fetchdate(cp + 74, &iep->cdate);
	fetchstr(cp + 80, lname, 20);
	fetchstr(cp + 100, fname, 20);
	fetchstr(cp + 120, junk, 30);
	fetchstr(cp + 150, junk, 30);
	fetchstr(cp + 180, junk, 30);
	fetchint(cp + 210, &val, 15);
	iep->totalpay = (double )val / 100.0;
	fetchint(cp + 225, &val, 15);
	iep->totaltax = (double )val / 100.0;
	fetchstr(cp + 240, flag, 1);
	iep->exempt = (flag[0] == 'Y' || flag[0] == 'y') ? 1 : 0;
	fetchint(cp + 241, &val, 2);
	iep->srate = (double )val / 100.0;
	fetchint(cp + 243, &val, 2);
	iep->hrate = (double )val / 100.0;
	/*
	 * Find the employee (based on the Tax ID)
	 */
	if ((ep = db_findemployeebytaxid(dbp, ppsno)) == NULL)
		berror("p2cload", "cannot find employee");
	iep->employee_id = ep->id;
	/*
	 * Now calculate the previous PRSI code because the Revenue
	 * doesn't provide this.
	 */
	if ((oiep = findtaxcredits(dbp, ep->id, now)) != NULL) {
		iep->prsicode = oiep->prsicode;
		oiep->prsicode = NULL;
	} else
		iep->prsicode = strdup(DEFAULT_PRSI_CODE);
	if (verbose) {
		printf("\n>>> PPS No: \"%s\"\n", ppsno);
		dumpp2c(iep);
	}
	/*
	 * Look to see if the entry already exists...
	 */
	if ((oiep = findtaxcredits(dbp, ep->id, iep->sdate)) != NULL &&
						oiep->sdate == iep->sdate) {
		/*
		 * Existing record - update it.
		 */
		if (verbose)
			printf("Updating...\n");
		iep->id = oiep->id;
		if (!noeffect && db_updateiepayebyid(dbp, iep, oiep->id) < 0)
			berror("p2cload", "cannot insert data");
	} else {
		/*
		 * Insert the new entry.
		 */
		if (verbose)
			printf("Creating a new record...\n");
		if (!noeffect && db_insertiepaye(dbp, iep) < 0)
			berror("p2cload", "cannot insert data");
	}
}

/*
 *
 */
void
doheader(char *cp)
{
	int tyear, lyear, date, unit;
	char empno[9], empname[31];

	fetchint(cp + 2, &tyear, 4);
	fetchstr(cp + 6, empno, 8);
	fetchstr(cp + 14, empname, 30);
	fetchint(cp + 44, &date, 6);
	fetchint(cp + 50, &lyear, 4);
	fetchint(cp + 54, &unit, 3);
	if (strcmp(comp->taxno, empno) != 0) {
		error("tax registration does not match selected company");
		exit(1);
	}
	if (verbose) {
		printf("Tax Year:\t\t%d\n", tyear);
		printf("Employer No:\t\t\"%s\"\n", empno);
		printf("Employer Name:\t\t\"%s\"\n", empname);
		printf("Date:\t\t\t%d\n", date);
		printf("Label Year:\t\t%d\n", lyear);
		printf("Unit No:\t\t%d\n", unit);
	}
}

/*
 *
 */
void
dotrailer(char *cp)
{
	int tsrcop, tmsrcop, twsrcop, ttaxcr, tmtaxcr, twtaxcr, nrec;
	char empno[9];

	fetchstr(cp + 2, empno, 8);
	fetchint(cp + 10, &tsrcop, 10);
	fetchint(cp + 20, &tmsrcop, 12);
	fetchint(cp + 32, &twsrcop, 12);
	fetchint(cp + 44, &ttaxcr, 10);
	fetchint(cp + 54, &tmtaxcr, 12);
	fetchint(cp + 66, &twtaxcr, 12);
	fetchint(cp + 78, &nrec, 5);
	if (verbose) {
		printf("\nEmployer No:\t\t\"%s\"\n", empno);
		printf("Total SRCOP:\t\t%d\n", tsrcop);
		printf("Tot. Monthly SRCOP:\t%d\n", tmsrcop);
		printf("Tot. Weekly SRCOP:\t%d\n", twsrcop);
		printf("Total Tax Credits:\t%d\n", ttaxcr);
		printf("Tot. Monthly Credits:\t%d\n", tmtaxcr);
		printf("Tot. Weekly Credits:\t%d\n", twtaxcr);
		printf("No. of Records:\t\t%d\n", nrec);
	}
}

/*
 *
 */
void
fetchstr(char *src, char *dst, int len)
{
	int ch, bflag = 0;

	while (len--) {
		ch = *src++;
		if (bflag == 0 && isspace(ch))
			continue;
		bflag = 1;
		*dst++ = ch;
	}
	if (bflag) {
		while (isspace(*--dst))
			;
		dst++;
	}
	*dst = '\0';
}

/*
 *
 */
void
fetchdate(char *src, int *dst)
{
	char date1[8], date2[8];

	*dst = 0;
	fetchstr(src, date1, 6);
	date1[6] = date2[6] = '\0';
	date2[0] = date1[4];
	date2[1] = date1[5];
	date2[2] = date1[2];
	date2[3] = date1[3];
	date2[4] = date1[0];
	date2[5] = date1[1];
	parsedate(date2, (time_t *)dst);
	/*
	 * Because parsedate sets the time to 6PM and DBOW sets it to noon,
	 * we need to adjust here...
	 */
	if (*dst > HOURS(6))
		*dst -= HOURS(6);
}

/*
 *
 */
void
fetchint(char *src, int *dst, int len)
{
	char maxint[16];

	*dst = 0;
	fetchstr(src, maxint, len);
	if (maxint[0] != '\0')
		*dst = atoi(maxint);
}

/*
 *
 */
void
error(char *errmsg)
{
	printf("\"%s\", line %d: %s.\n", fname, lineno, errmsg);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: p2cload [-c <comp>][-nv] <p2cfile>\n");
	exit(2);
}
