#ident "$Id: p30gen.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: p30gen.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: p30gen.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:43:24  dtynan
 * Added new program to handle generating a P30.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#include "p30.h"

int	noeffect;
int	verbose;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, year, ipaye, iprsi;
	char *cname;
	dbow_conn *dbp;
	double paye, prsi;
	struct db_company *comp;
	struct db_payroll *prp;
	struct db_p30_data *pdp;
	struct db_paye_p30 *pxp, *mypxp;
	struct tm *tmp;
	time_t when, idate;

	noeffect = verbose = 0;
	cname = NULL;
	time(&when);
	while ((i = getopt(argc, argv, "c:d:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				berror("p30gen", "bad cli date format");
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Find the company that we're processing...
	 */
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("p30gen", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	if (verbose)
		printf("Generating P30 data for %s", ctime(&when));
	/*
	 * What year are we talking about?
	 */
	if ((tmp = localtime(&when)) != NULL)
		year = tmp->tm_year;
	else
		year = 100;
	if (verbose)
		printf("Year: %d\n", year + 1900);
	tmp->tm_mday = 1;
	tmp->tm_hour = 12;
	idate = mktime(tmp);
	/*
	 * Calculate the P30s which have been filed to date.
	 */
	paye = prsi = 0.0;
	for (mypxp = NULL, pxp = findallp30s(dbp); pxp != NULL;
				pxp = db_findpaye_p30next(dbp, pxp)) {
		if ((tmp = localtime((time_t *)&pxp->idate)) == NULL ||
							tmp->tm_year != year)
			continue;
		if (mypxp == NULL || mypxp->idate < pxp->idate)
			mypxp = pxp;
		printf("> ID: %d\n", pxp->id);
		printf(">> PAYE: %d\n", pxp->paye);
		printf(">> PRSI: %d\n", pxp->prsi);
		paye += (double )pxp->paye;
		prsi += (double )pxp->prsi;
	}
	printf("PAYE paid to date: %.2f\n", paye);
	printf("PRSI paid to date: %.2f\n", prsi);
	/*
	 * Calculate the YTD PAYE and PRSI records...
	 */
	for (pdp = db_findp30_datafirst(dbp); pdp != NULL;
				pdp = db_findp30_datanext(dbp, pdp)) {
		if ((prp = db_findpayrollbyid(dbp, pdp->payroll_id)) == NULL)
			berror("p30gen", "can't find payroll record");
		if ((tmp = localtime((time_t *)&prp->pdate)) == NULL ||
							prp->pdate > when ||
							tmp->tm_year != year)
			continue;
		paye += pdp->paye_amt;
		prsi += pdp->prsi_amt;
	}
	printf("Total PAYE: %.2f\n", paye);
	printf("Total PRSI: %.2f\n", prsi);
	ipaye = (int )roundoff(paye, ROUNDOFF_INTEGER);
	iprsi = (int )roundoff(prsi, ROUNDOFF_INTEGER);
	printf("Net PAYE:  %d\n", ipaye);
	printf("Net PRSI:  %d\n", iprsi);
	/*
	 * Now collate the P30s which have been filed.
	 */
	for (mypxp = NULL, pxp = findallp30s(dbp); pxp != NULL;
				pxp = db_findpaye_p30next(dbp, pxp)) {
		if ((tmp = localtime((time_t *)&pxp->idate)) == NULL ||
							tmp->tm_year != year)
			continue;
		if (pxp->idate == idate)
			mypxp = pxp;
		printf("> ID: %d\n", pxp->id);
		printf(">> PAYE: %d\n", pxp->paye);
		printf(">> PRSI: %d\n", pxp->prsi);
		paye -= (double )pxp->paye;
		prsi -= (double )pxp->prsi;
	}
	ipaye = (int )roundoff(paye, ROUNDOFF_INTEGER);
	iprsi = (int )roundoff(prsi, ROUNDOFF_INTEGER);
	printf("Net PAYE:  %d\n", ipaye);
	printf("Net PRSI:  %d\n", iprsi);
	printf("Total Due: %d\n", ipaye + iprsi);
	if (mypxp != NULL) {
		printf("My ID: %d\n", mypxp->id);
		printf("PAYE Was: %d\n", mypxp->paye);
		printf("PRSI Was: %d\n", mypxp->prsi);
		pxp = mypxp;
		pxp->paye = ipaye;
		pxp->prsi = iprsi;
		if (!noeffect && db_updatepaye_p30byid(dbp, pxp, pxp->id) < 0)
			berror("p30gen", "cannot update p30 record");
	} else {
		printf("New record.\n");
		pxp = db_paye_p30alloc();
		pxp->idate = idate;
		pxp->sdate = 0;
		pxp->notice = strdup(makenotice(comp));
		pxp->paye = ipaye;
		pxp->prsi = iprsi;
		if (!noeffect && db_insertpaye_p30(dbp, pxp) < 0)
			berror("p30gen", "cannot insert p30 record");
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: p30gen [-c <comp>][-nv][-d <taxdate>]\n");
	exit(2);
}
