#ident "$Id: p30reminder.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: p30reminder.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2007, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: p30reminder.c,v $
 * Revision 1.5  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.4  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.3  2008/01/24 10:34:57  dtynan
 * Added the 'noeffect' option and cleaned up the P2C loader so
 * that it updates tables rather than inserting new ones.  There is an
 * issue here though, because it makes it impossible to rerun an old
 * payroll run with the old data.  It might be better to tweak the 'sdate'
 * if it's behind us and set it to the current date and always insert.
 *
 * Revision 1.2  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.1  2008/01/15 15:46:50  dtynan
 * First pass at utilities for integrating the Irish Revenue Online
 * Service (ROS) files with the payroll system.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#include "p30.h"

int	noeffect;
int	verbose;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	dbow_conn *dbp;
	struct db_company *comp;
	struct db_paye_p30 *p30p;
	struct tm *tmp;
	time_t when;

	noeffect = verbose = 0;
	cname = NULL;
	time(&when);
	while ((i = getopt(argc, argv, "c:d:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				berror("p30reminder", "bad cli date format");
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("p30reminder", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	if (verbose)
		printf("Generating P30 reminders for %s", ctime(&when));
	tmp = localtime(&when);
	tmp->tm_mday = 1;
	when = mktime(tmp);
	if ((p30p = db_findpaye_p30byidate(dbp, when)) == NULL) {
		p30p = db_paye_p30alloc();
		p30p->idate = when;
		p30p->sdate = 0;
		p30p->notice = strdup(makenotice(comp));
		p30p->paye = p30p->prsi = 0.0;
		if (!noeffect && db_insertpaye_p30(dbp, p30p) < 0)
			berror("p30reminder", "cannot insert p30 record");
	}
	db_dumppaye_p30(p30p, stdout);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: p30reminder [-c <comp>][-nv][-d <taxdate>]\n");
	exit(2);
}
