#ident "Kalopa: $Id: hours.c,v 1.5 2008/10/31 15:17:13 dtynan Exp $"

/*
 * $Id: hours.c,v 1.5 2008/10/31 15:17:13 dtynan Exp $
 *
 * Copyright (c) 2008, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: hours.c,v $
 * Revision 1.5  2008/10/31 15:17:13  dtynan
 * Extensive changes for version 0.8.2.
 *
 * Revision 1.4  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.3  2008/10/13 20:59:37  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.2  2008/07/18 11:07:38  dtynan
 * Fixed issue with isholiday/skipholiday where it wouldn't actually detect
 * bank holidays (only weekends).  It now interrogates the database.
 *
 * Revision 1.1  2008/07/08 16:44:27  dtynan
 * Added a new program to report consulting/contracting hours.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

int	noeffect;
int	verbose;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, soid, item;
	char *cname, dstr[64];
	double val;
	struct db_company *comp;
	struct db_hours *hp;
	dbow_conn *dbp;
	time_t first, when;

	noeffect = verbose = 0;
	cname = NULL;
	time(&when);
	first = 0;
	while ((i = getopt(argc, argv, "c:d:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				usage();
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("hours", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	for (hp = db_findhoursfirst(dbp); hp != NULL;
					hp = db_findhoursnext(dbp, hp)) {
		if (hp->hdate > first) {
			first = hp->hdate;
			soid  = hp->salesorder_id;
			item  = hp->item_id;
		}
	}
	if (first == 0)
		berror("hours", "no previous hour data");
	first += HOURS(24);
	hp = db_hoursalloc();
	hp->salesorder_id = soid;
	hp->item_id = item;
	hp->invoice_id = 0;
	while (first < when) {
		val = isholiday(first) ? 0.0 : 8.0;
		strftime(dstr, sizeof(dstr), "%D (%a)", localtime(&first));
		printf("%s ", dstr);
		sprintf(dstr, "%3.1f", val);
		val = atof(getanswer("Hours:", dstr));
		if (val > 0.0) {
			hp->id = 0;
			hp->hdate = first;
			hp->hours = (int )val;
			if (!noeffect && db_inserthours(dbp, hp) < 0)
				berror("hours", "cannot insert hours");
		}
		first += HOURS(24);
	}
	exit(0);
}

/*
	id mediumint(7) not null auto_increment primary key,
	salesorder_id mediumint(7) not null,
	item_id mediumint(7) not null,
	invoice_id mediumint(7),
	hdate date not null,
	hours tinyint(3) not null
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: hours [-c #][-d date][-nv]\n");
	exit(2);
}
