#ident "Kalopa: $Id: payslip.c,v 1.4 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: payslip.c,v 1.4 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2007, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: payslip.c,v $
 * Revision 1.4  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.3  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.2  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.1  2008/01/15 14:10:17  dtynan
 * First pass at producing a payroll system (currently heavily biased for
 * the Irish PAYE system).
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#define EURO		"\\(Eu"

int	verbose;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	struct db_company *comp;
	struct db_payroll *pp;
	struct db_employee *ep;
	struct db_contact *cp;
	struct tm *tmp;
	double totpay, totdeds;
	dbow_conn *dbp;
	FILE *fp;

	verbose = 0;
	cname = NULL;
	fp = stdout;
	while ((i = getopt(argc, argv, "c:v")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Find the company that we're processing...
	 */
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("payslip", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	/*
	 * Find the payroll entry...
	 */
	if ((pp = db_findpayrollbyid(dbp, atoi(argv[optind]))) == NULL)
		berror("payslip", "cannot find payroll entry");
	if ((ep = db_findemployeebyid(dbp, pp->employee_id)) == NULL)
		berror("payslip", "cannot find employee details");
	if ((cp = contact_findbyid(dbp, ep->contact_id)) == NULL)
		berror("payslip", "cannot find contact details");
	fprintf(fp, ".ds EmpName %s\n", orgname(cp));
	fprintf(fp, ".ds EmpNo %d\n", ep->id);
	fprintf(fp, ".ds EmpPPS %s\n", ep->taxid);
	fprintf(fp, ".ds Dept 111\n");
	fprintf(fp, ".ds Period %d\n", pp->period);
	if ((tmp = localtime((time_t *)&pp->pdate)) == NULL)
		berror("payslip", "cannot convert date");
	fprintf(fp, ".ds PayDate %d/%02d/%04d\n", tmp->tm_mday, tmp->tm_mon + 1,
						tmp->tm_year + 1900);
	fprintf(fp, ".ds PayDesc1 BASIC PAY\n");
	fprintf(fp, ".ds PayAmount1 %s\n", bal(pp->basicpay, NULL));
	/* FIXME */
	fprintf(fp, ".ds DedDesc1 P.A.Y.E.\n");
	fprintf(fp, ".ds DedAmount1 %s\n", bal(pp->ded1_amt, NULL));
	fprintf(fp, ".ds DedTotal1 %s\n", bal(pp->ded1_ytd, NULL));
	fprintf(fp, ".ds DedDesc2 P.R.S.I.\n");
	fprintf(fp, ".ds DedAmount2 %s\n", bal(pp->ded2_amt, NULL));
	fprintf(fp, ".ds DedTotal2 %s\n", bal(pp->ded2_ytd, NULL));
	totpay = pp->basicpay;
	fprintf(fp, ".ds GrossPay %s\n", bal(totpay, EURO));
	totdeds = pp->ded1_amt + pp->ded2_amt;
	fprintf(fp, ".ds TotalDeds %s\n", bal(totdeds, NULL));
	fprintf(fp, ".ds NettPay %s\n", bal(totpay - totdeds, EURO));
	fprintf(fp, ".ds PayMethod EFT\n");
	fprintf(fp, ".ds CumGrossPay %s\n", bal(pp->grossytd, EURO));
	fprintf(fp, ".ds CumTaxPay %s\n", bal(pp->grossytd, NULL));
	fprintf(fp, ".ds CumTaxCredit %s\n", bal(pp->note3_ytd, NULL));
	fprintf(fp, ".ds CumCutOff %s\n", bal(pp->note2_ytd, NULL));
	fprintf(fp, ".ds CumTaxPaid %s\n", bal(pp->ded1_ytd, EURO));
	fprintf(fp, ".ds TaxCode N\n");
	fprintf(fp, ".ds TaxCredit %s\n", bal(pp->note3_amt, NULL));
	fprintf(fp, ".ds TaxPRSICode %s\n", pp->comm1);
	fprintf(fp, ".ds TaxInsWeeks %s\n", pp->comm2);
	fprintf(fp, ".ds ComEyerPRSI %s\n", bal(pp->note1_amt, NULL));
	fprintf(fp, ".ds ComEyerPRSITD %s\n", bal(pp->note1_ytd, NULL));
	fprintf(fp, ".nr PrePrint 1\n");
	fprintf(fp, ".nr Company %d\n", comp->id);
	fprintf(fp, ".so %s.tmac\n", makepath("roff", "payslip", NULL));
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: payslip [-c <comp>][-v] <payroll.id>\n");
	exit(2);
}
