#ident "Kalopa: $Id: runpayroll.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: runpayroll.c,v 1.5 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2007, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: runpayroll.c,v $
 * Revision 1.5  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.4  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.3  2008/07/08 16:44:27  dtynan
 * Added a new program to report consulting/contracting hours.
 *
 * Revision 1.2  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.1  2008/01/15 14:10:17  dtynan
 * First pass at producing a payroll system (currently heavily biased for
 * the Irish PAYE system).
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#define SALARY_LIABILITY	3300
#define TAXES_LIABILITY		3420
#define PAYROLL_EXPENSE		6000
#define PRSI_EXPENSE		6100

int	period;
int	wkmon;
int	year;
int	noeffect;
int	verbose;
time_t	when;

extern	int	optind, opterr;
extern	char	*optarg;

void	payroll(struct db_employee *ep, struct db_salary *sp);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	struct db_company *comp;
	struct db_employee *ep;
	struct db_salary *sp;
	struct tm *tmp;
	dbow_conn *dbp;

	noeffect = verbose = period = 0;
	cname = NULL;
	time(&when);
	while ((i = getopt(argc, argv, "c:d:mwnv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				usage();
			break;

		case 'm':
			period = FREQ_MONTHLY;
			break;

		case 'w':
			period = FREQ_WEEKLY;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (period == 0)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Find the company that we're processing...
	 */
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("runpayroll", "cannot find company");
	if (verbose) {
		printf("Company: %s\n", comp->cname);
		printf("Running %s Payroll...\n", period == FREQ_WEEKLY ?
						"Weekly" : "Monthly");
	}
	/*
	 * Calculate the effective week/month and year of the payroll
	 * run.  FIXME:  The week is not computed correctly.
	 */
	if ((tmp = localtime(&when)) == NULL)
		berror("runpayroll", "cannot determine the date");
	year = tmp->tm_year + 1900;
	if (period == FREQ_MONTHLY)
		wkmon = tmp->tm_mon + 1;
	else
		wkmon = (tmp->tm_yday / 7) + 1;
	if (verbose)
		printf("Payroll period %d/%d\n", wkmon, year);
	/*
	 * Scan through the database of eligible employees...
	 */
	for (ep = db_findemployeefirst(dbp); ep != NULL;
					ep = db_findemployeenext(dbp, ep)) {
		/*
		 * Is the employee still with the company?
		 */
		if (ep->edate != 0 && ep->edate < when)
			continue;
		if ((sp = findsalary(dbp, ep->id, when)) == NULL) {
			/*
			 * No salary for the employee - probably not on the
			 * payroll at the effective date.
			 */
			continue;
		}
		/*
		 * Is this employee monthly (and we're running monthly)?
		 * Or vice-versa?
		 */
		if (sp->sfreq == period)
			payroll(ep, sp);
	}
	exit(0);
}

/*
 *
 */
void
payroll(struct db_employee *ep, struct db_salary *sp)
{
	double totalpay;
	double tax, srcop, credits, prsipay, prsidelta, erprsi, eeprsi;
	struct db_contact *cp;
	struct db_iepaye *iep;
	struct db_ieprsi *irp;
	struct db_payroll *pp, *opp;
	struct db_prbonus *bp;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct db_p30_data *pdp;
	struct tm *tmp;
	char descr[256];
	dbow_conn *dbp = getdbase();

	if ((cp = contact_findbyid(dbp, ep->contact_id)) == NULL)
		berror("runpayroll", "cannot find contact details");
	if (verbose)
		printf("**** Processing employee %s ****\n", orgname(cp));
	/*
	 * Stuff the payroll struct and save it.
	 */
	if ((pp = db_payrollalloc()) == NULL)
		berror("runpayroll", "cannot allocate payroll entry");
	pp->employee_id = ep->id;
	pp->pdate = when;
	pp->period = wkmon;
	pp->pay1_amt = pp->pay2_amt = pp->pay3_amt = pp->pay4_amt = 0.0;
	/*
	 * Look for any bonuses.
	 */
	printf("Salary: %.2f\n", sp->salary);
	if ((bp = findbonus(dbp, ep->id, when)) != NULL) {
		printf("Bonus: %.2f\n", bp->amount);
		pp->pay1_id = PAYROLL_BONUS;
		pp->pay1_amt = bp->amount;
	}
	/*
	 * Determine cumulative amounts...
	 */
	for (opp = db_findpayrollbyemployee_id(dbp, ep->id);
				opp != NULL;
				opp = db_findpayrollnext(dbp, opp)) {
		if ((tmp = localtime((time_t *)&opp->pdate)) == NULL)
			continue;
		if (tmp->tm_year + 1900 != year)
			continue;
		if (verbose)
			printf("Previous Payroll ID: %d\n", opp->id);
		pp->grossytd += opp->basicpay;
		pp->grossytd += opp->pay1_amt;
		pp->grossytd += opp->pay2_amt;
		pp->grossytd += opp->pay3_amt;
		pp->grossytd += opp->pay4_amt;
		if (opp->ded1_id != PAYE_DEDUCTION)
			berror("runpayroll", "invalid PAYE entry");
		pp->ded1_ytd += opp->ded1_amt;
		if (opp->ded2_id != 0)
			pp->ded2_ytd += opp->ded2_amt;
		if (opp->ded3_id != 0)
			pp->ded3_ytd += opp->ded3_amt;
		if (opp->ded4_id != 0)
			pp->ded4_ytd += opp->ded4_amt;
		if (opp->note1_id != 0)
			pp->note1_ytd += opp->note1_amt;
		if (opp->note2_id != 0)
			pp->note2_ytd += opp->note2_amt;
		if (opp->note3_id != 0)
			pp->note3_ytd += opp->note3_amt;
		if (opp->note4_id != 0)
			pp->note4_ytd += opp->note4_amt;
	}
	/*
	 * Find tax credits...
	 */
	if ((iep = findtaxcredits(dbp, ep->id, when)) == NULL)
		berror("runpayroll", "no tax credits for employee");
	if ((tmp = localtime((time_t *)&iep->cdate)) == NULL)
		berror("runpayroll", "cannot determine IEP date");
	if (tmp->tm_year + 1900 == year) {
		/*
		 * Add in any cumulative totals from the
		 * P2C data if we're in the same tax year
		 * as that of the certificate.
		 */
		pp->grossytd += iep->totalpay;
		pp->ded1_ytd += iep->totaltax;
	}
	pp->grossytd = roundoff(pp->grossytd, ROUNDOFF_NORMAL);
	/*
	 * Set some of the payroll "comment" fields for use
	 * by the payslip software.  'comm1' is the PRSI
	 * Code and 'comm2' is the "insurance weeks",
	 * whatever that is.  Currently defaulted to 6.
	 */
	pp->comm1 = strdup(iep->prsicode);
	pp->comm2 = strdup("6");
	/*
	 * Begin payroll/tax computation...
	 */
	totalpay = pp->basicpay = sp->salary;
	totalpay += pp->pay1_amt;
	totalpay += pp->pay2_amt;
	totalpay += pp->pay3_amt;
	totalpay += pp->pay4_amt;
	pp->grossytd += totalpay;
	pp->note2_id = NOTE_SRCOP;
	pp->note3_id = NOTE_CREDITS;
	if (period == FREQ_MONTHLY) {
		pp->note2_amt = iep->msrcop;
		pp->note3_amt = iep->mtaxcr;
		srcop = roundoff((double )wkmon * iep->msrcop, ROUNDOFF_NORMAL);
		credits = roundoff((double )wkmon*iep->mtaxcr, ROUNDOFF_NORMAL);
	} else {
		/*
		 * FIXME:  What about 53-week years?
		 */
		pp->note2_amt = iep->wsrcop;
		pp->note3_amt = iep->wtaxcr;
		srcop = roundoff((double )wkmon * iep->wsrcop, ROUNDOFF_NORMAL);
		credits = roundoff((double )wkmon*iep->wtaxcr, ROUNDOFF_NORMAL);
	}
	pp->note2_ytd = srcop;
	pp->note3_ytd = credits;
	/*
	 * Do the tax computation...
	 */
	if (pp->grossytd <= srcop) {
		/*
		 * Pay is below the standard-rate cutoff
		 * point (SRCOP) - all tax is at the lower
		 * rate.
		 */
		tax = pp->grossytd * iep->srate;
	} else {
		/*
		 * Tax up to the SRCOP at the low rate, and
		 * the balance at the higher rate.
		 */
		tax = srcop * iep->srate;
		tax += (pp->grossytd - srcop) * iep->hrate;
	}
	/*
	 * Calculate PRSI due...
	 */
	prsipay = totalpay;
	eeprsi = erprsi = 0.0;
	for (irp = findprsibands(dbp, iep->prsicode, when);
				irp != NULL && prsipay > 0.0;
				irp = db_findieprsinext(dbp, irp)) {
		if (period == FREQ_MONTHLY)
			irp->thresh = (irp->thresh * 52.0 / 12.0);
		if (prsipay > irp->thresh)
			prsidelta = irp->thresh;
		else
			prsidelta = prsipay;
		printf("PRSI Data:-\n");
		printf("> Delta:\t%.2f\n", prsidelta);
		printf("> ID:\t%d\n", irp->id);
		printf("> Code:\t%s\n", irp->prsicode);
		printf("> Thresh:\t%.2f\n", irp->thresh);
		printf("> Plimit:\t%.2f\n", irp->plimit);
		printf("> EEl:\t%.2f\n", irp->eelower);
		printf("> ERl:\t%.2f\n", irp->erlower);
		printf("> EEu:\t%.2f\n", irp->eeupper);
		printf("> ERu:\t%.2f\n", irp->erupper);
		if (pp->grossytd <= irp->plimit) {
			eeprsi += prsidelta * irp->eelower / 100.0;
			erprsi += prsidelta * irp->erlower / 100.0;
		} else {
			eeprsi += prsidelta * irp->eeupper / 100.0;
			erprsi += prsidelta * irp->erupper / 100.0;
		}
		printf("> EE: %.2f\n", eeprsi);
		printf("> ER: %.2f\n", erprsi);
		prsipay -= prsidelta;
	}
	printf("Gross Pay:\t%.2f\n", totalpay);
	printf("Cum. Pay:\t%.2f\n", pp->grossytd);
	printf("Cum. Tax:\t%.2f\n", pp->ded1_ytd);
	printf("SRCOP:\t\t%.2f\n", srcop);
	printf("Tax Balance:\t%.2f\n", tax);
	printf("Credits:\t%.2f\n", credits);
	printf("Tax to Date:\t%.2f\n", pp->ded1_ytd);
	if (credits > tax)
		tax = 0.0;
	else
		tax -= credits;
	tax -= pp->ded1_ytd;
	/*
	 * Round off the tax amounts.
	 */
	tax    = roundoff(tax, ROUNDOFF_NORMAL);
	eeprsi = roundoff(eeprsi, ROUNDOFF_NORMAL);
	erprsi = roundoff(erprsi, ROUNDOFF_NORMAL);
	if (verbose) {
		printf("Tax Due:\t%.2f\n", tax);
		printf("E'er PRSI:\t%.2f\n", erprsi);
		printf("E'ee PRSI:\t%.2f\n", eeprsi);
	}
	/*
	 * Now update the payroll record...
	 */
	pp->ded1_id = PAYE_DEDUCTION;
	pp->ded1_amt = tax;
	pp->ded1_ytd += tax;
	pp->ded2_id = PRSI_DEDUCTION;
	pp->ded2_amt = eeprsi;
	pp->ded2_ytd += eeprsi;
	pp->note1_id = NOTE_ERPRSI;
	pp->note1_amt = erprsi;
	pp->note1_ytd += erprsi;
	if (!noeffect && db_insertpayroll(dbp, pp) < 0)
		berror("runpayroll", "cannot insert payroll entry");
	if (verbose)
		printf("New payroll ID: %d\n", pp->id);
	/*
	 * Mark the bonus entry as "burnt".
	 */
	if (bp != NULL && !noeffect) {
		bp->payroll_id = pp->id;
		if (db_updateprbonusbyid(dbp, bp, bp->id) < 0)
			berror("runpayroll", "cannot update bonus");
		db_prbonusfree(bp);
		bp = NULL;
	}
	/*
	 * Create the journal entries.
	 */
	if ((jp = db_journalalloc()) == NULL)
		berror("runpayroll", "cannot allocate journal entry");
	jp->jdate = when;
	jp->folio = NULL;
	sprintf(descr, "Payroll for %s", orgname(cp));
	jp->descr = strdup(descr);
	jp->user_id = getuid();
	jp->state = 0;
	if (!noeffect && db_insertjournal(dbp, jp) < 0)
		berror("runpayroll", "cannot insert journal entry");
	if ((jip = db_journal_itemalloc()) == NULL)
		berror("runpayroll", "cannot allocate journal item");
	if (verbose)
		printf(">> Journal ID: %d\n", jp->id);
	jip->journal_id = jp->id;
	db_journalfree(jp);
	jip->item_id = 1;
	jip->nomacct_id = PAYROLL_EXPENSE;
	jip->drcrf = 0;
	jip->amount = totalpay;
	if (verbose)
		printf(">> Payroll Expense:\t%.2f\n", jip->amount);
	if (!noeffect && db_insertjournal_item(dbp, jip) < 0)
		berror("runpayroll", "cannot insert journal item");
	jip->item_id++;
	jip->nomacct_id = PRSI_EXPENSE;
	jip->drcrf = 0;
	jip->amount = erprsi;
	if (verbose)
		printf(">> PRSI Expense:\t%.2f\n", jip->amount);
	if (!noeffect && db_insertjournal_item(dbp, jip) < 0)
		berror("runpayroll", "cannot insert journal item");
	jip->item_id++;
	jip->nomacct_id = SALARY_LIABILITY;
	jip->drcrf = 1;
	jip->amount = totalpay - eeprsi - tax;
	if (verbose)
		printf(">> Salary Liability:\t%.2f\n", jip->amount);
	if (!noeffect && db_insertjournal_item(dbp, jip) < 0)
		berror("runpayroll", "cannot insert journal item");
	jip->item_id++;
	jip->nomacct_id = TAXES_LIABILITY;
	jip->drcrf = 1;
	jip->amount = eeprsi + erprsi + tax;
	if (verbose)
		printf(">> Tax Liability:\t%.2f\n", jip->amount);
	if (!noeffect && db_insertjournal_item(dbp, jip) < 0)
		berror("runpayroll", "cannot insert journal item");
	db_journal_itemfree(jip);
	/*
	 * Create a P30 data record so we can produce the
	 * end of month P30s.
	 */
	pdp = db_p30_dataalloc();
	pdp->payroll_id = pp->id;
	pdp->paye_amt = tax;
	pdp->prsi_amt = eeprsi + erprsi;
	printf("P30 PAYE: %.2f\n", pdp->paye_amt);
	printf("P30 PRSI: %.2f\n", pdp->prsi_amt);
	if (!noeffect && db_insertp30_data(dbp, pdp) < 0)
		berror("runpayroll", "cannot insert p30 data record");
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: runpayroll [-c <comp>][-d date][-nv]{-m|-w}\n");
	exit(2);
}
