/* */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

int	noeffect;
int	verbose;
int	errflag;
char	*fname;
char	input[1024];

struct db_company *comp;

void	error(char *, int, char *);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, acno, lineno, num;
	char *cname, *cp, *descr;
	struct db_bankacct *bap;
	struct db_bank_stmnt *bsp;
	struct db_bankdata *bdp;
	struct tm *tmp;
	double amount;
	time_t when, sdate, edate;
	dbow_conn *dbp;
	FILE *fp;

	/*
	 *
	 */
	verbose = noeffect = acno = sdate = edate = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "a:c:s:e:nv")) != EOF) {
		switch (i) {
		case 'a':
			if ((acno = atoi(optarg)) < 1)
				usage();
			break;

		case 'c':
			cname = optarg;
			break;

		case 's':
			if (parsedate(optarg, &sdate) < 0)
				usage();
			break;

		case 'e':
			if (parsedate(optarg, &edate) < 0)
				usage();
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1 || acno == 0 || (sdate == 0 && edate == 0))
		usage();
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("qifread", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	/*
	 * Find the appropriate bank account...
	 */
	if ((bap = db_findbankacctbyid(dbp, acno)) == NULL)
		berror("qifread", "cannot find bank account");
	if (verbose)
		printf("Bank Account:\t%s\n", bap->acname);
	/*
	 * Create the appropriate bank statement entry.
	 */
	if ((bsp = findstatement(dbp, bap->id, when)) != NULL)
		berror("qifread", "statement is already in the database");
	if ((bsp = db_bank_stmntalloc()) == NULL)
		berror("qifread", "cannot allocate a bank statement");
	bsp->bankacct_id = bap->id;
	if (sdate != 0) {
		bsp->sdate = sdate;
		tmp = localtime(&sdate);
		tmp->tm_mon++;
		bsp->edate = mktime(tmp) - DAYS(1);
	} else {
		bsp->edate = edate;
		tmp = localtime(&edate);
		tmp->tm_mon--;
		bsp->sdate = mktime(tmp) + DAYS(1);
	}
	if (verbose) {
		printf("Starts:\t%s", ctime((time_t *)&bsp->sdate));
		printf("Ends:\t%s", ctime((time_t *)&bsp->edate));
	}
	if (!noeffect && db_insertbank_stmnt(dbp, bsp))
		berror("qifread", "cannot create a bank statement");
	if (verbose)
		printf("New bank statement: %d\n", bsp->id);
	/*
	 * Now, start processing the QIF file...
	 */
	if ((fp = fopen(argv[optind], "r")) == NULL)
		berror("qifread", "cannot open file");
	lineno = 0;
	num = 0;
	when = 0;
	descr = NULL;
	amount = 0.0;
	while (fgets(input, sizeof(input), fp) != NULL) {
		if ((cp = strpbrk(input, "\n\r")) != NULL)
			*cp = '\0';
		cp = input;
		lineno++;
		switch (*cp++) {
		case '#':
			/*
			 * Ignore comments...
			 */
			break;

		case '!':
			/*
			 * File header.
			 */
			if (strncmp(cp, "Type:", 5) != 0) {
				error(fname, lineno, "unknown QIF file");
				break;
			}
			cp += 5;
			if (verbose)
				printf("File type: \"%s\"\n", cp);
			break;

		case 'A':
			/*
			 * Address record - we don't really care.
			 */
			break;

		case 'D':
			/*
			 * Date record.
			 */
			if (parsedate(cp, &when) < 0) {
				error(fname, lineno, "invalid date format");
				break;
			}
			break;

		case 'N':
			/*
			 * Number field.
			 */
			num = atoi(cp);
			break;

		case 'P':
			/*
			 * Description.
			 */
			descr = strdup(cp);
			break;

		case 'T':
			/*
			 * Amount.
			 */
			amount = atof(cp);
			break;

		case '^':
			/*
			 * End of record.
			 */
			if (errflag) {
				errflag = 0;
				break;
			}
			/*
			 * Create the new record.
			 */
			if ((bdp = db_bankdataalloc()) == NULL)
				berror("qifread", "bankdata alloc");
			bdp->bank_stmnt_id = bsp->id;
			bdp->journal_id = 0;
			bdp->rdate = when;
			bdp->number = num;
			bdp->descr = descr;
			bdp->amount = amount;
			printf(">Insert [%s], %.2f\n", descr, amount);
			if (!noeffect && db_insertbankdata(dbp, bdp) < 0)
				berror("qifread", "cannot insert bank data");
			num = 0;
			when = 0;
			descr = NULL;
			amount = 0.0;
			break;

		default:
			error(fname, lineno, "invalid QIF command");
			break;
		}
	}
	fclose(fp);
	exit(0);
}

/*
 *
 */
void
error(char *fname, int lno, char *msg)
{
	fprintf(stderr, "\"%s\", line %d: %s.\n", fname, lno, msg);
	errflag = 1;
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: qifread [-a #][-c <comp>][-s date][-e date][-nv] <file>\n");
	exit(2);
}
