#ident "Kalopa: $Id: acctview.c,v 1.16 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: acctview.c,v 1.16 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: acctview.c,v $
 * Revision 1.16  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.15  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.14  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.13  2008/01/15 15:33:01  dtynan
 * Extensive changes for new DBOW version (due out soon...)
 *
 * Revision 1.12  2007/10/29 17:24:34  dtynan
 * Various tweaks and fixes.
 *
 * Revision 1.11  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.10  2005/03/22 09:41:23  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.9  2005/03/08 16:38:59  dtynan
 * Added the journal ID number to the report outputs.
 *
 * Revision 1.8  2005/02/17 17:25:27  dtynan
 * Fixed some compiler warnings.
 *
 * Revision 1.7  2005/01/13 21:57:09  dtynan
 * Use nominal account table from main database.
 *
 * Revision 1.6  2004/12/09 20:23:39  dtynan
 * Cleaned up include mechanisms.
 *
 * Revision 1.5  2004/12/09 19:19:57  dtynan
 * Added a folio field to the output data.
 *
 * Revision 1.4  2004/09/08 10:43:12  dtynan
 * Fix a bug where 0.0 was shown as a negative balance and add a
 * "Closing Balance" line to all accounts.
 *
 * Revision 1.3  2004/09/03 13:46:43  dtynan
 * Created a text format for use without groff and made it the default.
 *
 * Revision 1.2  2004/08/26 17:15:35  dtynan
 * New version will display the transactions for any account.
 *
 * Revision 1.1  2004/08/07 15:31:24  dtynan
 * New program to produce a pro-forma bank statement.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"

int	verbose;
int	troff;
int	lineno;
char	*title;
double	bals[10000];
struct db_company *comp;

extern	int	optind, opterr;
extern	char	*optarg;

void	acctview(struct db_nomacct *, int);
void	heading(int, int);
void	usage();

struct	db_journal	*findjournalentries(dbow_conn *);
struct	db_journal_item	*findjournalitems(dbow_conn *, int);
struct	db_nomacct	*findnomaccts(dbow_conn *);

char	*month[12] = {
	"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, aflag, year, acctno;
	char *cname;
	struct tm *tmp;
	struct db_nomacct *np;
	dbow_conn *dbp;
	time_t now;

	/*
	 * Process command-line options.
	 */
	cname = NULL;
	time(&now);
	if ((tmp = localtime(&now)) != NULL)
		year = tmp->tm_year;
	else
		year = 104;
	opterr = verbose = aflag = troff = 0;
	while ((i = getopt(argc, argv, "ac:y:vt")) != EOF) {
		switch (i) {
		case 'a':
			/*
			 * Do all the accounts...
			 */
			aflag = 1;
			break;

		case 'c':
			cname = optarg;
			break;

		case 'y':
			if ((i = atoi(optarg)) < 1990 || i > 2099) {
				fprintf(stderr, "?Error - invalid year.\n");
				exit(2);
			}
			year = i - 1900;
			break;

		case 't':
			troff = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (!aflag && (argc - optind) != 1)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL) {
		fprintf(stderr, "?Error - cannot find specified company.\n");
		exit(1);
	}
	if (!aflag) {
		acctno = atoi(argv[optind]);
		if ((np = db_findnomacctbyid(dbp, acctno)) == NULL) {
			fprintf(stderr, "?Invalid nominal account number.\n");
			exit(1);
		}
		acctview(np, year);
	} else {
		for (np = findnomaccts(dbp); np != NULL;
					np = db_findnomacctnext(dbp, np)) {
			heading(-1, -1);
			acctview(np, year);
		}
	}
	exit(0);
}

/*
 *
 */
void
acctview(struct db_nomacct *np, int year)
{
	int sawflag = 0;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct tm *tmp;
	char tmpbal[20];
	double tot;
	dbow_conn *dbp = getdbase();

	title = strdup(np->aname);
	lineno = 0;
	tot = 0.0;
	for (jp = findjournalentries(dbp); jp != NULL;
				jp = db_findjournalnext(dbp, jp)) {
		if ((tmp = localtime((time_t *)&jp->jdate)) == NULL)
			continue;
		if (tmp->tm_year != year)
			continue;
		for (jip = findjournalitems(dbp, jp->id);
				jip != NULL;
				jip = db_findjournal_itemnext(dbp, jip)) {
			if (jip->nomacct_id != np->id)
				continue;
			heading(0, year);
			sawflag = 1;
			if (troff) {
				printf("%d-%s:", tmp->tm_mday,
							month[tmp->tm_mon]);
				if (jp->folio != NULL)
					printf("%s", jp->folio);
				printf(":%s:", jp->descr);
				if (jip->drcrf) {
					printf(":%.2f:", jip->amount);
					tot += jip->amount;
				} else {
					printf("%.2f::", jip->amount);
					tot -= jip->amount;
				}
				if (tot < 0.0)
					printf("(%.2f)\n", -tot);
				else
					printf("%.2f\n", tot);
				lineno++;
			} else {
				printf("|%2d-%s|%4d|", tmp->tm_mday,
							month[tmp->tm_mon],
							jp->id);
				if (jp->folio != NULL)
					printf("%4.4s|", jp->folio);
				else
					printf("    |");
				printf("%-28.28s|", jp->descr);
				if (jip->drcrf) {
					printf("         |%9.2f|", jip->amount);
					tot += jip->amount;
				} else {
					printf("%9.2f|         |", jip->amount);
					tot -= jip->amount;
				}
				if (tot < 0.0)
					sprintf(tmpbal, "(%.2f)", -tot);
				else
					sprintf(tmpbal, "%.2f ", tot);
				printf("%12s|\n", tmpbal);
			}
		}
	}
	if (!sawflag && tot == 0.0)
		return;
	heading(0, year);
	if (troff) {
		printf("::Balance Brought Forward:::");
		if (tot < 0.0)
			printf("(%.2f)\n", -tot);
		else
			printf("%.2f\n", tot);
		lineno++;
	} else {
		printf("|      |    |    |Balance Brought Forward     |         |         |");
		if (tot < 0.0)
			sprintf(tmpbal, "(%.2f)", -tot);
		else
			sprintf(tmpbal, "%.2f ", tot);
		printf("%12s|\n", tmpbal);
	}
	heading(-1, year);
	putchar('\n');
}

/*
 *
 */
void
heading(int type, int year)
{
	static int htype = -1, pageno = 0;

	if (year == -1) {
		htype = -1;
		return;
	}
	if (!troff) {
		if (htype == -1) {
			printf("%s for %s", title, comp->cname);
			if (year != 0)
				printf(" -- %d", year + 1900);
			putchar('\n');
		}
		if (type == -1 || htype == -1) {
			printf("+------+-------------------------------------");
			printf("-+---------+---------+------------+\n");
		}
		htype = 0;
		return;
	}
	if (type >= 0 && htype == -1) {
		printf(".nr PS 12\n.nr VS 14\n.sp 2c\n");
		htype = 0;
	}
	if (htype == type && lineno > 0 && lineno < 50) {
		lineno++;
		return;
	}
	if (lineno > 0)
		printf(".TE\n");
	if (type < 0)
		return;
	if (lineno > 0)
		printf(".bp\n.sp 2.5c\n");
	if (htype != type) {
		lineno = pageno = 0;
		htype = type;
	}
	printf("%s for %s", title, comp->cname);
	printf(" (Page %d)\n.sp\n.TS\n", ++pageno);
	switch (type) {
	case 0:
		printf("center box tab(:);\nc | c | c | c | c\n");
		printf("ri | lw4ib | r | r | r.\n");
		printf("Date:Details:Debit:Credit:Balance\n_\n");
		break;
	}
	lineno = 1;
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: acctview [-a][-v][-c <comp>] [-y year] [acctno]\n");
	exit(2);
}
