#ident "Kalopa: $Id: csvmake.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: csvmake.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: csvmake.c,v $
 * Revision 1.6  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.4  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2008/01/15 15:33:01  dtynan
 * Extensive changes for new DBOW version (due out soon...)
 *
 * Revision 1.2  2007/10/29 17:24:34  dtynan
 * Various tweaks and fixes.
 *
 * Revision 1.1  2006/07/31 13:58:32  dtynan
 * New application to dump the journal as a CSV file (for importing into
 * excel or gnumeric).
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

char	*month[12] = {
	"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};

int		verbose;
struct db_company *comp;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
void
csvemit(FILE *fp, char *cp, int leadf)
{
	int ch;

	if (leadf & 01)
		fputc(',', fp);
	if (leadf != 02 && cp == NULL)
		return;
	fputc('"', fp);
	if (cp != NULL) {
		while ((ch = *cp++) != '\0') {
			if (ch == '"')
				fputc('"', fp);
			fputc(ch, fp);
		}
	}
	fputc('"', fp);
	if (leadf & 02)
		fputc('\n', fp);
}

/*
 *
 */
void
dojdetails(FILE *fp, struct db_journal *jp, struct tm *tmp, int flg)
{
	fprintf(fp, "%d-%s,%d", tmp->tm_mday, month[tmp->tm_mon], jp->id);
	csvemit(fp, jp->folio, 1);
	csvemit(fp, jp->descr, flg ? 3 : 1);
}

/*
 *
 */
void
dojournal(int year)
{
	struct tm *tmp;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct db_nomacct *np;
	dbow_conn *dbp = getdbase();
	FILE *fp;

	if ((fp = fopen("csv/journal.csv", "w")) == NULL) {
		perror("csv/journal.csv");
		exit(1);
	}
	fprintf(fp, "DATE,J.ID,FOLIO,DESCRIPTION,A/C,DEBIT,CREDIT\n");
	for (jp = findjournalentries(dbp); jp != NULL;
				jp = db_findjournalnext(dbp, jp)) {
		if ((tmp = localtime((time_t *)&jp->jdate)) == NULL)
			continue;
		if (tmp->tm_year != year)
			continue;
		dojdetails(fp, jp, tmp, 1);
		for (jip = findjournalitems(dbp, jp->id);
				jip != NULL;
				jip = db_findjournal_itemnext(dbp, jip)) {
			fprintf(fp, ",,");
			np = db_findnomacctbyid(dbp, jip->nomacct_id);
			csvemit(fp, np == NULL ? "<Unknown>" : np->aname, 1);
			fprintf(fp, ",%d,", jip->nomacct_id);
			if (jip->drcrf)
				fputc(',', fp);
			fprintf(fp, "%.2f\n", jip->amount);
		}
		csvemit(fp, NULL, 2);
	}
	fclose(fp);
}

/*
 *
 */
void
donomacct(struct db_nomacct *np, int year)
{
	struct db_journal *jp;
	struct db_journal_item *jip;
	char tmpfname[512];
	struct tm *tmp;
	double drtot, crtot, tot;
	dbow_conn *dbp = getdbase();
	FILE *fp;

	sprintf(tmpfname, "csv/na%04d.csv", np->id);
	if ((fp = fopen(tmpfname, "w")) == NULL) {
		perror(tmpfname);
		exit(1);
	}
	fprintf(fp, "Nominal Account Number:,%d\n", np->id);
	fprintf(fp, "Nominal Account Name:");
	csvemit(fp, np->aname, 3);
	csvemit(fp, NULL, 2);
	fprintf(fp, "DATE,J.ID,FOLIO,DETAILS,DEBIT,CREDIT,BALANCE\n");
	tot = drtot = crtot = 0.0;
	for (jp = findjournalentries(dbp); jp != NULL;
				jp = db_findjournalnext(dbp, jp)) {
		if ((tmp = localtime((time_t *)&jp->jdate)) == NULL)
			continue;
		if (tmp->tm_year != year)
			continue;
		for (jip = findjournalitems(dbp, jp->id);
				jip != NULL;
				jip = db_findjournal_itemnext(dbp, jip)) {
			if (jip->nomacct_id != np->id)
				continue;
			dojdetails(fp, jp, tmp, 0);
			if (jip->drcrf) {
				fprintf(fp, ",,%.2f,", jip->amount);
				tot += jip->amount;
				crtot += jip->amount;
			} else {
				fprintf(fp, ",%.2f,,", jip->amount);
				tot -= jip->amount;
				drtot += jip->amount;
			}
			fprintf(fp, "%.2f\n", tot);
		}
	}
	csvemit(fp, NULL, 2);
	fprintf(fp, ",,,YEAR END TOTALS:,%.2f,%.2f,%.2f\n", drtot, crtot, tot);
	fclose(fp);
}

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, year;
	char *cname;
	struct tm *tmp;
	struct db_nomacct *np;
	dbow_conn *dbp;
	time_t now;

	/*
	 * Process command-line options.
	 */
	opterr = verbose = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:v")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) > 0) {
		if ((i = atoi(argv[optind])) < 1990 || i > 2099)
			usage();
		year = i - 1900;
	} else {
		time(&now);
		if ((tmp = localtime(&now)) != NULL)
			year = tmp->tm_year;
		else
			year = 104;
	}
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL) {
		fprintf(stderr, "?Error - cannot find specified company.\n");
		exit(1);
	}
	/*
	 * Create a CSV file for the journal entries.
	 */
	dojournal(year);
	for (np = findnomaccts(dbp); np != NULL;
					np = db_findnomacctnext(dbp, np)) {
		/*
		 * Create a CSV file for each nominal account.
		 */
		donomacct(np, year);
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: csvmake [-v][-c <comp>] [year]\n");
	exit(2);
}
