#ident "$Id: customers.c,v 1.8 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: customers.c,v 1.8 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: customers.c,v $
 * Revision 1.8  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.7  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.6  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.5  2008/01/15 15:33:02  dtynan
 * Extensive changes for new DBOW version (due out soon...)
 *
 * Revision 1.4  2006/08/25 11:27:14  dtynan
 * Lots of changes as per the end-of-year.
 *
 * Revision 1.3  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.2  2005/10/17 13:34:46  dtynan
 * Extensive changes to use the 'bal()' function for consistent output.
 *
 * Revision 1.1  2005/09/21 18:10:28  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"

int	verbose;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, days;
	char *cname;
	double crtot, drtot;
	dbow_conn *dbp;
	struct db_company *comp;
	struct db_customer *cusp;
	struct db_contact *conp;
	struct db_invoice *invp;
	time_t now;

	/*
	 * Process command-line options.
	 */
	cname = NULL;
	opterr = verbose = 0;
	crtot = drtot = 0.0;
	while ((i = getopt(argc, argv, "c:v")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("customers", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	/*
	 *
	 */
	time(&now);
	for (cusp = db_findcustomerfirst(dbp); cusp != NULL;
				cusp = db_findcustomernext(dbp, cusp)) {
		/*
		 * Find the contact details first...
		 */
		if ((conp = contact_findbyid(dbp, cusp->contact_id)) == NULL)
			berror("customers", "cannot find contact data for address");
		printf("%6d  ", cusp->id);
		printf("%-12.12s  ", conp->ccode);
		printf("%12.12s  ", bal(cusp->balance, NULL));
		printf("%s\n", orgname(conp));
		if (cusp->balance < 0.0)
			drtot -= cusp->balance;
		else
			crtot += cusp->balance;
		if (!verbose)
			continue;
		for (invp = db_findinvoicebycustomer_id(dbp, cusp->id);
					invp != NULL;
					invp = db_findinvoicenext(dbp, invp)) {
			if (invp->state < INVOICE_STATE_SENT)
				continue;
			days = (now - invp->idate) / (24*60*60);
			printf("\t> I%06d     %12.12s  [%d Days]\n",
						invp->id, bal(invp->htotal, NULL), days);
		}
	}
	printf("\t\t      ------------\n");
	printf("Credit Balance:\t     %12.2f\n", crtot);
	printf("Debit Balance:\t     %12.2f\n", drtot);
	printf("Total:\t\t      %12.12s\n", bal(crtot - drtot, NULL));
	printf("\t\t      ============\n");
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: customers [-v][-c <comp>]\n");
	exit(2);
}
