#ident "Kalopa: $Id: report.c,v 1.20 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: report.c,v 1.20 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: report.c,v $
 * Revision 1.20  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.19  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.18  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.17  2008/01/15 15:33:02  dtynan
 * Extensive changes for new DBOW version (due out soon...)
 *
 * Revision 1.16  2007/08/06 10:55:02  dtynan
 * Removed the hack prototypes for the two journal functions.
 *
 * Revision 1.15  2006/07/31 10:17:51  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.14  2005/10/17 13:34:46  dtynan
 * Extensive changes to use the 'bal()' function for consistent output.
 *
 * Revision 1.13  2005/08/01 10:53:08  dtynan
 * Added Debtors and Creditors Control Accounts to output.
 *
 * Revision 1.12  2005/03/22 09:41:24  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.11  2005/03/08 16:38:59  dtynan
 * Added the journal ID number to the report outputs.
 *
 * Revision 1.10  2005/02/17 17:25:28  dtynan
 * Fixed some compiler warnings.
 *
 * Revision 1.9  2005/01/13 21:57:10  dtynan
 * Use nominal account table from main database.
 *
 * Revision 1.8  2005/01/13 17:15:54  dtynan
 * Uses new function to display balances.
 *
 * Revision 1.7  2004/12/09 20:23:39  dtynan
 * Cleaned up include mechanisms.
 *
 * Revision 1.6  2004/12/09 19:19:58  dtynan
 * Added a folio field to the output data.
 *
 * Revision 1.5  2004/12/08 14:13:33  dtynan
 * Added 'folio' field to reports.
 *
 * Revision 1.4  2004/09/08 10:42:24  dtynan
 * Don't show retained P&L with a minus sign, use parens.
 *
 * Revision 1.3  2004/08/26 17:14:59  dtynan
 * Added new function to show negative balances with parentheses.
 *
 * Revision 1.2  2004/08/05 12:46:26  dtynan
 * Made each report type optional.  Also fixed bug in company selection.
 *
 * Revision 1.1  2004/08/05 12:32:09  dtynan
 * New components to produce management reports.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

int	verbose;
int	dflag;
int	cflag;
int	bflag;
int	pflag;
int	jflag;
int	lineno;
double	bals[10000];
struct db_company *comp;

extern	int	optind, opterr;
extern	char	*optarg;

void	heading(int, int);
void	usage();

char	*month[12] = {
	"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, year, flg;
	char *cname;
	dbow_conn *dbp;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct db_nomacct *np;
	struct db_customer *cusp;
	struct db_contact *conp;
	struct db_sledger *slp;
	double tot1, tot2;
	struct tm *tmp;
	time_t now;

	/*
	 * Process command-line options.
	 */
	opterr = verbose = 0;
	cname = NULL;
	bflag = pflag = jflag = 0;
	while ((i = getopt(argc, argv, "abCDpjc:v")) != EOF) {
		switch (i) {
		case 'b':
			bflag = 1;
			break;

		case 'C':
			cflag = 1;
			break;

		case 'D':
			dflag = 1;
			break;

		case 'p':
			pflag = 1;
			break;

		case 'j':
			jflag = 1;
			break;

		case 'a':
			bflag = 1;
			cflag = 1;
			dflag = 1;
			pflag = 1;
			jflag = 1;
			break;

		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (bflag == 0 && pflag == 0 && jflag == 0 && cflag == 0 && dflag == 0)
		bflag = 1;
	if ((argc - optind) > 0) {
		if ((i = atoi(argv[optind])) < 1990 || i > 2099)
			usage();
		year = i - 1900;
	} else {
		time(&now);
		if ((tmp = localtime(&now)) != NULL)
			year = tmp->tm_year;
		else
			year = 104;
	}
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("report", "cannot find specified company");
	/*
	 * Clear all balances and begin...
	 */
	for (i = 0; i < 10000; i++)
		bals[i] = 0.0;
	printf(".nr PS 12\n.nr VS 14\n.sp 2c\n");
	lineno = 0;
	for (jp = findjournalentries(dbp); jp != NULL;
				jp = db_findjournalnext(dbp, jp)) {
		if ((tmp = localtime((time_t *)&jp->jdate)) == NULL)
			continue;
		if (tmp->tm_year != year)
			continue;
		if (jflag) {
			heading(0, year);
			printf("%d-%s-%02d:", tmp->tm_mday,
					month[tmp->tm_mon],
					tmp->tm_year % 100);
			printf("%d:", jp->id);
			if (jp->folio != NULL)
				printf("%s", jp->folio);
			printf(":%s\n", jp->descr);
		}
		for (jip = findjournalitems(dbp, jp->id);
				jip != NULL;
				jip = db_findjournal_itemnext(dbp, jip)) {
			if (jflag) {
				if ((np = db_findnomacctbyid(dbp, jip->nomacct_id)) == NULL)
					printf(":::<Unknown>:%d:", jip->nomacct_id);
				else
					printf(":::%s:%d:", np->aname, jip->nomacct_id);
			}
			if (jip->drcrf) {
				bals[jip->nomacct_id] += jip->amount;
				if (jflag)
					putchar(':');
			} else
				bals[jip->nomacct_id] -= jip->amount;
			if (jflag) {
				printf("%.2f\n", jip->amount);
				lineno++;
			}
		}
		if (jflag) {
			printf("\n");
			lineno++;
		}
	}
	if (jflag) {
		heading(1, year);
		for (i = 0; i < 10000; i++) {
			if (bals[i] == 0.0 || (np = db_findnomacctbyid(dbp, i)) == NULL)
				continue;
			printf("%s:%d:%d:%s\n", np->aname, i, np->atype, bal(bals[i], NULL));
		}
	}
	if (bflag) {
		heading(2, year);
		tot1 = tot2 = 0.0;
		printf("\\fBFixed Assets\\fR\n");
		lineno++;
		for (i = 0; i < 10000; i++) {
			if (bals[i] == 0.0 || (np = db_findnomacctbyid(dbp, i)) == NULL || np->atype != ASSET_FIXED)
				continue;
			printf("%s (%04d):%s\n", np->aname, np->id, bal(-bals[i], NULL));
			lineno++;
			tot2 -= bals[i];
		}
		if (tot2 != 0.0) {
			printf("::_\n::%s\n", bal(tot2, NULL));
			lineno += 2;
		}
		tot1 += tot2;
		tot2 = 0.0;
		printf("\n\\fBCurrent Assets\\fR\n");
		lineno += 2;
		for (i = 0; i < 10000; i++) {
			if (bals[i] == 0.0 || (np = db_findnomacctbyid(dbp, i)) == NULL || np->atype != ASSET_CURRENT)
				continue;
			printf("%s (%04d):%s\n", np->aname, np->id, bal(-bals[i], NULL));
			lineno++;
			tot2 -= bals[i];
		}
		if (tot2 != 0.0) {
			printf("::_\n::%s\n", bal(tot2, NULL));
			lineno += 2;
		}
		tot1 += tot2;
		tot2 = 0.0;
		printf("\n::_\n\\fBAssets\\fR::%s\n::=\n", bal(tot1, NULL));
		printf("\n\\fBLiabilities\\fR\n");
		lineno += 6;
		for (i = 0; i < 10000; i++) {
			if (bals[i] == 0.0 || (np = db_findnomacctbyid(dbp, i)) == NULL || np->atype != LIABILITY)
				continue;
			printf("%s (%04d):%s\n", np->aname, np->id, bal(bals[i], NULL));
			lineno++;
			tot2 += bals[i];
		}
		tot1 -= tot2;
		printf("\n::_\n\\fBNet assets less liabilities\\fR::%s\n::=\n", bal(tot1, NULL));
		tot2 = 0.0;
		printf("\n\\fBCapital and Reserves\\fR\n");
		lineno += 6;
		for (i = 0; i < 10000; i++) {
			if (bals[i] == 0.0 || (np = db_findnomacctbyid(dbp, i)) == NULL || np->atype != OEQUITY)
				continue;
			printf("%s (%04d):%s\n", np->aname, np->id, bal(bals[i], NULL));
			lineno++;
			tot2 += bals[i];
		}
		printf("Profit and loss account:%s\n", bal(tot1 - tot2, NULL));
		printf("::_\n\\fBShareholders funds\\fR::%s\n::=\n", bal(tot1, NULL));
		lineno += 4;
	}
	if (pflag) {
		heading(3, year);
		tot1 = tot2 = 0.0;
		printf("\\fBTurnover\\fR\n");
		for (i = 0; i < 10000; i++) {
			if (bals[i] == 0.0 || (np = db_findnomacctbyid(dbp, i)) == NULL || np->atype != INCOME)
				continue;
			printf("%s:%s\n", np->aname, bal(bals[i], NULL));
			tot2 += bals[i];
		}
		if (tot2 != 0.0)
			printf("::_\n::%s\n", bal(tot2, NULL));
		tot1 += tot2;
		printf("\n\\fBCost of Sales\\fR\n");
		tot2 = 0.0;
		for (i = 0; i < 10000; i++) {
			if (bals[i] == 0.0 || (np = db_findnomacctbyid(dbp, i)) == NULL || np->atype != COSTOFSALES)
				continue;
			printf("%s:%s\n", np->aname, bal(-bals[i], NULL));
			tot2 -= bals[i];
		}
		if (tot2 != 0.0)
			printf("::_\n::%s\n", bal(tot2, NULL));
		tot1 -= tot2;
		printf("\n::_\n\\fBGross Profit\\fR::%s\n::=\n", bal(tot1, NULL));
		printf("\n\\fBExpenses\\fR\n");
		tot2 = 0.0;
		for (i = 0; i < 10000; i++) {
			if (bals[i] == 0.0 || (np = db_findnomacctbyid(dbp, i)) == NULL || np->atype != EXPENSE)
				continue;
			printf("%s:%s\n", np->aname, bal(-bals[i], NULL));
			tot2 -= bals[i];
		}
		if (tot2 != 0.0)
			printf("::_\n::%s\n", bal(tot2, NULL));
		printf("\n::_\n\\fB");
		if (tot1 > tot2)
			printf("Profit");
		else
			printf("Loss");
		printf(" on ordinary activities before taxation::%s\n::=\n", bal(tot1 - tot2, NULL));
	}
	if (cflag) {
		heading(4, year);
	}
	if (dflag) {
		for (cusp = db_findcustomerfirst(dbp); cusp != NULL;
				cusp = db_findcustomernext(dbp, cusp)) {
			/*
			 * Find the contact details first...
			 */
			flg = 0;
			tot1 = 0.0;
			if ((conp = contact_findbyid(dbp, cusp->contact_id)) == NULL)
				continue;
			for (slp = db_findsledgerbycustomer_id(dbp, cusp->id);
					slp != NULL;
					slp = db_findsledgernext(dbp, slp)) {
				tot1 += slp->amount;
				if ((tmp = localtime((time_t *)&slp->sldate)) == NULL)
					continue;
				if (tmp->tm_year != year)
					continue;
				heading(5, year);
				if (flg == 0) {
					printf("_\n:\\fB%s\\fP\n", orgname(conp));
					lineno += 2;
				}
				flg = 1;
				printf("%d-%s-%02d:%s:", tmp->tm_mday,
							month[tmp->tm_mon],
							tmp->tm_year % 100,
							slp->descr);
				if (slp->amount < 0.0)
					printf("%.2f::", -slp->amount);
				else
					printf(":%.2f:", slp->amount);
				printf("%s\n", bal(tot1, NULL));
				lineno++;
			}
		}
	}
	heading(-1, year);
	exit(0);
}

/*
 *
 */
void
heading(int type, int year)
{
	static int htype = 0, pageno = 0;

	if (htype == type && lineno > 0 && lineno < 50) {
		lineno++;
		return;
	}
	if (lineno > 0)
		printf(".TE\n");
	if (type < 0)
		return;
	if (lineno > 0)
		printf(".bp\n.sp 2.5c\n");
	if (htype != type) {
		lineno = pageno = 0;
		htype = type;
	}
	switch (type) {
	case 0:
		printf("Journal");
		break;

	case 1:
		printf("Statement of Accounts");
		break;

	case 2:
		printf("Balance Sheet");
		break;

	case 3:
		printf("Profit and Loss");
		break;

	case 4:
		printf("Creditors Control Account");
		break;

	case 5:
		printf("Debtors Control Account");
		break;
	}
	printf(" for %s", comp->cname);
	if (year != 0)
		printf(" -- %d", year + 1900);
	printf(" (Page %d)\n.sp\n.TS\n", ++pageno);
	switch (type) {
	case 0:
		printf("expand box tab(:);\nc | c | c | c | c | c | c\n");
		printf("ri | ri | ri | lw4ib | rip-2 | ri | ri.\n");
		printf("Date:J:F:Description:A/C:Debit:Credit\n_\n");
		break;

	case 1:
		printf("expand box tab(:);\nc | c | c | c\n");
		printf("lw4ib | rip-2 | rip-2 | ri.\n");
		printf("Account:No.:Type:Amount\n_\n");
		break;

	case 2:
	case 3:
		printf("expand tab(:);\nl r r.\n");
		break;

	case 4:
	case 5:
		printf("expand box tab(:);\nc | c | c | c | c\n");
		printf("ri | lw4 | ri | ri | ri.\n");
		printf("Date:Description:Debit:Credit:Balance\n");
		break;
	}
	lineno = 1;
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: report [-abCDpjv][-c <comp>] [year]\n");
	exit(2);
}
