#ident "Kalopa: $Id: revrec.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: revrec.c,v 1.6 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2006, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: revrec.c,v $
 * Revision 1.6  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.4  2008/01/15 18:10:43  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2008/01/15 15:33:02  dtynan
 * Extensive changes for new DBOW version (due out soon...)
 *
 * Revision 1.2  2007/10/29 17:24:34  dtynan
 * Various tweaks and fixes.
 *
 * Revision 1.1  2006/08/25 11:27:14  dtynan
 * Lots of changes as per the end-of-year.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

struct	db_journal	*findjournalentries(dbow_conn *);
struct	db_journal_item	*findjournalitems(dbow_conn *, int);

char	*month[12] = {
	"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, ndays, lastf, year, verbose;
	char *cname;
	dbow_conn *dbp;
	struct db_company *comp;
	struct db_invoice *ip;
	struct db_invoice_item *iip;
	struct tm *tmp, tm;
	double aval, hval, revamt, total;
	time_t now, eoy;

	/*
	 * Process command-line options.
	 */
	opterr = verbose = lastf = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:v")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) > 0) {
		if ((i = atoi(argv[optind])) < 1990 || i > 2099)
			usage();
		year = i - 1900;
	} else {
		time(&now);
		if ((tmp = localtime(&now)) != NULL)
			year = tmp->tm_year;
		else
			year = 104;
	}
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL) {
		fprintf(stderr, "?Error - cannot find specified company.\n");
		exit(1);
	}
	if (verbose)
		printf("Company: %s\n", comp->cname);
	if (verbose)
		printf("Year: %d\n", year + 1900);
	tm.tm_mday = 1;
	tm.tm_mon = 0;
	tm.tm_year = year + 1;
	tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
	tm.tm_isdst = -1;
	eoy = mktime(&tm);
	/*
	 * Find all invoices for the specified period.  What we
	 * want here are invoices for the current year where (rdate
	 * + runlen) run into next year.
	 *
	 * HACK: What about last years RevRec?  We need to recognize it
	 * now.
	 */
	total = 0.0;
	for (ip = db_findinvoicefirst(dbp); ip != NULL;
					ip = db_findinvoicenext(dbp, ip)) {
		if ((tmp = localtime((time_t *)&ip->idate)) == NULL ||
							tmp->tm_year != year)
			continue;
		/*
		 * OK, the invoice applies this year.
		 */
		iip = db_findinvoice_itembyinvoice_id(dbp, ip->id);
		for (; iip != NULL; iip = db_findinvoice_itemnext(dbp, iip)) {
			now = iip->rdate + DAYS(iip->runlen - 1);
			if (now < eoy)
				continue;
			if ((tmp = localtime((time_t *)&iip->rdate)) == NULL)
				continue;
			/*
			 * Calculate the actual line-item value, then do
			 * a currency conversion to determine the local
			 * value.  'ndays' is the number of days remaining
			 * on this line item until the year-end.
			 *
			 * Credit notes must be treated very carefully
			 * for revenue recognition.
			 */
			invoice_total(ip, iip, &aval, &hval);
			ndays = (iip->rdate + DAYS(iip->runlen) - eoy) / DAYS(1);
			revamt = (hval * (double )ndays) / (double )iip->runlen;
			if (lastf == 0) {
				printf("Invoice        Date      Gross Amt");
				printf("    Deferred\tFraction\n-------   ");
				printf("     ----      ---------    ------");
				printf("--\t--------\n");
			} else if (lastf != ip->id)
				putchar('\n');
			lastf = ip->id;
			i = (ip->cninv_id != 0) ? 'C' : 'I';
			printf("%c%06d:%d   %2d/%02d/%04d  %10.2f  %10.2f\t(%d/%d)\n",
						i, ip->id, iip->item_id,
						tmp->tm_mday, tmp->tm_mon + 1,
						tmp->tm_year + 1900,
						hval, revamt, ndays,
						iip->runlen);
			total += revamt;
		}
	}
	printf("\t\t\t__________\nTOTAL deferred revenue: %10.2f\n", total);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: revrec [-v][-c <comp>] [year]\n");
	exit(2);
}
