#ident "$Id: main.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: main.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: main.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:47:26  dtynan
 * An application to pull relevant information from the database and
 * generate the Irish Revenue Online Service (ROS) data files.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

#include "ros.h"

/*
 * Command-line options...
 */
struct	cmdopts	{
	char	*name;
	char	*ext;
	int	(*func)(FILE *, struct db_company *, time_t);
} options[] = {
	{"ct1", ".ct1", doct1},
	{"p30", ".p30", dop30},
	{"p35l", ".p35", dop35l},
	{"p45", ".p45", dop45},
	{"vat3", ".vt3", dovat3},
	{NULL, NULL, NULL}
};

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname, *fname, *date, fnstr[32];
	struct db_company *comp;
	struct cmdopts *optp = NULL;
	struct tm *tmp;
	time_t when;
	FILE *fp;

	fname = NULL;
	cname = NULL;
	noeffect = verbose = 0;
	date = NULL;
	while ((i = getopt(argc, argv, "c:d:nvo:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			date = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		case 'o':
			fname = optarg;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1)
		usage();
	for (optp = options; optp->name != NULL; optp++)
		if (strcmp(optp->name, argv[optind]) == 0)
			break;
	if (optp->name == NULL)
		usage();
	if (date != NULL)
		parsedate(date, &when);
	else
		time(&when);
	tmp = localtime(&when);
	if (verbose) {
		printf("Processing on %d/%04d\n", tmp->tm_mon + 1,
							tmp->tm_year + 1900);
	}
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	if ((comp = db_findcompanyfirst(dbp), 1)) == NULL)
		berror("ros", "cannot find company");
	if (fname == NULL) {
		if ((i = strlen(comp->taxno) - 4) < 0)
			i = 0;
		sprintf(fnstr, "%s%02d%02d%s", comp->taxno + i,
							tmp->tm_year % 100,
							tmp->tm_mon + 1,
							optp->ext);
		fname = makepath("ros", optp->name, fnstr, NULL);
		if (verbose)
			printf("Output filename: %s\n", fname);
	}
	if (strcmp(fname, "-") == 0)
		fp = stdout;
	else if ((fp = fopen(fname, "w")) == NULL) {
		fprintf(stderr, "ros: file open: ");
		perror(fname);
		exit(1);
	}
	if (optp->func(fp, comp, when) < 0) {
		fclose(fp);
		if (fp != stdout)
			unlink(fname);
		exit(1);
	}
	fclose(fp);
	exit(0);
}

/*
 *
 */
void
usage()
{
	struct cmdopts *optp;

	fprintf(stderr, "Usage: ros [-c #][-d date][-nv][-o <file>] <type>\n");
	fprintf(stderr, "Where type is one of the following:\n");
	for (optp = options; optp->name != NULL; optp++)
		fprintf(stderr, "\t%s\n", optp->name);
	exit(2);
}
