#ident "$Id: p30.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: p30.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: p30.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:47:26  dtynan
 * An application to pull relevant information from the database and
 * generate the Irish Revenue Online Service (ROS) data files.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

#include "../paye.ie/p30.h"

#include "ros.h"

/*
 * Definitions for P30 types...
 */
#define P30_FORM_VERSION	1
#define P30_LANGUAGE_ENGLISH	"E"
#define P30_LANGUAGE_IRISH	"G"
#define P30_CURRENCY_EURO	"E"
#define P30_CURRENCY_POUND	"P"
#define P30_TYPE_ORIGINAL	0
#define P30_TYPE_SUPPLEMENTARY	1

/*
 *
 */
int
dop30(FILE *fp, struct db_company *comp, time_t when)
{
	int m, y;
	char datestr[16];
	struct db_paye_p30 *p30p;
	dbow_conn *dbp;
	struct xml *xp;
	struct tm *tmp;

	if ((tmp = localtime(&when)) == NULL)
		return(-1);
	m = tmp->tm_mon + 1;
	y = tmp->tm_year + 1900;
	dbp = getdbase();
	/*
	 * Get the ancillary data and calculate the appropriate PAYE
	 * and PRSI for the period.
	 */
	if ((p30p = db_findpaye_p30byidate(dbp, (int )when)) == NULL) {
		bwarn("ros", "cannot find PAYE record for period");
		return(-1);
	}
	if (verbose) {
		printf("P30 Data:-\nNotice:\t%s\n", p30p->notice);
		printf("Period:\t\t%2d/%04d\n", m, y);
		printf("PAYE:\t\t%7d\n", p30p->paye);
		printf("PRSI:\t\t%7d\n\t\t-------\n", p30p->prsi);
		printf("Total:\t\t%7d\n", p30p->paye + p30p->prsi);
		printf("\t\t=======\n");
	}
	/*
	 * Now produce the XML...
	 */
	xp = openxml(fp);
	xmlbegin(xp, "P30File");
	xmlstr(xp, "language", P30_LANGUAGE_ENGLISH);
	xmlstr(xp, "currency", P30_CURRENCY_EURO);
	xmlnum(xp, "formversion", P30_FORM_VERSION);
	xmlbegin(xp, "P30");
	xmlnum(xp, "period", 0);		/* FIXME */
	sprintf(datestr, "01/%02d/%04d", m, y);
	xmlstr(xp, "start", datestr);
	xmlnum(xp, "PAYE", p30p->paye);
	xmlnum(xp, "PRSI", p30p->prsi);
	xmlnum(xp, "total", p30p->paye + p30p->prsi);
	xmlnum(xp, "type", P30_TYPE_ORIGINAL);
	xmlend(xp, "P30");
	employer(xp, comp);
	xmlend(xp, "P30File");
	closexml(xp);
	return(0);
}
