#ident "$Id: p45.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: p45.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: p45.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:47:26  dtynan
 * An application to pull relevant information from the database and
 * generate the Irish Revenue Online Service (ROS) data files.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

#include "ros.h"

/*
 * Definitions for P30 types...
 */
#define P45_FORM_VERSION	2
#define P45_LANGUAGE_ENGLISH	"E"
#define P45_LANGUAGE_IRISH	"G"
#define P45_CURRENCY_EURO	"E"
#define P45_CURRENCY_POUND	"P"
#define P45_PRINTER_LASER	0
#define P45_PRINTER_DOTMATRIX	1
#define P45_TYPE_ORIGINAL	0
#define P45_TYPE_SUPPLEMENTARY	1

/*
 *
 */
int
dop45(FILE *fp, struct db_company *comp, time_t when)
{
	char datestr[16];
	dbow_conn *dbp;
	struct xml *xp;

#if 0
	/*
	 * Get the ancillary data and calculate the appropriate PAYE
	 * and PRSI for the period.
	 */
	dbp = getdbase();
	if ((p30p = db_findpaye_p30byidate(dbp, (int )when)) == NULL) {
		bwarn("ros", "cannot find PAYE record for period");
		return(-1);
	}
	if (verbose) {
		printf("P30 Data:-\nNotice:\t%s\n", p30p->notice);
		printf("Period:\t\t%2d/%04d\n", m, y);
		printf("PAYE:\t\t%7d\n", p30p->paye);
		printf("PRSI:\t\t%7d\n\t\t-------\n", p30p->prsi);
		printf("Total:\t\t%7d\n", p30p->paye + p30p->prsi);
		printf("\t\t=======\n");
	}
#endif
	/*
	 * Now produce the XML...
	 */
	xp = openxml(fp);
	xmlbegin(xp, "P45File");
	xmlnum(xp, "formversion", P45_FORM_VERSION);
	xmlstr(xp, "language", P45_LANGUAGE_ENGLISH);
	xmlstr(xp, "currency", P45_CURRENCY_EURO);
	xmlnum(xp, "printer", P45_PRINTER_LASER);
	employer(xp, comp);
	xmlbegin(xp, "P45");
	xmlbegin(xp, "Employee");
	xmlstr(xp, "surname", "Jones");
	xmlstr(xp, "firstname", "Brian");
	xmlstr(xp, "ppsn", "1111111L");
	xmlend(xp, "Employee");
	xmlbegin(xp, "Employment");
	xmlstr(xp, "end", "31/08/2001");
	xmlstr(xp, "unit", "000");
	xmlend(xp, "Employment");
	xmlbegin(xp, "Pay");
	xmlstr(xp, "freq", "m");
	xmlstr(xp, "period", "8");
	xmlstr(xp, "taxcredit", "87900");
	xmlstr(xp, "cutoff", "21212");
	xmlstr(xp, "emergency", "false");
	xmlend(xp, "Pay");
	xmlbegin(xp, "TaxDetails");
	xmlstr(xp, "totalpay", "512345");
	xmlstr(xp, "totaltax", "234567");
	xmlend(xp, "TaxDetails");
	xmlbegin(xp, "PRSI");
	xmlstr(xp, "total", "111111");
	xmlstr(xp, "employee", "555555");
	xmlstr(xp, "weeks", "12");
	xmlbegin(xp, "PRSIClass");
	xmlstr(xp, "class", "A");
	xmlstr(xp, "weeks", "12");
	xmlend(xp, "PRSIClass");
	xmlend(xp, "PRSI");
	xmlend(xp, "P45");
	xmlbegin(xp, "P45Supp");
	xmlbegin(xp, "Employee");
	xmlstr(xp, "surname", "Boy");
	xmlstr(xp, "firstnames", "Tom");
	xmlstr(xp, "ppsn", "1234567Q");
	xmlend(xp, "Employee");
	xmlbegin(xp, "Employment");
	xmlstr(xp, "end", "31/08/2000");
	xmlstr(xp, "unit", "000");
	xmlend(xp, "Employment");
	xmlbegin(xp, "Pay");
	xmlstr(xp, "freq", "m");
	xmlstr(xp, "period", "5");
	xmlstr(xp, "taxcredit", "90000");
	xmlstr(xp, "cutoff", "21212");
	xmlstr(xp, "emergency", "false");
	xmlend(xp, "Pay");
	xmlbegin(xp, "TaxDetails");
	xmlstr(xp, "totalpay", "50000");
	xmlstr(xp, "totaltax", "12000");
	xmlend(xp, "TaxDetails");
	xmlbegin(xp, "PRSI");
	xmlstr(xp, "total", "20000");
	xmlstr(xp, "weeks", "12");
	xmlbegin(xp, "PRSIClass");
	xmlstr(xp, "class", "A");
	xmlstr(xp, "weeks", "5");
	xmlend(xp, "PRSIClass");
	xmlbegin(xp, "PRSIClass");
	xmlstr(xp, "class", "CI");
	xmlend(xp, "PRSIClass");
	xmlend(xp, "PRSI");
	xmlend(xp, "P45Supp");
	xmlend(xp, "P45File");
	closexml(xp);
	return(0);
}
