#ident "$Id: vat3.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: vat3.c,v 1.3 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: vat3.c,v $
 * Revision 1.3  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.2  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.1  2008/07/08 16:47:26  dtynan
 * An application to pull relevant information from the database and
 * generate the Irish Revenue Online Service (ROS) data files.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

#include "ros.h"

#define VAT3_FORM_VERSION	1
#define VAT3_LANGUAGE_ENGLISH	"E"
#define VAT3_LANGUAGE_IRISH	"G"

/*
 *
 */
int
dovat3(FILE *fp, struct db_company *comp, time_t when)
{
	int sales, purchs, gfrom, gto;
	struct xml *xp;
	struct db_vatret *vp;
	dbow_conn *dbp;

	/*
	 * Calculate form data.
	 */
	dbp = getdbase();
	if ((vp = findvatreturn(dbp, (int )when)) == NULL) {
		bwarn("ros", "cannot find VAT record for period");
		return(-1);
	}
	sales = vp->sales;
	purchs = vp->purchs;
	gfrom = vp->goodsfrom;
	gto = vp->goodsto;
	/*
	 * Now produce the XML form...
	 */
	xp = openxml(fp);
	xmlbegin(xp, "VAT3");
	xmlstr(xp, "name", comp->cname);
	xmlstr(xp, "regnum", comp->taxno);
	xmlnum(xp, "filefreq", VAT3_FREQ_BIMONTHLY);
	xmldate(xp, "startdate", vp->sdate);
	xmldate(xp, "enddate", vp->edate);
	xmlnum(xp, "type", vp->vtype);
	xmlnum(xp, "sales", sales);
	xmlnum(xp, "purchs", purchs);
	xmlnum(xp, "goodsfrom", gfrom);
	xmlnum(xp, "goodsto", gto);
	xmlnum(xp, "formversion", VAT3_FORM_VERSION);
	xmlstr(xp, "language", VAT3_LANGUAGE_ENGLISH);
	xmlstr(xp, "currency", VAT3_CURRENCY_EURO);
	xmlend(xp, "VAT3");
	closexml(xp);
	return(0);
}
