#ident "$Id: xml.c,v 1.1 2008/07/08 16:47:26 dtynan Exp $"

/*
 * $Id: xml.c,v 1.1 2008/07/08 16:47:26 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: xml.c,v $
 * Revision 1.1  2008/07/08 16:47:26  dtynan
 * An application to pull relevant information from the database and
 * generate the Irish Revenue Online Service (ROS) data files.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

#include "ros.h"

#define XMLVER	"1.0"
#define XMLENC	"UTF-8"

#define XS_BEGIN	0
#define XS_INTAG	1
#define XS_INBLOCK	2

/*
 *
 */
struct xml *
openxml(FILE *fp)
{
	struct xml *xp;

	if ((xp = (struct xml *)malloc(sizeof(struct xml))) == NULL) {
		perror("ros xml");
		exit(1);
	}
	xp->fp = fp;
	xp->state = XS_BEGIN;
	xp->level = 0;
	fprintf(fp, "<?xml version=\"%s\" encoding=\"%s\"?>\n", XMLVER, XMLENC);
	return(xp);
}

/*
 *
 */
void
closexml(struct xml *xp)
{
	fclose(xp->fp);
	free((char *)xp);
}

/*
 *
 */
void
xmlbegin(struct xml *xp, char *name)
{
	int i;

	if (xp->state == XS_INTAG)
		fprintf(xp->fp, ">\n");
	for (i = 0; i < xp->level; i++)
		fprintf(xp->fp, "    ");
	fprintf(xp->fp, "<%s", name);
	xp->state = XS_INTAG;
	xp->level++;
}

/*
 *
 */
void
xmlend(struct xml *xp, char *name)
{
	int i;

	switch (xp->state) {
	case XS_INTAG:
		xp->level--;
		fprintf(xp->fp, "/>\n");
		xp->state = XS_INBLOCK;
		break;

	case XS_INBLOCK:
		xp->level--;
		for (i = 0; i < xp->level; i++)
			fprintf(xp->fp, "    ");
		fprintf(xp->fp, "</%s>\n", name);
		break;
	}
}

/*
 *
 */
void
xmlstr(struct xml *xp, char *name, char *value)
{
	fprintf(xp->fp, " %s=\"%.30s\"", name, value);
}

/*
 *
 */
void
xmlnum(struct xml *xp, char *name, int value)
{
	fprintf(xp->fp, " %s=\"%d\"", name, value);
}

/*
 *
 */
void
xmldate(struct xml *xp, char *name, time_t when)
{
	char datestr[16];
	struct tm *tmp;

	tmp = localtime(&when);
	sprintf(datestr, "%02d/%02d/%04d", tmp->tm_mday, tmp->tm_mon + 1,
							tmp->tm_year + 1900);
	xmlstr(xp, name, datestr);
}
