#ident "Kalopa: $Id: arreport.c,v 1.16 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: arreport.c,v 1.16 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: arreport.c,v $
 * Revision 1.16  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.15  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.14  2008/07/08 17:10:17  dtynan
 * Extensive changes.
 *
 * Revision 1.13  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.12  2007/10/29 18:48:47  dtynan
 * Cleaned up the output format and fixed some of the net/gross bugs.
 *
 * Revision 1.11  2007/10/29 17:23:39  dtynan
 * Extensive changes to handle hourly billing and various other tweaks.
 *
 * Revision 1.10  2007/08/07 07:47:32  dtynan
 * Change to show "home" balance rather than invoice balance and show the
 * total outstanding at the bottom.
 *
 * Revision 1.9  2007/06/19 22:02:06  dtynan
 * Updated with extra components.
 *
 * Revision 1.8  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.7  2005/03/22 09:41:24  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.6  2005/02/17 16:23:21  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.5  2004/12/09 20:29:16  dtynan
 * Use new mechanism for determining organisation name.
 *
 * Revision 1.4  2004/08/04 13:15:24  dtynan
 * Make sure the company number is handled properly.
 *
 * Revision 1.3  2004/07/12 19:07:11  dtynan
 * Extensive changes for new directory structure and other things.
 *
 * Revision 1.2  2004/05/12 15:21:04  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.1  2004/01/05 19:05:48  dtynan
 * Added program to show outstanding debtors.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

int	srs;
int	verbose;
time_t	now;

void	usage();

/*
 *
 */
void
sendreport(struct db_invoice *invp)
{
	struct db_customer *cusp;
	struct db_contact *cp;
	dbow_conn *dbp = getdbase();

	if ((cusp = db_findcustomerbyid(dbp, invp->customer_id)) == NULL)
		berror("arreport", "cannot find customer record for invoice");
	if ((cp = contact_findbyid(dbp, cusp->contact_id)) == NULL)
		berror("arreport", "cannot find customer contact details");
	if (srs == 0) {
		printf("The following invoices are outstanding:\n");
		srs = 1;
	}
	if (srs != invp->state) {
		putchar('\n');
		switch (srs = invp->state) {
		case INVOICE_STATE_SENT:
			printf(">> Current (not yet due):\n\n");
			break;

		case INVOICE_STATE_LATE1:
			printf(">> Over Thirty Days:\n\n");
			break;

		case INVOICE_STATE_LATE2:
			printf(">> Over Forty Five Days:\n\n");
			break;

		case INVOICE_STATE_LATE3:
			printf(">> Over Sixty Days:\n\n");
			break;

		case INVOICE_STATE_LATE4:
			printf(">> Over Ninety Days:\n\n");
			break;

		case INVOICE_STATE_LATE5:
			printf(">> Over One Hundred And Twenty Days:\n");
			break;
		}
		printf(" Invoice #    Gross          Net      Age    Debtor\n");
		printf(" ---------    -----          ---      ---    ------\n");
	}
	printf("  %s   %10.2f   %10.2f   %3d    %-30.30s\n",
			invoice_number(invp->id),
			invp->htotal, invp->htotal - invp->vatttl,
			(now - invp->idate) / DAYS(1),
			orgname(cp));
}

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, s, ninv;
	char *cname;
	dbow_conn *dbp;
	struct db_invoice *invp;
	struct db_company *comp;
	double net, gross, avg;

	opterr = verbose = 0;
	cname = NULL;
	time(&now);
	while ((i = getopt(argc, argv, "c:qv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
		}
	}
	srs = ninv = 0;
	avg = 0.0;
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("arreport", "cannot find company");
	net = gross = 0.0;
	for (s = INVOICE_STATE_SENT; s <= INVOICE_STATE_LATE5; s++) {
		for (invp = db_findinvoicebystate(dbp, s); invp != NULL;
					invp = db_findinvoicenext(dbp, invp)) {
			/*
			 * Ignore cash invoices.
			 */
			if (invp->customer_id == 0)
				continue;
			sendreport(invp);
			net += invp->htotal - invp->vatttl;
			gross += invp->htotal;
			avg += (double )((now - invp->idate)/DAYS(1));
			ninv++;
		}
	}
	if (srs > 0) {
		avg /= (double )ninv;
		printf("\nTOTALS:     %10.2f   %10.2f    %4.1f",gross,net,avg);
		printf("\n\nPlease follow up on these invoices as soon as po");
		printf("ssible.\n\n--\nThe Accounts Department\n");
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: arreport [-c <comp>] [-v]\n");
	exit(2);
}
