#ident "$Id: crnote.c,v 1.8 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: crnote.c,v 1.8 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: crnote.c,v $
 * Revision 1.8  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.7  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.6  2008/01/23 16:29:21  dtynan
 * Added the ability to run the apps without effect (-n) and fixed some
 * effective-date issues with invoice postings.
 *
 * Revision 1.5  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2008/01/15 15:34:59  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.3  2006/10/01 10:32:33  dtynan
 * Major changes for VAT.
 *
 * Revision 1.2  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.1  2005/12/18 16:11:53  dtynan
 * Added a new program to generate credit notes.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#define PROGNAME	"crnote"

int	noeffect;
int	verbose;

extern	int	optind, opterr;
extern	char	*optarg;

void	creditnote(char *, time_t);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	time_t when;

	/*
	 * Process command-line options.
	 */
	cname = NULL;
	opterr = noeffect = verbose = 0;
	time(&when);
	while ((i = getopt(argc, argv, "c:d:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				usage();
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) < 1)
		usage();
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	/*
	 *
	 */
	while (optind < argc)
		creditnote(argv[optind++], when);
	exit(0);
}

/*
 *
 */
void
creditnote(char *inv, time_t when)
{
	int invno, itemno;
	static int cnitemno = 0;
	char *cp;
	struct db_invoice *ip;
	struct db_invoice_item *iip;
	static struct db_customer *cusp = NULL;
	static struct db_invoice *cnp = NULL;
	dbow_conn *dbp = getdbase();

	printf("Credit note against: %s\n", inv);
	if ((cp = strchr(inv, ':')) != NULL) {
		*cp++ = '\0';
		itemno = atoi(cp);
	} else
		itemno = -1;
	invno = atoi(inv);
	printf("%d (%d)\n", invno, itemno);
	if ((ip = db_findinvoicebyid(dbp, invno)) == NULL)
		berror(PROGNAME, "cannot find invoice");
	if (cusp == NULL) {
		cusp = db_findcustomerbyid(dbp, ip->customer_id);
		if (cusp == NULL)
			berror(PROGNAME, "cannot find customer for invoice");
	} else if (cusp->id != ip->customer_id)
		berror(PROGNAME, "invoice is for a different customer");
	if (cnp == NULL) {
		cnp = ip;
		cnp->cninv_id = ip->id;
		cnp->id = 0;
		time((time_t *)&cnp->idate);
		cnp->total = cnp->htotal = cnp->vatttl = cnp->shipping = 0.0;
		cnp->state = INVOICE_STATE_NEW;
		if (!noeffect && db_insertinvoice(dbp, cnp) < 0)
			berror(PROGNAME, "cannot create credit note");
		printf("CR #%d\n", cnp->id);
	} else if (cnp->cninv_id != ip->id)
		berror(PROGNAME, "cannot issue a credit note against more than one invoice.");
	else
		db_invoicefree(ip);
	if (itemno == -1)
		iip = db_findinvoice_itembyinvoice_id(dbp, invno);
	else
		iip = findinvoiceitem(dbp, invno, itemno);
	if (iip == NULL)
		berror(PROGNAME, "cannot find invoice item");
	while (iip != NULL) {
		iip->unitc = -iip->unitc;
		iip->invoice_id = cnp->id;
		iip->item_id = ++cnitemno;
		if (!noeffect && db_insertinvoice_item(dbp, iip) < 0)
			berror(PROGNAME, "cannot create credit note item");
		iip = db_findinvoice_itemnext(dbp, iip);
	}
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: %s [-nv][-c <comp>][-d date] <invno[:item]> ...\n", PROGNAME);
	exit(2);
}
