#ident "Kalopa: $Id: geninvoice.c,v 1.44 2008/10/15 15:11:13 dtynan Exp $"

/*
 * $Id: geninvoice.c,v 1.44 2008/10/15 15:11:13 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: geninvoice.c,v $
 * Revision 1.44  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.43  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.42  2008/07/08 17:10:17  dtynan
 * Extensive changes.
 *
 * Revision 1.41  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.40  2008/01/15 15:34:59  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.39  2007/10/29 17:23:39  dtynan
 * Extensive changes to handle hourly billing and various other tweaks.
 *
 * Revision 1.38  2006/10/01 10:32:33  dtynan
 * Major changes for VAT.
 *
 * Revision 1.37  2006/08/25 11:27:14  dtynan
 * Lots of changes as per the end-of-year.
 *
 * Revision 1.36  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.35  2006/03/06 12:27:04  dtynan
 * Fixed bug where a lack of an IBAN number would cause a core dump.
 *
 * Revision 1.34  2005/12/19 23:07:35  dtynan
 * Changed to now output VAT information.
 *
 * Revision 1.33  2005/12/19 22:18:24  dtynan
 * Fixed issues with the way the VAT code letter was printed.
 *
 * Revision 1.32  2005/10/17 13:35:27  dtynan
 * First pass at a multicurrency invoicing system.
 *
 * Revision 1.31  2005/09/21 19:16:25  dtynan
 * Added the facility to create a credit note.
 *
 * Revision 1.30  2005/09/21 18:10:28  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.29  2005/04/11 21:16:32  dtynan
 * Cleaned up the code a bit and also ensured that without the -p option
 * to geninvoice, the output goes to stdout.  Could be nice to have a -o
 * option for this (and statement.c).
 *
 * Revision 1.28  2005/04/11 16:06:31  dtynan
 * Changed code to use new function for printing address blocks in troff.
 *
 * Revision 1.27  2005/04/07 17:57:42  dtynan
 * The 'PrePrint' variable should be set to 1 if run manually.
 *
 * Revision 1.26  2005/03/22 09:41:25  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.25  2005/03/09 11:22:13  dtynan
 * Removed custcode from the customer table.  Now replaced with ccode
 * from the contact table.
 *
 * Revision 1.24  2005/03/08 16:38:19  dtynan
 * Added a new function to auto-generate a Google AdSense invoice (once
 * payment has been received) and removed the vatcode() function.
 *
 * Revision 1.23  2005/03/05 19:28:52  dtynan
 * Fixed bug where country wasn't printed on invoice.
 *
 * Revision 1.22  2005/03/03 12:05:25  dtynan
 * Added IBAN details to invoices.
 *
 * Revision 1.21  2005/02/17 16:23:21  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.20  2004/12/09 20:23:36  dtynan
 * Cleaned up include mechanisms.
 *
 * Revision 1.19  2004/12/09 20:03:42  dtynan
 * Make sure we emit the organisation name when creating an invoice.
 *
 * Revision 1.18  2004/10/30 19:04:11  dtynan
 * New invoice schema uses a number register to specify the company.
 *
 * Revision 1.17  2004/08/26 17:24:10  dtynan
 * Minor changes and tweaks - nothing serious.
 *
 * Revision 1.16  2004/08/05 12:49:58  dtynan
 * Fixed bug where customer code was being defined twice.
 *
 * Revision 1.15  2004/08/04 13:15:24  dtynan
 * Make sure the company number is handled properly.
 *
 * Revision 1.14  2004/08/04 11:01:49  dtynan
 * Various changes for new & improved system.  Made the invoice header
 * company-specific, modified the primary key of the journal and removed
 * extraneous income types.
 *
 * Revision 1.13  2004/07/27 13:12:17  dtynan
 * Changed output filename and fixed cmdline option handling.
 *
 * Revision 1.12  2004/07/16 19:23:32  dtynan
 * Extensive changes to split the invoicing into posting and generating.
 * Also fixed some bugs in the site billing code, added a default nominal
 * account (5060) and fixed some bugs in the invoice emailing code.  Added
 * makepath() calls to generate the correct paths.
 *
 * Revision 1.11  2004/07/14 15:00:33  dtynan
 * Changed name of column in 'country' table.
 *
 * Revision 1.10  2004/07/12 19:07:12  dtynan
 * Extensive changes for new directory structure and other things.
 *
 * Revision 1.9  2004/05/12 15:21:05  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.8  2004/01/05 18:09:17  dtynan
 * Lots of changes to help automate invoice generation and emailing.
 *
 * Revision 1.7  2004/01/05 11:53:44  dtynan
 * Various changes around the filesystem layout and other minor details.
 *
 * Revision 1.6  2003/11/26 20:04:19  dtynan
 * Fixed groff output to include .so command.
 *
 * Revision 1.5  2003/11/08 16:13:54  dtynan
 * Fixed a couple of portability issues.
 *
 * Revision 1.4  2003/11/08 15:23:22  dtynan
 * Added a new file for generating website invoices, and modified the
 * geninvoice code to automatically post invoices.
 *
 * Revision 1.3  2003/11/07 15:58:10  dtynan
 * Fixed implementation of invoice generation and emailing thereof.
 *
 * Revision 1.2  2003/10/31 17:18:05  dtynan
 * Various changes and fixes for new/improved accounting system.
 *
 * Revision 1.1  2003/10/31 14:35:15  dtynan
 * First pass at automated billing software.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

int	verbose;
int	crnote;
int	psflag;
int	lno;
char	*ipath;
double	subttl;

struct	vatamt	*vat;

extern	int	optind, opterr;
extern	char	*optarg;

void	doitem(dbow_conn *, FILE *, struct db_invoice_item *, char *);
void	usage();

struct	db_vat	*db_findvatcodes(dbow_conn *);

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, curr;
	char *cname, *currlabel, cmd[512];
	dbow_conn *dbp;
	struct db_company *comp;
	struct db_invoice *invp;
	struct db_invoice_item *iip;
	struct db_customer *cusp;
	struct db_bankacct *bap;
	struct db_currency *curp;
	struct vatamt *vp;
	struct tm *tmp;
	FILE *fp;

	psflag = verbose = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:pv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'p':
			/*
			 * Generate postscript rather than troff.
			 */
			psflag = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1)
		usage();
	loadconfig(cname);
	/*
	 * Find the company that we're processing...
	 */
	vat = vat_init();
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("geninvoice", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	if ((i = atoi(argv[optind++])) < 1)
		berror("geninvoice", "invalid invoice number");
	if (verbose)
		printf("Searching for invoice #%d...\n", i);
	invp = db_findinvoicebyid(dbp, i);
	if (invp == NULL)
		berror("geninvoice", "cannot find invoice");
	if (verbose)
		printf("Generating invoice #%d...\n", invp->id);
	lno = 0;
	subttl = 0.0;
	if (psflag) {
		sprintf(cmd, "groff -s -t -Tps");
		if ((fp = popen(cmd, "w")) == NULL)
			berror("geninvoice", "popen");
	} else
		fp = stdout;
	if (invp->cninv_id != 0) {
		crnote = 1;
		fprintf(fp, ".ds CrNote %s\n", crnote_number(invp->id));
		fprintf(fp, ".ds InvNo %s\n", invoice_number(invp->cninv_id));
	} else {
		crnote = 0;
		fprintf(fp, ".ds InvNo %s\n", invoice_number(invp->id));
	}
	tmp = localtime((time_t *)&invp->idate);
	fprintf(fp, ".ds InvDate %s\n",
		prdate(tmp->tm_mday, tmp->tm_mon + 1, tmp->tm_year + 1900));
	fprintf(fp, ".ds InvPX 1\n.ds InvPY 1\n");
	if (comp->vatno != NULL)
		fprintf(fp, ".ds VATNo %s\n", comp->vatno);
	if (invp->customer_id == 0) {
		/*
		 * Cash sale.
		 */
		cusp = NULL;
	} else if ((cusp=db_findcustomerbyid(dbp, invp->customer_id)) == NULL)
		berror("geninvoice", "cannot find customer record for invoice");
	if (cusp != NULL && cusp->currency_id > 0)
		curr = cusp->currency_id;
	else
		curr = HOME_CURRENCY;
	if ((curp = findcurrency(dbp, curr)) == NULL)
		berror("geninvoice", "cannot find customer currency");
	if (cusp == NULL) {
		fprintf(fp, ".ds Name CASH SALE\n");
		fprintf(fp, ".ds ORef CS/%d\n", invp->id);
	} else {
		gentroffaddr(fp, dbp, cusp->contact_id, cusp->cperson_id);
		fprintf(fp, ".ds ORef R%d/%d\n", cusp->id, invp->id);
	}
	fprintf(fp, ".ds Curr %s\n", curp->lname);
	if (invp->salesorder_id != 0)
		fprintf(fp, ".ds Order %d\n", invp->salesorder_id);
	if (invp->yref != NULL)
		fprintf(fp, ".ds YRef %s\n", invp->yref);
	tmp = localtime((time_t *)&invp->odate);
	fprintf(fp, ".ds ODate %d/%02d/%04d\n",
			tmp->tm_mday, tmp->tm_mon + 1, tmp->tm_year + 1900);
	if (invp->descr != NULL)
		fprintf(fp, ".ds Desc %s\n", invp->descr);
	if (cusp != NULL && cusp->vat != NULL)
		fprintf(fp, ".ds YVAT %s\n", cusp->vat);
	else
		fprintf(fp, ".ds YVAT N/A\n");
	if ((bap = db_findbankacctbyid(dbp, 1)) != NULL && bap->iban != NULL)
		fprintf(fp, ".ds IBAN %s\n", bap->iban);
	iip = db_findinvoice_itembyinvoice_id(dbp, invp->id);
	currlabel = (curp->rofflabel != NULL) ? curp->rofflabel : "";
	while (iip != NULL) {
		doitem(dbp, fp, iip, currlabel);
		iip = db_findinvoice_itemnext(dbp, iip);
	}
	vat_total(vat);
	if (cusp != NULL) {
		if (cusp->terms != NULL)
			fprintf(fp, ".ds Terms %s\n", cusp->terms);
		else
			fprintf(fp, ".ds Terms Net 30 Days\n");
	} else
		fprintf(fp, ".ds Terms CWO - Cash With Order\n");
	fprintf(fp, ".ds SubTTL %s\n", bal(subttl, currlabel));
	fprintf(fp, ".ds Pkg %s\n", bal(invp->shipping, NULL));
	fprintf(fp, ".ds VAT %s\n", bal(invp->vatttl, NULL));
	fprintf(fp, ".ds Total %s\n", bal(invp->total, currlabel));
	for (i = 1, vp = vat; vp != NULL; vp = vp->next, i++) {
		fprintf(fp, ".ds VCode%d %c\n", i, vp->vcode);
		fprintf(fp, ".ds VRate%d %s\n", i, bal(vp->rate, NULL));
		if (vp->total == 0.0)
			continue;
		fprintf(fp, ".ds VNet%d %s\n", i, bal(vp->total, currlabel));
		fprintf(fp, ".ds VAmt%d %s\n", i, bal(vp->amount, currlabel));
	}
	fprintf(fp, ".ds Company %s\n", comp->tag);
	fprintf(fp, ".nr PrePrint %d\n", psflag ? 0 : 1);
	if (crnote)
		fprintf(fp, ".so %s.tmac\n", makepath("roff", "crnote", NULL));
	else
		fprintf(fp, ".so %s.tmac\n", makepath("roff", "invoice", NULL));
	if (psflag)
		pclose(fp);
	db_invoicefree(invp);
	exit(0);
}

/*
 *
 */
void
doitem(dbow_conn *dbp, FILE *fp, struct db_invoice_item *iip, char *clabel)
{
	char *cp, *xp;
	double ttl;

	if ((cp = strchr(iip->descr, '\n')) != NULL)
		*cp++ = '\0';
	ttl = iip->quant * iip->unitc;
	ttl = (100.0 - iip->disc) * ttl / 100.0;
	vat_lineitem(vat, ttl, iip->vcode, 0, VATDATA_NOPOST);
	if (crnote)
		ttl = -ttl;
	subttl += ttl;

	fprintf(fp, ".ds L%02da %s\n", ++lno, roffize(iip->descr));
	fprintf(fp, ".ds L%02db %c\n", lno, vatcode(iip->vcode));
	fprintf(fp, ".ds L%02dc %.2f\n.ds L", lno, iip->quant);
	if (crnote)
		fprintf(fp, "%02dd %s\n", lno, bal(-iip->unitc, NULL));
	else
		fprintf(fp, "%02dd %s\n", lno, bal(iip->unitc, NULL));
	fprintf(fp, ".ds L%02de %.2f\n", lno, iip->disc);
	if (lno > 1)
		clabel = NULL;
	fprintf(fp, ".ds L%02df %s\n", lno, bal(ttl, clabel));

	while (cp != NULL) {
		xp = cp;
		if ((cp = strchr(xp, '\n')) != NULL)
			*cp++ = '\0';
		fprintf(fp, ".ds L%02da \\ \\ \\ \\ %s\n", ++lno, roffize(xp));
	}
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: geninvoice [-c <comp>][-p][-v] <invno>\n");
	exit(2);
}
