#ident "Kalopa: $Id: invemail.c,v 1.21 2008/10/31 15:17:13 dtynan Exp $"

/*
 * $Id: invemail.c,v 1.21 2008/10/31 15:17:13 dtynan Exp $
 *
 * Copyright (c) 2004, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: invemail.c,v $
 * Revision 1.21  2008/10/31 15:17:13  dtynan
 * Extensive changes for version 0.8.2.
 *
 * Revision 1.20  2008/10/31 15:08:42  dtynan
 * Fixed a consistent bug where -c was being used with a numeric value.
 *
 * Revision 1.19  2008/10/15 15:11:13  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.18  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.17  2008/01/15 15:34:59  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.16  2006/10/01 10:32:33  dtynan
 * Major changes for VAT.
 *
 * Revision 1.15  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.14  2005/10/16 10:02:55  dtynan
 * Fixed bug where email address was incorrectly formed.
 *
 * Revision 1.13  2005/08/15 12:04:10  dtynan
 * Minor tweaks to fix some MIME issues.
 *
 * Revision 1.12  2005/08/12 18:57:50  dtynan
 * Changed to use a slightly different format - now an invoice is
 * sent with a statement the first time but only a statement is
 * sent as a reminder.
 *
 * Revision 1.11  2005/03/22 09:41:25  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.10  2005/02/22 08:21:27  dtynan
 * Fixed bug in processing of 'To:' field.
 *
 * Revision 1.9  2005/02/17 16:23:22  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.8  2004/12/09 20:29:16  dtynan
 * Use new mechanism for determining organisation name.
 *
 * Revision 1.7  2004/12/09 20:23:36  dtynan
 * Cleaned up include mechanisms.
 *
 * Revision 1.6  2004/09/07 12:00:24  dtynan
 * Fixed issue where geninvoice was being called without specifying
 * the company number (which defaults to 1).
 *
 * Revision 1.5  2004/08/04 13:46:30  dtynan
 * Fixed minor bug where -P option was changed to -p in geninvoice.
 *
 * Revision 1.4  2004/07/16 19:23:32  dtynan
 * Extensive changes to split the invoicing into posting and generating.
 * Also fixed some bugs in the site billing code, added a default nominal
 * account (5060) and fixed some bugs in the invoice emailing code.  Added
 * makepath() calls to generate the correct paths.
 *
 * Revision 1.3  2004/07/12 19:07:12  dtynan
 * Extensive changes for new directory structure and other things.
 *
 * Revision 1.2  2004/01/05 19:05:21  dtynan
 * Send reminders to the correct person.
 *
 * Revision 1.1  2004/01/05 18:09:17  dtynan
 * Lots of changes to help automate invoice generation and emailing.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

/*
 *
 */
void
invoicemail(struct db_company *comp, struct db_customer *cusp, struct db_invoice *invp, char *file)
{
	int doneto = 0;
	struct db_contact *contp;
	struct db_contact_person *cpp;
	char to[256], cmd[1024], *cp, *email, *email_ovr;
	dbow_conn *dbp = getdbase();
	FILE *fp;

	/*
	 * Find the customer record and the contact details...
	 */
	if ((contp = contact_findbyid(dbp, cusp->contact_id)) == NULL)
		berror("invoicemail", "cannot find customer contact details");
	if (cusp->cperson_id > 0)
		cpp = person_findbyid(dbp, cusp->cperson_id);
	else
		cpp = NULL;
	/*
	 * How do they want their bill?  The 'email_override' config value
	 * allows us to hard-code the email output to a specific person.
	 * This is mostly for test purposes.
	 */
	email = NULL;
	email_ovr = getconfig("email_override", NULL);
	switch (cusp->billtype) {
	case CUST_BTYPE_NONE:
		/*
		 * Doesn't want to receive any invoices!  Send it to
		 * the administrator instead.
		 */
		break;

	case CUST_BTYPE_POST:
		/*
		 * We have to post invoices to this customer.
		 * For now, email it to the administrator who can
		 * handle it manually.
		 */
		break;

	case CUST_BTYPE_FAX:
		/*
		 * This customer receives invoices via fax.
		 * For now, email it to the administrator who can
		 * handle it manually.
		 */
		if (cpp != NULL && cpp->fax != NULL)
			email = makefaxemail(cpp->fax);
		else if (contp->fax != NULL)
			email = makefaxemail(contp->fax);
		if (email == NULL)
			bwarn("invoicemail", "no contact fax for customer");
		break;

	case CUST_BTYPE_EMAIL:
		/*
		 * This customer accepts invoices via email.  First see
		 * if there is a contact person for the customer and
		 * if they have an email address.  If not, use the
		 * main email address for the customer.  If all else
		 * fails, email it to the administrator.
		 */
		if (cpp != NULL && cpp->email != NULL) {
			if (email_ovr == NULL)
				email = cpp->email;
			else
				email = email_ovr;
			if (cpp->fname == NULL) {
				sprintf(to, "%s <%s>", cpp->lname, email);
				email_setarg(1, "Valued Customer");
			} else {
				sprintf(to, "%s %s <%s>", cpp->fname,
							cpp->lname, email);
				email_setarg(1, cpp->fname);
			}
			doneto = 1;
		} else if (contp->email != NULL)
			email = contp->email;
		else
			bwarn("invoicemail", "no contact email for customer");
		break;
	}
	if (email_ovr != NULL)
		email = email_ovr;
	else if (email == NULL)
		email = getconfig("from_email", "beanie");
	if (!doneto) {
		sprintf(to, "%s <%s>", orgname(contp), email);
		email_setarg(1, "Valued Customer");
	}
	if ((fp = fopen(makepath("email", file, NULL), "r")) == NULL) {
		bwarn("invoicemail", "cannot open template file");
		return;
	}
	if (invp != NULL) {
		if (invp->cninv_id != 0) {
			email_setarg(2, crnote_number(invp->id));
			email_setarg(3, invoice_number(invp->cninv_id));
		} else
			email_setarg(2, invoice_number(invp->id));
	} else
		email_setarg(2, "");
	if (invp == NULL)
		sprintf(cmd, "Account Statement from %s", comp->cname);
	else if (invp->cninv_id != 0) {
		sprintf(cmd, "Credit Note %s from %s", crnote_number(invp->id),
								comp->cname);
	} else {
		sprintf(cmd, "Invoice %s from %s", invoice_number(invp->id),
								comp->cname);
	}
	email_open(getconfig("from_eaddr", "beanie"), to, cmd);
	while (fgets(cmd, sizeof(cmd), fp) != NULL) {
		if ((cp = strchr(cmd, '\n')) != NULL)
			*cp = '\0';
		if (*cmd == '#')
			continue;
		email_text(cmd);
	}
	fclose(fp);
	email_text("");
	email_text("");
	sprintf(cmd, "%s -c %s -p %s | %s",
					makepath("bin", "statement", NULL),
					comp->tag, contp->ccode,
					makepdf(NULL));
	if ((fp = popen(cmd, "r")) == NULL)
		berror("invemail", "statement pipe");
	email_fptr(fp, "statement.pdf", "application/pdf");
	pclose(fp);
	if (invp != NULL) {
		sprintf(cmd, "%s -c %s -p %d | %s",
					makepath("bin", "geninvoice", NULL),
					comp->tag, invp->id,
					makepdf(NULL));
		if ((fp = popen(cmd, "r")) == NULL)
			berror("invemail", "geninvoice pipe");
		if (invp->cninv_id != 0)
			sprintf(cmd, "crnote%d.pdf", invp->id);
		else
			sprintf(cmd, "invoice%d.pdf", invp->id);
		email_fptr(fp, cmd, "application/pdf");
		pclose(fp);
	}
	email_close();
}
