#ident "Kalopa: $Id: postinvoice.c,v 1.33 2008/10/31 15:17:13 dtynan Exp $"

/*
 * $Id: postinvoice.c,v 1.33 2008/10/31 15:17:13 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: postinvoice.c,v $
 * Revision 1.33  2008/10/31 15:17:13  dtynan
 * Extensive changes for version 0.8.2.
 *
 * Revision 1.32  2008/10/20 17:53:25  dtynan
 * Added a document for installation (quite basic at this point), removed
 * some debug output from tpadjust(), and fixed a lingering issue with the
 * way invoice due-dates were calculated.
 *
 * Revision 1.31  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.30  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.29  2008/07/18 11:07:38  dtynan
 * Fixed issue with isholiday/skipholiday where it wouldn't actually detect
 * bank holidays (only weekends).  It now interrogates the database.
 *
 * Revision 1.28  2008/07/08 17:10:18  dtynan
 * Extensive changes.
 *
 * Revision 1.27  2008/01/23 16:29:21  dtynan
 * Added the ability to run the apps without effect (-n) and fixed some
 * effective-date issues with invoice postings.
 *
 * Revision 1.26  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.25  2007/08/07 07:48:13  dtynan
 * When posting a credit note, don't add the invoice description to the
 * Sales Ledger, add "Credit Note" instead.
 *
 * Revision 1.24  2006/10/01 10:32:33  dtynan
 * Major changes for VAT.
 *
 * Revision 1.23  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.22  2005/12/19 22:17:43  dtynan
 * Added rudimentary support for VAT - still need to sort out some
 * multicurrency issues.
 *
 * Revision 1.21  2005/10/17 15:41:47  dtynan
 * Tweaked the roundoff routine.
 *
 * Revision 1.20  2005/10/17 15:12:42  dtynan
 * Use the new roundoff function for accurate discount calculations.
 *
 * Revision 1.19  2005/10/17 13:35:27  dtynan
 * First pass at a multicurrency invoicing system.
 *
 * Revision 1.18  2005/09/21 18:10:28  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.17  2005/08/12 18:57:50  dtynan
 * Changed to use a slightly different format - now an invoice is
 * sent with a statement the first time but only a statement is
 * sent as a reminder.
 *
 * Revision 1.16  2005/04/11 13:52:04  dtynan
 * Added code to post invoice data to sales ledger.
 *
 * Revision 1.15  2005/03/22 09:41:25  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.14  2005/03/09 11:22:13  dtynan
 * Removed custcode from the customer table.  Now replaced with ccode
 * from the contact table.
 *
 * Revision 1.13  2005/02/17 17:25:28  dtynan
 * Fixed some compiler warnings.
 *
 * Revision 1.12  2005/02/17 16:23:22  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.11  2005/01/13 18:19:22  dtynan
 * Added code to correctly handle prepayments.
 *
 * Revision 1.10  2005/01/13 17:50:31  dtynan
 * Debit the customer record by the invoice amount.
 *
 * Revision 1.9  2004/12/09 20:33:57  dtynan
 * Fixed bug in include files.
 *
 * Revision 1.8  2004/12/09 20:29:16  dtynan
 * Use new mechanism for determining organisation name.
 *
 * Revision 1.7  2004/12/08 14:30:37  dtynan
 * Fixed a bug.
 *
 * Revision 1.6  2004/12/08 14:29:42  dtynan
 * Now adds the invoice # to the journal folio and calculates the invoice
 * total.
 *
 * Revision 1.5  2004/09/07 12:00:24  dtynan
 * Fixed issue where geninvoice was being called without specifying
 * the company number (which defaults to 1).
 *
 * Revision 1.4  2004/08/26 17:24:10  dtynan
 * Minor changes and tweaks - nothing serious.
 *
 * Revision 1.3  2004/08/05 12:51:53  dtynan
 * Added code to post an invoice to the journal.
 *
 * Revision 1.2  2004/08/04 13:15:24  dtynan
 * Make sure the company number is handled properly.
 *
 * Revision 1.1  2004/07/16 19:23:33  dtynan
 * Extensive changes to split the invoicing into posting and generating.
 * Also fixed some bugs in the site billing code, added a default nominal
 * account (5060) and fixed some bugs in the invoice emailing code.  Added
 * makepath() calls to generate the correct paths.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

struct	jitem	{
	int	acctno;
	double	amount;
} jitem[20];

int		noeffect;
int		verbose;
struct db_company *comp;
struct	vatamt	*vat;

extern	int	optind, opterr;
extern	char	*optarg;

void	postinvoice(struct db_invoice *);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	dbow_conn *dbp;
	struct db_invoice *invp;

	verbose = noeffect = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("postinvoice", "cannot find company");
	vat = vat_init(dbp);
	/*
	 * Post all the invoices that aren't complete.
	 */
	invp = db_findinvoicebystate(dbp, INVOICE_STATE_NEW);
	while (invp != NULL) {
		postinvoice(invp);
		invp = db_findinvoicenext(dbp, invp);
	}
	exit(0);
}

/*
 *
 */
void
postinvoice(struct db_invoice *invp)
{
	int i, njitem, curr;
	struct jitem *xp;
	struct db_journal *jp;
	struct db_journal_item *jip;
	struct db_invoice *pip;
	struct db_invoice_item *iip;
	struct db_customer *cusp;
	struct db_contact *cp;
	struct db_sledger *slp;
	char descr[256], invid[64];
	double hval, aval, ttl;
	dbow_conn *dbp = getdbase();

	if (verbose)
		printf("Posting invoice/crnote #%d to journal...\n", invp->id);
	vat_clear(vat);
	/*
	 * Find the name of the customer and get their default currency.
	 */
	if (invp->customer_id == 0) {
		/*
		 * Cash sale.
		 */
		cusp = NULL;
	} else if ((cusp = db_findcustomerbyid(dbp, invp->customer_id)) == NULL)
		berror("postinvoice", "cannot find customer record");
	if (cusp != NULL && cusp->currency_id > 0)
		curr = cusp->currency_id;
	else
		curr = HOME_CURRENCY;
	if (invp->currency_id == 0)
		invp->currency_id = curr;
	/*
	 * Update the invoice fields, including the 'due date' and the
	 * invoice state (via the dunning system).
	 */
	if (invp->idate == 0)
		time((time_t *)&invp->idate);
	dunning(invp);
	invp->vatttl = 0.0;
	invp->shipping = 0.0;
	if (verbose)
		printf("Due date: %s", ctime((time_t *)&invp->ddate));
	/*
	 * Generate the journal entry title
	 */
	if (cusp != NULL) {
		cp = contact_findbyid(dbp, cusp->contact_id);
		sprintf(descr, "%s for %s", invp->cninv_id != 0 ?
				"Credit Note" : "Invoice", orgname(cp));
	} else {
		sprintf(descr, "Cash %s", invp->cninv_id != 0 ?
				"Credit Note" : "Invoice");
	}
	sprintf(invid, "%d", invp->id);
	if (verbose)
		printf("Posting title: [%s] (%s)\n", descr, invid);
	/*
	 * Allocate (and stuff) a journal entry.
	 */
	if ((jp = db_journalalloc()) == NULL)
		berror("postinvoice", "cannot create journal entry");
	jp->jdate = invp->idate;
	jp->folio = strdup(invid);
	jp->descr = strdup(descr);
	jp->user_id = getuid();
	jp->state = 0;
	if (!noeffect && db_insertjournal(dbp, jp) < 0)
		berror("postinvoice", "cannot create journal entry");
	/*
	 * Find out how many different nominal accounts are needed.
	 * The first is for accounts-receivable which is debited.
	 * 'aval' is the actual value of the invoice item in the
	 * customer currency, and 'hval' is the home value of the
	 * invoice in our local currency.  Note that the customer
	 * will be invoiced in their currency, but the posting will
	 * be in our currency.  When payment is made (in their
	 * currency), we use N/A "9400; currency gains and losses"
	 * to account for any discrepancy.
	 */
	njitem = 1;
	xp = jitem;
	xp->acctno = (cusp == NULL) ? NA_CASHSALES : NA_DEBTORSCA;
	xp->amount = ttl = 0.0;
#ifdef VAT_CALCULATE
	xp++;
	njitem++;
	xp->acctno = NA_VATACCOUNT;
	xp->amount = 0.0;
#endif
	iip = db_findinvoice_itembyinvoice_id(dbp, invp->id);
	while (iip != NULL) {
		invoice_total(invp, iip, &aval, &hval);
		ttl += aval;
		for (i = 0, xp = jitem; i < njitem; i++, xp++)
			if (xp->acctno == iip->nomacct_id)
				break;
		if (i == njitem && ++njitem < 20) {
			jitem[i].acctno = iip->nomacct_id;
			jitem[i].amount = 0.0;
		}
		jitem[i].amount += hval;
		jitem[0].amount += hval;
		vat_lineitem(vat, hval, iip->vcode, jp->id, VATDATA_SALES);
		iip = db_findinvoice_itemnext(dbp, iip);
	}
	/*
	 * Set the invoice total.  This is done here because we can
	 * add multiple invoice line items to an invoice over the
	 * course of a business day so we don't need to keep
	 * recomputing the invoice total until we post the invoice.
	 * Also remember what the home value of the invoice is, for
	 * computing currency gains and losses later on.
	 *
	 * HACK: We need to add a journal line item for shipping
	 * charges.  They get totalled in the first line item but
	 * don't appear anywhere else.
	 */
#ifdef VAT_CALCULATE
	invp->vatttl = vat_total(vat);
	jitem[1].amount = currconv(invp->vatttl, invp->currency_id);
	jitem[0].amount += jitem[1].amount;
#endif
	jitem[0].amount += currconv(invp->shipping, invp->currency_id);
	invp->total = ttl + invp->vatttl + invp->shipping;
	invp->htotal = jitem[0].amount;
	/*
	 * Now allocate (and stuff) each journal item entry.
	 */
	jip = db_journal_itemalloc();
	jip->journal_id = jp->id;
	db_journalfree(jp);
	for (i = 0, xp = jitem; i < njitem; i++, xp++) {
		if (verbose)
			printf("%d - %.2f\n", xp->acctno, xp->amount);
		jip->item_id = i + 1;
		jip->nomacct_id = xp->acctno;
		if (xp->amount < 0.0) {
			jip->drcrf = (i == 0) ? 1 : 0;
			jip->amount = -xp->amount;
		} else {
			jip->drcrf = (i == 0) ? 0 : 1;
			jip->amount = xp->amount;
		}
		if (!noeffect && db_insertjournal_item(dbp, jip) < 0)
			berror("postinvoice", "cannot create journal item");
	}
	db_journal_itemfree(jip);
	/*
	 * Update the customer balance.  If they are still in credit,
	 * then show the prepayment amount as going away.
	 */
	if (cusp!=NULL && invp->total>=0.0 && cusp->balance>=invp->total) {
		/*
		 * Allocate a journal entry for recognising the prepayment.
		 */
		sprintf(descr, "Prepayment on invoice #%d", invp->id);
		if (!noeffect) {
			journal_entry(&invp->idate, NULL, descr, 0,
					NA_PREPAYMENTS, NA_DEBTORSCA,
					invp->htotal);
		}
		invp->state = INVOICE_STATE_PAID;
	}
	if (cusp != NULL) {
		cusp->balance -= invp->total;
		if (verbose)
			printf("New customer balance: %.2f\n", cusp->balance);
	}
	/*
	 * If this is a credit note and it covers the full amount of the
	 * credited invoice, close them both out now.
	 */
	if (invp->cninv_id != 0) {
		if ((pip = db_findinvoicebyid(dbp, invp->cninv_id)) == NULL)
			berror("postinvoice", "cannot find parent invoice");
		if (invp->total <= -pip->total) {
			pip->state = INVOICE_STATE_PAID;
			invp->state = INVOICE_STATE_PAID;
		}
	}
	/*
	 * Update the database with the invoice changes.
	 */
	if (!noeffect) {
		if (db_updateinvoicebyid(dbp, invp, invp->id) < 0)
			berror("postinvoice", "cannot update invoice");
		if (cusp!=NULL && db_updatecustomerbyid(dbp, cusp, cusp->id)<0)
			berror("postinvoice", "cannot update customer");
	}
	/*
	 * Add a record to the sales ledger.
	 */
	if ((slp = db_sledgeralloc()) == NULL)
		berror("postinvoice", "cannot generate sales ledger entry");
	slp->customer_id = invp->customer_id;
	slp->sldate = invp->idate;
	slp->folio = strdup(invid);
	if (invp->cninv_id != 0)
		slp->descr = strdup("Credit Note");
	else
		slp->descr = strdup(invp->descr);
	slp->amount = -invp->total;
	if (!noeffect && db_insertsledger(dbp, slp) < 0)
		berror("postinvoice", "cannot create sales ledger entry");
	db_sledgerfree(slp);
	/*
	 * Send a copy of the invoice via the most appropriate method.
	 */
	if (!noeffect && cusp != NULL) {
		if (invp->cninv_id != 0)
			invoicemail(comp, cusp, invp, "crnote");
		else
			invoicemail(comp, cusp, invp, "invoice");
	}
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: postinvoice [-c <comp>][-nv]\n");
	exit(2);
}
