#ident "Kalopa$Id: runhours.c,v 1.8 2008/10/31 15:08:42 dtynan Exp $"

/*
 * $Id: runhours.c,v 1.8 2008/10/31 15:08:42 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: runhours.c,v $
 * Revision 1.8  2008/10/31 15:08:42  dtynan
 * Fixed a consistent bug where -c was being used with a numeric value.
 *
 * Revision 1.7  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.6  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.5  2008/01/23 16:29:21  dtynan
 * Added the ability to run the apps without effect (-n) and fixed some
 * effective-date issues with invoice postings.
 *
 * Revision 1.4  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2008/01/15 15:34:59  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.2  2007/10/29 17:34:50  dtynan
 * Fixed item no. bug.
 *
 * Revision 1.1  2007/10/29 17:23:39  dtynan
 * Extensive changes to handle hourly billing and various other tweaks.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

#define PROGNAME	"runhours"

int		noeffect;
int		verbose;
time_t		effdate;

void	invoiceitem(struct db_salesorder *, struct db_sorder_item *, struct db_invoice *, int, time_t, time_t, int);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, hours, itemno = 0;
	char *cname;
	struct db_hours *hp;
	struct db_salesorder *sop;
	struct db_sorder_item *sip;
	struct db_invoice *invp;
	time_t wstart, wend, wlast;
	struct tm *tmp;
	dbow_conn *dbp;

	/*
	 * Process command-line arguments.
	 */
	verbose = noeffect = 0;
	cname = NULL;
	time(&effdate);
	while ((i = getopt(argc, argv, "c:d:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &effdate) < 0)
				usage();
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (loadconfig(cname) < 0)
		berror(PROGNAME, "cannot load configuration file");
	if (verbose)
		printf("Effective Date:\t%s", ctime(&effdate));
	/*
	 * Run through the open sales orders and see if anything needs
	 * to be done.
	 */
	wstart = wend = hours = 0;
	sop = NULL;
	sip = NULL;
	invp = NULL;
	dbp = getdbase();
	for (hp = findhours(dbp); hp != NULL; hp = db_findhoursnext(dbp, hp)) {
		/*
		 * Start by finding the sales order.
		 */
		if (sop == NULL || sop->id != hp->salesorder_id) {
			if (sop != NULL)
				db_salesorderfree(sop);
			if ((sop = db_findsalesorderbyid(dbp,
						hp->salesorder_id)) == NULL) {
				berror(PROGNAME, "cannot find salesorder");
			}
		}
		/*
		 * Now find the line item on the sales order.
		 */
		if (sip == NULL || sip->salesorder_id != hp->salesorder_id ||
						sip->item_id != hp->item_id) {
			if (sip != NULL)
				db_sorder_itemfree(sip);
			if ((sip = findsalesorderitem(dbp, sop->id,
							hp->item_id)) == NULL) {
				berror(PROGNAME, "cannot find SO item");
			}
		}
		/*
		 * OK, make an invoice if we don't already have one.
		 */
		if (invp == NULL || invp->salesorder_id != sop->id) {
			/*
			 * Haven't created an invoice for this
			 * customer yet.  Do so now.
			 */
			if (invp != NULL)
				db_invoicefree(invp);
			if ((invp = db_invoicealloc()) == NULL)
				berror(PROGNAME, "db_invoicealloc");
			itemno = 0;
			invp->idate = effdate;
			invp->ddate = invp->idate - 1;
			invp->odate = sop->odate;
			invp->customer_id = sop->customer_id;
			invp->salesorder_id = sop->id;
			invp->yref = strdup(sop->yref);
			invp->descr = strdup(sop->descr);
			invp->state = INVOICE_STATE_NEW;
			if (!noeffect && db_insertinvoice(dbp, invp) < 0)
				berror(PROGNAME, "cannot insert invoice");
			if (verbose)
				printf("New invoice #%d...\n", invp->id);
		}
		/*
		 * Do we have a week-end?  If so, process the
		 * existing invoice and start again...
		 */
		if (wend > 0 && hp->hdate > wend) {
			invoiceitem(sop, sip, invp, ++itemno, wstart,
							wlast, hours);
			wstart = wend = hours = 0;
		}
		/*
		 * Do we have a week start?  If not, take this hour
		 * record as our week start and work forward.
		 */
		if (wstart == 0) {
			wstart = hp->hdate;
			tmp = localtime(&wstart);
			wend = wstart + DAYS(5 - tmp->tm_wday) + HOURS(8);
		}
		/*
		 * OK, all the hard work is done.  Now just accrue
		 * the hours worked and then tag this hourly record
		 * with the appropriate invoice number.
		 */
		hours += hp->hours;
		hp->invoice_id = invp->id;
		if (!noeffect && db_updatehoursbyid(dbp, hp, hp->id) < 0)
			berror(PROGNAME, "cannot update hours");
		wlast = hp->hdate;
	}
	if (wstart != 0)
		invoiceitem(sop, sip, invp, ++itemno, wstart, wlast, hours);
	exit(0);
}

/*
 *
 */
void
invoiceitem(struct db_salesorder *sop, struct db_sorder_item *sip, struct db_invoice *invp, int itemno, time_t wstart, time_t wend, int hours)
{
	int sday, smon, syear, eday, emon, eyear;
	struct db_invoice_item *iip;
	struct db_product *pp;
	struct tm *tmp;
	char descr[512], *cp;
	dbow_conn *dbp = getdbase();

	if (verbose) {
		printf(">>> Invoice line item...\n");
		printf("Contract week start: %s", ctime(&wstart));
		printf("Contract week end: %s", ctime(&wend));
		printf("Contract week hours: %d\n", hours);
	}
	/*
	 * Find the product in case we need product pricing info.
	 */
	if ((pp = db_findproductbyid(dbp, sip->product_id)) == NULL)
		berror(PROGNAME, "db_findproductbyid");
	if ((iip = db_invoice_itemalloc()) == NULL)
		berror(PROGNAME, "db_invoice_itemalloc");
	iip->invoice_id = invp->id;
	iip->item_id = itemno;
	iip->nomacct_id = pp->nomacct_id;
	/*
	 * Calculate the window for this invoice.
	 */
	if (pp->idescr == NULL)
		sprintf(descr, "%s", sip->descr);
	else
		sprintf(descr, "%s\n%s", pp->idescr, sip->descr);
	cp = strchr(descr, '\0');
	*cp++ = '\n';
	tmp = localtime(&wstart);
	sday = tmp->tm_mday;
	smon = tmp->tm_mon + 1;
	syear = tmp->tm_year + 1900;
	tmp = localtime(&wend);
	eday = tmp->tm_mday;
	emon = tmp->tm_mon + 1;
	eyear = tmp->tm_year + 1900;
	if (syear == eyear) {
		syear = 0;
		if (smon == emon) {
			emon = 0;
			if (sday == eday)
				sday = 0;
		}
	}
	strcpy(cp, prdate(sday, smon, syear));
	cp = strchr(cp, '\0');
	if (sday > 0) {
		*cp++ = ' ';
		*cp++ = '-';
		*cp++ = ' ';
	}
	strcpy(cp, prdate(eday, emon, eyear));
	iip->descr = strdup(descr);
	iip->rdate = wstart;
	iip->runlen = 1 + (wend - wstart + 86399) / 86400;
	iip->vcode = sip->vcode;
	iip->quant = (double )hours / 8.0;
	iip->disc = sip->disc;
	if (sip->unitc == 0.0)
		iip->unitc = pp->retail;
	else
		iip->unitc = sip->unitc;
	if (verbose) {
		printf(">> Item #%d...\n", iip->item_id);
		printf(">> Description: [%s]\n", iip->descr);
		printf(">> Quantity: %.2f\n", iip->quant);
		printf(">> Amount: %.2f\n", iip->unitc);
	}
	if (!noeffect && db_insertinvoice_item(dbp, iip) < 0)
		berror(PROGNAME, "cannot insert invoice item");
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: runhours [-c #][-d effdate][-nv]\n");
	exit(2);
}
