#ident "Kalopa$Id: runorders.c,v 1.13 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: runorders.c,v 1.13 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: runorders.c,v $
 * Revision 1.13  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.12  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.11  2008/01/23 16:29:21  dtynan
 * Added the ability to run the apps without effect (-n) and fixed some
 * effective-date issues with invoice postings.
 *
 * Revision 1.10  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.9  2008/01/15 15:34:59  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.8  2007/11/11 14:14:53  dtynan
 * Use a new method for updating the salesorder_item table.
 *
 * Revision 1.7  2007/10/29 17:23:39  dtynan
 * Extensive changes to handle hourly billing and various other tweaks.
 *
 * Revision 1.6  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.5  2005/12/19 21:15:49  dtynan
 * Added code to correctly calculate rdate/runlen for invoice items.
 *
 * Revision 1.4  2005/09/21 18:10:28  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 *
 * Revision 1.3  2005/08/12 19:37:15  dtynan
 * Minor tweaks prior to first tagged release.
 *
 * Revision 1.2  2005/04/03 15:21:45  dtynan
 * Extensive changes to new sales order system.
 *
 * Revision 1.1  2005/03/31 10:16:11  dtynan
 * First pass at producing a generic billing system based on what's in
 * the sales order table.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "beanie.h"
#include "hacks.h"

int		noeffect;
int		verbose;
time_t		effdate;

void	makeinvoice(struct db_salesorder *, struct db_sorder_item *);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	struct db_salesorder *sop;
	struct db_sorder_item *sip;
	dbow_conn *dbp;

	/*
	 * Process command-line arguments.
	 */
	verbose = noeffect = 0;
	cname = NULL;
	time(&effdate);
	while ((i = getopt(argc, argv, "c:d:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &effdate) < 0)
				usage();
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	loadconfig(cname);
	if (verbose)
		printf("Effective Date:\t%s", ctime(&effdate));
	dbp = getdbase();
	/*
	 * Run through the open sales orders and see if anything needs
	 * to be done.
	 */
	for (sop = db_findsalesorderfirst(dbp); sop != NULL;
				sop = db_findsalesordernext(dbp, sop)) {
		for (sip = db_findsorder_itembysalesorder_id(dbp,
				sop->id); sip != NULL;
				sip = db_findsorder_itemnext(dbp, sip)) {
			if (sip->state != SALESORDER_READY)
				continue;
			if (sip->cdate != 0 && sip->cdate <= effdate) {
				sip->state = SALESORDER_DONE;
				if (!noeffect && updateorderitem(dbp, sip) < 0)
					berror("runorders", "order update");
				continue;
			}
			if (sip->rdate < effdate)
				makeinvoice(sop, sip);
		}
	}
	exit(0);
}

/*
 *
 */
void
makeinvoice(struct db_salesorder *sop, struct db_sorder_item *sip)
{
	static int itemno = 0;
	static struct db_invoice *invp = NULL;
	int sday, smon, syear, eday, emon, eyear;
	struct db_invoice_item *iip;
	struct db_product *pp;
	struct tm *tmp;
	int edate;
	char descr[512], *cp;
	dbow_conn *dbp = getdbase();

	if (verbose)
		printf(">>>>> ID:    %d.%d\n",sop->id,sip->item_id);
	if (invp == NULL || invp->salesorder_id != sop->id) {
		/*
		 * Haven't created an invoice for this customer yet.
		 * Do so now.
		 */
		if (invp != NULL)
			db_invoicefree(invp);
		if ((invp = db_invoicealloc()) == NULL)
			berror("runorders", "db_invoicealloc");
		itemno = 0;
		invp->idate = effdate;
		invp->ddate = invp->idate - 1;
		invp->odate = sop->odate;
		invp->customer_id = sop->customer_id;
		invp->salesorder_id = sop->id;
		invp->yref = strdup(sop->yref);
		invp->descr = strdup(sop->descr);
		invp->state = INVOICE_STATE_NEW;
		if (!noeffect && db_insertinvoice(dbp, invp) < 0)
			berror("runorders", "db_insertinvoice");
		if (verbose)
			printf("New invoice #%d...\n", invp->id);
	}
	/*
	 * Find the product in case we need product pricing info.
	 */
	if ((pp = db_findproductbyid(dbp, sip->product_id)) == NULL)
		berror("runorders", "db_findproductbyid");
	if ((iip = db_invoice_itemalloc()) == NULL)
		berror("runorders", "db_invoice_itemalloc");
	iip->invoice_id = invp->id;
	iip->item_id = ++itemno;
	iip->nomacct_id = pp->nomacct_id;
	/*
	 * Calculate the window for this invoice (if any).
	 */
	if (sip->rdate <= sop->odate)
		sip->rdate = sop->odate;
	else
		sip->rdate += DAYS(1);
	tmp = localtime((time_t *)&sip->rdate);
	sday = tmp->tm_mday;
	smon = tmp->tm_mon + 1;
	syear = tmp->tm_year + 1900;
	switch (sip->freq) {
	case FREQ_DAILY:
		tmp->tm_mday += 1;
		break;

	case FREQ_WEEKLY:
		tmp->tm_mday += 7;
		break;

	case FREQ_WEEKLY2:
		tmp->tm_mday += 14;
		break;

	case FREQ_WEEKLY4:
		tmp->tm_mday += 28;
		break;

	case FREQ_MONTHLY:
		tmp->tm_mon += 1;
		break;

	case FREQ_MONTHLY2:
		tmp->tm_mon += 2;
		break;

	case FREQ_QUARTERLY:
		tmp->tm_mon += 3;
		break;

	case FREQ_BIANNUAL:
		tmp->tm_mon += 6;
		break;

	case FREQ_ANNUALLY:
		tmp->tm_mon += 12;
		break;
	}
	tmp->tm_mday--;
	edate = mktime(tmp);
	if (pp->idescr == NULL)
		sprintf(descr, "%s", sip->descr);
	else
		sprintf(descr, "%s\n%s", pp->idescr, sip->descr);
	if (sip->freq != FREQ_ONCE) {
		cp = strchr(descr, '\0');
		*cp++ = '\n';
		tmp = localtime((time_t *)&edate);
		eday = tmp->tm_mday;
		emon = tmp->tm_mon + 1;
		eyear = tmp->tm_year + 1900;
		if (syear == eyear) {
			syear = 0;
			if (smon == emon) {
				emon = 0;
				if (sday == eday)
					sday = 0;
			}
		}
		strcpy(cp, prdate(sday, smon, syear));
		cp = strchr(cp, '\0');
		*cp++ = ' ';
		*cp++ = '-';
		*cp++ = ' ';
		strcpy(cp, prdate(eday, emon, eyear));
	}
	iip->descr = strdup(descr);
	iip->rdate = sip->rdate;
	iip->runlen = 1 + (edate - sip->rdate + 86399) / 86400;
	iip->vcode = sip->vcode;
	iip->quant = sip->quant;
	iip->disc = sip->disc;
	if (sip->unitc == 0.0)
		iip->unitc = pp->retail;
	else
		iip->unitc = sip->unitc;
	if (verbose) {
		printf(">> Item #%d...\n", iip->item_id);
		printf(">> Description: [%s]\n", iip->descr);
		printf(">> Amount: %.2f\n", iip->unitc);
	}
	if (!noeffect && db_insertinvoice_item(dbp, iip) < 0)
		berror("runorders", "db_insertinvoice_item");
	/*
	 * Now update the sales order item so we don't run it again.
	 */
	switch (sip->freq) {
	case FREQ_ONCE:
		sip->state = SALESORDER_DONE;
		time(&sip->cdate);
		break;

	case FREQ_MANUALLY:
		sip->state = SALESORDER_BLOCKED;
		break;

	default:
		sip->rdate = edate;
		break;
	}
	printf("SIPDATE: %s", ctime(&sip->rdate));
	if (!noeffect && newrundate(dbp, sip) < 0)
		berror("runorders", "updateorderitem");
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: runorders [-c #][-d effdate][-nv]\n");
	exit(2);
}
