#ident "Kalopa: $Id: sendinvoice.c,v 1.20 2008/10/31 15:17:13 dtynan Exp $"

/*
 * $Id: sendinvoice.c,v 1.20 2008/10/31 15:17:13 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: sendinvoice.c,v $
 * Revision 1.20  2008/10/31 15:17:13  dtynan
 * Extensive changes for version 0.8.2.
 *
 * Revision 1.19  2008/10/20 17:53:25  dtynan
 * Added a document for installation (quite basic at this point), removed
 * some debug output from tpadjust(), and fixed a lingering issue with the
 * way invoice due-dates were calculated.
 *
 * Revision 1.18  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.17  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.16  2008/07/18 13:05:53  dtynan
 * Added the 'noeffect' flag and some additional verbose output.
 *
 * Revision 1.15  2008/07/18 11:07:38  dtynan
 * Fixed issue with isholiday/skipholiday where it wouldn't actually detect
 * bank holidays (only weekends).  It now interrogates the database.
 *
 * Revision 1.14  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.13  2006/10/01 10:32:33  dtynan
 * Major changes for VAT.
 *
 * Revision 1.12  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.11  2005/08/12 18:57:50  dtynan
 * Changed to use a slightly different format - now an invoice is
 * sent with a statement the first time but only a statement is
 * sent as a reminder.
 *
 * Revision 1.10  2005/03/22 09:41:25  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.9  2005/02/17 16:23:22  dtynan
 * Extensive changes to support new customer/contact database
 * format and to add user IDs to journal entries.
 *
 * Revision 1.8  2004/12/09 20:23:36  dtynan
 * Cleaned up include mechanisms.
 *
 * Revision 1.7  2004/09/07 12:00:24  dtynan
 * Fixed issue where geninvoice was being called without specifying
 * the company number (which defaults to 1).
 *
 * Revision 1.6  2004/08/04 13:15:25  dtynan
 * Make sure the company number is handled properly.
 *
 * Revision 1.5  2004/07/16 19:23:33  dtynan
 * Extensive changes to split the invoicing into posting and generating.
 * Also fixed some bugs in the site billing code, added a default nominal
 * account (5060) and fixed some bugs in the invoice emailing code.  Added
 * makepath() calls to generate the correct paths.
 *
 * Revision 1.4  2004/05/12 15:21:05  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.3  2004/01/05 18:09:17  dtynan
 * Lots of changes to help automate invoice generation and emailing.
 *
 * Revision 1.2  2004/01/05 11:53:44  dtynan
 * Various changes around the filesystem layout and other minor details.
 *
 * Revision 1.1  2003/12/09 15:43:02  dtynan
 * Placeholder.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

/*
 * Linked-list of customers with late invoices.
 */
struct	cuslist	{
	struct	cuslist		*next;
	struct	db_customer	*cusp;
	int			howlate;
};

/*
 * List of reminder files.
 */
char *reminder[9] = {
	NULL, NULL, NULL,
	"reminder1",
	"reminder2",
	"reminder3",
	"reminder4",
	"reminder5",
	"reminder5"
};

int		noeffect;
int		verbose;
struct	cuslist	*head;
struct	cuslist	*tail;

void	usage();

/*
 * Send reminder emails (and .PDF statement attachments) to customers who
 * are overdue.
 */
int
main(int argc, char *argv[])
{
	int i, terms;
	char *cp, *cname;
	dbow_conn *dbp;
	struct cuslist *clp;
	struct db_company *comp;
	struct db_invoice *invp;

	opterr = verbose = noeffect = 0;
	cname = NULL;
	head = tail = NULL;
	while ((i = getopt(argc, argv, "c:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
		}
	}
	loadconfig(cname);
	dbp = getdbase();
	/*
	 * Find the company that we're processing...
	 */
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("remind", "cannot find company");
	invp = findoverdueinvoice(dbp);
	while (invp != NULL) {
		/*
		 * Hard-code the payment terms to thirty days (should
		 * really come from the customer record).  Search the
		 * local list to see if we've found an invoice for this
		 * customer already.
		 */
		terms = 30;
		for (clp = head; clp != NULL; clp = clp->next)
			if (clp->cusp->id == invp->customer_id)
				break;
		if (clp == NULL) {
			/*
			 * First invoice for this guy.
			 */
			clp = (struct cuslist *)malloc(sizeof(clp));
			if (clp == NULL)
				berror("remind", "cannot allocate memory");
			clp->next = NULL;
			clp->cusp = db_findcustomerbyid(dbp, invp->customer_id);
			if (clp->cusp == NULL)
				berror("remind", "cannot find customer");
			clp->howlate = 0;
			/*
			 * Add this guy.
			 */
			if (head == NULL)
				head = clp;
			else
				tail->next = clp;
			tail = clp;
		}
		if (verbose) {
			printf("ID: %d (state=%d), ", invp->id, invp->state);
			printf("due date: %s", ctime((time_t *)&invp->ddate));
		}
		if (clp->howlate < invp->state)
			clp->howlate = invp->state;
		dunning(invp);
		if (invp->state == INVOICE_STATE_LATE5 &&
				(clp->cusp->cusflag & CUST_CREDITHOLD) == 0) {
			/*
			 * Put this fecker on Credit Hold.
			 */
			clp->cusp->cusflag |= CUST_CREDITHOLD;
			if (!noeffect && db_updatecustomerbyid(dbp, clp->cusp,
							clp->cusp->id) < 0) {
				berror("remind", "cannot update customer");
			}
			if (verbose) {
				printf("Customer ID %d on CREDIT HOLD.\n",
							clp->cusp->id);
			}
		}
		if (verbose) {
			printf("New state: %d, due: %s", invp->state,
					ctime((time_t *)&invp->ddate));
		}
		if (!noeffect && db_updateinvoicebyid(dbp, invp, invp->id) < 0)
			berror("remind", "cannot update invoice");
		invp = db_findinvoicenext(dbp, invp);
	}
	for (clp = head; clp != NULL; clp = clp->next) {
		if ((cp = reminder[clp->howlate]) == NULL)
			continue;
		/*
		 * Send a reminder email with a copy of the account
		 * statement.
		 */
		if (!noeffect)
			invoicemail(comp, clp->cusp, NULL, cp);
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: remind [-c <comp>][-v]\n");
	exit(2);
}
