#ident "Kalopa: $Id: statement.c,v 1.15 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: statement.c,v 1.15 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: statement.c,v $
 * Revision 1.15  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.14  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.13  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.12  2008/01/15 15:34:59  dtynan
 * Extensive changes for new DBOW version.
 *
 * Revision 1.11  2007/06/19 22:02:06  dtynan
 * Updated with extra components.
 *
 * Revision 1.10  2006/10/01 10:32:33  dtynan
 * Major changes for VAT.
 *
 * Revision 1.9  2006/09/12 07:42:31  dtynan
 * Fixed bug in statement generation, where the customer code could not be
 * retrieved correctly.
 *
 * Revision 1.8  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.7  2005/10/17 13:35:27  dtynan
 * First pass at a multicurrency invoicing system.
 *
 * Revision 1.6  2005/08/15 12:04:10  dtynan
 * Minor tweaks to fix some MIME issues.
 *
 * Revision 1.5  2005/08/15 11:31:40  dtynan
 * Troff address block was going to the wrong place.
 *
 * Revision 1.4  2005/08/12 18:59:03  dtynan
 * Fixed some minor bugs with the handling of negative numbers.
 *
 * Revision 1.3  2005/04/11 21:17:08  dtynan
 * Added a -p option to generate PostScript (via troff) also made some
 * cleanup changes to the output.
 *
 * Revision 1.2  2005/04/11 20:52:27  dtynan
 * Deal properly with negative numbers.
 *
 * Revision 1.1  2005/04/11 17:51:57  dtynan
 * New program to produce a customer statement.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

int		verbose;

extern	int	optind, opterr;
extern	char	*optarg;

void	genstatement(FILE *, struct db_customer *);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, aflag, psflag, warnf, sawfirst, didfirst, row;
	struct db_company *comp;
	struct db_customer *cusp;
	struct db_contact *conp;
	struct db_sledger *sp;
	struct db_invoice *ip;
	struct db_currency *curp;
	struct db_bankacct *bap;
	double total = 0.0;
	double ac, a3, a6, a9;
	time_t sdate, edate, now;
	dbow_conn *dbp;
	char *cname, cmd[512], *currlabel;
	FILE *fp;

	time(&now);
	edate = (now / 86400) * 86400 + 82799;
	aflag = verbose = psflag = warnf = 0;
	cname = NULL;
	sdate = now - DAYS(120);
	ac = a3 = a6 = a9 = 0.0;
	while ((i = getopt(argc, argv, "ac:ps:e:v")) != EOF) {
		switch (i) {
		case 'a':
			aflag = 1;
			break;

		case 'c':
			cname = optarg;
			break;

		case 'p':
			psflag = 1;
			break;

		case 's':
			if (parsedate(optarg, &sdate) < 0)
				usage();
			break;

		case 'e':
			if (parsedate(optarg, &edate) < 0)
				usage();
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1)
		usage();
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("statement", "cannot find company");
	if (verbose) {
		printf("Company: %s\n", comp->cname);
		printf("Start on %s", ctime(&sdate));
		printf("End on %s", ctime(&edate));
	}
	/*
	 * Do the report for each customer...
	 */
	if ((conp = contact_findcode(dbp, argv[optind])) == NULL) {
		fprintf(stderr,"?Cannot find code: %s\n", argv[optind]);
		exit(1);
	}
	cusp = db_findcustomerbycontact_id(dbp, conp->id);
	if (cusp == NULL) {
		fprintf(stderr, "?Contact is not a customer.\n");
		exit(1);
	}
	if ((curp = findcurrency(dbp, cusp->currency_id)) == NULL)
		berror("statement", "cannot find customer currency");
	currlabel = (curp->rofflabel != NULL) ? curp->rofflabel : "";
	/*
	 * Where is this statement going?
	 */
	if (psflag) {
		sprintf(cmd, "groff -s -t -Tps");
		if ((fp = popen(cmd, "w")) == NULL)
			berror("geninvoice", "popen");
	} else
		fp = stdout;
	/*
	 * Pull all the appropriate sales ledger records...
	 */
	fprintf(fp, ".ds StDate %s\n.ds StPX 1\n.ds StPY 1\n", prtime(&now));
	fprintf(fp, ".ds Curr %s\n", curp->lname);
	gentroffaddr(fp, dbp, cusp->contact_id, cusp->cperson_id);
	total = 0.0;
	sawfirst = didfirst = row = 0;
	bap = db_findbankacctbyid(dbp, 1);
	if (bap != NULL && bap->iban != NULL)
		fprintf(fp, ".ds IBAN %s\n", bap->iban);
	for (sp = findsledgerentries(dbp, cusp->id);
					sp != NULL && sp->sldate <= edate;
					sp = db_findsledgernext(dbp, sp)) {
		if (!sawfirst) {
			if (sdate < sp->sldate)
				sdate = sp->sldate;
			sawfirst = 1;
		}
		total += sp->amount;
		if (sp->sldate < sdate)
			continue;
		if (!didfirst) {
			fprintf(fp, ".ds ODate %s\n", prabbrev(&sdate));
			fprintf(fp, ".ds OBal %s\n", bal(total - sp->amount,
								currlabel));
			didfirst = 1;
		}
		fprintf(fp, ".ds L%02da %s\n", ++row, prabbrev((time_t *)&sp->sldate));
		if (sp->folio != NULL)
			fprintf(fp, ".ds L%02db %s\n", row, sp->folio);
		if (sp->descr != NULL)
			fprintf(fp, ".ds L%02dc %s\n", row, sp->descr);
		if (sp->amount < 0.0) {
			fprintf(fp, ".ds L%02dd %s\n", row, bal(-sp->amount,
									NULL));
		} else  {
			fprintf(fp, ".ds L%02de %s\n", row, bal(sp->amount,
									NULL));
		}
		fprintf(fp, ".ds L%02df %s\n", row, bal(total, NULL));
	}
	if (!didfirst)
		fprintf(fp, ".ds ODate %s\n.ds OBal 0.00\n", prabbrev(&sdate));
	fprintf(fp, ".ds CBal %s\n", bal(total, currlabel));
	fprintf(fp, ".ds CDate %s\n", prabbrev(&edate));
	for (ip = db_findinvoicebycustomer_id(dbp, cusp->id);
					ip != NULL;
					ip = db_findinvoicenext(dbp, ip)) {
		switch (ip->state) {
		case INVOICE_STATE_SENT:
			ac += ip->total;
			break;

		case INVOICE_STATE_LATE1:
		case INVOICE_STATE_LATE2:
			a3 += ip->total;
			break;

		case INVOICE_STATE_LATE3:
			warnf = 1;
			a6 += ip->total;
			break;

		case INVOICE_STATE_LATE4:
		case INVOICE_STATE_LATE5:
			warnf = 1;
			a9 += ip->total;
			break;
		}
	}
	if (ac > 0.0)
		fprintf(fp, ".ds ACurr %s\n", bal(ac, currlabel));
	if (a3 > 0.0)
		fprintf(fp, ".ds A30 %s\n", bal(a3, currlabel));
	if (a6 > 0.0)
		fprintf(fp, ".ds A60 %s\n", bal(a6, currlabel));
	if (a9 > 0.0)
		fprintf(fp, ".ds A90 %s\n", bal(a9, currlabel));
	fprintf(fp, ".ds Company %s\n", comp->tag);
	fprintf(fp, ".nr PrePrint %d\n", psflag ? 0 : 1);
	fprintf(fp, ".so %s.tmac\n", makepath("roff", "statement", NULL));
	fprintf(fp, ".TS\nbox;\nlb.\n");
	if (warnf) {
		fprintf(fp, "Please Note: Your account is currently delinquen");
		fprintf(fp, "t.\nPlease pay all outstanding invoices to preve");
		fprintf(fp, "nt an\ninterruption in service.\n");
	} else if (bap != NULL && bap->iban != NULL) {
		fprintf(fp, "Please Note: You can pay by Electronic Funds Tra");
		fprintf(fp, "nsfer\nto \\fI%s\\fB.  Please send an", bap->iban);
		fprintf(fp, "\nemail to \\fIaccounts@kalopa.net\\fB to confir");
		fprintf(fp, "m EFT payment.\n");
	} else {
		fprintf(fp, "If you wish to pay by Electronic Funds Transfer,");
		fprintf(fp, " please\ncontact the accounts department at \\fI");
		fprintf(fp, "accounts@kalopa.net\\fP\nfor further details.\n");
	}
	fprintf(fp, ".TE\n");
	if (psflag)
		pclose(fp);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: statement [-c #][-v][-s DATE][-e DATE] <custcode>\n");
	exit(2);
}
