#ident "Kalopa: $Id: sharecert.c,v 1.6 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: sharecert.c,v 1.6 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: sharecert.c,v $
 * Revision 1.6  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.4  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2008/01/15 15:37:40  dtynan
 * Fixed issues with new table names.
 *
 * Revision 1.2  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.1  2005/08/01 10:30:33  dtynan
 * New program to generate share certificate data.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

int	verbose;
int	psflag;

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname, *cp, cmd[512];
	dbow_conn *dbp;
	struct db_company *comp;
	struct db_sharecert *scp;
	struct tm *tmp;
	FILE *fp;

	psflag = verbose = 0;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:pv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'p':
			/*
			 * Generate postscript rather than troff.
			 */
			psflag = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if ((argc - optind) != 1)
		usage();
	/*
	 * Find the company that we're processing...
	 */
	loadconfig(cname);
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("sharecert", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	/*
	 * Now collect all the data...
	 */
	if ((i = atoi(argv[optind++])) < 1)
		berror("sharecert", "invalid share certificate number");
	if (verbose)
		printf("Searching for share cert #%d...\n", i);
	if ((scp = db_findsharecertbyid(dbp, i)) == NULL)
		berror("sharecert", "cannot find share certificate");
	/*
	 * Find out where to send the troff data.
	 */
	if (psflag) {
		sprintf(cmd, "groff -s -t -P-l -Tps");
		if ((fp = popen(cmd, "w")) == NULL)
			berror("sharecert", "popen");
	} else
		fp = stdout;
	/*
	 * Now start to generate the output...
	 */
	fprintf(fp, ".ds Company ");
	for (cp = comp->cname; (i = *cp) != '\0'; cp++) {
		if (islower(i))
			i = toupper(i);
		fputc(i, fp);
	}
	fprintf(fp, "\n.ds CertNo %d\n", scp->id);
	gentroffaddrline(fp, dbp, scp->contact_id);
	makenumberdigit(cmd, sizeof(cmd), scp->nshares);
	fprintf(fp, ".ds Number %s\n", cmd);
	makenumbertext(cmd, sizeof(cmd), scp->nshares);
	fprintf(fp, ".ds NumText%s\n", cmd);
	fprintf(fp, ".ds Price %.2f\n", scp->price);
	tmp = localtime((time_t *)&scp->sdate);
	fprintf(fp, ".ds Date %s day of ", prdate(tmp->tm_mday, 0, 0));
	fprintf(fp, "%s, ", prdate(0, tmp->tm_mon + 1, 0));
	fprintf(fp, "%s\n", prdate(0, 0, tmp->tm_year + 1900));
	fprintf(fp, ".so %s\n", makepath("roff", "sharecert.tmac", NULL));
	/*
	 * We're done!
	 */
	if (psflag)
		pclose(fp);
	db_sharecertfree(scp);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: sharecert [-c <comp>][-p][-v] <certno>\n");
	exit(2);
}
