#ident "Kalopa: $Id: calcdate.c,v 1.6 2008/01/15 18:10:44 dtynan Exp $"

/*
 * $Id: calcdate.c,v 1.6 2008/01/15 18:10:44 dtynan Exp $
 *
 * Copyright (c) 1996, Claddagh Films Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: calcdate.c,v $
 * Revision 1.6  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.5  2005/03/22 09:41:28  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.4  2003/11/08 16:10:13  dtynan
 * Added old accounting system functions.
 *
 * Revision 1.3  2003/04/09 17:36:52  dtynan
 * Coalesced all the different source trees and folded in new changes.
 *
 * Revision 1.2  1997/01/05 03:31:16  dtynan
 * Baseline check-in - most functionality is now present.
 *
 * Revision 1.1  1997/01/03  21:37:22  dtynan
 * Initial check-in.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "beanie.h"

void	usage(char *);

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, d, m, y, dflag, mflag, yflag;

	/*
	 * Initialize and get the command-line arguments.
	 */
	dflag = mflag = yflag = 0;
	while ((i = getopt(argc, argv, "dmy")) != EOF) {
		switch (i) {
		case 'd':
			dflag = 1;
			break;
		case 'm':
			mflag = 1;
			break;
		case 'y':
			yflag = 1;
			break;
		default:
			usage("invalid option");
			break;
		}
	}
	if ((argc - optind) != 4)
		usage("incorrect number of arguments");
	if (!dflag && !mflag && !yflag)
		dflag = mflag = yflag = 1;
	/*
	 * Determine the start day, month and year.
	 */
	d = atoi(argv[optind++]);
	if ((m = atoi(argv[optind++])) < 1 || m > 12)
		usage("specified month is out of range");
	if ((y = atoi(argv[optind++])) < 100)
		y += 1900;
	if (y > 2199 || y < 1990)
		usage("specified year is out of range");
	if (d < 1 || d > mdays(m, y))
		usage("specified day is out of range");
	calcdate(&d, &m, &y, argv[optind]);
	/*
	 * Print the results and exit.
	 */
	if (dflag)
		printf("%d ", d);
	if (mflag)
		printf("%d ", m);
	if (yflag)
		printf("%d", y);
	putchar('\n');
	exit(0);
}

/*
 *
 */
void
usage(char *umsg)
{
	fprintf(stderr, "?Error - %s.\n", umsg);
	fprintf(stderr, "Usage: calcdate [-d|-m|-y] day mon year off\n");
	exit(2);
}
