#ident "Kalopa: $Id: holidays.c,v 1.7 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: holidays.c,v 1.7 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: holidays.c,v $
 * Revision 1.7  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.6  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.5  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.4  2006/08/25 11:27:15  dtynan
 * Lots of changes as per the end-of-year.
 *
 * Revision 1.3  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.2  2005/08/02 20:28:07  dtynan
 * Fixed minor glitches with the Easter calculation, added the US holidays
 * (still need UK holidays) and added some more command-line options.
 *
 * Revision 1.1  2005/08/02 18:01:56  dtynan
 * Added new utility to calculate public holidays for a given year and
 * update the database accordingly.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"
#include "hacks.h"

#if HOME_COUNTRY == 101
/*
 * Holiday definitions for Ireland.  There are nine public holidays
 * a year.  Interestingly, St. Patricks Day isn't an official bank
 * holiday, although some places are closed.
 */
#define NEWYEARSDAY
#define GOODFRIDAY
#define EASTERMONDAY
#define MAYDAY
#define FIRSTMONJUN
#define FIRSTMONAUG
#define LASTMONOCT
#define CHRISTMAS
#define STEPHENSDAY
#endif

#if HOME_COUNTRY == 219
/*
 * Holiday definitions for the United States.  There are nine public
 * holidays a year as defined by the New York Stock Exchange
 * (www.nyse.com).  There are more federal holidays, but these are
 * the "official" ones.
 */
#define NEWYEARSDAY
#define MARTINLUTHERKING
#define WASHINGTONSBIRTHDAY
#define GOODFRIDAY
#define MEMORIALDAY
#define INDEPENDENCEDAY
#define LABORDAY
#define THANKSGIVING
#define CHRISTMAS
#endif

/*
 * The Brits refer to St. Stephens Day as Boxing Day.
 */
#ifdef BOXINGDAY
#define STEPHENSDAY
#endif

int	verbose;
int	testmode;

extern	int	optind, opterr;
extern	char	*optarg;

void	genholiday(time_t *, char *);
void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, year, g, c, h;
	char *cname;
	struct tm *tmp;
	time_t now;
	struct db_holiday *hp;
	dbow_conn *dbp;

	verbose = testmode = 0;
	/*
	 * Initialize the tm struct...
	 */
	time(&now);
	tmp = localtime(&now);
	year = tmp->tm_year + 1900;
	cname = NULL;
	while ((i = getopt(argc, argv, "c:n1234vy:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			/*
			 * Test mode doesn't make any changes to
			 * the database - useful for testing the
			 * veracity of the algorithms.
			 */
			testmode = 1;
			break;

		case '1':
		case '2':
		case '3':
		case '4':
			/*
			 * This is so we can run this out of an overnight
			 * script without having to calculate the year.
			 * In other words, calculate a future holiday year.
			 */
			year += (i - '0');
			break;

		case 'y':
			/*
			 * Specify a particular year.  We can't really do
			 * anything before 1971 (and why should we?) because
			 * lots of algorithms changed over the years.
			 * We assume things are good until the next
			 * millenium and frankly, I don't care if they
			 * aren't!
			 */
			if ((year = atoi(optarg)) < 1971 || year > 9999)
				usage();
			break;

		case 'v':
			/*
			 * Be a bit verbose about what's happening.
			 */
			verbose = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (loadconfig(cname) < 0)
		berror("holidays", "cannot load configuration file");
	if ((dbp = getdbase()) == NULL)
		berror("holidays", "cannot connect to database");
	/*
	 * Remove all the entries for this year already in the database.
	 */
	if (testmode)
		printf("TEST MODE.\n");
	for (hp = db_findholidayfirst(dbp); hp != NULL;
					hp = db_findholidaynext(dbp, hp)) {
		tmp = localtime((time_t *)&hp->hdate);
		if (tmp->tm_year + 1900 != year)
			continue;
		if (verbose)
			printf("Deleting %s (%d)\n", hp->descr, year);
		if (!testmode)
			db_deleteholidaybyid(dbp, hp->id);
	}
	/*
	 * This is *SO* country-specific.  Not much you can do about
	 * that, really.
	 */
	if (verbose)
		printf("Calculating public holidays in %d\n", year);
	tmp->tm_year = year - 1900;
#ifdef NEWYEARSDAY
	/*
	 * New Years Day (1st of January or first Monday if it
	 * falls on a weekend)
	 */
	tmp->tm_mday = 1;
	tmp->tm_mon = 0;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday == 0 || tmp->tm_wday == 6);
	genholiday(&now, "New Years Day");
#endif
#ifdef MARTINLUTHERKING
	/*
	 * Birthday of Martin Luther King.  Defined as the "third
	 * Monday in January."
	 */
	tmp->tm_mday = 1;
	tmp->tm_mon = 0;
	for (i = 0; i < 3; tmp->tm_mday++) {
		now = mktime(tmp);
		if (tmp->tm_wday == 1)
			i++;
	}
	genholiday(&now, "Martin Luther King Day");
#endif
#ifdef WASHINGTONSBIRTHDAY
	/*
	 * Washington's Birthday.  Originally February 22nd, now
	 * defined as the "third Monday in February."  Also
	 * known as President's Day.
	 */
	tmp->tm_mday = 1;
	tmp->tm_mon = 1;
	for (i = 0; i < 3; tmp->tm_mday++) {
		now = mktime(tmp);
		if (tmp->tm_wday == 1)
			i++;
	}
	genholiday(&now, "Washington's Birthday");
#endif
#ifdef STPATRICKSDAY
	/*
	 * St. Patricks Day (17th of March or first Monday if it
	 * falls on a weekend)
	 */
	tmp->tm_mday = 17;
	tmp->tm_mon = 2;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday == 0 || tmp->tm_wday == 6);
	genholiday(&now, "St. Patrick's Day");
#endif
	/*
	 * Good Friday & Easter Monday...
	 * Easter is tricky to calculate.  It's related to moon phases,
	 * the spring equinox and other things.  We use the John Horton
	 * Conway algorithm which is extremely effective.  Note the
	 * following about the Conway algorith: "Except when the
	 * formula gives Apr 19 you should take Apr 18, and when
	 * it gives Apr 18 and G>=12 you should take Apr 17.  Easter
	 * is then the following Sunday, since Easter always falls
	 * on the next Sunday that is _strictly later_ than the
	 * Paschal Full Moon."
	 */
	g = (year % 19) + 1;
	h = year / 100;
	c = -h + (h/4) + (8*(h+11)/25);
	i = 50 - ((11*g+c) % 30);
	if (i <= 31) {
		/*
		 * Easter is in March.
		 */
		tmp->tm_mday = i;
		tmp->tm_mon = 2;
	} else {
		/*
		 * Easter is in April.
		 */
		if ((i -= 31) == 19)
			i = 18;
		else if (i == 18 && g >= 12)
			i = 17;
		tmp->tm_mday = i;
		tmp->tm_mon = 3;
	}
	/*
	 * Find the next Sunday after this date.
	 */
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday != 0);
	/*
	 * Now back up two days to get to Good Friday.  Keep the
	 * current day because tm_mday now points to Easter Monday.
	 */
	i = tmp->tm_mday;
	tmp->tm_mday -= 3;
	now = mktime(tmp);
#ifdef GOODFRIDAY
	genholiday(&now, "Good Friday");
#endif
#ifdef EASTERMONDAY
	tmp->tm_mday = i;
	now = mktime(tmp);
	genholiday(&now, "Easter Monday");
#endif
#ifdef MAYDAY
	/*
	 * May Bank Holiday (first Monday in May)
	 */
	tmp->tm_mday = 1;
	tmp->tm_mon = 4;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday != 1);
	genholiday(&now, "May Bank Holiday");
#endif
#ifdef MEMORIALDAY
	/*
	 * US Memorial Day (last Monday in May)
	 */
	tmp->tm_mday = 31;
	tmp->tm_mon = 4;
	do {
		now = mktime(tmp);
		tmp->tm_mday--;
	} while (tmp->tm_wday != 1);
	genholiday(&now, "Memorial Day");
#endif
#ifdef FIRSTMONJUN
	/*
	 * June Bank Holiday (first Monday in June)
	 */
	tmp->tm_mday = 1;
	tmp->tm_mon = 5;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday != 1);
	genholiday(&now, "June Bank Holiday");
#endif
#ifdef INDEPENDENCEDAY
	/*
	 * US Independence Day, July 4 (or first Monday
	 * if it falls on a weekend).
	 */
	tmp->tm_mday = 4;
	tmp->tm_mon = 6;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday == 0 || tmp->tm_wday == 6);
	genholiday(&now, "Independence Day");
#endif
#ifdef FIRSTMONAUG
	/*
	 * August Bank Holiday (first Monday in August)
	 */
	tmp->tm_mday = 1;
	tmp->tm_mon = 7;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday != 1);
	genholiday(&now, "August Bank Holiday");
#endif
#ifdef LABORDAY
	/*
	 * US LAbor Day (first Monday in September)
	 */
	tmp->tm_mday = 1;
	tmp->tm_mon = 8;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday != 1);
	genholiday(&now, "Labor Day");
#endif
#ifdef LASTMONOCT
	/*
	 * October Bank Holiday (last Monday in October)
	 */
	tmp->tm_mday = 31;
	tmp->tm_mon = 9;
	do {
		now = mktime(tmp);
		tmp->tm_mday--;
	} while (tmp->tm_wday != 1);
	genholiday(&now, "October Bank Holiday");
#endif
#ifdef THANKSGIVING
	/*
	 * Thanksgiving Day (fourth Thursday in November).
	 */
	tmp->tm_mday = 1;
	tmp->tm_mon = 10;
	for (i = 0; i < 4; tmp->tm_mday++) {
		now = mktime(tmp);
		if (tmp->tm_wday == 4)
			i++;
	}
	genholiday(&now, "Thanksgiving");
#endif
	/*
	 * Christmas Day (25th of December or first Monday if it falls
	 * on a Weekend) and St. Stephen's Day (26th of December or
	 * first Monday/Tuesday if it falls on a weekend).
	 */
	tmp->tm_mday = 25;
	tmp->tm_mon = 11;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday == 0 || tmp->tm_wday == 6);
	i = tmp->tm_mday;
#ifdef CHRISTMAS
	genholiday(&now, "Christmas Day");
#endif
#ifdef STEPHENSDAY
	tmp->tm_mday = i;
	do {
		now = mktime(tmp);
		tmp->tm_mday++;
	} while (tmp->tm_wday == 0 || tmp->tm_wday == 6);
	genholiday(&now, "St. Stephen's Day");
#endif
	exit(0);
}

/*
 *
 */
void
genholiday(time_t *when, char *name)
{
	struct db_holiday *hp;

	if (verbose)
		printf("%30s:  %s", name, ctime(when));
	if ((hp = db_holidayalloc()) == NULL)
		return;
	hp->hdate = *when;
	hp->descr = strdup(name);
	if (!testmode)
		db_insertholiday(getdbase(), hp);
	db_holidayfree(hp);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: holidays [-c <comp>][-n][-1234][-y year][-v]\n");
	exit(2);
}
