#ident "Kalopa: $Id: mkletter.c,v 1.6 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: mkletter.c,v 1.6 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: mkletter.c,v $
 * Revision 1.6  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.4  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.2  2005/03/22 09:41:28  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.1  2005/03/05 19:22:52  dtynan
 * Added a utility to generate a template letter.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <paths.h>

#include "beanie.h"
#include "hacks.h"

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, row;
	char *cname, *ccode, *dname, *ref, fname[24], *cp, *xp;
	dbow_conn *dbp;
	struct stat statb;
	struct tm *tmp;
	struct db_company *comp;
	struct db_contact *contp;
	struct db_country *cyp;
	time_t when;
	FILE *fp;

	opterr = 0;
	time(&when);
	cname = ref = NULL;
	while ((i = getopt(argc, argv, "c:d:r:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'd':
			if (parsedate(optarg, &when) < 0)
				usage();
			break;

		case 'r':
			ref = optarg;
			break;

		default:
			usage();
		}
	}
	/*
	 * Find the company that we're processing...
	 */
	if (loadconfig(cname) < 0)
		berror("mkletter", "cannot load configuration file");
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("mkletter", "cannot find company");
	/*
	 * Find the contact name.
	 */
	if ((argc - optind) != 1)
		usage();
	ccode = argv[optind];
	if ((contp = contact_findcode(dbp, ccode)) == NULL) {
		fprintf(stderr, "?Error - cannot find contact: %s\n", ccode);
		exit(1);
	}
	for (cp = contp->ccode, xp = fname; (i = *cp++) != '\0';) {
		if (isupper(i))
			i = tolower(i);
		*xp++ = i;
	}
	*xp++ = '-';
	*xp = '\0';
	tmp = localtime(&when);
	strftime(xp, sizeof(fname) - 12, "%Y%m%dA.r", tmp);
	dname = makepath("letters", fname, NULL);
	xp = dname + strlen(dname) - 3;
	for (row = 0; row < 27; row++) {
		if (row == 0)
			strcpy(xp, ".r");
		else
			sprintf(xp, "%c.r", row + 'a' - 1);
		if (stat(dname, &statb) < 0)
			break;
	}
	if (row == 27) {
		fprintf(stderr, "?Error - too many letters today!\n");
		exit(1);
	}
	if ((fp = fopen(dname, "w")) == NULL) {
		fprintf(stderr, "?mkletter: ");
		perror(dname);
		exit(1);
	}
	fprintf(fp, ".ds Date %s\n", prdate(tmp->tm_mday,
						tmp->tm_mon + 1,
						tmp->tm_year + 1900));
	row = 0;
#if 0
	if (cusp->cperson_id > 0) {
		if ((cpp = person_findbyid(dbp, cusp->cperson_id)) == NULL)
			berror("geninvoice", "cannot find person record");
		if (cpp->fname == NULL)
			fprintf(fp, ".ds Name %s\n", cpp->lname);
		else
			fprintf(fp, ".ds Name %s %s\n", cpp->fname, cpp->lname);
		row++;
		if (cpp->title != NULL)
			fprintf(fp, ".ds Addr%d %s\n", row++, cpp->title);
	}
	if ((cp = contact_findbyid(dbp, cusp->contact_id)) == NULL)
		berror("geninvoice", "cannot find contact data for invoice");
#endif
	if (row == 0)
		fprintf(fp, ".ds Name %s\n", orgname(contp));
	else
		fprintf(fp, ".ds Addr%d %s\n", row, orgname(contp));
	row++;
	cyp = db_findcountrybyid(dbp, contp->id);
	if (contp->a1 != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, contp->a1);
	if (contp->a2 != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, contp->a2);
	if (contp->a3 != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, contp->a3);
	if (contp->city != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, contp->city);
	if (contp->state_county != NULL) {
		fprintf(fp, ".ds Addr%d %s", row++, contp->state_county);
		if (contp->postcode_zip != NULL)
			fprintf(fp, " %s", contp->postcode_zip);
		fputc('\n', fp);
	}
	if (cyp != NULL)
		fprintf(fp, ".ds Addr%d %s\n", row++, cyp->cname);
	fprintf(fp, ".ds Greet Dear Sir/Madam,\n");
	if (ref != NULL)
		fprintf(fp, ".ds Ref %s\n", ref);
	else
		fprintf(fp, ".ds Ref\n");
	fprintf(fp, ".\\\"\n");
	fprintf(fp, ".nr Company %d\n", comp->id);
	fprintf(fp, ".so %s\n", makepath("roff", "letter.tmac", NULL));
	fprintf(fp, ".\\\"\n");
	fprintf(fp, "<Message Body Here>\n");
	fprintf(fp, ".SN\n");
	fclose(fp);
	sprintf(fname, "+%d", row + 8);
	printf("fname: %s\n", fname);
	printf("dname: %s\n", dname);
	execl(_PATH_VI, "vi", fname, dname, NULL);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: mkletter [-c <comp>] [-d date] [-r ref] <ccode>\n");
	exit(2);
}
