#ident "$Id: mklists.c,v 1.6 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: mklists.c,v 1.6 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: mklists.c,v $
 * Revision 1.6  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.4  2008/01/16 11:02:29  dtynan
 * Fixed broken tag for company struct.
 *
 * Revision 1.3  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.2  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.1  2005/09/21 18:10:28  dtynan
 * Extensive changes prior to first public release, including an
 * import/export mechanism.
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "beanie.h"
#include "hacks.h"

void	usage();

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, mode, verbose;
	char *cname, *ofile;
	dbow_conn *dbp;
	FILE *fp = stdout;
	struct db_company *comp;
	struct db_customer *cp;
	struct db_contact *conp;
	struct db_contact_person *cpp;

	/*
	 * Handle start-up options...
	 */
	opterr = verbose = mode = 0;
	cname = ofile = NULL;
	while ((i = getopt(argc, argv, "c:vo:-:")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'v':
			verbose = 1;
			break;

		case 'o':
			ofile = optarg;
			break;

		case '-':
			if (strcmp(optarg, "customers") == 0)
				mode = 1;
			else
				usage();
		default:
			usage();
			break;
		}
	}
	/*
	 * Find the company that we're processing...
	 */
	if (loadconfig(cname) < 0)
		berror("mklists", "cannot load configuration file");
	dbp = getdbase();
	if ((comp = db_findcompanyfirst(dbp)) == NULL)
		berror("mklists", "cannot find company");
	if (verbose)
		printf("Company: %s\n", comp->cname);
	/*
	 * Open the output channel.
	 */
	if (ofile != NULL && (fp = fopen(ofile, "w")) == NULL) {
		perror(ofile);
		exit(1);
	}
	/*
	 * Do header block.
	 */
	fprintf(fp, "#\n# WARNING: Autogenerated from accounting data.\n#\n");
	/*
	 * Pull all the customer records...
	 */
	for (cp = db_findcustomerfirst(dbp); cp != NULL;
					cp = db_findcustomernext(dbp, cp)) {
		if (cp->cperson_id > 0 && (cpp = person_findbyid(dbp,
						cp->cperson_id)) != NULL &&
						cpp->email != NULL)
			fprintf(fp, "CP%d:%s\n", cp->id, cpp->email);
		else if (cp->contact_id > 0 && (conp = contact_findbyid(dbp,
						cp->contact_id)) != NULL) {
			if (conp->email == NULL) {
				fprintf(fp, "# No details for %s\n",
								orgname(conp));
			} else
				fprintf(fp, "%s\n", conp->email);
		} else {
			fprintf(stderr, "?Error - no contact details for %d.\n",
						cp->id);
		}
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: mklists [-c <comp>][-v][-o <file>] {--customers}\n");
	exit(2);
}
