#ident "$Id: newcompany.c,v 1.5 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: newcompany.c,v 1.5 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: newcompany.c,v $
 * Revision 1.5  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.4  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.3  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.2  2005/08/12 18:55:17  dtynan
 * Added a new shell script to produce the SQL for creating a new
 * company.
 *
 * Revision 1.1  2005/03/30 11:25:59  dtynan
 * New program for creating a new company.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "beanie.h"

#define PROGNAME	"newcompany"

int	noeffect;
int	verbose;

extern	char	sql_data[];

extern	int	optind, opterr;
extern	char	*optarg;

void	error(const char *);

#define INSTALL		"\n\n\
For more information, read the INSTALL document.\n"

#define USAGE	"\n\
Usage:\tnewcompany [-v][-n] -c <short_name>\n\
\n\
Please specify the short-name handle of the company (such as 'fedex') using\n\
the '-c' option, similar to the following:\n\
\n\
./newcompany -c fedex\n\
\n\
You can use the '-v' option to print verbose output during execution and the\n\
'-n' option to do a dummy-run.  As a general rule, passing the '-n' option to\n\
any beanie application will suppress any actual database or update activity."

#define MESSAGE1	"\
I was unable to find a database configuration for the named company.  The\n\
first step in creating a new company is to create a MySQL database for it.\n\
You must then create a .beanierc file in your home directory (see the INSTALL\n\
file for a sample file).  You will need the hostname where the database is,\n\
the username and password for the accounting system to access the database,\n\
and the name of the database."

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i;
	char *cname;
	dbow_conn *dbp;
	struct db_company *comp;

	/*
	 * Process command-line options.
	 */
	cname = NULL;
	opterr = noeffect = verbose = 0;
	while ((i = getopt(argc, argv, "c:nv")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 'n':
			noeffect = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		default:
			error(USAGE);
			break;
		}
	}
	if (cname == NULL)
		error(USAGE);
	if (loadconfig(cname) < 0)
		fprintf(stderr, MESSAGE1);
	dbp = getdbase();
	printf("[%s]\n", sql_data);
	if (dbow_query(dbp, sql_data) < 0) {
		fprintf(stderr, "?Error - cannot generate SQL...\n");
		exit(1);
	}
	exit(0);
}

/*
 *
 */
void
error(const char *msg)
{
	fprintf(stderr, msg);
	fprintf(stderr, INSTALL);
	exit(1);
}
