#ident "Kalopa: $Id: prnomacct.c,v 1.6 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: prnomacct.c,v 1.6 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: prnomacct.c,v $
 * Revision 1.6  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.5  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.4  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.3  2008/01/15 15:36:02  dtynan
 * Changed output format slightly.
 *
 * Revision 1.2  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.1  2005/02/22 12:14:52  dtynan
 * Made 'prnomacct' an executable.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <syslog.h>
#include <time.h>
#include <string.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

#define PROGNAME	"prnomacct"

struct	db_nomacct	*findnomaccts(dbow_conn *);

struct	nacct	{
	int	number;
	char	*type;
	char	*name;
} nacct[110];

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
void
dump(int col)
{
	int i;
	static int page = 0;
	struct nacct *ap;

	page++;
	printf(".sp 1.5c\n.TS\ncenter box;\n");
	printf("cb cb cb | cb cb cb\nl li lb | l li lb.\n");
	printf("Acct.\tT\tDescription\tAcct.\tT\tDescription\n_\n");
	for (i = 0; i < col/2; i++) {
		ap = &nacct[i];
		if (ap->name != NULL) {
			printf("%04d\t%s\t%s\t", ap->number, ap->type,
							ap->name);
		} else
			printf("\t\t\t");
		ap = &nacct[i + col/2];
		if (ap->name != NULL) {
			printf("%04d\t%s\t%s\t", ap->number, ap->type,
							ap->name);
		}
		printf("\n");
	}
	printf(".TE\n.bp\n");
}

/*
 *
 */
int
main(int argc, char *argv[])
{
	int i, troff;
	char *cname;
	struct db_nomacct *np;
	struct nacct *ap;
	dbow_conn *dbp;

	/*
	 * Process command-line options.
	 */
	cname = NULL;
	opterr = troff = 0;
	while ((i = getopt(argc, argv, "c:t")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		case 't':
			troff = 1;
			break;

		default:
			usage();
			break;
		}
	}
	if (loadconfig(cname) < 0)
		berror(PROGNAME, "cannot load configuration file");
	dbp = getdbase();
	/*
	 * Start printing the nominal account names...
	 */
	for (ap = nacct; ap < &nacct[110]; ap++)
		ap->name = NULL;
	for (i = 0, np = findnomaccts(dbp); np != NULL;
					np = db_findnomacctnext(dbp, np)) {
		if (i >= 120) {
			dump(i);
			i = 0;
		}
		ap = &nacct[i];
		ap->number = np->id;
		switch (np->atype) {
		case INCOME:
			ap->type = "IN";
			break;

		case EXPENSE:
			ap->type = "EX";
			break;

		case ASSET_FIXED:
			ap->type = "FA";
			break;

		case ASSET_CURRENT:
			ap->type = "CA";
			break;

		case LIABILITY_CURRENT:
			ap->type = "CL";
			break;

		case LIABILITY_LONG:
			ap->type = "LL";
			break;

		case OEQUITY:
			ap->type = "OE";
			break;

		case COSTOFSALES:
			ap->type = "CS";
			break;
		}
		ap->name = np->aname;
		if (troff) {
			i++;
			np->aname = NULL;
		} else {
			printf("%04d\t%s\t%s\n", ap->number, ap->type,
							ap->name);
		}
	}
	if (i > 0)
		dump(i);
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: %s [-t][-c <comp>]\n", PROGNAME);
	exit(2);
}
