#ident "Kalopa: $Id: xeparse.c,v 1.11 2008/10/15 15:11:14 dtynan Exp $"

/*
 * $Id: xeparse.c,v 1.11 2008/10/15 15:11:14 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: xeparse.c,v $
 * Revision 1.11  2008/10/15 15:11:14  dtynan
 * Mammoth change to convert from old-style database configuration to new
 * schema where an RC file defines the database access.
 *
 * Revision 1.10  2008/10/13 20:59:38  dtynan
 * Extensive changes (part 1) for change to database configuration.
 *
 * Revision 1.9  2008/01/15 18:10:44  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.8  2006/07/31 10:17:52  dtynan
 * Massive changeover to a more Ruby/Rails primary key naming convention.
 *
 * Revision 1.7  2005/03/22 09:41:28  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.6  2004/12/09 20:23:40  dtynan
 * Cleaned up include mechanisms.
 *
 * Revision 1.5  2004/08/04 13:16:56  dtynan
 * Changed handling of beanie_init.
 *
 * Revision 1.4  2004/05/12 15:21:23  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.3  2003/11/08 16:36:06  dtynan
 * Fixed a couple of bugs in the date processing.
 *
 * Revision 1.2  2003/11/08 15:06:52  dtynan
 * Updated to use new beanie_init function.
 *
 * Revision 1.1  2003/11/07 20:14:44  dtynan
 * Program to take an XE (www.xe.net) currency email and update the
 * currency database accordingly.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <syslog.h>
#include <time.h>
#include <string.h>
#include <ctype.h>

#include "beanie.h"
#include "hacks.h"

char	input[512];
time_t	when;

#define PROGNAME	"xeparse"

extern	int	optind, opterr;
extern	char	*optarg;

void	usage();

/*
 *
 */
time_t
parsetime(char *cp)
{
	struct tm *tmp;
	time_t now;

	while (isspace(*cp))
		cp++;
	time(&now);
	if (strlen(cp) < 19 || (tmp = localtime(&now)) == NULL)
		return(0);
	cp[4] = cp[7] = cp[10] = cp[13] = cp[16] = cp[19] = '\0';
	tmp->tm_wday = tmp->tm_yday = tmp->tm_gmtoff = 0;
	tmp->tm_isdst = -1;
	tmp->tm_year = atoi(cp +  0) - 1900;
	tmp->tm_mon  = atoi(cp +  5) - 1;
	tmp->tm_mday = atoi(cp +  8);
	tmp->tm_hour = atoi(cp + 11);
	tmp->tm_min  = atoi(cp + 14);
	tmp->tm_sec  = atoi(cp + 17);
	if ((now = mktime(tmp)) < 0)
		return(0);
	return(now);
}

/*
 *
 */
void
process(char *cp)
{
	int i;
	char ccode[4], *cname;
	struct db_currency *curp;
	double val;
	dbow_conn *dbp = getdbase();

	for (i = 0; i < 3; i++)
		ccode[i] = *cp++;
	ccode[i] = '\0';
	cp++;
	cname = cp;
	cp += 30;
	while (isspace(*cp))
		*cp-- = '\0';
	cp = cname + 53;
	while (isspace(*cp))
		cp++;
	val = atof(cp);
	if ((curp = db_findcurrencybysname(dbp, ccode)) == NULL)
		return;
	curp->rate = val;
	curp->cdate = when;
	db_updatecurrencybyid(dbp, curp, curp->id);
	db_currencyfree(curp);
}

/*
 * And so It begins...
 */
int
main(int argc, char *argv[])
{
	int i, state;
	char *cname, *cp;

	/*
	 * Process command-line options.
	 */
	cname = NULL;
	opterr = 0;
	while ((i = getopt(argc, argv, "c:t")) != EOF) {
		switch (i) {
		case 'c':
			cname = optarg;
			break;

		default:
			usage();
			break;
		}
	}
	if (loadconfig(cname) < 0)
		berror(PROGNAME, "cannot load configuration file");
	state = 0;
	while (fgets(input, sizeof(input)-2, stdin) != NULL) {
		if ((cp = strpbrk(input, "\r\n")) != NULL)
			*cp = '\0';
		cp = input;
		switch (state) {
		case 0:
			if (*cp == '\0')
				state = 1;
			break;

		case 1:
			if (strncasecmp("rates as of", cp, 11) != 0)
				break;
			state = 2;
			when = parsetime(cp + 11);
			sprintf(input, "currency update for %s", ctime(&when));
			syslog(LOG_INFO, "%s: %s", PROGNAME, input);
			break;

		case 2:
			if (strncmp("==========", cp, 10) == 0)
				state = 3;
			break;

		case 3:
			if (*cp == '\0') {
				state = 4;
				break;
			}
			process(cp);
			break;
		}
	}
	exit(0);
}

/*
 *
 */
void
usage()
{
	fprintf(stderr, "Usage: %s [-c <comp>]\n", PROGNAME);
	exit(2);
}
